/******************************************************************
   IP*Works! SSL V9 C++ Edition
   Copyright (c) 2013 /n software inc. - All rights reserved.
*******************************************************************/

#ifndef _IPWORKSSSL_FTPS_H_
#define _IPWORKSSSL_FTPS_H_

#define IPWORKSSSL_ONLY_TYPES
#include "ipworksssl.h"
#include "ipworksssl.key"

//FirewallTypes
#define FW_NONE                                            0
#define FW_TUNNEL                                          1
#define FW_SOCKS4                                          2
#define FW_SOCKS5                                          3

//CertStoreTypes
#define CST_USER                                           0
#define CST_MACHINE                                        1
#define CST_PFXFILE                                        2
#define CST_PFXBLOB                                        3
#define CST_JKSFILE                                        4
#define CST_JKSBLOB                                        5
#define CST_PEMKEY_FILE                                    6
#define CST_PEMKEY_BLOB                                    7
#define CST_PUBLIC_KEY_FILE                                8
#define CST_PUBLIC_KEY_BLOB                                9
#define CST_SSHPUBLIC_KEY_BLOB                             10
#define CST_P7BFILE                                        11
#define CST_P7BBLOB                                        12
#define CST_SSHPUBLIC_KEY_FILE                             13

//TSSLStartMode
#define SSL_AUTOMATIC                                      0
#define SSL_IMPLICIT                                       1
#define SSL_EXPLICIT                                       2
#define SSL_NONE                                           3

//TFTPTransferModes
#define TM_DEFAULT                                         0
#define TM_ASCII                                           1
#define TM_BINARY                                          2


extern "C" void* IPWORKSSSL_CALL IPWorksSSL_FTPS_Create(PIPWORKSSSL_CALLBACK lpSink, void *lpContext, char *lpOemKey);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_FTPS_Destroy(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_FTPS_CheckIndex(void *lpObj, int propid, int arridx);
extern "C" void* IPWORKSSSL_CALL IPWorksSSL_FTPS_Get(void *lpObj, int propid, int arridx, int *lpcbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_FTPS_Set(void *lpObj, int propid, int arridx, const void *val, int cbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_FTPS_Do(void *lpObj, int methid, int cparam, void *param[], int cbparam[]);
extern "C" char* IPWORKSSSL_CALL IPWorksSSL_FTPS_GetLastError(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_FTPS_GetLastErrorCode(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_FTPS_StaticInit(void *hInst);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_FTPS_StaticDestroy();

#ifdef WIN32
#include <windows.h>
#pragma warning(disable:4311) 
#pragma warning(disable:4312) 
#endif

typedef struct {
  const char* ConnectionEvent;
  int StatusCode;
  const char* Description;
  int reserved;
} FTPSConnectionStatusEventParams;

typedef struct {
  const char* DirEntry;
  const char* FileName;
  int IsDir;
  ns_int64 *pFileSize;
  const char* FileTime;
  int reserved;
} FTPSDirListEventParams;

typedef struct {
  int Direction;
  int reserved;
} FTPSEndTransferEventParams;

typedef struct {
  int ErrorCode;
  const char* Description;
  int reserved;
} FTPSErrorEventParams;

typedef struct {
  int Direction;
  const char* Message;
  int reserved;
} FTPSPITrailEventParams;

typedef struct {
  const char* CertEncoded;
  const char* CertSubject;
  const char* CertIssuer;
  const char* Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} FTPSSSLServerAuthenticationEventParams;

typedef struct {
  const char* Message;
  int reserved;
} FTPSSSLStatusEventParams;

typedef struct {
  int Direction;
  int reserved;
} FTPSStartTransferEventParams;

typedef struct {
  int Direction;
  ns_int64 *pBytesTransferred;
  int PercentDone;
  const char* Text;
  int lenText;
  int reserved;
} FTPSTransferEventParams;



class FTPS {
  
  public: //events
  
    virtual int FireConnectionStatus(FTPSConnectionStatusEventParams *e) {return 0;}
    virtual int FireDirList(FTPSDirListEventParams *e) {return 0;}
    virtual int FireEndTransfer(FTPSEndTransferEventParams *e) {return 0;}
    virtual int FireError(FTPSErrorEventParams *e) {return 0;}
    virtual int FirePITrail(FTPSPITrailEventParams *e) {return 0;}
    virtual int FireSSLServerAuthentication(FTPSSSLServerAuthenticationEventParams *e) {return 0;}
    virtual int FireSSLStatus(FTPSSSLStatusEventParams *e) {return 0;}
    virtual int FireStartTransfer(FTPSStartTransferEventParams *e) {return 0;}
    virtual int FireTransfer(FTPSTransferEventParams *e) {return 0;}


  protected:

    void *m_pObj;
    
    static int IPWORKSSSL_CALL FTPSEventSink(void *lpObj, int event_id, int cparam, void *param[], int cbparam[]) {
      int ret_code = 0;
      if (event_id > 10000) return ((FTPS*)lpObj)->FTPSEventSinkW(event_id - 10000, cparam, param, cbparam);
      switch (event_id) {
         case 1: {
            FTPSConnectionStatusEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]),  0};
            ret_code = ((FTPS*)lpObj)->FireConnectionStatus(&e);
            break;
         }
         case 2: {
            FTPSDirListEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (ns_int64*)IPS64CAST(param[3]), (char*)IPS64CAST(param[4]),  0};
            ret_code = ((FTPS*)lpObj)->FireDirList(&e);
            break;
         }
         case 3: {
            FTPSEndTransferEventParams e = {(int)IPS64CAST(param[0]),  0};
            ret_code = ((FTPS*)lpObj)->FireEndTransfer(&e);
            break;
         }
         case 4: {
            FTPSErrorEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((FTPS*)lpObj)->FireError(&e);
            break;
         }
         case 5: {
            FTPSPITrailEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((FTPS*)lpObj)->FirePITrail(&e);
            break;
         }
         case 6: {
            FTPSSSLServerAuthenticationEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = ((FTPS*)lpObj)->FireSSLServerAuthentication(&e);
            param[4] = (void*)IPS64CAST(e.Accept);
            break;
         }
         case 7: {
            FTPSSSLStatusEventParams e = {(char*)IPS64CAST(param[0]),  0};
            ret_code = ((FTPS*)lpObj)->FireSSLStatus(&e);
            break;
         }
         case 8: {
            FTPSStartTransferEventParams e = {(int)IPS64CAST(param[0]),  0};
            ret_code = ((FTPS*)lpObj)->FireStartTransfer(&e);
            break;
         }
         case 9: {
            FTPSTransferEventParams e = {(int)IPS64CAST(param[0]), (ns_int64*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (int)IPS64CAST(cbparam[3]),  0};
            ret_code = ((FTPS*)lpObj)->FireTransfer(&e);
            break;
         }

      }
      return ret_code;
    }

    virtual int FTPSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {return 0;}

  public:

    FTPS(char *lpOemKey = (char*)IPWORKSSSL_OEMKEY_17) {
      m_pObj = IPWorksSSL_FTPS_Create(FTPSEventSink, (void*)this, (char*)lpOemKey);
    }

    virtual ~FTPS() {
      IPWorksSSL_FTPS_Destroy(m_pObj);
    }

  public:

    inline char *GetLastError() {
      return IPWorksSSL_FTPS_GetLastError(m_pObj);
    }
    
    inline int GetLastErrorCode() {
      return IPWorksSSL_FTPS_GetLastErrorCode(m_pObj);
    }

    inline char *VERSION() {
      return (char*)IPWorksSSL_FTPS_Get(m_pObj, 0, 0, 0);
    }

  public: //properties

    inline char* GetAccount() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 1, 0, 0);
      return (char*)val;
    }

    inline int SetAccount(const char *lpAccount) {
      return IPWorksSSL_FTPS_Set(m_pObj, 1, 0, (void*)lpAccount, 0);
    }


    inline int SetCommand(const char *lpCommand) {
      return IPWorksSSL_FTPS_Set(m_pObj, 2, 0, (void*)lpCommand, 0);
    }

    inline int GetConnected() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 3, 0, 0);
      return (int)(long)val;
    }
    inline int SetConnected(int bConnected) {
      void* val = (void*)IPS64CAST(bConnected);
      return IPWorksSSL_FTPS_Set(m_pObj, 3, 0, val, 0);
    }
    inline int GetDirListCount() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 4, 0, 0);
      return (int)(long)val;
    }

    inline char* GetDirListEntry(int iEntryIndex) {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 5, iEntryIndex, 0);
      return (char*)val;
    }


    inline char* GetDirListFileName(int iEntryIndex) {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 6, iEntryIndex, 0);
      return (char*)val;
    }


    inline ns_int64 GetDirListFileSize(int iEntryIndex) {
      ns_int64 *pval = (ns_int64*)IPWorksSSL_FTPS_Get(m_pObj, 7, iEntryIndex, 0);
      return *pval;
    }


    inline char* GetDirListFileTime(int iEntryIndex) {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 8, iEntryIndex, 0);
      return (char*)val;
    }


    inline int GetDirListIsDir(int iEntryIndex) {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 9, iEntryIndex, 0);
      return (int)(long)val;
    }

    inline int GetFileExists() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 10, 0, 0);
      return (int)(long)val;
    }

    inline ns_int64 GetFileSize() {
      ns_int64 *pval = (ns_int64*)IPWorksSSL_FTPS_Get(m_pObj, 11, 0, 0);
      return *pval;
    }


    inline char* GetFileTime() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 12, 0, 0);
      return (char*)val;
    }


    inline int GetFirewallAutoDetect() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 13, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallAutoDetect(int bFirewallAutoDetect) {
      void* val = (void*)IPS64CAST(bFirewallAutoDetect);
      return IPWorksSSL_FTPS_Set(m_pObj, 13, 0, val, 0);
    }
    inline int GetFirewallType() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 14, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallType(int iFirewallType) {
      void* val = (void*)IPS64CAST(iFirewallType);
      return IPWorksSSL_FTPS_Set(m_pObj, 14, 0, val, 0);
    }
    inline char* GetFirewallHost() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 15, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallHost(const char *lpFirewallHost) {
      return IPWorksSSL_FTPS_Set(m_pObj, 15, 0, (void*)lpFirewallHost, 0);
    }

    inline char* GetFirewallPassword() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 16, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallPassword(const char *lpFirewallPassword) {
      return IPWorksSSL_FTPS_Set(m_pObj, 16, 0, (void*)lpFirewallPassword, 0);
    }

    inline int GetFirewallPort() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 17, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallPort(int lFirewallPort) {
      void* val = (void*)IPS64CAST(lFirewallPort);
      return IPWorksSSL_FTPS_Set(m_pObj, 17, 0, val, 0);
    }
    inline char* GetFirewallUser() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 18, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallUser(const char *lpFirewallUser) {
      return IPWorksSSL_FTPS_Set(m_pObj, 18, 0, (void*)lpFirewallUser, 0);
    }

    inline int GetIdle() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 19, 0, 0);
      return (int)(long)val;
    }

    inline char* GetLastReply() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 20, 0, 0);
      return (char*)val;
    }


    inline char* GetLocalFile() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 21, 0, 0);
      return (char*)val;
    }

    inline int SetLocalFile(const char *lpLocalFile) {
      return IPWorksSSL_FTPS_Set(m_pObj, 21, 0, (void*)lpLocalFile, 0);
    }

    inline char* GetLocalHost() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 22, 0, 0);
      return (char*)val;
    }

    inline int SetLocalHost(const char *lpLocalHost) {
      return IPWorksSSL_FTPS_Set(m_pObj, 22, 0, (void*)lpLocalHost, 0);
    }

    inline int GetOverwrite() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 23, 0, 0);
      return (int)(long)val;
    }
    inline int SetOverwrite(int bOverwrite) {
      void* val = (void*)IPS64CAST(bOverwrite);
      return IPWorksSSL_FTPS_Set(m_pObj, 23, 0, val, 0);
    }
    inline int GetPassive() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 24, 0, 0);
      return (int)(long)val;
    }
    inline int SetPassive(int bPassive) {
      void* val = (void*)IPS64CAST(bPassive);
      return IPWorksSSL_FTPS_Set(m_pObj, 24, 0, val, 0);
    }
    inline char* GetPassword() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 25, 0, 0);
      return (char*)val;
    }

    inline int SetPassword(const char *lpPassword) {
      return IPWorksSSL_FTPS_Set(m_pObj, 25, 0, (void*)lpPassword, 0);
    }

    inline char* GetRemoteFile() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 26, 0, 0);
      return (char*)val;
    }

    inline int SetRemoteFile(const char *lpRemoteFile) {
      return IPWorksSSL_FTPS_Set(m_pObj, 26, 0, (void*)lpRemoteFile, 0);
    }

    inline char* GetRemoteHost() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 27, 0, 0);
      return (char*)val;
    }

    inline int SetRemoteHost(const char *lpRemoteHost) {
      return IPWorksSSL_FTPS_Set(m_pObj, 27, 0, (void*)lpRemoteHost, 0);
    }

    inline char* GetRemotePath() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 28, 0, 0);
      return (char*)val;
    }

    inline int SetRemotePath(const char *lpRemotePath) {
      return IPWorksSSL_FTPS_Set(m_pObj, 28, 0, (void*)lpRemotePath, 0);
    }

    inline int GetRemotePort() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 29, 0, 0);
      return (int)(long)val;
    }
    inline int SetRemotePort(int lRemotePort) {
      void* val = (void*)IPS64CAST(lRemotePort);
      return IPWorksSSL_FTPS_Set(m_pObj, 29, 0, val, 0);
    }
    inline int GetSSLAcceptServerCertEncoded(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_FTPS_Get(m_pObj, 30, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }

    inline int SetSSLAcceptServerCertEncoded(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_FTPS_Set(m_pObj, 30, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }

    inline int GetSSLCertEncoded(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_FTPS_Get(m_pObj, 31, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }

    inline int SetSSLCertEncoded(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_FTPS_Set(m_pObj, 31, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }

    inline int GetSSLCertStore(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_FTPS_Get(m_pObj, 32, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }

    inline int SetSSLCertStore(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_FTPS_Set(m_pObj, 32, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }

    inline char* GetSSLCertStorePassword() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 33, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertStorePassword(const char *lpSSLCertStorePassword) {
      return IPWorksSSL_FTPS_Set(m_pObj, 33, 0, (void*)lpSSLCertStorePassword, 0);
    }

    inline int GetSSLCertStoreType() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 34, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLCertStoreType(int iSSLCertStoreType) {
      void* val = (void*)IPS64CAST(iSSLCertStoreType);
      return IPWorksSSL_FTPS_Set(m_pObj, 34, 0, val, 0);
    }
    inline char* GetSSLCertSubject() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 35, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertSubject(const char *lpSSLCertSubject) {
      return IPWorksSSL_FTPS_Set(m_pObj, 35, 0, (void*)lpSSLCertSubject, 0);
    }

    inline int GetSSLServerCertEncoded(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_FTPS_Get(m_pObj, 36, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }


    inline int GetSSLStartMode() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 37, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLStartMode(int iSSLStartMode) {
      void* val = (void*)IPS64CAST(iSSLStartMode);
      return IPWorksSSL_FTPS_Set(m_pObj, 37, 0, val, 0);
    }
    inline char* GetStartByte() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 38, 0, 0);
      return (char*)val;
    }

    inline int SetStartByte(const char *lpStartByte) {
      return IPWorksSSL_FTPS_Set(m_pObj, 38, 0, (void*)lpStartByte, 0);
    }

    inline int GetTimeout() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 39, 0, 0);
      return (int)(long)val;
    }
    inline int SetTimeout(int iTimeout) {
      void* val = (void*)IPS64CAST(iTimeout);
      return IPWorksSSL_FTPS_Set(m_pObj, 39, 0, val, 0);
    }
    inline int GetTransferMode() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 40, 0, 0);
      return (int)(long)val;
    }
    inline int SetTransferMode(int iTransferMode) {
      void* val = (void*)IPS64CAST(iTransferMode);
      return IPWorksSSL_FTPS_Set(m_pObj, 40, 0, val, 0);
    }
    inline char* GetUser() {
      void* val = IPWorksSSL_FTPS_Get(m_pObj, 41, 0, 0);
      return (char*)val;
    }

    inline int SetUser(const char *lpUser) {
      return IPWorksSSL_FTPS_Set(m_pObj, 41, 0, (void*)lpUser, 0);
    }


  public: //methods

    inline int Abort() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_FTPS_Do(m_pObj, 2, 0, param, cbparam);
      
      
    }
    inline int Append() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_FTPS_Do(m_pObj, 3, 0, param, cbparam);
      
      
    }
    inline char* Config(const char* lpszConfigurationString) {
      void *param[1+1] = {(void*)IPS64CAST(lpszConfigurationString), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_FTPS_Do(m_pObj, 4, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline int DeleteFile(const char* lpszFileName) {
      void *param[1+1] = {(void*)IPS64CAST(lpszFileName), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_FTPS_Do(m_pObj, 5, 1, param, cbparam);
      
      
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_FTPS_Do(m_pObj, 6, 0, param, cbparam);
      
      
    }
    inline int Download() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_FTPS_Do(m_pObj, 7, 0, param, cbparam);
      
      
    }
    inline int Interrupt() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_FTPS_Do(m_pObj, 8, 0, param, cbparam);
      
      
    }
    inline int ListDirectory() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_FTPS_Do(m_pObj, 9, 0, param, cbparam);
      
      
    }
    inline int ListDirectoryLong() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_FTPS_Do(m_pObj, 10, 0, param, cbparam);
      
      
    }
    inline int Logoff() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_FTPS_Do(m_pObj, 11, 0, param, cbparam);
      
      
    }
    inline int Logon() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_FTPS_Do(m_pObj, 12, 0, param, cbparam);
      
      
    }
    inline int MakeDirectory(const char* lpszNewDir) {
      void *param[1+1] = {(void*)IPS64CAST(lpszNewDir), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_FTPS_Do(m_pObj, 13, 1, param, cbparam);
      
      
    }
    inline int RemoveDirectory(const char* lpszDirName) {
      void *param[1+1] = {(void*)IPS64CAST(lpszDirName), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_FTPS_Do(m_pObj, 14, 1, param, cbparam);
      
      
    }
    inline int RenameFile(const char* lpszNewName) {
      void *param[1+1] = {(void*)IPS64CAST(lpszNewName), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_FTPS_Do(m_pObj, 15, 1, param, cbparam);
      
      
    }
    inline int StoreUnique() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_FTPS_Do(m_pObj, 18, 0, param, cbparam);
      
      
    }
    inline int Upload() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_FTPS_Do(m_pObj, 19, 0, param, cbparam);
      
      
    }

};


#ifdef WIN32 //UNICODE

typedef struct {
  LPWSTR ConnectionEvent;
  int StatusCode;
  LPWSTR Description;
  int reserved;
} FTPSConnectionStatusEventParamsW;

typedef struct {
  LPWSTR DirEntry;
  LPWSTR FileName;
  int IsDir;
  ns_int64 *pFileSize;
  LPWSTR FileTime;
  int reserved;
} FTPSDirListEventParamsW;

typedef struct {
  int Direction;
  int reserved;
} FTPSEndTransferEventParamsW;

typedef struct {
  int ErrorCode;
  LPWSTR Description;
  int reserved;
} FTPSErrorEventParamsW;

typedef struct {
  int Direction;
  LPWSTR Message;
  int reserved;
} FTPSPITrailEventParamsW;

typedef struct {
  LPWSTR CertEncoded;
  LPWSTR CertSubject;
  LPWSTR CertIssuer;
  LPWSTR Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} FTPSSSLServerAuthenticationEventParamsW;

typedef struct {
  LPWSTR Message;
  int reserved;
} FTPSSSLStatusEventParamsW;

typedef struct {
  int Direction;
  int reserved;
} FTPSStartTransferEventParamsW;

typedef struct {
  int Direction;
  ns_int64 *pBytesTransferred;
  int PercentDone;
  LPWSTR Text;
  int lenText;
  int reserved;
} FTPSTransferEventParamsW;



class FTPSW : public FTPS {

  public: //properties
  
    inline LPWSTR GetAccount() {
      return (LPWSTR)IPWorksSSL_FTPS_Get(m_pObj, 10000+1, 0, 0);
    }

    inline int SetAccount(LPWSTR lpAccount) {
      return IPWorksSSL_FTPS_Set(m_pObj, 10000+1, 0, (void*)lpAccount, 0);
    }



    inline int SetCommand(LPWSTR lpCommand) {
      return IPWorksSSL_FTPS_Set(m_pObj, 10000+2, 0, (void*)lpCommand, 0);
    }





    inline LPWSTR GetDirListEntry(int iEntryIndex) {
      return (LPWSTR)IPWorksSSL_FTPS_Get(m_pObj, 10000+5, iEntryIndex, 0);
    }



    inline LPWSTR GetDirListFileName(int iEntryIndex) {
      return (LPWSTR)IPWorksSSL_FTPS_Get(m_pObj, 10000+6, iEntryIndex, 0);
    }





    inline LPWSTR GetDirListFileTime(int iEntryIndex) {
      return (LPWSTR)IPWorksSSL_FTPS_Get(m_pObj, 10000+8, iEntryIndex, 0);
    }









    inline LPWSTR GetFileTime() {
      return (LPWSTR)IPWorksSSL_FTPS_Get(m_pObj, 10000+12, 0, 0);
    }







    inline LPWSTR GetFirewallHost() {
      return (LPWSTR)IPWorksSSL_FTPS_Get(m_pObj, 10000+15, 0, 0);
    }

    inline int SetFirewallHost(LPWSTR lpFirewallHost) {
      return IPWorksSSL_FTPS_Set(m_pObj, 10000+15, 0, (void*)lpFirewallHost, 0);
    }

    inline LPWSTR GetFirewallPassword() {
      return (LPWSTR)IPWorksSSL_FTPS_Get(m_pObj, 10000+16, 0, 0);
    }

    inline int SetFirewallPassword(LPWSTR lpFirewallPassword) {
      return IPWorksSSL_FTPS_Set(m_pObj, 10000+16, 0, (void*)lpFirewallPassword, 0);
    }



    inline LPWSTR GetFirewallUser() {
      return (LPWSTR)IPWorksSSL_FTPS_Get(m_pObj, 10000+18, 0, 0);
    }

    inline int SetFirewallUser(LPWSTR lpFirewallUser) {
      return IPWorksSSL_FTPS_Set(m_pObj, 10000+18, 0, (void*)lpFirewallUser, 0);
    }



    inline LPWSTR GetLastReply() {
      return (LPWSTR)IPWorksSSL_FTPS_Get(m_pObj, 10000+20, 0, 0);
    }



    inline LPWSTR GetLocalFile() {
      return (LPWSTR)IPWorksSSL_FTPS_Get(m_pObj, 10000+10000+21, 0, 0);
    }

    inline int SetLocalFile(LPWSTR lpLocalFile) {
      return IPWorksSSL_FTPS_Set(m_pObj, 10000+10000+21, 0, (void*)lpLocalFile, 0);
    }

    inline LPWSTR GetLocalHost() {
      return (LPWSTR)IPWorksSSL_FTPS_Get(m_pObj, 10000+22, 0, 0);
    }

    inline int SetLocalHost(LPWSTR lpLocalHost) {
      return IPWorksSSL_FTPS_Set(m_pObj, 10000+22, 0, (void*)lpLocalHost, 0);
    }





    inline LPWSTR GetPassword() {
      return (LPWSTR)IPWorksSSL_FTPS_Get(m_pObj, 10000+25, 0, 0);
    }

    inline int SetPassword(LPWSTR lpPassword) {
      return IPWorksSSL_FTPS_Set(m_pObj, 10000+25, 0, (void*)lpPassword, 0);
    }

    inline LPWSTR GetRemoteFile() {
      return (LPWSTR)IPWorksSSL_FTPS_Get(m_pObj, 10000+26, 0, 0);
    }

    inline int SetRemoteFile(LPWSTR lpRemoteFile) {
      return IPWorksSSL_FTPS_Set(m_pObj, 10000+26, 0, (void*)lpRemoteFile, 0);
    }

    inline LPWSTR GetRemoteHost() {
      return (LPWSTR)IPWorksSSL_FTPS_Get(m_pObj, 10000+27, 0, 0);
    }

    inline int SetRemoteHost(LPWSTR lpRemoteHost) {
      return IPWorksSSL_FTPS_Set(m_pObj, 10000+27, 0, (void*)lpRemoteHost, 0);
    }

    inline LPWSTR GetRemotePath() {
      return (LPWSTR)IPWorksSSL_FTPS_Get(m_pObj, 10000+28, 0, 0);
    }

    inline int SetRemotePath(LPWSTR lpRemotePath) {
      return IPWorksSSL_FTPS_Set(m_pObj, 10000+28, 0, (void*)lpRemotePath, 0);
    }



    inline LPWSTR GetSSLAcceptServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_FTPS_Get(m_pObj, 10000+30, 0, 0);
    }

    inline int SetSSLAcceptServerCertEncoded(LPWSTR lpSSLAcceptServerCertEncoded) {
      return IPWorksSSL_FTPS_Set(m_pObj, 10000+30, 0, (void*)lpSSLAcceptServerCertEncoded, 0);
    }
    inline int GetSSLAcceptServerCertEncodedB(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_FTPS_Get(m_pObj, 30, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }
    inline int SetSSLAcceptServerCertEncodedB(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_FTPS_Set(m_pObj, 30, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }
    inline LPWSTR GetSSLCertEncoded() {
      return (LPWSTR)IPWorksSSL_FTPS_Get(m_pObj, 10000+31, 0, 0);
    }

    inline int SetSSLCertEncoded(LPWSTR lpSSLCertEncoded) {
      return IPWorksSSL_FTPS_Set(m_pObj, 10000+31, 0, (void*)lpSSLCertEncoded, 0);
    }
    inline int GetSSLCertEncodedB(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_FTPS_Get(m_pObj, 31, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }
    inline int SetSSLCertEncodedB(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_FTPS_Set(m_pObj, 31, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }
    inline LPWSTR GetSSLCertStore() {
      return (LPWSTR)IPWorksSSL_FTPS_Get(m_pObj, 10000+32, 0, 0);
    }

    inline int SetSSLCertStore(LPWSTR lpSSLCertStore) {
      return IPWorksSSL_FTPS_Set(m_pObj, 10000+32, 0, (void*)lpSSLCertStore, 0);
    }
    inline int GetSSLCertStoreB(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_FTPS_Get(m_pObj, 32, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }
    inline int SetSSLCertStoreB(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_FTPS_Set(m_pObj, 32, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }
    inline LPWSTR GetSSLCertStorePassword() {
      return (LPWSTR)IPWorksSSL_FTPS_Get(m_pObj, 10000+33, 0, 0);
    }

    inline int SetSSLCertStorePassword(LPWSTR lpSSLCertStorePassword) {
      return IPWorksSSL_FTPS_Set(m_pObj, 10000+33, 0, (void*)lpSSLCertStorePassword, 0);
    }



    inline LPWSTR GetSSLCertSubject() {
      return (LPWSTR)IPWorksSSL_FTPS_Get(m_pObj, 10000+35, 0, 0);
    }

    inline int SetSSLCertSubject(LPWSTR lpSSLCertSubject) {
      return IPWorksSSL_FTPS_Set(m_pObj, 10000+35, 0, (void*)lpSSLCertSubject, 0);
    }

    inline LPWSTR GetSSLServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_FTPS_Get(m_pObj, 10000+36, 0, 0);
    }


    inline int GetSSLServerCertEncodedB(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_FTPS_Get(m_pObj, 36, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }



    inline LPWSTR GetStartByte() {
      return (LPWSTR)IPWorksSSL_FTPS_Get(m_pObj, 10000+38, 0, 0);
    }

    inline int SetStartByte(LPWSTR lpStartByte) {
      return IPWorksSSL_FTPS_Set(m_pObj, 10000+38, 0, (void*)lpStartByte, 0);
    }





    inline LPWSTR GetUser() {
      return (LPWSTR)IPWorksSSL_FTPS_Get(m_pObj, 10000+41, 0, 0);
    }

    inline int SetUser(LPWSTR lpUser) {
      return IPWorksSSL_FTPS_Set(m_pObj, 10000+41, 0, (void*)lpUser, 0);
    }



  public: //events
  
    virtual int FireConnectionStatus(FTPSConnectionStatusEventParamsW *e) {return 0;}
    virtual int FireDirList(FTPSDirListEventParamsW *e) {return 0;}
    virtual int FireEndTransfer(FTPSEndTransferEventParamsW *e) {return 0;}
    virtual int FireError(FTPSErrorEventParamsW *e) {return 0;}
    virtual int FirePITrail(FTPSPITrailEventParamsW *e) {return 0;}
    virtual int FireSSLServerAuthentication(FTPSSSLServerAuthenticationEventParamsW *e) {return 0;}
    virtual int FireSSLStatus(FTPSSSLStatusEventParamsW *e) {return 0;}
    virtual int FireStartTransfer(FTPSStartTransferEventParamsW *e) {return 0;}
    virtual int FireTransfer(FTPSTransferEventParamsW *e) {return 0;}


  protected:
  
    virtual int FTPSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {
    	int ret_code = 0;
      switch (event_id) {
         case 1: {
            FTPSConnectionStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]),  0};
            ret_code = FireConnectionStatus(&e);
            break;
         }
         case 2: {
            FTPSDirListEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (ns_int64*)IPS64CAST(param[3]), (LPWSTR)IPS64CAST(param[4]),  0};
            ret_code = FireDirList(&e);
            break;
         }
         case 3: {
            FTPSEndTransferEventParamsW e = {(int)IPS64CAST(param[0]),  0};
            ret_code = FireEndTransfer(&e);
            break;
         }
         case 4: {
            FTPSErrorEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireError(&e);
            break;
         }
         case 5: {
            FTPSPITrailEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FirePITrail(&e);
            break;
         }
         case 6: {
            FTPSSSLServerAuthenticationEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = FireSSLServerAuthentication(&e);
            param[4] = (void*)(e.Accept);
            break;
         }
         case 7: {
            FTPSSSLStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]),  0};
            ret_code = FireSSLStatus(&e);
            break;
         }
         case 8: {
            FTPSStartTransferEventParamsW e = {(int)IPS64CAST(param[0]),  0};
            ret_code = FireStartTransfer(&e);
            break;
         }
         case 9: {
            FTPSTransferEventParamsW e = {(int)IPS64CAST(param[0]), (ns_int64*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (int)IPS64CAST(cbparam[3]),  0};
            ret_code = FireTransfer(&e);
            break;
         }

      }
      return ret_code;
    }
  
  public: //event overrides

    virtual int FireConnectionStatus(FTPSConnectionStatusEventParams *e) {return -10000;}
    virtual int FireDirList(FTPSDirListEventParams *e) {return -10000;}
    virtual int FireEndTransfer(FTPSEndTransferEventParams *e) {return -10000;}
    virtual int FireError(FTPSErrorEventParams *e) {return -10000;}
    virtual int FirePITrail(FTPSPITrailEventParams *e) {return -10000;}
    virtual int FireSSLServerAuthentication(FTPSSSLServerAuthenticationEventParams *e) {return -10000;}
    virtual int FireSSLStatus(FTPSSSLStatusEventParams *e) {return -10000;}
    virtual int FireStartTransfer(FTPSStartTransferEventParams *e) {return -10000;}
    virtual int FireTransfer(FTPSTransferEventParams *e) {return -10000;}

  public: //methods

    inline int Abort() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_FTPS_Do(m_pObj, 10000+2, 0, param, cbparam);
      
    }
    inline int Append() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_FTPS_Do(m_pObj, 10000+3, 0, param, cbparam);
      
    }
    inline LPWSTR Config(LPWSTR lpszConfigurationString) {
      void *param[1+1] = {(void*)lpszConfigurationString, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_FTPS_Do(m_pObj, 10000+4, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline int DeleteFile(LPWSTR lpszFileName) {
      void *param[1+1] = {(void*)lpszFileName, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_FTPS_Do(m_pObj, 10000+5, 1, param, cbparam);
      
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_FTPS_Do(m_pObj, 10000+6, 0, param, cbparam);
      
    }
    inline int Download() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_FTPS_Do(m_pObj, 10000+7, 0, param, cbparam);
      
    }
    inline int Interrupt() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_FTPS_Do(m_pObj, 10000+8, 0, param, cbparam);
      
    }
    inline int ListDirectory() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_FTPS_Do(m_pObj, 10000+9, 0, param, cbparam);
      
    }
    inline int ListDirectoryLong() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_FTPS_Do(m_pObj, 10000+10, 0, param, cbparam);
      
    }
    inline int Logoff() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_FTPS_Do(m_pObj, 10000+11, 0, param, cbparam);
      
    }
    inline int Logon() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_FTPS_Do(m_pObj, 10000+12, 0, param, cbparam);
      
    }
    inline int MakeDirectory(LPWSTR lpszNewDir) {
      void *param[1+1] = {(void*)lpszNewDir, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_FTPS_Do(m_pObj, 10000+13, 1, param, cbparam);
      
    }
    inline int RemoveDirectory(LPWSTR lpszDirName) {
      void *param[1+1] = {(void*)lpszDirName, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_FTPS_Do(m_pObj, 10000+14, 1, param, cbparam);
      
    }
    inline int RenameFile(LPWSTR lpszNewName) {
      void *param[1+1] = {(void*)lpszNewName, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_FTPS_Do(m_pObj, 10000+15, 1, param, cbparam);
      
    }
    inline int StoreUnique() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_FTPS_Do(m_pObj, 10000+18, 0, param, cbparam);
      
    }
    inline int Upload() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_FTPS_Do(m_pObj, 10000+19, 0, param, cbparam);
      
    }

};

#endif //WIN32

#endif //_IPWORKSSSL_FTPS_H_




