/******************************************************************
   IP*Works! SSL V9 C++ Edition
   Copyright (c) 2013 /n software inc. - All rights reserved.
*******************************************************************/

#ifndef _IPWORKSSSL_HTMLMAILERS_H_
#define _IPWORKSSSL_HTMLMAILERS_H_

#define IPWORKSSSL_ONLY_TYPES
#include "ipworksssl.h"
#include "ipworksssl.key"

//AuthMechanisms
#define AM_USER_PASSWORD                                   0
#define AM_CRAMMD5                                         1
#define AM_NTLM                                            2
#define AM_KERBEROS                                        6

//FirewallTypes
#define FW_NONE                                            0
#define FW_TUNNEL                                          1
#define FW_SOCKS4                                          2
#define FW_SOCKS5                                          3

//TImportance
#define MI_UNSPECIFIED                                     0
#define MI_HIGH                                            1
#define MI_NORMAL                                          2
#define MI_LOW                                             3

//EmailRecipientTypes
#define RT_TO                                              0
#define RT_CC                                              1
#define RT_BCC                                             2

//TPriority
#define EP_UNSPECIFIED                                     0
#define EP_NORMAL                                          1
#define EP_URGENT                                          2
#define EP_NON_URGENT                                      3

//TSensitivity
#define ES_UNSPECIFIED                                     0
#define ES_PERSONAL                                        1
#define ES_PRIVATE                                         2
#define ES_COMPANY_CONFIDENTIAL                            3

//CertStoreTypes
#define CST_USER                                           0
#define CST_MACHINE                                        1
#define CST_PFXFILE                                        2
#define CST_PFXBLOB                                        3
#define CST_JKSFILE                                        4
#define CST_JKSBLOB                                        5
#define CST_PEMKEY_FILE                                    6
#define CST_PEMKEY_BLOB                                    7
#define CST_PUBLIC_KEY_FILE                                8
#define CST_PUBLIC_KEY_BLOB                                9
#define CST_SSHPUBLIC_KEY_BLOB                             10
#define CST_P7BFILE                                        11
#define CST_P7BBLOB                                        12
#define CST_SSHPUBLIC_KEY_FILE                             13

//TSSLStartMode
#define SSL_AUTOMATIC                                      0
#define SSL_IMPLICIT                                       1
#define SSL_EXPLICIT                                       2
#define SSL_NONE                                           3


extern "C" void* IPWORKSSSL_CALL IPWorksSSL_HTMLMailerS_Create(PIPWORKSSSL_CALLBACK lpSink, void *lpContext, char *lpOemKey);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_HTMLMailerS_Destroy(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_HTMLMailerS_CheckIndex(void *lpObj, int propid, int arridx);
extern "C" void* IPWORKSSSL_CALL IPWorksSSL_HTMLMailerS_Get(void *lpObj, int propid, int arridx, int *lpcbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_HTMLMailerS_Set(void *lpObj, int propid, int arridx, const void *val, int cbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_HTMLMailerS_Do(void *lpObj, int methid, int cparam, void *param[], int cbparam[]);
extern "C" char* IPWORKSSSL_CALL IPWorksSSL_HTMLMailerS_GetLastError(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_HTMLMailerS_GetLastErrorCode(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_HTMLMailerS_StaticInit(void *hInst);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_HTMLMailerS_StaticDestroy();

#ifdef WIN32
#include <windows.h>
#pragma warning(disable:4311) 
#pragma warning(disable:4312) 
#endif

typedef struct {
  const char* ConnectionEvent;
  int StatusCode;
  const char* Description;
  int reserved;
} HTMLMailerSConnectionStatusEventParams;

typedef struct {
  int Direction;
  int reserved;
} HTMLMailerSEndTransferEventParams;

typedef struct {
  int ErrorCode;
  const char* Description;
  int reserved;
} HTMLMailerSErrorEventParams;

typedef struct {
  int Direction;
  const char* Message;
  int reserved;
} HTMLMailerSPITrailEventParams;

typedef struct {
  const char* CertEncoded;
  const char* CertSubject;
  const char* CertIssuer;
  const char* Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} HTMLMailerSSSLServerAuthenticationEventParams;

typedef struct {
  const char* Message;
  int reserved;
} HTMLMailerSSSLStatusEventParams;

typedef struct {
  int Direction;
  int reserved;
} HTMLMailerSStartTransferEventParams;

typedef struct {
  int Direction;
  ns_int64 *pBytesTransferred;
  int PercentDone;
  int reserved;
} HTMLMailerSTransferEventParams;



class HTMLMailerS {
  
  public: //events
  
    virtual int FireConnectionStatus(HTMLMailerSConnectionStatusEventParams *e) {return 0;}
    virtual int FireEndTransfer(HTMLMailerSEndTransferEventParams *e) {return 0;}
    virtual int FireError(HTMLMailerSErrorEventParams *e) {return 0;}
    virtual int FirePITrail(HTMLMailerSPITrailEventParams *e) {return 0;}
    virtual int FireSSLServerAuthentication(HTMLMailerSSSLServerAuthenticationEventParams *e) {return 0;}
    virtual int FireSSLStatus(HTMLMailerSSSLStatusEventParams *e) {return 0;}
    virtual int FireStartTransfer(HTMLMailerSStartTransferEventParams *e) {return 0;}
    virtual int FireTransfer(HTMLMailerSTransferEventParams *e) {return 0;}


  protected:

    void *m_pObj;
    
    static int IPWORKSSSL_CALL HTMLMailerSEventSink(void *lpObj, int event_id, int cparam, void *param[], int cbparam[]) {
      int ret_code = 0;
      if (event_id > 10000) return ((HTMLMailerS*)lpObj)->HTMLMailerSEventSinkW(event_id - 10000, cparam, param, cbparam);
      switch (event_id) {
         case 1: {
            HTMLMailerSConnectionStatusEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]),  0};
            ret_code = ((HTMLMailerS*)lpObj)->FireConnectionStatus(&e);
            break;
         }
         case 2: {
            HTMLMailerSEndTransferEventParams e = {(int)IPS64CAST(param[0]),  0};
            ret_code = ((HTMLMailerS*)lpObj)->FireEndTransfer(&e);
            break;
         }
         case 3: {
            HTMLMailerSErrorEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((HTMLMailerS*)lpObj)->FireError(&e);
            break;
         }
         case 4: {
            HTMLMailerSPITrailEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((HTMLMailerS*)lpObj)->FirePITrail(&e);
            break;
         }
         case 5: {
            HTMLMailerSSSLServerAuthenticationEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = ((HTMLMailerS*)lpObj)->FireSSLServerAuthentication(&e);
            param[4] = (void*)IPS64CAST(e.Accept);
            break;
         }
         case 6: {
            HTMLMailerSSSLStatusEventParams e = {(char*)IPS64CAST(param[0]),  0};
            ret_code = ((HTMLMailerS*)lpObj)->FireSSLStatus(&e);
            break;
         }
         case 7: {
            HTMLMailerSStartTransferEventParams e = {(int)IPS64CAST(param[0]),  0};
            ret_code = ((HTMLMailerS*)lpObj)->FireStartTransfer(&e);
            break;
         }
         case 8: {
            HTMLMailerSTransferEventParams e = {(int)IPS64CAST(param[0]), (ns_int64*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]),  0};
            ret_code = ((HTMLMailerS*)lpObj)->FireTransfer(&e);
            break;
         }

      }
      return ret_code;
    }

    virtual int HTMLMailerSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {return 0;}

  public:

    HTMLMailerS(char *lpOemKey = (char*)IPWORKSSSL_OEMKEY_42) {
      m_pObj = IPWorksSSL_HTMLMailerS_Create(HTMLMailerSEventSink, (void*)this, (char*)lpOemKey);
    }

    virtual ~HTMLMailerS() {
      IPWorksSSL_HTMLMailerS_Destroy(m_pObj);
    }

  public:

    inline char *GetLastError() {
      return IPWorksSSL_HTMLMailerS_GetLastError(m_pObj);
    }
    
    inline int GetLastErrorCode() {
      return IPWorksSSL_HTMLMailerS_GetLastErrorCode(m_pObj);
    }

    inline char *VERSION() {
      return (char*)IPWorksSSL_HTMLMailerS_Get(m_pObj, 0, 0, 0);
    }

  public: //properties

    inline int GetAttachmentCount() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 1, 0, 0);
      return (int)(long)val;
    }
    inline int SetAttachmentCount(int iAttachmentCount) {
      void* val = (void*)IPS64CAST(iAttachmentCount);
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 1, 0, val, 0);
    }
    inline char* GetAttachmentFile(int iAttachmentIndex) {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 2, iAttachmentIndex, 0);
      return (char*)val;
    }

    inline int SetAttachmentFile(int iAttachmentIndex, const char *lpAttachmentFile) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 2, iAttachmentIndex, (void*)lpAttachmentFile, 0);
    }

    inline char* GetAttachmentName(int iAttachmentIndex) {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 3, iAttachmentIndex, 0);
      return (char*)val;
    }

    inline int SetAttachmentName(int iAttachmentIndex, const char *lpAttachmentName) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 3, iAttachmentIndex, (void*)lpAttachmentName, 0);
    }

    inline int GetAuthMechanism() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 4, 0, 0);
      return (int)(long)val;
    }
    inline int SetAuthMechanism(int iAuthMechanism) {
      void* val = (void*)IPS64CAST(iAuthMechanism);
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 4, 0, val, 0);
    }
    inline char* GetBCc() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 5, 0, 0);
      return (char*)val;
    }

    inline int SetBCc(const char *lpBCc) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 5, 0, (void*)lpBCc, 0);
    }

    inline char* GetCc() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 6, 0, 0);
      return (char*)val;
    }

    inline int SetCc(const char *lpCc) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 6, 0, (void*)lpCc, 0);
    }


    inline int SetCommand(const char *lpCommand) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 7, 0, (void*)lpCommand, 0);
    }

    inline int GetConnected() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 8, 0, 0);
      return (int)(long)val;
    }
    inline int SetConnected(int bConnected) {
      void* val = (void*)IPS64CAST(bConnected);
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 8, 0, val, 0);
    }
    inline char* GetDeliveryNotificationTo() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 9, 0, 0);
      return (char*)val;
    }

    inline int SetDeliveryNotificationTo(const char *lpDeliveryNotificationTo) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 9, 0, (void*)lpDeliveryNotificationTo, 0);
    }

    inline int GetFirewallAutoDetect() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 10, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallAutoDetect(int bFirewallAutoDetect) {
      void* val = (void*)IPS64CAST(bFirewallAutoDetect);
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10, 0, val, 0);
    }
    inline int GetFirewallType() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 11, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallType(int iFirewallType) {
      void* val = (void*)IPS64CAST(iFirewallType);
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 11, 0, val, 0);
    }
    inline char* GetFirewallHost() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 12, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallHost(const char *lpFirewallHost) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 12, 0, (void*)lpFirewallHost, 0);
    }

    inline char* GetFirewallPassword() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 13, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallPassword(const char *lpFirewallPassword) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 13, 0, (void*)lpFirewallPassword, 0);
    }

    inline int GetFirewallPort() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 14, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallPort(int lFirewallPort) {
      void* val = (void*)IPS64CAST(lFirewallPort);
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 14, 0, val, 0);
    }
    inline char* GetFirewallUser() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 15, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallUser(const char *lpFirewallUser) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 15, 0, (void*)lpFirewallUser, 0);
    }

    inline char* GetFrom() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 16, 0, 0);
      return (char*)val;
    }

    inline int SetFrom(const char *lpFrom) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 16, 0, (void*)lpFrom, 0);
    }

    inline char* GetHTMLFile() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 17, 0, 0);
      return (char*)val;
    }

    inline int SetHTMLFile(const char *lpHTMLFile) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 17, 0, (void*)lpHTMLFile, 0);
    }

    inline int GetIdle() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 18, 0, 0);
      return (int)(long)val;
    }

    inline int GetImageCount() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 19, 0, 0);
      return (int)(long)val;
    }
    inline int SetImageCount(int iImageCount) {
      void* val = (void*)IPS64CAST(iImageCount);
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 19, 0, val, 0);
    }
    inline char* GetImageFile(int iImageIndex) {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 20, iImageIndex, 0);
      return (char*)val;
    }

    inline int SetImageFile(int iImageIndex, const char *lpImageFile) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 20, iImageIndex, (void*)lpImageFile, 0);
    }

    inline char* GetImageId(int iImageIndex) {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 21, iImageIndex, 0);
      return (char*)val;
    }

    inline int SetImageId(int iImageIndex, const char *lpImageId) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 21, iImageIndex, (void*)lpImageId, 0);
    }

    inline char* GetImageType(int iImageIndex) {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 22, iImageIndex, 0);
      return (char*)val;
    }

    inline int SetImageType(int iImageIndex, const char *lpImageType) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 22, iImageIndex, (void*)lpImageType, 0);
    }

    inline int GetImportance() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 23, 0, 0);
      return (int)(long)val;
    }
    inline int SetImportance(int iImportance) {
      void* val = (void*)IPS64CAST(iImportance);
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 23, 0, val, 0);
    }
    inline char* GetLastReply() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 24, 0, 0);
      return (char*)val;
    }


    inline char* GetLocalHost() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 25, 0, 0);
      return (char*)val;
    }

    inline int SetLocalHost(const char *lpLocalHost) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 25, 0, (void*)lpLocalHost, 0);
    }

    inline int GetMailPort() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 26, 0, 0);
      return (int)(long)val;
    }
    inline int SetMailPort(int lMailPort) {
      void* val = (void*)IPS64CAST(lMailPort);
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 26, 0, val, 0);
    }
    inline char* GetMailServer() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 27, 0, 0);
      return (char*)val;
    }

    inline int SetMailServer(const char *lpMailServer) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 27, 0, (void*)lpMailServer, 0);
    }

    inline char* GetMessageDate() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 28, 0, 0);
      return (char*)val;
    }

    inline int SetMessageDate(const char *lpMessageDate) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 28, 0, (void*)lpMessageDate, 0);
    }

    inline char* GetMessageHTML() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 29, 0, 0);
      return (char*)val;
    }

    inline int SetMessageHTML(const char *lpMessageHTML) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 29, 0, (void*)lpMessageHTML, 0);
    }

    inline char* GetMessageId() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 30, 0, 0);
      return (char*)val;
    }

    inline int SetMessageId(const char *lpMessageId) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 30, 0, (void*)lpMessageId, 0);
    }

    inline int GetMessageRecipientCount() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 31, 0, 0);
      return (int)(long)val;
    }
    inline int SetMessageRecipientCount(int iMessageRecipientCount) {
      void* val = (void*)IPS64CAST(iMessageRecipientCount);
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 31, 0, val, 0);
    }
    inline char* GetMessageRecipientAddress(int iRecipientIndex) {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 32, iRecipientIndex, 0);
      return (char*)val;
    }

    inline int SetMessageRecipientAddress(int iRecipientIndex, const char *lpMessageRecipientAddress) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 32, iRecipientIndex, (void*)lpMessageRecipientAddress, 0);
    }

    inline char* GetMessageRecipientName(int iRecipientIndex) {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 33, iRecipientIndex, 0);
      return (char*)val;
    }

    inline int SetMessageRecipientName(int iRecipientIndex, const char *lpMessageRecipientName) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 33, iRecipientIndex, (void*)lpMessageRecipientName, 0);
    }

    inline char* GetMessageRecipientOptions(int iRecipientIndex) {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 34, iRecipientIndex, 0);
      return (char*)val;
    }

    inline int SetMessageRecipientOptions(int iRecipientIndex, const char *lpMessageRecipientOptions) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 34, iRecipientIndex, (void*)lpMessageRecipientOptions, 0);
    }

    inline int GetMessageRecipientType(int iRecipientIndex) {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 35, iRecipientIndex, 0);
      return (int)(long)val;
    }
    inline int SetMessageRecipientType(int iRecipientIndex, int iMessageRecipientType) {
      void* val = (void*)IPS64CAST(iMessageRecipientType);
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 35, iRecipientIndex, val, 0);
    }
    inline char* GetMessageText() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 36, 0, 0);
      return (char*)val;
    }

    inline int SetMessageText(const char *lpMessageText) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 36, 0, (void*)lpMessageText, 0);
    }

    inline char* GetOtherHeaders() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 37, 0, 0);
      return (char*)val;
    }

    inline int SetOtherHeaders(const char *lpOtherHeaders) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 37, 0, (void*)lpOtherHeaders, 0);
    }

    inline int GetParseHTML() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 38, 0, 0);
      return (int)(long)val;
    }
    inline int SetParseHTML(int bParseHTML) {
      void* val = (void*)IPS64CAST(bParseHTML);
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 38, 0, val, 0);
    }
    inline char* GetPassword() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 39, 0, 0);
      return (char*)val;
    }

    inline int SetPassword(const char *lpPassword) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 39, 0, (void*)lpPassword, 0);
    }

    inline int GetPriority() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 40, 0, 0);
      return (int)(long)val;
    }
    inline int SetPriority(int iPriority) {
      void* val = (void*)IPS64CAST(iPriority);
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 40, 0, val, 0);
    }
    inline char* GetReadReceiptTo() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 41, 0, 0);
      return (char*)val;
    }

    inline int SetReadReceiptTo(const char *lpReadReceiptTo) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 41, 0, (void*)lpReadReceiptTo, 0);
    }

    inline char* GetReplyTo() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 42, 0, 0);
      return (char*)val;
    }

    inline int SetReplyTo(const char *lpReplyTo) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 42, 0, (void*)lpReplyTo, 0);
    }

    inline char* GetSendTo() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 43, 0, 0);
      return (char*)val;
    }

    inline int SetSendTo(const char *lpSendTo) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 43, 0, (void*)lpSendTo, 0);
    }

    inline int GetSensitivity() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 44, 0, 0);
      return (int)(long)val;
    }
    inline int SetSensitivity(int iSensitivity) {
      void* val = (void*)IPS64CAST(iSensitivity);
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 44, 0, val, 0);
    }
    inline int GetSSLAcceptServerCertEncoded(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_HTMLMailerS_Get(m_pObj, 45, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }

    inline int SetSSLAcceptServerCertEncoded(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 45, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }

    inline int GetSSLCertEncoded(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_HTMLMailerS_Get(m_pObj, 46, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }

    inline int SetSSLCertEncoded(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 46, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }

    inline int GetSSLCertStore(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_HTMLMailerS_Get(m_pObj, 47, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }

    inline int SetSSLCertStore(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 47, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }

    inline char* GetSSLCertStorePassword() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 48, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertStorePassword(const char *lpSSLCertStorePassword) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 48, 0, (void*)lpSSLCertStorePassword, 0);
    }

    inline int GetSSLCertStoreType() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 49, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLCertStoreType(int iSSLCertStoreType) {
      void* val = (void*)IPS64CAST(iSSLCertStoreType);
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 49, 0, val, 0);
    }
    inline char* GetSSLCertSubject() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 50, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertSubject(const char *lpSSLCertSubject) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 50, 0, (void*)lpSSLCertSubject, 0);
    }

    inline int GetSSLServerCertEncoded(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_HTMLMailerS_Get(m_pObj, 51, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }


    inline int GetSSLStartMode() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 52, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLStartMode(int iSSLStartMode) {
      void* val = (void*)IPS64CAST(iSSLStartMode);
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 52, 0, val, 0);
    }
    inline char* GetSubject() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 53, 0, 0);
      return (char*)val;
    }

    inline int SetSubject(const char *lpSubject) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 53, 0, (void*)lpSubject, 0);
    }

    inline int GetTimeout() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 54, 0, 0);
      return (int)(long)val;
    }
    inline int SetTimeout(int iTimeout) {
      void* val = (void*)IPS64CAST(iTimeout);
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 54, 0, val, 0);
    }
    inline char* GetUser() {
      void* val = IPWorksSSL_HTMLMailerS_Get(m_pObj, 55, 0, 0);
      return (char*)val;
    }

    inline int SetUser(const char *lpUser) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 55, 0, (void*)lpUser, 0);
    }


  public: //methods

    inline int AddAttachment(const char* lpszFileName) {
      void *param[1+1] = {(void*)IPS64CAST(lpszFileName), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_HTMLMailerS_Do(m_pObj, 2, 1, param, cbparam);
      
      
    }
    inline char* Config(const char* lpszConfigurationString) {
      void *param[1+1] = {(void*)IPS64CAST(lpszConfigurationString), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_HTMLMailerS_Do(m_pObj, 3, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline int Connect() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_HTMLMailerS_Do(m_pObj, 4, 0, param, cbparam);
      
      
    }
    inline int Disconnect() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_HTMLMailerS_Do(m_pObj, 5, 0, param, cbparam);
      
      
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_HTMLMailerS_Do(m_pObj, 6, 0, param, cbparam);
      
      
    }
    inline int Interrupt() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_HTMLMailerS_Do(m_pObj, 7, 0, param, cbparam);
      
      
    }
    inline int ProcessQueue(const char* lpszQueueDir) {
      void *param[1+1] = {(void*)IPS64CAST(lpszQueueDir), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_HTMLMailerS_Do(m_pObj, 8, 1, param, cbparam);
      
      
    }
    inline char* Queue(const char* lpszQueueDir) {
      void *param[1+1] = {(void*)IPS64CAST(lpszQueueDir), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_HTMLMailerS_Do(m_pObj, 9, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline int ResetHeaders() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_HTMLMailerS_Do(m_pObj, 10, 0, param, cbparam);
      
      
    }
    inline int Send() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_HTMLMailerS_Do(m_pObj, 11, 0, param, cbparam);
      
      
    }

};


#ifdef WIN32 //UNICODE

typedef struct {
  LPWSTR ConnectionEvent;
  int StatusCode;
  LPWSTR Description;
  int reserved;
} HTMLMailerSConnectionStatusEventParamsW;

typedef struct {
  int Direction;
  int reserved;
} HTMLMailerSEndTransferEventParamsW;

typedef struct {
  int ErrorCode;
  LPWSTR Description;
  int reserved;
} HTMLMailerSErrorEventParamsW;

typedef struct {
  int Direction;
  LPWSTR Message;
  int reserved;
} HTMLMailerSPITrailEventParamsW;

typedef struct {
  LPWSTR CertEncoded;
  LPWSTR CertSubject;
  LPWSTR CertIssuer;
  LPWSTR Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} HTMLMailerSSSLServerAuthenticationEventParamsW;

typedef struct {
  LPWSTR Message;
  int reserved;
} HTMLMailerSSSLStatusEventParamsW;

typedef struct {
  int Direction;
  int reserved;
} HTMLMailerSStartTransferEventParamsW;

typedef struct {
  int Direction;
  ns_int64 *pBytesTransferred;
  int PercentDone;
  int reserved;
} HTMLMailerSTransferEventParamsW;



class HTMLMailerSW : public HTMLMailerS {

  public: //properties
  


    inline LPWSTR GetAttachmentFile(int iAttachmentIndex) {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+2, iAttachmentIndex, 0);
    }

    inline int SetAttachmentFile(int iAttachmentIndex, LPWSTR lpAttachmentFile) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+2, iAttachmentIndex, (void*)lpAttachmentFile, 0);
    }

    inline LPWSTR GetAttachmentName(int iAttachmentIndex) {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+3, iAttachmentIndex, 0);
    }

    inline int SetAttachmentName(int iAttachmentIndex, LPWSTR lpAttachmentName) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+3, iAttachmentIndex, (void*)lpAttachmentName, 0);
    }



    inline LPWSTR GetBCc() {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+5, 0, 0);
    }

    inline int SetBCc(LPWSTR lpBCc) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+5, 0, (void*)lpBCc, 0);
    }

    inline LPWSTR GetCc() {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+6, 0, 0);
    }

    inline int SetCc(LPWSTR lpCc) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+6, 0, (void*)lpCc, 0);
    }



    inline int SetCommand(LPWSTR lpCommand) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+7, 0, (void*)lpCommand, 0);
    }



    inline LPWSTR GetDeliveryNotificationTo() {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+9, 0, 0);
    }

    inline int SetDeliveryNotificationTo(LPWSTR lpDeliveryNotificationTo) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+9, 0, (void*)lpDeliveryNotificationTo, 0);
    }





    inline LPWSTR GetFirewallHost() {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+12, 0, 0);
    }

    inline int SetFirewallHost(LPWSTR lpFirewallHost) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+12, 0, (void*)lpFirewallHost, 0);
    }

    inline LPWSTR GetFirewallPassword() {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+13, 0, 0);
    }

    inline int SetFirewallPassword(LPWSTR lpFirewallPassword) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+13, 0, (void*)lpFirewallPassword, 0);
    }



    inline LPWSTR GetFirewallUser() {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+15, 0, 0);
    }

    inline int SetFirewallUser(LPWSTR lpFirewallUser) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+15, 0, (void*)lpFirewallUser, 0);
    }

    inline LPWSTR GetFrom() {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+16, 0, 0);
    }

    inline int SetFrom(LPWSTR lpFrom) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+16, 0, (void*)lpFrom, 0);
    }

    inline LPWSTR GetHTMLFile() {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+17, 0, 0);
    }

    inline int SetHTMLFile(LPWSTR lpHTMLFile) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+17, 0, (void*)lpHTMLFile, 0);
    }





    inline LPWSTR GetImageFile(int iImageIndex) {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+20, iImageIndex, 0);
    }

    inline int SetImageFile(int iImageIndex, LPWSTR lpImageFile) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+20, iImageIndex, (void*)lpImageFile, 0);
    }

    inline LPWSTR GetImageId(int iImageIndex) {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+21, iImageIndex, 0);
    }

    inline int SetImageId(int iImageIndex, LPWSTR lpImageId) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+21, iImageIndex, (void*)lpImageId, 0);
    }

    inline LPWSTR GetImageType(int iImageIndex) {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+22, iImageIndex, 0);
    }

    inline int SetImageType(int iImageIndex, LPWSTR lpImageType) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+22, iImageIndex, (void*)lpImageType, 0);
    }



    inline LPWSTR GetLastReply() {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+24, 0, 0);
    }



    inline LPWSTR GetLocalHost() {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+25, 0, 0);
    }

    inline int SetLocalHost(LPWSTR lpLocalHost) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+25, 0, (void*)lpLocalHost, 0);
    }



    inline LPWSTR GetMailServer() {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+27, 0, 0);
    }

    inline int SetMailServer(LPWSTR lpMailServer) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+27, 0, (void*)lpMailServer, 0);
    }

    inline LPWSTR GetMessageDate() {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+28, 0, 0);
    }

    inline int SetMessageDate(LPWSTR lpMessageDate) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+28, 0, (void*)lpMessageDate, 0);
    }

    inline LPWSTR GetMessageHTML() {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+29, 0, 0);
    }

    inline int SetMessageHTML(LPWSTR lpMessageHTML) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+29, 0, (void*)lpMessageHTML, 0);
    }

    inline LPWSTR GetMessageId() {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+30, 0, 0);
    }

    inline int SetMessageId(LPWSTR lpMessageId) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+30, 0, (void*)lpMessageId, 0);
    }



    inline LPWSTR GetMessageRecipientAddress(int iRecipientIndex) {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+32, iRecipientIndex, 0);
    }

    inline int SetMessageRecipientAddress(int iRecipientIndex, LPWSTR lpMessageRecipientAddress) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+32, iRecipientIndex, (void*)lpMessageRecipientAddress, 0);
    }

    inline LPWSTR GetMessageRecipientName(int iRecipientIndex) {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+33, iRecipientIndex, 0);
    }

    inline int SetMessageRecipientName(int iRecipientIndex, LPWSTR lpMessageRecipientName) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+33, iRecipientIndex, (void*)lpMessageRecipientName, 0);
    }

    inline LPWSTR GetMessageRecipientOptions(int iRecipientIndex) {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+34, iRecipientIndex, 0);
    }

    inline int SetMessageRecipientOptions(int iRecipientIndex, LPWSTR lpMessageRecipientOptions) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+34, iRecipientIndex, (void*)lpMessageRecipientOptions, 0);
    }



    inline LPWSTR GetMessageText() {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+36, 0, 0);
    }

    inline int SetMessageText(LPWSTR lpMessageText) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+36, 0, (void*)lpMessageText, 0);
    }

    inline LPWSTR GetOtherHeaders() {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+37, 0, 0);
    }

    inline int SetOtherHeaders(LPWSTR lpOtherHeaders) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+37, 0, (void*)lpOtherHeaders, 0);
    }



    inline LPWSTR GetPassword() {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+39, 0, 0);
    }

    inline int SetPassword(LPWSTR lpPassword) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+39, 0, (void*)lpPassword, 0);
    }



    inline LPWSTR GetReadReceiptTo() {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+41, 0, 0);
    }

    inline int SetReadReceiptTo(LPWSTR lpReadReceiptTo) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+41, 0, (void*)lpReadReceiptTo, 0);
    }

    inline LPWSTR GetReplyTo() {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+42, 0, 0);
    }

    inline int SetReplyTo(LPWSTR lpReplyTo) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+42, 0, (void*)lpReplyTo, 0);
    }

    inline LPWSTR GetSendTo() {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+43, 0, 0);
    }

    inline int SetSendTo(LPWSTR lpSendTo) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+43, 0, (void*)lpSendTo, 0);
    }



    inline LPWSTR GetSSLAcceptServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+45, 0, 0);
    }

    inline int SetSSLAcceptServerCertEncoded(LPWSTR lpSSLAcceptServerCertEncoded) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+45, 0, (void*)lpSSLAcceptServerCertEncoded, 0);
    }
    inline int GetSSLAcceptServerCertEncodedB(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_HTMLMailerS_Get(m_pObj, 45, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }
    inline int SetSSLAcceptServerCertEncodedB(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 45, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }
    inline LPWSTR GetSSLCertEncoded() {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+46, 0, 0);
    }

    inline int SetSSLCertEncoded(LPWSTR lpSSLCertEncoded) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+46, 0, (void*)lpSSLCertEncoded, 0);
    }
    inline int GetSSLCertEncodedB(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_HTMLMailerS_Get(m_pObj, 46, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }
    inline int SetSSLCertEncodedB(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 46, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }
    inline LPWSTR GetSSLCertStore() {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+47, 0, 0);
    }

    inline int SetSSLCertStore(LPWSTR lpSSLCertStore) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+47, 0, (void*)lpSSLCertStore, 0);
    }
    inline int GetSSLCertStoreB(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_HTMLMailerS_Get(m_pObj, 47, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }
    inline int SetSSLCertStoreB(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 47, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }
    inline LPWSTR GetSSLCertStorePassword() {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+48, 0, 0);
    }

    inline int SetSSLCertStorePassword(LPWSTR lpSSLCertStorePassword) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+48, 0, (void*)lpSSLCertStorePassword, 0);
    }



    inline LPWSTR GetSSLCertSubject() {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+50, 0, 0);
    }

    inline int SetSSLCertSubject(LPWSTR lpSSLCertSubject) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+50, 0, (void*)lpSSLCertSubject, 0);
    }

    inline LPWSTR GetSSLServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+51, 0, 0);
    }


    inline int GetSSLServerCertEncodedB(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_HTMLMailerS_Get(m_pObj, 51, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }



    inline LPWSTR GetSubject() {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+53, 0, 0);
    }

    inline int SetSubject(LPWSTR lpSubject) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+53, 0, (void*)lpSubject, 0);
    }



    inline LPWSTR GetUser() {
      return (LPWSTR)IPWorksSSL_HTMLMailerS_Get(m_pObj, 10000+55, 0, 0);
    }

    inline int SetUser(LPWSTR lpUser) {
      return IPWorksSSL_HTMLMailerS_Set(m_pObj, 10000+55, 0, (void*)lpUser, 0);
    }



  public: //events
  
    virtual int FireConnectionStatus(HTMLMailerSConnectionStatusEventParamsW *e) {return 0;}
    virtual int FireEndTransfer(HTMLMailerSEndTransferEventParamsW *e) {return 0;}
    virtual int FireError(HTMLMailerSErrorEventParamsW *e) {return 0;}
    virtual int FirePITrail(HTMLMailerSPITrailEventParamsW *e) {return 0;}
    virtual int FireSSLServerAuthentication(HTMLMailerSSSLServerAuthenticationEventParamsW *e) {return 0;}
    virtual int FireSSLStatus(HTMLMailerSSSLStatusEventParamsW *e) {return 0;}
    virtual int FireStartTransfer(HTMLMailerSStartTransferEventParamsW *e) {return 0;}
    virtual int FireTransfer(HTMLMailerSTransferEventParamsW *e) {return 0;}


  protected:
  
    virtual int HTMLMailerSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {
    	int ret_code = 0;
      switch (event_id) {
         case 1: {
            HTMLMailerSConnectionStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]),  0};
            ret_code = FireConnectionStatus(&e);
            break;
         }
         case 2: {
            HTMLMailerSEndTransferEventParamsW e = {(int)IPS64CAST(param[0]),  0};
            ret_code = FireEndTransfer(&e);
            break;
         }
         case 3: {
            HTMLMailerSErrorEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireError(&e);
            break;
         }
         case 4: {
            HTMLMailerSPITrailEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FirePITrail(&e);
            break;
         }
         case 5: {
            HTMLMailerSSSLServerAuthenticationEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = FireSSLServerAuthentication(&e);
            param[4] = (void*)(e.Accept);
            break;
         }
         case 6: {
            HTMLMailerSSSLStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]),  0};
            ret_code = FireSSLStatus(&e);
            break;
         }
         case 7: {
            HTMLMailerSStartTransferEventParamsW e = {(int)IPS64CAST(param[0]),  0};
            ret_code = FireStartTransfer(&e);
            break;
         }
         case 8: {
            HTMLMailerSTransferEventParamsW e = {(int)IPS64CAST(param[0]), (ns_int64*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]),  0};
            ret_code = FireTransfer(&e);
            break;
         }

      }
      return ret_code;
    }
  
  public: //event overrides

    virtual int FireConnectionStatus(HTMLMailerSConnectionStatusEventParams *e) {return -10000;}
    virtual int FireEndTransfer(HTMLMailerSEndTransferEventParams *e) {return -10000;}
    virtual int FireError(HTMLMailerSErrorEventParams *e) {return -10000;}
    virtual int FirePITrail(HTMLMailerSPITrailEventParams *e) {return -10000;}
    virtual int FireSSLServerAuthentication(HTMLMailerSSSLServerAuthenticationEventParams *e) {return -10000;}
    virtual int FireSSLStatus(HTMLMailerSSSLStatusEventParams *e) {return -10000;}
    virtual int FireStartTransfer(HTMLMailerSStartTransferEventParams *e) {return -10000;}
    virtual int FireTransfer(HTMLMailerSTransferEventParams *e) {return -10000;}

  public: //methods

    inline int AddAttachment(LPWSTR lpszFileName) {
      void *param[1+1] = {(void*)lpszFileName, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_HTMLMailerS_Do(m_pObj, 10000+2, 1, param, cbparam);
      
    }
    inline LPWSTR Config(LPWSTR lpszConfigurationString) {
      void *param[1+1] = {(void*)lpszConfigurationString, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_HTMLMailerS_Do(m_pObj, 10000+3, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline int Connect() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_HTMLMailerS_Do(m_pObj, 10000+4, 0, param, cbparam);
      
    }
    inline int Disconnect() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_HTMLMailerS_Do(m_pObj, 10000+5, 0, param, cbparam);
      
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_HTMLMailerS_Do(m_pObj, 10000+6, 0, param, cbparam);
      
    }
    inline int Interrupt() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_HTMLMailerS_Do(m_pObj, 10000+7, 0, param, cbparam);
      
    }
    inline int ProcessQueue(LPWSTR lpszQueueDir) {
      void *param[1+1] = {(void*)lpszQueueDir, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_HTMLMailerS_Do(m_pObj, 10000+8, 1, param, cbparam);
      
    }
    inline LPWSTR Queue(LPWSTR lpszQueueDir) {
      void *param[1+1] = {(void*)lpszQueueDir, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_HTMLMailerS_Do(m_pObj, 10000+9, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline int ResetHeaders() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_HTMLMailerS_Do(m_pObj, 10000+10, 0, param, cbparam);
      
    }
    inline int Send() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_HTMLMailerS_Do(m_pObj, 10000+11, 0, param, cbparam);
      
    }

};

#endif //WIN32

#endif //_IPWORKSSSL_HTMLMAILERS_H_




