/******************************************************************
   IP*Works! SSL V9 C++ Edition
   Copyright (c) 2013 /n software inc. - All rights reserved.
*******************************************************************/

#ifndef _IPWORKSSSL_IMAPS_H_
#define _IPWORKSSSL_IMAPS_H_

#define IPWORKSSSL_ONLY_TYPES
#include "ipworksssl.h"
#include "ipworksssl.key"

//AuthMechanisms
#define AM_USER_PASSWORD                                   0
#define AM_CRAMMD5                                         1
#define AM_NTLM                                            2
#define AM_SASLPLAIN                                       4
#define AM_SASLDIGEST_MD5                                  5
#define AM_KERBEROS                                        6

//FirewallTypes
#define FW_NONE                                            0
#define FW_TUNNEL                                          1
#define FW_SOCKS4                                          2
#define FW_SOCKS5                                          3

//EmailRecipientTypes
#define RT_TO                                              0
#define RT_CC                                              1
#define RT_BCC                                             2

//CertStoreTypes
#define CST_USER                                           0
#define CST_MACHINE                                        1
#define CST_PFXFILE                                        2
#define CST_PFXBLOB                                        3
#define CST_JKSFILE                                        4
#define CST_JKSBLOB                                        5
#define CST_PEMKEY_FILE                                    6
#define CST_PEMKEY_BLOB                                    7
#define CST_PUBLIC_KEY_FILE                                8
#define CST_PUBLIC_KEY_BLOB                                9
#define CST_SSHPUBLIC_KEY_BLOB                             10
#define CST_P7BFILE                                        11
#define CST_P7BBLOB                                        12
#define CST_SSHPUBLIC_KEY_FILE                             13

//TSSLStartMode
#define SSL_AUTOMATIC                                      0
#define SSL_IMPLICIT                                       1
#define SSL_EXPLICIT                                       2
#define SSL_NONE                                           3


extern "C" void* IPWORKSSSL_CALL IPWorksSSL_IMAPS_Create(PIPWORKSSSL_CALLBACK lpSink, void *lpContext, char *lpOemKey);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_IMAPS_Destroy(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_IMAPS_CheckIndex(void *lpObj, int propid, int arridx);
extern "C" void* IPWORKSSSL_CALL IPWorksSSL_IMAPS_Get(void *lpObj, int propid, int arridx, int *lpcbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_IMAPS_Set(void *lpObj, int propid, int arridx, const void *val, int cbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_IMAPS_Do(void *lpObj, int methid, int cparam, void *param[], int cbparam[]);
extern "C" char* IPWORKSSSL_CALL IPWorksSSL_IMAPS_GetLastError(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_IMAPS_GetLastErrorCode(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_IMAPS_StaticInit(void *hInst);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_IMAPS_StaticDestroy();

#ifdef WIN32
#include <windows.h>
#pragma warning(disable:4311) 
#pragma warning(disable:4312) 
#endif

typedef struct {
  const char* ConnectionEvent;
  int StatusCode;
  const char* Description;
  int reserved;
} IMAPSConnectionStatusEventParams;

typedef struct {
  int Direction;
  int reserved;
} IMAPSEndTransferEventParams;

typedef struct {
  int ErrorCode;
  const char* Description;
  int reserved;
} IMAPSErrorEventParams;

typedef struct {
  const char* Field;
  const char* Value;
  int reserved;
} IMAPSHeaderEventParams;

typedef struct {
  const char* Mailbox;
  const char* User;
  const char* Rights;
  int reserved;
} IMAPSMailboxACLEventParams;

typedef struct {
  const char* Mailbox;
  const char* Separator;
  const char* Flags;
  int reserved;
} IMAPSMailboxListEventParams;

typedef struct {
  const char* MessageId;
  const char* Subject;
  const char* MessageDate;
  const char* From;
  const char* Flags;
  ns_int64 *pSize;
  int reserved;
} IMAPSMessageInfoEventParams;

typedef struct {
  const char* PartId;
  ns_int64 *pSize;
  const char* ContentType;
  const char* Filename;
  const char* ContentEncoding;
  const char* Parameters;
  const char* MultipartMode;
  const char* ContentId;
  const char* ContentDisposition;
  int reserved;
} IMAPSMessagePartEventParams;

typedef struct {
  int Direction;
  const char* Message;
  int reserved;
} IMAPSPITrailEventParams;

typedef struct {
  const char* CertEncoded;
  const char* CertSubject;
  const char* CertIssuer;
  const char* Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} IMAPSSSLServerAuthenticationEventParams;

typedef struct {
  const char* Message;
  int reserved;
} IMAPSSSLStatusEventParams;

typedef struct {
  int Direction;
  int reserved;
} IMAPSStartTransferEventParams;

typedef struct {
  int Direction;
  ns_int64 *pBytesTransferred;
  int PercentDone;
  const char* Text;
  int reserved;
} IMAPSTransferEventParams;



class IMAPS {
  
  public: //events
  
    virtual int FireConnectionStatus(IMAPSConnectionStatusEventParams *e) {return 0;}
    virtual int FireEndTransfer(IMAPSEndTransferEventParams *e) {return 0;}
    virtual int FireError(IMAPSErrorEventParams *e) {return 0;}
    virtual int FireHeader(IMAPSHeaderEventParams *e) {return 0;}
    virtual int FireMailboxACL(IMAPSMailboxACLEventParams *e) {return 0;}
    virtual int FireMailboxList(IMAPSMailboxListEventParams *e) {return 0;}
    virtual int FireMessageInfo(IMAPSMessageInfoEventParams *e) {return 0;}
    virtual int FireMessagePart(IMAPSMessagePartEventParams *e) {return 0;}
    virtual int FirePITrail(IMAPSPITrailEventParams *e) {return 0;}
    virtual int FireSSLServerAuthentication(IMAPSSSLServerAuthenticationEventParams *e) {return 0;}
    virtual int FireSSLStatus(IMAPSSSLStatusEventParams *e) {return 0;}
    virtual int FireStartTransfer(IMAPSStartTransferEventParams *e) {return 0;}
    virtual int FireTransfer(IMAPSTransferEventParams *e) {return 0;}


  protected:

    void *m_pObj;
    
    static int IPWORKSSSL_CALL IMAPSEventSink(void *lpObj, int event_id, int cparam, void *param[], int cbparam[]) {
      int ret_code = 0;
      if (event_id > 10000) return ((IMAPS*)lpObj)->IMAPSEventSinkW(event_id - 10000, cparam, param, cbparam);
      switch (event_id) {
         case 1: {
            IMAPSConnectionStatusEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]),  0};
            ret_code = ((IMAPS*)lpObj)->FireConnectionStatus(&e);
            break;
         }
         case 2: {
            IMAPSEndTransferEventParams e = {(int)IPS64CAST(param[0]),  0};
            ret_code = ((IMAPS*)lpObj)->FireEndTransfer(&e);
            break;
         }
         case 3: {
            IMAPSErrorEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((IMAPS*)lpObj)->FireError(&e);
            break;
         }
         case 4: {
            IMAPSHeaderEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((IMAPS*)lpObj)->FireHeader(&e);
            break;
         }
         case 5: {
            IMAPSMailboxACLEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]),  0};
            ret_code = ((IMAPS*)lpObj)->FireMailboxACL(&e);
            break;
         }
         case 6: {
            IMAPSMailboxListEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]),  0};
            ret_code = ((IMAPS*)lpObj)->FireMailboxList(&e);
            break;
         }
         case 7: {
            IMAPSMessageInfoEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (char*)IPS64CAST(param[4]), (ns_int64*)IPS64CAST(param[5]),  0};
            ret_code = ((IMAPS*)lpObj)->FireMessageInfo(&e);
            break;
         }
         case 8: {
            IMAPSMessagePartEventParams e = {(char*)IPS64CAST(param[0]), (ns_int64*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (char*)IPS64CAST(param[4]), (char*)IPS64CAST(param[5]), (char*)IPS64CAST(param[6]), (char*)IPS64CAST(param[7]), (char*)IPS64CAST(param[8]),  0};
            ret_code = ((IMAPS*)lpObj)->FireMessagePart(&e);
            break;
         }
         case 9: {
            IMAPSPITrailEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((IMAPS*)lpObj)->FirePITrail(&e);
            break;
         }
         case 10: {
            IMAPSSSLServerAuthenticationEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = ((IMAPS*)lpObj)->FireSSLServerAuthentication(&e);
            param[4] = (void*)IPS64CAST(e.Accept);
            break;
         }
         case 11: {
            IMAPSSSLStatusEventParams e = {(char*)IPS64CAST(param[0]),  0};
            ret_code = ((IMAPS*)lpObj)->FireSSLStatus(&e);
            break;
         }
         case 12: {
            IMAPSStartTransferEventParams e = {(int)IPS64CAST(param[0]),  0};
            ret_code = ((IMAPS*)lpObj)->FireStartTransfer(&e);
            break;
         }
         case 13: {
            IMAPSTransferEventParams e = {(int)IPS64CAST(param[0]), (ns_int64*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]),  0};
            ret_code = ((IMAPS*)lpObj)->FireTransfer(&e);
            break;
         }

      }
      return ret_code;
    }

    virtual int IMAPSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {return 0;}

  public:

    IMAPS(char *lpOemKey = (char*)IPWORKSSSL_OEMKEY_31) {
      m_pObj = IPWorksSSL_IMAPS_Create(IMAPSEventSink, (void*)this, (char*)lpOemKey);
    }

    virtual ~IMAPS() {
      IPWorksSSL_IMAPS_Destroy(m_pObj);
    }

  public:

    inline char *GetLastError() {
      return IPWorksSSL_IMAPS_GetLastError(m_pObj);
    }
    
    inline int GetLastErrorCode() {
      return IPWorksSSL_IMAPS_GetLastErrorCode(m_pObj);
    }

    inline char *VERSION() {
      return (char*)IPWorksSSL_IMAPS_Get(m_pObj, 0, 0, 0);
    }

  public: //properties

    inline int GetAuthMechanism() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 1, 0, 0);
      return (int)(long)val;
    }
    inline int SetAuthMechanism(int iAuthMechanism) {
      void* val = (void*)IPS64CAST(iAuthMechanism);
      return IPWorksSSL_IMAPS_Set(m_pObj, 1, 0, val, 0);
    }
    inline int GetAutoDecodeParts() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 2, 0, 0);
      return (int)(long)val;
    }
    inline int SetAutoDecodeParts(int bAutoDecodeParts) {
      void* val = (void*)IPS64CAST(bAutoDecodeParts);
      return IPWorksSSL_IMAPS_Set(m_pObj, 2, 0, val, 0);
    }

    inline int SetCommand(const char *lpCommand) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 3, 0, (void*)lpCommand, 0);
    }

    inline int GetConnected() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 4, 0, 0);
      return (int)(long)val;
    }
    inline int SetConnected(int bConnected) {
      void* val = (void*)IPS64CAST(bConnected);
      return IPWorksSSL_IMAPS_Set(m_pObj, 4, 0, val, 0);
    }
    inline ns_int64 GetEndByte() {
      ns_int64 *pval = (ns_int64*)IPWorksSSL_IMAPS_Get(m_pObj, 5, 0, 0);
      return *pval;
    }

    inline int SetEndByte(ns_int64 lEndByte) {
      void* val = (void*)(&lEndByte);
      return IPWorksSSL_IMAPS_Set(m_pObj, 5, 0, val, 0);
    }

    inline int GetFirewallAutoDetect() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 6, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallAutoDetect(int bFirewallAutoDetect) {
      void* val = (void*)IPS64CAST(bFirewallAutoDetect);
      return IPWorksSSL_IMAPS_Set(m_pObj, 6, 0, val, 0);
    }
    inline int GetFirewallType() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 7, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallType(int iFirewallType) {
      void* val = (void*)IPS64CAST(iFirewallType);
      return IPWorksSSL_IMAPS_Set(m_pObj, 7, 0, val, 0);
    }
    inline char* GetFirewallHost() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 8, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallHost(const char *lpFirewallHost) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 8, 0, (void*)lpFirewallHost, 0);
    }

    inline char* GetFirewallPassword() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 9, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallPassword(const char *lpFirewallPassword) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 9, 0, (void*)lpFirewallPassword, 0);
    }

    inline int GetFirewallPort() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 10, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallPort(int lFirewallPort) {
      void* val = (void*)IPS64CAST(lFirewallPort);
      return IPWorksSSL_IMAPS_Set(m_pObj, 10, 0, val, 0);
    }
    inline char* GetFirewallUser() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 11, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallUser(const char *lpFirewallUser) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 11, 0, (void*)lpFirewallUser, 0);
    }

    inline int GetIdle() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 12, 0, 0);
      return (int)(long)val;
    }

    inline char* GetLastReply() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 13, 0, 0);
      return (char*)val;
    }


    inline char* GetLocalFile() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 14, 0, 0);
      return (char*)val;
    }

    inline int SetLocalFile(const char *lpLocalFile) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 14, 0, (void*)lpLocalFile, 0);
    }

    inline char* GetLocalHost() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 15, 0, 0);
      return (char*)val;
    }

    inline int SetLocalHost(const char *lpLocalHost) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 15, 0, (void*)lpLocalHost, 0);
    }

    inline char* GetMailbox() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 16, 0, 0);
      return (char*)val;
    }

    inline int SetMailbox(const char *lpMailbox) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 16, 0, (void*)lpMailbox, 0);
    }

    inline char* GetMailboxFlags() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 17, 0, 0);
      return (char*)val;
    }


    inline int GetMailboxListCount() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 18, 0, 0);
      return (int)(long)val;
    }

    inline char* GetMailboxListFlags(int iMailboxListIndex) {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 19, iMailboxListIndex, 0);
      return (char*)val;
    }


    inline char* GetMailboxListName(int iMailboxListIndex) {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 20, iMailboxListIndex, 0);
      return (char*)val;
    }


    inline char* GetMailboxListSeparator(int iMailboxListIndex) {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 21, iMailboxListIndex, 0);
      return (char*)val;
    }


    inline int GetMailPort() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 22, 0, 0);
      return (int)(long)val;
    }
    inline int SetMailPort(int lMailPort) {
      void* val = (void*)IPS64CAST(lMailPort);
      return IPWorksSSL_IMAPS_Set(m_pObj, 22, 0, val, 0);
    }
    inline char* GetMailServer() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 23, 0, 0);
      return (char*)val;
    }

    inline int SetMailServer(const char *lpMailServer) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 23, 0, (void*)lpMailServer, 0);
    }

    inline int GetMessage(char *&lpMessage, int &lenMessage) {
      lpMessage = (char*)IPWorksSSL_IMAPS_Get(m_pObj, 24, 0, &lenMessage);
      return lpMessage ? 0 : lenMessage;
    }

    inline int SetMessage(const char *lpMessage, int lenMessage) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 24, 0, (void*)lpMessage, lenMessage);
    }

    inline char* GetMessageContentEncoding() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 25, 0, 0);
      return (char*)val;
    }


    inline char* GetMessageContentType() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 26, 0, 0);
      return (char*)val;
    }


    inline int GetMessageCount() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 27, 0, 0);
      return (int)(long)val;
    }

    inline char* GetMessageDate() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 28, 0, 0);
      return (char*)val;
    }


    inline char* GetMessageDeliveryTime() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 29, 0, 0);
      return (char*)val;
    }


    inline char* GetMessageFlags() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 30, 0, 0);
      return (char*)val;
    }

    inline int SetMessageFlags(const char *lpMessageFlags) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 30, 0, (void*)lpMessageFlags, 0);
    }

    inline char* GetMessageFrom() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 31, 0, 0);
      return (char*)val;
    }


    inline int GetMessageHeaderCount() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 32, 0, 0);
      return (int)(long)val;
    }
    inline int SetMessageHeaderCount(int iMessageHeaderCount) {
      void* val = (void*)IPS64CAST(iMessageHeaderCount);
      return IPWorksSSL_IMAPS_Set(m_pObj, 32, 0, val, 0);
    }
    inline char* GetMessageHeaderField(int iMessageHeaderIndex) {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 33, iMessageHeaderIndex, 0);
      return (char*)val;
    }

    inline int SetMessageHeaderField(int iMessageHeaderIndex, const char *lpMessageHeaderField) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 33, iMessageHeaderIndex, (void*)lpMessageHeaderField, 0);
    }

    inline char* GetMessageHeaderValue(int iMessageHeaderIndex) {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 34, iMessageHeaderIndex, 0);
      return (char*)val;
    }

    inline int SetMessageHeaderValue(int iMessageHeaderIndex, const char *lpMessageHeaderValue) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 34, iMessageHeaderIndex, (void*)lpMessageHeaderValue, 0);
    }

    inline char* GetMessageHeadersString() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 35, 0, 0);
      return (char*)val;
    }

    inline int SetMessageHeadersString(const char *lpMessageHeadersString) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 35, 0, (void*)lpMessageHeadersString, 0);
    }

    inline char* GetMessageId() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 36, 0, 0);
      return (char*)val;
    }


    inline int GetMessageInfoCount() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 37, 0, 0);
      return (int)(long)val;
    }

    inline char* GetMessageInfoDate(int iMessageInfoIndex) {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 38, iMessageInfoIndex, 0);
      return (char*)val;
    }


    inline char* GetMessageInfoFlags(int iMessageInfoIndex) {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 39, iMessageInfoIndex, 0);
      return (char*)val;
    }


    inline char* GetMessageInfoFrom(int iMessageInfoIndex) {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 40, iMessageInfoIndex, 0);
      return (char*)val;
    }


    inline char* GetMessageInfoMessageId(int iMessageInfoIndex) {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 41, iMessageInfoIndex, 0);
      return (char*)val;
    }


    inline ns_int64 GetMessageInfoSize(int iMessageInfoIndex) {
      ns_int64 *pval = (ns_int64*)IPWorksSSL_IMAPS_Get(m_pObj, 42, iMessageInfoIndex, 0);
      return *pval;
    }


    inline char* GetMessageInfoSubject(int iMessageInfoIndex) {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 43, iMessageInfoIndex, 0);
      return (char*)val;
    }


    inline char* GetMessageInReplyTo() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 44, 0, 0);
      return (char*)val;
    }


    inline char* GetMessageNetId() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 45, 0, 0);
      return (char*)val;
    }


    inline int GetMessagePartCount() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 46, 0, 0);
      return (int)(long)val;
    }

    inline char* GetMessagePartContentDisposition(int iMessagePartIndex) {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 47, iMessagePartIndex, 0);
      return (char*)val;
    }


    inline char* GetMessagePartContentEncoding(int iMessagePartIndex) {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 48, iMessagePartIndex, 0);
      return (char*)val;
    }


    inline char* GetMessagePartContentId(int iMessagePartIndex) {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 49, iMessagePartIndex, 0);
      return (char*)val;
    }


    inline char* GetMessagePartContentType(int iMessagePartIndex) {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 50, iMessagePartIndex, 0);
      return (char*)val;
    }


    inline char* GetMessagePartFilename(int iMessagePartIndex) {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 51, iMessagePartIndex, 0);
      return (char*)val;
    }


    inline char* GetMessagePartId(int iMessagePartIndex) {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 52, iMessagePartIndex, 0);
      return (char*)val;
    }


    inline char* GetMessagePartMultipartMode(int iMessagePartIndex) {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 53, iMessagePartIndex, 0);
      return (char*)val;
    }


    inline char* GetMessagePartParameters(int iMessagePartIndex) {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 54, iMessagePartIndex, 0);
      return (char*)val;
    }


    inline ns_int64 GetMessagePartSize(int iMessagePartIndex) {
      ns_int64 *pval = (ns_int64*)IPWorksSSL_IMAPS_Get(m_pObj, 55, iMessagePartIndex, 0);
      return *pval;
    }


    inline int GetMessageRecipientCount() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 56, 0, 0);
      return (int)(long)val;
    }

    inline char* GetMessageRecipientAddress(int iRecipientIndex) {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 57, iRecipientIndex, 0);
      return (char*)val;
    }


    inline char* GetMessageRecipientName(int iRecipientIndex) {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 58, iRecipientIndex, 0);
      return (char*)val;
    }


    inline char* GetMessageRecipientOptions(int iRecipientIndex) {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 59, iRecipientIndex, 0);
      return (char*)val;
    }


    inline int GetMessageRecipientType(int iRecipientIndex) {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 60, iRecipientIndex, 0);
      return (int)(long)val;
    }

    inline char* GetMessageReplyTo() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 61, 0, 0);
      return (char*)val;
    }


    inline char* GetMessageSender() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 62, 0, 0);
      return (char*)val;
    }


    inline char* GetMessageSet() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 63, 0, 0);
      return (char*)val;
    }

    inline int SetMessageSet(const char *lpMessageSet) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 63, 0, (void*)lpMessageSet, 0);
    }

    inline ns_int64 GetMessageSize() {
      ns_int64 *pval = (ns_int64*)IPWorksSSL_IMAPS_Get(m_pObj, 64, 0, 0);
      return *pval;
    }


    inline char* GetMessageSubject() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 65, 0, 0);
      return (char*)val;
    }


    inline char* GetMessageText() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 66, 0, 0);
      return (char*)val;
    }

    inline int SetMessageText(const char *lpMessageText) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 66, 0, (void*)lpMessageText, 0);
    }

    inline int GetOverwrite() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 67, 0, 0);
      return (int)(long)val;
    }
    inline int SetOverwrite(int bOverwrite) {
      void* val = (void*)IPS64CAST(bOverwrite);
      return IPWorksSSL_IMAPS_Set(m_pObj, 67, 0, val, 0);
    }
    inline char* GetPassword() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 68, 0, 0);
      return (char*)val;
    }

    inline int SetPassword(const char *lpPassword) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 68, 0, (void*)lpPassword, 0);
    }

    inline int GetPeekMode() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 69, 0, 0);
      return (int)(long)val;
    }
    inline int SetPeekMode(int bPeekMode) {
      void* val = (void*)IPS64CAST(bPeekMode);
      return IPWorksSSL_IMAPS_Set(m_pObj, 69, 0, val, 0);
    }
    inline int GetRecentMessageCount() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 70, 0, 0);
      return (int)(long)val;
    }

    inline char* GetSortCriteria() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 71, 0, 0);
      return (char*)val;
    }

    inline int SetSortCriteria(const char *lpSortCriteria) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 71, 0, (void*)lpSortCriteria, 0);
    }

    inline int GetSSLAcceptServerCertEncoded(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_IMAPS_Get(m_pObj, 72, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }

    inline int SetSSLAcceptServerCertEncoded(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 72, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }

    inline int GetSSLCertEncoded(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_IMAPS_Get(m_pObj, 73, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }

    inline int SetSSLCertEncoded(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 73, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }

    inline int GetSSLCertStore(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_IMAPS_Get(m_pObj, 74, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }

    inline int SetSSLCertStore(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 74, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }

    inline char* GetSSLCertStorePassword() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 75, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertStorePassword(const char *lpSSLCertStorePassword) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 75, 0, (void*)lpSSLCertStorePassword, 0);
    }

    inline int GetSSLCertStoreType() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 76, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLCertStoreType(int iSSLCertStoreType) {
      void* val = (void*)IPS64CAST(iSSLCertStoreType);
      return IPWorksSSL_IMAPS_Set(m_pObj, 76, 0, val, 0);
    }
    inline char* GetSSLCertSubject() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 77, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertSubject(const char *lpSSLCertSubject) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 77, 0, (void*)lpSSLCertSubject, 0);
    }

    inline int GetSSLServerCertEncoded(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_IMAPS_Get(m_pObj, 78, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }


    inline int GetSSLStartMode() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 79, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLStartMode(int iSSLStartMode) {
      void* val = (void*)IPS64CAST(iSSLStartMode);
      return IPWorksSSL_IMAPS_Set(m_pObj, 79, 0, val, 0);
    }
    inline ns_int64 GetStartByte() {
      ns_int64 *pval = (ns_int64*)IPWorksSSL_IMAPS_Get(m_pObj, 80, 0, 0);
      return *pval;
    }

    inline int SetStartByte(ns_int64 lStartByte) {
      void* val = (void*)(&lStartByte);
      return IPWorksSSL_IMAPS_Set(m_pObj, 80, 0, val, 0);
    }

    inline int GetTimeout() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 81, 0, 0);
      return (int)(long)val;
    }
    inline int SetTimeout(int iTimeout) {
      void* val = (void*)IPS64CAST(iTimeout);
      return IPWorksSSL_IMAPS_Set(m_pObj, 81, 0, val, 0);
    }
    inline int GetUIDMode() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 82, 0, 0);
      return (int)(long)val;
    }
    inline int SetUIDMode(int bUIDMode) {
      void* val = (void*)IPS64CAST(bUIDMode);
      return IPWorksSSL_IMAPS_Set(m_pObj, 82, 0, val, 0);
    }
    inline char* GetUIDValidity() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 83, 0, 0);
      return (char*)val;
    }


    inline char* GetUser() {
      void* val = IPWorksSSL_IMAPS_Get(m_pObj, 84, 0, 0);
      return (char*)val;
    }

    inline int SetUser(const char *lpUser) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 84, 0, (void*)lpUser, 0);
    }


  public: //methods

    inline int AddMessageFlags(const char* lpszFlags) {
      void *param[1+1] = {(void*)IPS64CAST(lpszFlags), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 2, 1, param, cbparam);
      
      
    }
    inline int AppendToMailbox() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 3, 0, param, cbparam);
      
      
    }
    inline int CheckMailbox() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 4, 0, param, cbparam);
      
      
    }
    inline int CloseMailbox() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 5, 0, param, cbparam);
      
      
    }
    inline char* Config(const char* lpszConfigurationString) {
      void *param[1+1] = {(void*)IPS64CAST(lpszConfigurationString), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_IMAPS_Do(m_pObj, 6, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline int Connect() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 7, 0, param, cbparam);
      
      
    }
    inline int CopyToMailbox() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 8, 0, param, cbparam);
      
      
    }
    inline int CreateMailbox() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 9, 0, param, cbparam);
      
      
    }
    inline int DeleteFromMailbox() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10, 0, param, cbparam);
      
      
    }
    inline int DeleteMailbox() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 11, 0, param, cbparam);
      
      
    }
    inline int DeleteMailboxACL(const char* lpszUser) {
      void *param[1+1] = {(void*)IPS64CAST(lpszUser), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 12, 1, param, cbparam);
      
      
    }
    inline int Disconnect() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 13, 0, param, cbparam);
      
      
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 14, 0, param, cbparam);
      
      
    }
    inline int ExamineMailbox() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 15, 0, param, cbparam);
      
      
    }
    inline int ExpungeMailbox() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 16, 0, param, cbparam);
      
      
    }
    inline int FetchMessageHeaders() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 17, 0, param, cbparam);
      
      
    }
    inline int FetchMessageInfo() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 18, 0, param, cbparam);
      
      
    }
    inline int FetchMessagePart(const char* lpszPartId) {
      void *param[1+1] = {(void*)IPS64CAST(lpszPartId), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 19, 1, param, cbparam);
      
      
    }
    inline int FetchMessagePartHeaders(const char* lpszPartId) {
      void *param[1+1] = {(void*)IPS64CAST(lpszPartId), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 20, 1, param, cbparam);
      
      
    }
    inline int FetchMessageText() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 21, 0, param, cbparam);
      
      
    }
    inline int GetMailboxACL() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 22, 0, param, cbparam);
      
      
    }
    inline int Interrupt() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 23, 0, param, cbparam);
      
      
    }
    inline int ListMailboxes() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 24, 0, param, cbparam);
      
      
    }
    inline int ListSubscribedMailboxes() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 25, 0, param, cbparam);
      
      
    }
    inline char* LocalizeDate(const char* lpszDateTime) {
      void *param[1+1] = {(void*)IPS64CAST(lpszDateTime), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_IMAPS_Do(m_pObj, 26, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline int MoveToMailbox() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 27, 0, param, cbparam);
      
      
    }
    inline int Noop() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 28, 0, param, cbparam);
      
      
    }
    inline int RenameMailbox(const char* lpszNewName) {
      void *param[1+1] = {(void*)IPS64CAST(lpszNewName), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 29, 1, param, cbparam);
      
      
    }
    inline int ResetMessageFlags() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 30, 0, param, cbparam);
      
      
    }
    inline int SearchMailbox(const char* lpszSearchCriteria) {
      void *param[1+1] = {(void*)IPS64CAST(lpszSearchCriteria), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 31, 1, param, cbparam);
      
      
    }
    inline int SelectMailbox() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 32, 0, param, cbparam);
      
      
    }
    inline int SetMailboxACL(const char* lpszUser, const char* lpszRights) {
      void *param[2+1] = {(void*)IPS64CAST(lpszUser), (void*)IPS64CAST(lpszRights), 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 33, 2, param, cbparam);
      
      
    }
    inline int SubscribeMailbox() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 35, 0, param, cbparam);
      
      
    }
    inline int UnsetMessageFlags() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 36, 0, param, cbparam);
      
      
    }
    inline int UnsubscribeMailbox() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 37, 0, param, cbparam);
      
      
    }

};


#ifdef WIN32 //UNICODE

typedef struct {
  LPWSTR ConnectionEvent;
  int StatusCode;
  LPWSTR Description;
  int reserved;
} IMAPSConnectionStatusEventParamsW;

typedef struct {
  int Direction;
  int reserved;
} IMAPSEndTransferEventParamsW;

typedef struct {
  int ErrorCode;
  LPWSTR Description;
  int reserved;
} IMAPSErrorEventParamsW;

typedef struct {
  LPWSTR Field;
  LPWSTR Value;
  int reserved;
} IMAPSHeaderEventParamsW;

typedef struct {
  LPWSTR Mailbox;
  LPWSTR User;
  LPWSTR Rights;
  int reserved;
} IMAPSMailboxACLEventParamsW;

typedef struct {
  LPWSTR Mailbox;
  LPWSTR Separator;
  LPWSTR Flags;
  int reserved;
} IMAPSMailboxListEventParamsW;

typedef struct {
  LPWSTR MessageId;
  LPWSTR Subject;
  LPWSTR MessageDate;
  LPWSTR From;
  LPWSTR Flags;
  ns_int64 *pSize;
  int reserved;
} IMAPSMessageInfoEventParamsW;

typedef struct {
  LPWSTR PartId;
  ns_int64 *pSize;
  LPWSTR ContentType;
  LPWSTR Filename;
  LPWSTR ContentEncoding;
  LPWSTR Parameters;
  LPWSTR MultipartMode;
  LPWSTR ContentId;
  LPWSTR ContentDisposition;
  int reserved;
} IMAPSMessagePartEventParamsW;

typedef struct {
  int Direction;
  LPWSTR Message;
  int reserved;
} IMAPSPITrailEventParamsW;

typedef struct {
  LPWSTR CertEncoded;
  LPWSTR CertSubject;
  LPWSTR CertIssuer;
  LPWSTR Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} IMAPSSSLServerAuthenticationEventParamsW;

typedef struct {
  LPWSTR Message;
  int reserved;
} IMAPSSSLStatusEventParamsW;

typedef struct {
  int Direction;
  int reserved;
} IMAPSStartTransferEventParamsW;

typedef struct {
  int Direction;
  ns_int64 *pBytesTransferred;
  int PercentDone;
  LPWSTR Text;
  int reserved;
} IMAPSTransferEventParamsW;



class IMAPSW : public IMAPS {

  public: //properties
  






    inline int SetCommand(LPWSTR lpCommand) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 10000+3, 0, (void*)lpCommand, 0);
    }









    inline LPWSTR GetFirewallHost() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+8, 0, 0);
    }

    inline int SetFirewallHost(LPWSTR lpFirewallHost) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 10000+8, 0, (void*)lpFirewallHost, 0);
    }

    inline LPWSTR GetFirewallPassword() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+9, 0, 0);
    }

    inline int SetFirewallPassword(LPWSTR lpFirewallPassword) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 10000+9, 0, (void*)lpFirewallPassword, 0);
    }



    inline LPWSTR GetFirewallUser() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+11, 0, 0);
    }

    inline int SetFirewallUser(LPWSTR lpFirewallUser) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 10000+11, 0, (void*)lpFirewallUser, 0);
    }



    inline LPWSTR GetLastReply() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+13, 0, 0);
    }



    inline LPWSTR GetLocalFile() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+10000+14, 0, 0);
    }

    inline int SetLocalFile(LPWSTR lpLocalFile) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 10000+10000+14, 0, (void*)lpLocalFile, 0);
    }

    inline LPWSTR GetLocalHost() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+15, 0, 0);
    }

    inline int SetLocalHost(LPWSTR lpLocalHost) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 10000+15, 0, (void*)lpLocalHost, 0);
    }

    inline LPWSTR GetMailbox() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+16, 0, 0);
    }

    inline int SetMailbox(LPWSTR lpMailbox) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 10000+16, 0, (void*)lpMailbox, 0);
    }

    inline LPWSTR GetMailboxFlags() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+17, 0, 0);
    }





    inline LPWSTR GetMailboxListFlags(int iMailboxListIndex) {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+19, iMailboxListIndex, 0);
    }



    inline LPWSTR GetMailboxListName(int iMailboxListIndex) {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+20, iMailboxListIndex, 0);
    }



    inline LPWSTR GetMailboxListSeparator(int iMailboxListIndex) {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+21, iMailboxListIndex, 0);
    }





    inline LPWSTR GetMailServer() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+23, 0, 0);
    }

    inline int SetMailServer(LPWSTR lpMailServer) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 10000+23, 0, (void*)lpMailServer, 0);
    }

    inline LPWSTR GetMessage() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+24, 0, 0);
    }

    inline int SetMessage(LPWSTR lpMessage) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 10000+24, 0, (void*)lpMessage, 0);
    }
    inline int GetMessageB(char *&lpMessage, int &lenMessage) {
      lpMessage = (char*)IPWorksSSL_IMAPS_Get(m_pObj, 24, 0, &lenMessage);
      return lpMessage ? 0 : lenMessage;
    }
    inline int SetMessageB(const char *lpMessage, int lenMessage) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 24, 0, (void*)lpMessage, lenMessage);
    }
    inline LPWSTR GetMessageContentEncoding() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+25, 0, 0);
    }



    inline LPWSTR GetMessageContentType() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+26, 0, 0);
    }





    inline LPWSTR GetMessageDate() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+28, 0, 0);
    }



    inline LPWSTR GetMessageDeliveryTime() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+29, 0, 0);
    }



    inline LPWSTR GetMessageFlags() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+30, 0, 0);
    }

    inline int SetMessageFlags(LPWSTR lpMessageFlags) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 10000+30, 0, (void*)lpMessageFlags, 0);
    }

    inline LPWSTR GetMessageFrom() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+31, 0, 0);
    }





    inline LPWSTR GetMessageHeaderField(int iMessageHeaderIndex) {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+33, iMessageHeaderIndex, 0);
    }

    inline int SetMessageHeaderField(int iMessageHeaderIndex, LPWSTR lpMessageHeaderField) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 10000+33, iMessageHeaderIndex, (void*)lpMessageHeaderField, 0);
    }

    inline LPWSTR GetMessageHeaderValue(int iMessageHeaderIndex) {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+34, iMessageHeaderIndex, 0);
    }

    inline int SetMessageHeaderValue(int iMessageHeaderIndex, LPWSTR lpMessageHeaderValue) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 10000+34, iMessageHeaderIndex, (void*)lpMessageHeaderValue, 0);
    }

    inline LPWSTR GetMessageHeadersString() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+35, 0, 0);
    }

    inline int SetMessageHeadersString(LPWSTR lpMessageHeadersString) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 10000+35, 0, (void*)lpMessageHeadersString, 0);
    }

    inline LPWSTR GetMessageId() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+36, 0, 0);
    }





    inline LPWSTR GetMessageInfoDate(int iMessageInfoIndex) {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+38, iMessageInfoIndex, 0);
    }



    inline LPWSTR GetMessageInfoFlags(int iMessageInfoIndex) {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+39, iMessageInfoIndex, 0);
    }



    inline LPWSTR GetMessageInfoFrom(int iMessageInfoIndex) {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+40, iMessageInfoIndex, 0);
    }



    inline LPWSTR GetMessageInfoMessageId(int iMessageInfoIndex) {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+41, iMessageInfoIndex, 0);
    }





    inline LPWSTR GetMessageInfoSubject(int iMessageInfoIndex) {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+43, iMessageInfoIndex, 0);
    }



    inline LPWSTR GetMessageInReplyTo() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+44, 0, 0);
    }



    inline LPWSTR GetMessageNetId() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+45, 0, 0);
    }





    inline LPWSTR GetMessagePartContentDisposition(int iMessagePartIndex) {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+47, iMessagePartIndex, 0);
    }



    inline LPWSTR GetMessagePartContentEncoding(int iMessagePartIndex) {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+48, iMessagePartIndex, 0);
    }



    inline LPWSTR GetMessagePartContentId(int iMessagePartIndex) {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+49, iMessagePartIndex, 0);
    }



    inline LPWSTR GetMessagePartContentType(int iMessagePartIndex) {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+50, iMessagePartIndex, 0);
    }



    inline LPWSTR GetMessagePartFilename(int iMessagePartIndex) {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+51, iMessagePartIndex, 0);
    }



    inline LPWSTR GetMessagePartId(int iMessagePartIndex) {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+52, iMessagePartIndex, 0);
    }



    inline LPWSTR GetMessagePartMultipartMode(int iMessagePartIndex) {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+53, iMessagePartIndex, 0);
    }



    inline LPWSTR GetMessagePartParameters(int iMessagePartIndex) {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+54, iMessagePartIndex, 0);
    }







    inline LPWSTR GetMessageRecipientAddress(int iRecipientIndex) {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+57, iRecipientIndex, 0);
    }



    inline LPWSTR GetMessageRecipientName(int iRecipientIndex) {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+58, iRecipientIndex, 0);
    }



    inline LPWSTR GetMessageRecipientOptions(int iRecipientIndex) {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+59, iRecipientIndex, 0);
    }





    inline LPWSTR GetMessageReplyTo() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+61, 0, 0);
    }



    inline LPWSTR GetMessageSender() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+62, 0, 0);
    }



    inline LPWSTR GetMessageSet() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+63, 0, 0);
    }

    inline int SetMessageSet(LPWSTR lpMessageSet) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 10000+63, 0, (void*)lpMessageSet, 0);
    }



    inline LPWSTR GetMessageSubject() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+65, 0, 0);
    }



    inline LPWSTR GetMessageText() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+66, 0, 0);
    }

    inline int SetMessageText(LPWSTR lpMessageText) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 10000+66, 0, (void*)lpMessageText, 0);
    }



    inline LPWSTR GetPassword() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+68, 0, 0);
    }

    inline int SetPassword(LPWSTR lpPassword) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 10000+68, 0, (void*)lpPassword, 0);
    }





    inline LPWSTR GetSortCriteria() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+71, 0, 0);
    }

    inline int SetSortCriteria(LPWSTR lpSortCriteria) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 10000+71, 0, (void*)lpSortCriteria, 0);
    }

    inline LPWSTR GetSSLAcceptServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+72, 0, 0);
    }

    inline int SetSSLAcceptServerCertEncoded(LPWSTR lpSSLAcceptServerCertEncoded) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 10000+72, 0, (void*)lpSSLAcceptServerCertEncoded, 0);
    }
    inline int GetSSLAcceptServerCertEncodedB(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_IMAPS_Get(m_pObj, 72, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }
    inline int SetSSLAcceptServerCertEncodedB(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 72, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }
    inline LPWSTR GetSSLCertEncoded() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+73, 0, 0);
    }

    inline int SetSSLCertEncoded(LPWSTR lpSSLCertEncoded) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 10000+73, 0, (void*)lpSSLCertEncoded, 0);
    }
    inline int GetSSLCertEncodedB(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_IMAPS_Get(m_pObj, 73, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }
    inline int SetSSLCertEncodedB(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 73, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }
    inline LPWSTR GetSSLCertStore() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+74, 0, 0);
    }

    inline int SetSSLCertStore(LPWSTR lpSSLCertStore) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 10000+74, 0, (void*)lpSSLCertStore, 0);
    }
    inline int GetSSLCertStoreB(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_IMAPS_Get(m_pObj, 74, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }
    inline int SetSSLCertStoreB(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 74, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }
    inline LPWSTR GetSSLCertStorePassword() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+75, 0, 0);
    }

    inline int SetSSLCertStorePassword(LPWSTR lpSSLCertStorePassword) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 10000+75, 0, (void*)lpSSLCertStorePassword, 0);
    }



    inline LPWSTR GetSSLCertSubject() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+77, 0, 0);
    }

    inline int SetSSLCertSubject(LPWSTR lpSSLCertSubject) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 10000+77, 0, (void*)lpSSLCertSubject, 0);
    }

    inline LPWSTR GetSSLServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+78, 0, 0);
    }


    inline int GetSSLServerCertEncodedB(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_IMAPS_Get(m_pObj, 78, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }









    inline LPWSTR GetUIDValidity() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+83, 0, 0);
    }



    inline LPWSTR GetUser() {
      return (LPWSTR)IPWorksSSL_IMAPS_Get(m_pObj, 10000+84, 0, 0);
    }

    inline int SetUser(LPWSTR lpUser) {
      return IPWorksSSL_IMAPS_Set(m_pObj, 10000+84, 0, (void*)lpUser, 0);
    }



  public: //events
  
    virtual int FireConnectionStatus(IMAPSConnectionStatusEventParamsW *e) {return 0;}
    virtual int FireEndTransfer(IMAPSEndTransferEventParamsW *e) {return 0;}
    virtual int FireError(IMAPSErrorEventParamsW *e) {return 0;}
    virtual int FireHeader(IMAPSHeaderEventParamsW *e) {return 0;}
    virtual int FireMailboxACL(IMAPSMailboxACLEventParamsW *e) {return 0;}
    virtual int FireMailboxList(IMAPSMailboxListEventParamsW *e) {return 0;}
    virtual int FireMessageInfo(IMAPSMessageInfoEventParamsW *e) {return 0;}
    virtual int FireMessagePart(IMAPSMessagePartEventParamsW *e) {return 0;}
    virtual int FirePITrail(IMAPSPITrailEventParamsW *e) {return 0;}
    virtual int FireSSLServerAuthentication(IMAPSSSLServerAuthenticationEventParamsW *e) {return 0;}
    virtual int FireSSLStatus(IMAPSSSLStatusEventParamsW *e) {return 0;}
    virtual int FireStartTransfer(IMAPSStartTransferEventParamsW *e) {return 0;}
    virtual int FireTransfer(IMAPSTransferEventParamsW *e) {return 0;}


  protected:
  
    virtual int IMAPSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {
    	int ret_code = 0;
      switch (event_id) {
         case 1: {
            IMAPSConnectionStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]),  0};
            ret_code = FireConnectionStatus(&e);
            break;
         }
         case 2: {
            IMAPSEndTransferEventParamsW e = {(int)IPS64CAST(param[0]),  0};
            ret_code = FireEndTransfer(&e);
            break;
         }
         case 3: {
            IMAPSErrorEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireError(&e);
            break;
         }
         case 4: {
            IMAPSHeaderEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireHeader(&e);
            break;
         }
         case 5: {
            IMAPSMailboxACLEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]),  0};
            ret_code = FireMailboxACL(&e);
            break;
         }
         case 6: {
            IMAPSMailboxListEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]),  0};
            ret_code = FireMailboxList(&e);
            break;
         }
         case 7: {
            IMAPSMessageInfoEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (LPWSTR)IPS64CAST(param[4]), (ns_int64*)IPS64CAST(param[5]),  0};
            ret_code = FireMessageInfo(&e);
            break;
         }
         case 8: {
            IMAPSMessagePartEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (ns_int64*)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (LPWSTR)IPS64CAST(param[4]), (LPWSTR)IPS64CAST(param[5]), (LPWSTR)IPS64CAST(param[6]), (LPWSTR)IPS64CAST(param[7]), (LPWSTR)IPS64CAST(param[8]),  0};
            ret_code = FireMessagePart(&e);
            break;
         }
         case 9: {
            IMAPSPITrailEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FirePITrail(&e);
            break;
         }
         case 10: {
            IMAPSSSLServerAuthenticationEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = FireSSLServerAuthentication(&e);
            param[4] = (void*)(e.Accept);
            break;
         }
         case 11: {
            IMAPSSSLStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]),  0};
            ret_code = FireSSLStatus(&e);
            break;
         }
         case 12: {
            IMAPSStartTransferEventParamsW e = {(int)IPS64CAST(param[0]),  0};
            ret_code = FireStartTransfer(&e);
            break;
         }
         case 13: {
            IMAPSTransferEventParamsW e = {(int)IPS64CAST(param[0]), (ns_int64*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]),  0};
            ret_code = FireTransfer(&e);
            break;
         }

      }
      return ret_code;
    }
  
  public: //event overrides

    virtual int FireConnectionStatus(IMAPSConnectionStatusEventParams *e) {return -10000;}
    virtual int FireEndTransfer(IMAPSEndTransferEventParams *e) {return -10000;}
    virtual int FireError(IMAPSErrorEventParams *e) {return -10000;}
    virtual int FireHeader(IMAPSHeaderEventParams *e) {return -10000;}
    virtual int FireMailboxACL(IMAPSMailboxACLEventParams *e) {return -10000;}
    virtual int FireMailboxList(IMAPSMailboxListEventParams *e) {return -10000;}
    virtual int FireMessageInfo(IMAPSMessageInfoEventParams *e) {return -10000;}
    virtual int FireMessagePart(IMAPSMessagePartEventParams *e) {return -10000;}
    virtual int FirePITrail(IMAPSPITrailEventParams *e) {return -10000;}
    virtual int FireSSLServerAuthentication(IMAPSSSLServerAuthenticationEventParams *e) {return -10000;}
    virtual int FireSSLStatus(IMAPSSSLStatusEventParams *e) {return -10000;}
    virtual int FireStartTransfer(IMAPSStartTransferEventParams *e) {return -10000;}
    virtual int FireTransfer(IMAPSTransferEventParams *e) {return -10000;}

  public: //methods

    inline int AddMessageFlags(LPWSTR lpszFlags) {
      void *param[1+1] = {(void*)lpszFlags, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+2, 1, param, cbparam);
      
    }
    inline int AppendToMailbox() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+3, 0, param, cbparam);
      
    }
    inline int CheckMailbox() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+4, 0, param, cbparam);
      
    }
    inline int CloseMailbox() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+5, 0, param, cbparam);
      
    }
    inline LPWSTR Config(LPWSTR lpszConfigurationString) {
      void *param[1+1] = {(void*)lpszConfigurationString, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_IMAPS_Do(m_pObj, 10000+6, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline int Connect() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+7, 0, param, cbparam);
      
    }
    inline int CopyToMailbox() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+8, 0, param, cbparam);
      
    }
    inline int CreateMailbox() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+9, 0, param, cbparam);
      
    }
    inline int DeleteFromMailbox() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+10, 0, param, cbparam);
      
    }
    inline int DeleteMailbox() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+11, 0, param, cbparam);
      
    }
    inline int DeleteMailboxACL(LPWSTR lpszUser) {
      void *param[1+1] = {(void*)lpszUser, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+12, 1, param, cbparam);
      
    }
    inline int Disconnect() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+13, 0, param, cbparam);
      
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+14, 0, param, cbparam);
      
    }
    inline int ExamineMailbox() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+15, 0, param, cbparam);
      
    }
    inline int ExpungeMailbox() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+16, 0, param, cbparam);
      
    }
    inline int FetchMessageHeaders() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+17, 0, param, cbparam);
      
    }
    inline int FetchMessageInfo() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+18, 0, param, cbparam);
      
    }
    inline int FetchMessagePart(LPWSTR lpszPartId) {
      void *param[1+1] = {(void*)lpszPartId, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+19, 1, param, cbparam);
      
    }
    inline int FetchMessagePartHeaders(LPWSTR lpszPartId) {
      void *param[1+1] = {(void*)lpszPartId, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+20, 1, param, cbparam);
      
    }
    inline int FetchMessageText() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+21, 0, param, cbparam);
      
    }
    inline int GetMailboxACL() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+22, 0, param, cbparam);
      
    }
    inline int Interrupt() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+23, 0, param, cbparam);
      
    }
    inline int ListMailboxes() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+24, 0, param, cbparam);
      
    }
    inline int ListSubscribedMailboxes() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+25, 0, param, cbparam);
      
    }
    inline LPWSTR LocalizeDate(LPWSTR lpszDateTime) {
      void *param[1+1] = {(void*)lpszDateTime, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_IMAPS_Do(m_pObj, 10000+26, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline int MoveToMailbox() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+27, 0, param, cbparam);
      
    }
    inline int Noop() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+28, 0, param, cbparam);
      
    }
    inline int RenameMailbox(LPWSTR lpszNewName) {
      void *param[1+1] = {(void*)lpszNewName, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+29, 1, param, cbparam);
      
    }
    inline int ResetMessageFlags() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+30, 0, param, cbparam);
      
    }
    inline int SearchMailbox(LPWSTR lpszSearchCriteria) {
      void *param[1+1] = {(void*)lpszSearchCriteria, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+31, 1, param, cbparam);
      
    }
    inline int SelectMailbox() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+32, 0, param, cbparam);
      
    }
    inline int SetMailboxACL(LPWSTR lpszUser, LPWSTR lpszRights) {
      void *param[2+1] = {(void*)lpszUser, (void*)lpszRights, 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+33, 2, param, cbparam);
      
    }
    inline int SubscribeMailbox() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+35, 0, param, cbparam);
      
    }
    inline int UnsetMessageFlags() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+36, 0, param, cbparam);
      
    }
    inline int UnsubscribeMailbox() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IMAPS_Do(m_pObj, 10000+37, 0, param, cbparam);
      
    }

};

#endif //WIN32

#endif //_IPWORKSSSL_IMAPS_H_




