/******************************************************************
   IP*Works! SSL V9 C++ Edition
   Copyright (c) 2013 /n software inc. - All rights reserved.
*******************************************************************/

#ifndef _IPWORKSSSL_IPDAEMONS_H_
#define _IPWORKSSSL_IPDAEMONS_H_

#define IPWORKSSSL_ONLY_TYPES
#include "ipworksssl.h"
#include "ipworksssl.key"

//CertStoreTypes
#define CST_USER                                           0
#define CST_MACHINE                                        1
#define CST_PFXFILE                                        2
#define CST_PFXBLOB                                        3
#define CST_JKSFILE                                        4
#define CST_JKSBLOB                                        5
#define CST_PEMKEY_FILE                                    6
#define CST_PEMKEY_BLOB                                    7
#define CST_PUBLIC_KEY_FILE                                8
#define CST_PUBLIC_KEY_BLOB                                9
#define CST_SSHPUBLIC_KEY_BLOB                             10
#define CST_P7BFILE                                        11
#define CST_P7BBLOB                                        12
#define CST_SSHPUBLIC_KEY_FILE                             13

//TSSLStartMode
#define SSL_AUTOMATIC                                      0
#define SSL_IMPLICIT                                       1
#define SSL_EXPLICIT                                       2
#define SSL_NONE                                           3


extern "C" void* IPWORKSSSL_CALL IPWorksSSL_IPDaemonS_Create(PIPWORKSSSL_CALLBACK lpSink, void *lpContext, char *lpOemKey);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_IPDaemonS_Destroy(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_IPDaemonS_CheckIndex(void *lpObj, int propid, int arridx);
extern "C" void* IPWORKSSSL_CALL IPWorksSSL_IPDaemonS_Get(void *lpObj, int propid, int arridx, int *lpcbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_IPDaemonS_Set(void *lpObj, int propid, int arridx, const void *val, int cbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_IPDaemonS_Do(void *lpObj, int methid, int cparam, void *param[], int cbparam[]);
extern "C" char* IPWORKSSSL_CALL IPWorksSSL_IPDaemonS_GetLastError(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_IPDaemonS_GetLastErrorCode(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_IPDaemonS_StaticInit(void *hInst);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_IPDaemonS_StaticDestroy();

#ifdef WIN32
#include <windows.h>
#pragma warning(disable:4311) 
#pragma warning(disable:4312) 
#endif

typedef struct {
  int ConnectionId;
  int StatusCode;
  const char* Description;
  int reserved;
} IPDaemonSConnectedEventParams;

typedef struct {
  const char* Address;
  int Port;
  int Accept;
  int reserved;
} IPDaemonSConnectionRequestEventParams;

typedef struct {
  int ConnectionId;
  const char* Text;
  int EOL;
  int lenText;
  int reserved;
} IPDaemonSDataInEventParams;

typedef struct {
  int ConnectionId;
  int StatusCode;
  const char* Description;
  int reserved;
} IPDaemonSDisconnectedEventParams;

typedef struct {
  int ConnectionId;
  int ErrorCode;
  const char* Description;
  int reserved;
} IPDaemonSErrorEventParams;

typedef struct {
  int ConnectionId;
  int reserved;
} IPDaemonSReadyToSendEventParams;

typedef struct {
  int ConnectionId;
  const char* CertEncoded;
  const char* CertSubject;
  const char* CertIssuer;
  const char* Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} IPDaemonSSSLClientAuthenticationEventParams;

typedef struct {
  int ConnectionId;
  const char* Message;
  int reserved;
} IPDaemonSSSLStatusEventParams;



class IPDaemonS {
  
  public: //events
  
    virtual int FireConnected(IPDaemonSConnectedEventParams *e) {return 0;}
    virtual int FireConnectionRequest(IPDaemonSConnectionRequestEventParams *e) {return 0;}
    virtual int FireDataIn(IPDaemonSDataInEventParams *e) {return 0;}
    virtual int FireDisconnected(IPDaemonSDisconnectedEventParams *e) {return 0;}
    virtual int FireError(IPDaemonSErrorEventParams *e) {return 0;}
    virtual int FireReadyToSend(IPDaemonSReadyToSendEventParams *e) {return 0;}
    virtual int FireSSLClientAuthentication(IPDaemonSSSLClientAuthenticationEventParams *e) {return 0;}
    virtual int FireSSLStatus(IPDaemonSSSLStatusEventParams *e) {return 0;}


  protected:

    void *m_pObj;
    
    static int IPWORKSSSL_CALL IPDaemonSEventSink(void *lpObj, int event_id, int cparam, void *param[], int cbparam[]) {
      int ret_code = 0;
      if (event_id > 10000) return ((IPDaemonS*)lpObj)->IPDaemonSEventSinkW(event_id - 10000, cparam, param, cbparam);
      switch (event_id) {
         case 1: {
            IPDaemonSConnectedEventParams e = {(int)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]),  0};
            ret_code = ((IPDaemonS*)lpObj)->FireConnected(&e);
            break;
         }
         case 2: {
            IPDaemonSConnectionRequestEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (int)IPS64CAST(param[2]),  0};
            ret_code = ((IPDaemonS*)lpObj)->FireConnectionRequest(&e);
            param[2] = (void*)IPS64CAST(e.Accept);
            break;
         }
         case 3: {
            IPDaemonSDataInEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (int)IPS64CAST(cbparam[1]),  0};
            ret_code = ((IPDaemonS*)lpObj)->FireDataIn(&e);
            break;
         }
         case 4: {
            IPDaemonSDisconnectedEventParams e = {(int)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]),  0};
            ret_code = ((IPDaemonS*)lpObj)->FireDisconnected(&e);
            break;
         }
         case 5: {
            IPDaemonSErrorEventParams e = {(int)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]),  0};
            ret_code = ((IPDaemonS*)lpObj)->FireError(&e);
            break;
         }
         case 6: {
            IPDaemonSReadyToSendEventParams e = {(int)IPS64CAST(param[0]),  0};
            ret_code = ((IPDaemonS*)lpObj)->FireReadyToSend(&e);
            break;
         }
         case 7: {
            IPDaemonSSSLClientAuthenticationEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (char*)IPS64CAST(param[4]), (int)IPS64CAST(param[5]), (int)IPS64CAST(cbparam[1]),  0};
            ret_code = ((IPDaemonS*)lpObj)->FireSSLClientAuthentication(&e);
            param[5] = (void*)IPS64CAST(e.Accept);
            break;
         }
         case 8: {
            IPDaemonSSSLStatusEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((IPDaemonS*)lpObj)->FireSSLStatus(&e);
            break;
         }

      }
      return ret_code;
    }

    virtual int IPDaemonSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {return 0;}

  public:

    IPDaemonS(char *lpOemKey = (char*)IPWORKSSSL_OEMKEY_12) {
      m_pObj = IPWorksSSL_IPDaemonS_Create(IPDaemonSEventSink, (void*)this, (char*)lpOemKey);
    }

    virtual ~IPDaemonS() {
      IPWorksSSL_IPDaemonS_Destroy(m_pObj);
    }

  public:

    inline char *GetLastError() {
      return IPWorksSSL_IPDaemonS_GetLastError(m_pObj);
    }
    
    inline int GetLastErrorCode() {
      return IPWorksSSL_IPDaemonS_GetLastErrorCode(m_pObj);
    }

    inline char *VERSION() {
      return (char*)IPWorksSSL_IPDaemonS_Get(m_pObj, 0, 0, 0);
    }

  public: //properties

    inline int GetConnectionBacklog() {
      void* val = IPWorksSSL_IPDaemonS_Get(m_pObj, 1, 0, 0);
      return (int)(long)val;
    }
    inline int SetConnectionBacklog(int iConnectionBacklog) {
      void* val = (void*)IPS64CAST(iConnectionBacklog);
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 1, 0, val, 0);
    }
    inline int GetConnectionCount() {
      void* val = IPWorksSSL_IPDaemonS_Get(m_pObj, 2, 0, 0);
      return (int)(long)val;
    }

    inline int GetAcceptData(int iConnectionId) {
      void* val = IPWorksSSL_IPDaemonS_Get(m_pObj, 3, iConnectionId, 0);
      return (int)(long)val;
    }
    inline int SetAcceptData(int iConnectionId, int bAcceptData) {
      void* val = (void*)IPS64CAST(bAcceptData);
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 3, iConnectionId, val, 0);
    }
    inline int GetBytesSent(int iConnectionId) {
      void* val = IPWorksSSL_IPDaemonS_Get(m_pObj, 4, iConnectionId, 0);
      return (int)(long)val;
    }

    inline int GetConnected(int iConnectionId) {
      void* val = IPWorksSSL_IPDaemonS_Get(m_pObj, 5, iConnectionId, 0);
      return (int)(long)val;
    }
    inline int SetConnected(int iConnectionId, int bConnected) {
      void* val = (void*)IPS64CAST(bConnected);
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 5, iConnectionId, val, 0);
    }
    inline char* GetConnectionId(int iConnectionId) {
      void* val = IPWorksSSL_IPDaemonS_Get(m_pObj, 6, iConnectionId, 0);
      return (char*)val;
    }



    inline int SetDataToSend(int iConnectionId, const char *lpDataToSend, int lenDataToSend) {
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 7, iConnectionId, (void*)lpDataToSend, lenDataToSend);
    }

    inline int GetEOL(int iConnectionId, char *&lpEOL, int &lenEOL) {
      lpEOL = (char*)IPWorksSSL_IPDaemonS_Get(m_pObj, 8, iConnectionId, &lenEOL);
      return lpEOL ? 0 : lenEOL;
    }

    inline int SetEOL(int iConnectionId, const char *lpEOL, int lenEOL) {
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 8, iConnectionId, (void*)lpEOL, lenEOL);
    }

    inline char* GetLocalAddress(int iConnectionId) {
      void* val = IPWorksSSL_IPDaemonS_Get(m_pObj, 9, iConnectionId, 0);
      return (char*)val;
    }


    inline char* GetRemoteHost(int iConnectionId) {
      void* val = IPWorksSSL_IPDaemonS_Get(m_pObj, 10, iConnectionId, 0);
      return (char*)val;
    }


    inline int GetRemotePort(int iConnectionId) {
      void* val = IPWorksSSL_IPDaemonS_Get(m_pObj, 11, iConnectionId, 0);
      return (int)(long)val;
    }

    inline int GetSingleLineMode(int iConnectionId) {
      void* val = IPWorksSSL_IPDaemonS_Get(m_pObj, 12, iConnectionId, 0);
      return (int)(long)val;
    }
    inline int SetSingleLineMode(int iConnectionId, int bSingleLineMode) {
      void* val = (void*)IPS64CAST(bSingleLineMode);
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 12, iConnectionId, val, 0);
    }
    inline int GetTimeout(int iConnectionId) {
      void* val = IPWorksSSL_IPDaemonS_Get(m_pObj, 13, iConnectionId, 0);
      return (int)(long)val;
    }
    inline int SetTimeout(int iConnectionId, int iTimeout) {
      void* val = (void*)IPS64CAST(iTimeout);
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 13, iConnectionId, val, 0);
    }
    inline int GetDefaultEOL(char *&lpDefaultEOL, int &lenDefaultEOL) {
      lpDefaultEOL = (char*)IPWorksSSL_IPDaemonS_Get(m_pObj, 14, 0, &lenDefaultEOL);
      return lpDefaultEOL ? 0 : lenDefaultEOL;
    }

    inline int SetDefaultEOL(const char *lpDefaultEOL, int lenDefaultEOL) {
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 14, 0, (void*)lpDefaultEOL, lenDefaultEOL);
    }

    inline int GetDefaultMaxLineLength() {
      void* val = IPWorksSSL_IPDaemonS_Get(m_pObj, 15, 0, 0);
      return (int)(long)val;
    }
    inline int SetDefaultMaxLineLength(int iDefaultMaxLineLength) {
      void* val = (void*)IPS64CAST(iDefaultMaxLineLength);
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 15, 0, val, 0);
    }
    inline int GetDefaultSingleLineMode() {
      void* val = IPWorksSSL_IPDaemonS_Get(m_pObj, 16, 0, 0);
      return (int)(long)val;
    }
    inline int SetDefaultSingleLineMode(int bDefaultSingleLineMode) {
      void* val = (void*)IPS64CAST(bDefaultSingleLineMode);
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 16, 0, val, 0);
    }
    inline int GetDefaultTimeout() {
      void* val = IPWorksSSL_IPDaemonS_Get(m_pObj, 17, 0, 0);
      return (int)(long)val;
    }
    inline int SetDefaultTimeout(int iDefaultTimeout) {
      void* val = (void*)IPS64CAST(iDefaultTimeout);
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 17, 0, val, 0);
    }
    inline int GetKeepAlive() {
      void* val = IPWorksSSL_IPDaemonS_Get(m_pObj, 18, 0, 0);
      return (int)(long)val;
    }
    inline int SetKeepAlive(int bKeepAlive) {
      void* val = (void*)IPS64CAST(bKeepAlive);
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 18, 0, val, 0);
    }
    inline int GetLinger() {
      void* val = IPWorksSSL_IPDaemonS_Get(m_pObj, 19, 0, 0);
      return (int)(long)val;
    }
    inline int SetLinger(int bLinger) {
      void* val = (void*)IPS64CAST(bLinger);
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 19, 0, val, 0);
    }
    inline int GetListening() {
      void* val = IPWorksSSL_IPDaemonS_Get(m_pObj, 20, 0, 0);
      return (int)(long)val;
    }
    inline int SetListening(int bListening) {
      void* val = (void*)IPS64CAST(bListening);
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 20, 0, val, 0);
    }
    inline char* GetLocalHost() {
      void* val = IPWorksSSL_IPDaemonS_Get(m_pObj, 21, 0, 0);
      return (char*)val;
    }

    inline int SetLocalHost(const char *lpLocalHost) {
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 21, 0, (void*)lpLocalHost, 0);
    }

    inline int GetLocalPort() {
      void* val = IPWorksSSL_IPDaemonS_Get(m_pObj, 22, 0, 0);
      return (int)(long)val;
    }
    inline int SetLocalPort(int lLocalPort) {
      void* val = (void*)IPS64CAST(lLocalPort);
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 22, 0, val, 0);
    }
    inline int GetSSLAuthenticateClients() {
      void* val = IPWorksSSL_IPDaemonS_Get(m_pObj, 23, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLAuthenticateClients(int bSSLAuthenticateClients) {
      void* val = (void*)IPS64CAST(bSSLAuthenticateClients);
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 23, 0, val, 0);
    }
    inline int GetSSLCertEncoded(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_IPDaemonS_Get(m_pObj, 24, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }

    inline int SetSSLCertEncoded(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 24, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }

    inline int GetSSLCertStore(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_IPDaemonS_Get(m_pObj, 25, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }

    inline int SetSSLCertStore(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 25, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }

    inline char* GetSSLCertStorePassword() {
      void* val = IPWorksSSL_IPDaemonS_Get(m_pObj, 26, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertStorePassword(const char *lpSSLCertStorePassword) {
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 26, 0, (void*)lpSSLCertStorePassword, 0);
    }

    inline int GetSSLCertStoreType() {
      void* val = IPWorksSSL_IPDaemonS_Get(m_pObj, 27, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLCertStoreType(int iSSLCertStoreType) {
      void* val = (void*)IPS64CAST(iSSLCertStoreType);
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 27, 0, val, 0);
    }
    inline char* GetSSLCertSubject() {
      void* val = IPWorksSSL_IPDaemonS_Get(m_pObj, 28, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertSubject(const char *lpSSLCertSubject) {
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 28, 0, (void*)lpSSLCertSubject, 0);
    }

    inline int GetSSLStartMode() {
      void* val = IPWorksSSL_IPDaemonS_Get(m_pObj, 29, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLStartMode(int iSSLStartMode) {
      void* val = (void*)IPS64CAST(iSSLStartMode);
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 29, 0, val, 0);
    }

  public: //methods

    inline char* Config(const char* lpszConfigurationString) {
      void *param[1+1] = {(void*)IPS64CAST(lpszConfigurationString), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_IPDaemonS_Do(m_pObj, 2, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline int Disconnect(int iConnectionId) {
      void *param[1+1] = {(void*)IPS64CAST(iConnectionId), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_IPDaemonS_Do(m_pObj, 3, 1, param, cbparam);
      
      
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IPDaemonS_Do(m_pObj, 4, 0, param, cbparam);
      
      
    }
    inline int Interrupt(int iConnectionId) {
      void *param[1+1] = {(void*)IPS64CAST(iConnectionId), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_IPDaemonS_Do(m_pObj, 5, 1, param, cbparam);
      
      
    }
    inline int Send(int iConnectionId, const char* lpText, int lenText) {
      void *param[2+1] = {(void*)IPS64CAST(iConnectionId), (void*)IPS64CAST(lpText), 0};
      int cbparam[2+1] = {0, lenText, 0};
      return IPWorksSSL_IPDaemonS_Do(m_pObj, 6, 2, param, cbparam);
      
      
    }
    inline int SendFile(int iConnectionId, const char* lpszFileName) {
      void *param[2+1] = {(void*)IPS64CAST(iConnectionId), (void*)IPS64CAST(lpszFileName), 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_IPDaemonS_Do(m_pObj, 7, 2, param, cbparam);
      
      
    }
    inline int SendLine(int iConnectionId, const char* lpszText) {
      void *param[2+1] = {(void*)IPS64CAST(iConnectionId), (void*)IPS64CAST(lpszText), 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_IPDaemonS_Do(m_pObj, 8, 2, param, cbparam);
      
      
    }
    inline int Shutdown() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IPDaemonS_Do(m_pObj, 9, 0, param, cbparam);
      
      
    }
    inline int StartSSL(int iConnectionId) {
      void *param[1+1] = {(void*)IPS64CAST(iConnectionId), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_IPDaemonS_Do(m_pObj, 10, 1, param, cbparam);
      
      
    }

};


#ifdef WIN32 //UNICODE

typedef struct {
  int ConnectionId;
  int StatusCode;
  LPWSTR Description;
  int reserved;
} IPDaemonSConnectedEventParamsW;

typedef struct {
  LPWSTR Address;
  int Port;
  int Accept;
  int reserved;
} IPDaemonSConnectionRequestEventParamsW;

typedef struct {
  int ConnectionId;
  LPWSTR Text;
  int EOL;
  int lenText;
  int reserved;
} IPDaemonSDataInEventParamsW;

typedef struct {
  int ConnectionId;
  int StatusCode;
  LPWSTR Description;
  int reserved;
} IPDaemonSDisconnectedEventParamsW;

typedef struct {
  int ConnectionId;
  int ErrorCode;
  LPWSTR Description;
  int reserved;
} IPDaemonSErrorEventParamsW;

typedef struct {
  int ConnectionId;
  int reserved;
} IPDaemonSReadyToSendEventParamsW;

typedef struct {
  int ConnectionId;
  LPWSTR CertEncoded;
  LPWSTR CertSubject;
  LPWSTR CertIssuer;
  LPWSTR Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} IPDaemonSSSLClientAuthenticationEventParamsW;

typedef struct {
  int ConnectionId;
  LPWSTR Message;
  int reserved;
} IPDaemonSSSLStatusEventParamsW;



class IPDaemonSW : public IPDaemonS {

  public: //properties
  










    inline LPWSTR GetConnectionId(int iConnectionId) {
      return (LPWSTR)IPWorksSSL_IPDaemonS_Get(m_pObj, 10000+6, iConnectionId, 0);
    }





    inline int SetDataToSend(int iConnectionId, LPWSTR lpDataToSend) {
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 10000+7, iConnectionId, (void*)lpDataToSend, 0);
    }

    inline int SetDataToSendB(int iConnectionId, const char *lpDataToSend, int lenDataToSend) {
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 7, iConnectionId, (void*)lpDataToSend, lenDataToSend);
    }
    inline LPWSTR GetEOL(int iConnectionId) {
      return (LPWSTR)IPWorksSSL_IPDaemonS_Get(m_pObj, 10000+8, iConnectionId, 0);
    }

    inline int SetEOL(int iConnectionId, LPWSTR lpEOL) {
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 10000+8, iConnectionId, (void*)lpEOL, 0);
    }
    inline int GetEOLB(int iConnectionId, char *&lpEOL, int &lenEOL) {
      lpEOL = (char*)IPWorksSSL_IPDaemonS_Get(m_pObj, 8, iConnectionId, &lenEOL);
      return lpEOL ? 0 : lenEOL;
    }
    inline int SetEOLB(int iConnectionId, const char *lpEOL, int lenEOL) {
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 8, iConnectionId, (void*)lpEOL, lenEOL);
    }
    inline LPWSTR GetLocalAddress(int iConnectionId) {
      return (LPWSTR)IPWorksSSL_IPDaemonS_Get(m_pObj, 10000+9, iConnectionId, 0);
    }



    inline LPWSTR GetRemoteHost(int iConnectionId) {
      return (LPWSTR)IPWorksSSL_IPDaemonS_Get(m_pObj, 10000+10, iConnectionId, 0);
    }









    inline LPWSTR GetDefaultEOL() {
      return (LPWSTR)IPWorksSSL_IPDaemonS_Get(m_pObj, 10000+14, 0, 0);
    }

    inline int SetDefaultEOL(LPWSTR lpDefaultEOL) {
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 10000+14, 0, (void*)lpDefaultEOL, 0);
    }
    inline int GetDefaultEOLB(char *&lpDefaultEOL, int &lenDefaultEOL) {
      lpDefaultEOL = (char*)IPWorksSSL_IPDaemonS_Get(m_pObj, 14, 0, &lenDefaultEOL);
      return lpDefaultEOL ? 0 : lenDefaultEOL;
    }
    inline int SetDefaultEOLB(const char *lpDefaultEOL, int lenDefaultEOL) {
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 14, 0, (void*)lpDefaultEOL, lenDefaultEOL);
    }












    inline LPWSTR GetLocalHost() {
      return (LPWSTR)IPWorksSSL_IPDaemonS_Get(m_pObj, 10000+21, 0, 0);
    }

    inline int SetLocalHost(LPWSTR lpLocalHost) {
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 10000+21, 0, (void*)lpLocalHost, 0);
    }





    inline LPWSTR GetSSLCertEncoded() {
      return (LPWSTR)IPWorksSSL_IPDaemonS_Get(m_pObj, 10000+24, 0, 0);
    }

    inline int SetSSLCertEncoded(LPWSTR lpSSLCertEncoded) {
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 10000+24, 0, (void*)lpSSLCertEncoded, 0);
    }
    inline int GetSSLCertEncodedB(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_IPDaemonS_Get(m_pObj, 24, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }
    inline int SetSSLCertEncodedB(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 24, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }
    inline LPWSTR GetSSLCertStore() {
      return (LPWSTR)IPWorksSSL_IPDaemonS_Get(m_pObj, 10000+25, 0, 0);
    }

    inline int SetSSLCertStore(LPWSTR lpSSLCertStore) {
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 10000+25, 0, (void*)lpSSLCertStore, 0);
    }
    inline int GetSSLCertStoreB(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_IPDaemonS_Get(m_pObj, 25, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }
    inline int SetSSLCertStoreB(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 25, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }
    inline LPWSTR GetSSLCertStorePassword() {
      return (LPWSTR)IPWorksSSL_IPDaemonS_Get(m_pObj, 10000+26, 0, 0);
    }

    inline int SetSSLCertStorePassword(LPWSTR lpSSLCertStorePassword) {
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 10000+26, 0, (void*)lpSSLCertStorePassword, 0);
    }



    inline LPWSTR GetSSLCertSubject() {
      return (LPWSTR)IPWorksSSL_IPDaemonS_Get(m_pObj, 10000+28, 0, 0);
    }

    inline int SetSSLCertSubject(LPWSTR lpSSLCertSubject) {
      return IPWorksSSL_IPDaemonS_Set(m_pObj, 10000+28, 0, (void*)lpSSLCertSubject, 0);
    }





  public: //events
  
    virtual int FireConnected(IPDaemonSConnectedEventParamsW *e) {return 0;}
    virtual int FireConnectionRequest(IPDaemonSConnectionRequestEventParamsW *e) {return 0;}
    virtual int FireDataIn(IPDaemonSDataInEventParamsW *e) {return 0;}
    virtual int FireDisconnected(IPDaemonSDisconnectedEventParamsW *e) {return 0;}
    virtual int FireError(IPDaemonSErrorEventParamsW *e) {return 0;}
    virtual int FireReadyToSend(IPDaemonSReadyToSendEventParamsW *e) {return 0;}
    virtual int FireSSLClientAuthentication(IPDaemonSSSLClientAuthenticationEventParamsW *e) {return 0;}
    virtual int FireSSLStatus(IPDaemonSSSLStatusEventParamsW *e) {return 0;}


  protected:
  
    virtual int IPDaemonSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {
    	int ret_code = 0;
      switch (event_id) {
         case 1: {
            IPDaemonSConnectedEventParamsW e = {(int)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]),  0};
            ret_code = FireConnected(&e);
            break;
         }
         case 2: {
            IPDaemonSConnectionRequestEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (int)IPS64CAST(param[2]),  0};
            ret_code = FireConnectionRequest(&e);
            param[2] = (void*)(e.Accept);
            break;
         }
         case 3: {
            IPDaemonSDataInEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (int)IPS64CAST(cbparam[1]),  0};
            ret_code = FireDataIn(&e);
            break;
         }
         case 4: {
            IPDaemonSDisconnectedEventParamsW e = {(int)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]),  0};
            ret_code = FireDisconnected(&e);
            break;
         }
         case 5: {
            IPDaemonSErrorEventParamsW e = {(int)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]),  0};
            ret_code = FireError(&e);
            break;
         }
         case 6: {
            IPDaemonSReadyToSendEventParamsW e = {(int)IPS64CAST(param[0]),  0};
            ret_code = FireReadyToSend(&e);
            break;
         }
         case 7: {
            IPDaemonSSSLClientAuthenticationEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (LPWSTR)IPS64CAST(param[4]), (int)IPS64CAST(param[5]), (int)IPS64CAST(cbparam[1]),  0};
            ret_code = FireSSLClientAuthentication(&e);
            param[5] = (void*)(e.Accept);
            break;
         }
         case 8: {
            IPDaemonSSSLStatusEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireSSLStatus(&e);
            break;
         }

      }
      return ret_code;
    }
  
  public: //event overrides

    virtual int FireConnected(IPDaemonSConnectedEventParams *e) {return -10000;}
    virtual int FireConnectionRequest(IPDaemonSConnectionRequestEventParams *e) {return -10000;}
    virtual int FireDataIn(IPDaemonSDataInEventParams *e) {return -10000;}
    virtual int FireDisconnected(IPDaemonSDisconnectedEventParams *e) {return -10000;}
    virtual int FireError(IPDaemonSErrorEventParams *e) {return -10000;}
    virtual int FireReadyToSend(IPDaemonSReadyToSendEventParams *e) {return -10000;}
    virtual int FireSSLClientAuthentication(IPDaemonSSSLClientAuthenticationEventParams *e) {return -10000;}
    virtual int FireSSLStatus(IPDaemonSSSLStatusEventParams *e) {return -10000;}

  public: //methods

    inline LPWSTR Config(LPWSTR lpszConfigurationString) {
      void *param[1+1] = {(void*)lpszConfigurationString, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_IPDaemonS_Do(m_pObj, 10000+2, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline int Disconnect(int iConnectionId) {
      void *param[1+1] = {(void*)iConnectionId, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_IPDaemonS_Do(m_pObj, 10000+3, 1, param, cbparam);
      
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IPDaemonS_Do(m_pObj, 10000+4, 0, param, cbparam);
      
    }
    inline int Interrupt(int iConnectionId) {
      void *param[1+1] = {(void*)iConnectionId, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_IPDaemonS_Do(m_pObj, 10000+5, 1, param, cbparam);
      
    }
    inline int Send(int iConnectionId, LPWSTR lpText, int lenText) {
      void *param[2+1] = {(void*)iConnectionId, (void*)lpText, 0};
      int cbparam[2+1] = {0, lenText, 0};
      return IPWorksSSL_IPDaemonS_Do(m_pObj, 10000+6, 2, param, cbparam);
      
    }
    inline int SendFile(int iConnectionId, LPWSTR lpszFileName) {
      void *param[2+1] = {(void*)iConnectionId, (void*)lpszFileName, 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_IPDaemonS_Do(m_pObj, 10000+7, 2, param, cbparam);
      
    }
    inline int SendLine(int iConnectionId, LPWSTR lpszText) {
      void *param[2+1] = {(void*)iConnectionId, (void*)lpszText, 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_IPDaemonS_Do(m_pObj, 10000+8, 2, param, cbparam);
      
    }
    inline int Shutdown() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_IPDaemonS_Do(m_pObj, 10000+9, 0, param, cbparam);
      
    }
    inline int StartSSL(int iConnectionId) {
      void *param[1+1] = {(void*)iConnectionId, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_IPDaemonS_Do(m_pObj, 10000+10, 1, param, cbparam);
      
    }

};

#endif //WIN32

#endif //_IPWORKSSSL_IPDAEMONS_H_




