/******************************************************************
   IP*Works! SSL V9 C++ Edition
   Copyright (c) 2013 /n software inc. - All rights reserved.
*******************************************************************/

#ifndef _IPWORKSSSL_LDAPS_H_
#define _IPWORKSSSL_LDAPS_H_

#define IPWORKSSSL_ONLY_TYPES
#include "ipworksssl.h"
#include "ipworksssl.key"

//LDAPAttributeModOps
#define AMO_ADD                                            0
#define AMO_DELETE                                         1
#define AMO_REPLACE                                        2

//AuthMechanisms
#define AM_SIMPLE                                          0
#define AM_DIGEST_MD5                                      1
#define AM_NEGOTIATE                                       2

//FirewallTypes
#define FW_NONE                                            0
#define FW_TUNNEL                                          1
#define FW_SOCKS4                                          2
#define FW_SOCKS5                                          3

//TLDAPDerefAliases
#define SDA_NEVER                                          0
#define SDA_IN_SEARCHING                                   1
#define SDA_FINDING_BASE_OBJECT                            2
#define SDA_ALWAYS                                         3

//TLDAPSearchScope
#define SS_BASE_OBJECT                                     0
#define SS_SINGLE_LEVEL                                    1
#define SS_WHOLE_SUBTREE                                   2

//CertStoreTypes
#define CST_USER                                           0
#define CST_MACHINE                                        1
#define CST_PFXFILE                                        2
#define CST_PFXBLOB                                        3
#define CST_JKSFILE                                        4
#define CST_JKSBLOB                                        5
#define CST_PEMKEY_FILE                                    6
#define CST_PEMKEY_BLOB                                    7
#define CST_PUBLIC_KEY_FILE                                8
#define CST_PUBLIC_KEY_BLOB                                9
#define CST_SSHPUBLIC_KEY_BLOB                             10
#define CST_P7BFILE                                        11
#define CST_P7BBLOB                                        12
#define CST_SSHPUBLIC_KEY_FILE                             13

//TSSLStartMode
#define SSL_AUTOMATIC                                      0
#define SSL_IMPLICIT                                       1
#define SSL_EXPLICIT                                       2
#define SSL_NONE                                           3


extern "C" void* IPWORKSSSL_CALL IPWorksSSL_LDAPS_Create(PIPWORKSSSL_CALLBACK lpSink, void *lpContext, char *lpOemKey);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_LDAPS_Destroy(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_LDAPS_CheckIndex(void *lpObj, int propid, int arridx);
extern "C" void* IPWORKSSSL_CALL IPWorksSSL_LDAPS_Get(void *lpObj, int propid, int arridx, int *lpcbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_LDAPS_Set(void *lpObj, int propid, int arridx, const void *val, int cbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_LDAPS_Do(void *lpObj, int methid, int cparam, void *param[], int cbparam[]);
extern "C" char* IPWORKSSSL_CALL IPWorksSSL_LDAPS_GetLastError(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_LDAPS_GetLastErrorCode(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_LDAPS_StaticInit(void *hInst);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_LDAPS_StaticDestroy();

#ifdef WIN32
#include <windows.h>
#pragma warning(disable:4311) 
#pragma warning(disable:4312) 
#endif

typedef struct {
  int StatusCode;
  const char* Description;
  int reserved;
} LDAPSConnectedEventParams;

typedef struct {
  const char* ConnectionEvent;
  int StatusCode;
  const char* Description;
  int reserved;
} LDAPSConnectionStatusEventParams;

typedef struct {
  int StatusCode;
  const char* Description;
  int reserved;
} LDAPSDisconnectedEventParams;

typedef struct {
  int ErrorCode;
  const char* Description;
  int reserved;
} LDAPSErrorEventParams;

typedef struct {
  int MessageId;
  const char* DN;
  int ResultCode;
  const char* Description;
  const char* ResponseName;
  const char* ResponseValue;
  int lenResponseValue;
  int reserved;
} LDAPSExtendedResponseEventParams;

typedef struct {
  int MessageId;
  const char* DN;
  int ResultCode;
  const char* Description;
  int reserved;
} LDAPSResultEventParams;

typedef struct {
  int MessageId;
  const char* DN;
  int ResultCode;
  const char* Description;
  int reserved;
} LDAPSSearchCompleteEventParams;

typedef struct {
  int MessageId;
  const char* DN;
  int ResultCode;
  const char* Description;
  int CancelSearch;
  int reserved;
} LDAPSSearchPageEventParams;

typedef struct {
  int MessageId;
  const char* DN;
  int reserved;
} LDAPSSearchResultEventParams;

typedef struct {
  int MessageId;
  const char* LdapUrl;
  int reserved;
} LDAPSSearchResultReferenceEventParams;

typedef struct {
  const char* CertEncoded;
  const char* CertSubject;
  const char* CertIssuer;
  const char* Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} LDAPSSSLServerAuthenticationEventParams;

typedef struct {
  const char* Message;
  int reserved;
} LDAPSSSLStatusEventParams;



class LDAPS {
  
  public: //events
  
    virtual int FireConnected(LDAPSConnectedEventParams *e) {return 0;}
    virtual int FireConnectionStatus(LDAPSConnectionStatusEventParams *e) {return 0;}
    virtual int FireDisconnected(LDAPSDisconnectedEventParams *e) {return 0;}
    virtual int FireError(LDAPSErrorEventParams *e) {return 0;}
    virtual int FireExtendedResponse(LDAPSExtendedResponseEventParams *e) {return 0;}
    virtual int FireResult(LDAPSResultEventParams *e) {return 0;}
    virtual int FireSearchComplete(LDAPSSearchCompleteEventParams *e) {return 0;}
    virtual int FireSearchPage(LDAPSSearchPageEventParams *e) {return 0;}
    virtual int FireSearchResult(LDAPSSearchResultEventParams *e) {return 0;}
    virtual int FireSearchResultReference(LDAPSSearchResultReferenceEventParams *e) {return 0;}
    virtual int FireSSLServerAuthentication(LDAPSSSLServerAuthenticationEventParams *e) {return 0;}
    virtual int FireSSLStatus(LDAPSSSLStatusEventParams *e) {return 0;}


  protected:

    void *m_pObj;
    
    static int IPWORKSSSL_CALL LDAPSEventSink(void *lpObj, int event_id, int cparam, void *param[], int cbparam[]) {
      int ret_code = 0;
      if (event_id > 10000) return ((LDAPS*)lpObj)->LDAPSEventSinkW(event_id - 10000, cparam, param, cbparam);
      switch (event_id) {
         case 1: {
            LDAPSConnectedEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((LDAPS*)lpObj)->FireConnected(&e);
            break;
         }
         case 2: {
            LDAPSConnectionStatusEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]),  0};
            ret_code = ((LDAPS*)lpObj)->FireConnectionStatus(&e);
            break;
         }
         case 3: {
            LDAPSDisconnectedEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((LDAPS*)lpObj)->FireDisconnected(&e);
            break;
         }
         case 4: {
            LDAPSErrorEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((LDAPS*)lpObj)->FireError(&e);
            break;
         }
         case 5: {
            LDAPSExtendedResponseEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (char*)IPS64CAST(param[4]), (char*)IPS64CAST(param[5]), (int)IPS64CAST(cbparam[5]),  0};
            ret_code = ((LDAPS*)lpObj)->FireExtendedResponse(&e);
            break;
         }
         case 6: {
            LDAPSResultEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]),  0};
            ret_code = ((LDAPS*)lpObj)->FireResult(&e);
            break;
         }
         case 7: {
            LDAPSSearchCompleteEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]),  0};
            ret_code = ((LDAPS*)lpObj)->FireSearchComplete(&e);
            break;
         }
         case 8: {
            LDAPSSearchPageEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (int)IPS64CAST(param[4]),  0};
            ret_code = ((LDAPS*)lpObj)->FireSearchPage(&e);
            param[4] = (void*)IPS64CAST(e.CancelSearch);
            break;
         }
         case 9: {
            LDAPSSearchResultEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((LDAPS*)lpObj)->FireSearchResult(&e);
            break;
         }
         case 10: {
            LDAPSSearchResultReferenceEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((LDAPS*)lpObj)->FireSearchResultReference(&e);
            break;
         }
         case 11: {
            LDAPSSSLServerAuthenticationEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = ((LDAPS*)lpObj)->FireSSLServerAuthentication(&e);
            param[4] = (void*)IPS64CAST(e.Accept);
            break;
         }
         case 12: {
            LDAPSSSLStatusEventParams e = {(char*)IPS64CAST(param[0]),  0};
            ret_code = ((LDAPS*)lpObj)->FireSSLStatus(&e);
            break;
         }

      }
      return ret_code;
    }

    virtual int LDAPSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {return 0;}

  public:

    LDAPS(char *lpOemKey = (char*)IPWORKSSSL_OEMKEY_33) {
      m_pObj = IPWorksSSL_LDAPS_Create(LDAPSEventSink, (void*)this, (char*)lpOemKey);
    }

    virtual ~LDAPS() {
      IPWorksSSL_LDAPS_Destroy(m_pObj);
    }

  public:

    inline char *GetLastError() {
      return IPWorksSSL_LDAPS_GetLastError(m_pObj);
    }
    
    inline int GetLastErrorCode() {
      return IPWorksSSL_LDAPS_GetLastErrorCode(m_pObj);
    }

    inline char *VERSION() {
      return (char*)IPWorksSSL_LDAPS_Get(m_pObj, 0, 0, 0);
    }

  public: //properties

    inline int GetAcceptData() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 1, 0, 0);
      return (int)(long)val;
    }
    inline int SetAcceptData(int bAcceptData) {
      void* val = (void*)IPS64CAST(bAcceptData);
      return IPWorksSSL_LDAPS_Set(m_pObj, 1, 0, val, 0);
    }
    inline int GetAttrCount() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 2, 0, 0);
      return (int)(long)val;
    }
    inline int SetAttrCount(int iAttrCount) {
      void* val = (void*)IPS64CAST(iAttrCount);
      return IPWorksSSL_LDAPS_Set(m_pObj, 2, 0, val, 0);
    }
    inline char* GetAttrType(int iAttrIndex) {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 3, iAttrIndex, 0);
      return (char*)val;
    }

    inline int SetAttrType(int iAttrIndex, const char *lpAttrType) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 3, iAttrIndex, (void*)lpAttrType, 0);
    }

    inline int GetAttrModOp(int iAttrIndex) {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 4, iAttrIndex, 0);
      return (int)(long)val;
    }
    inline int SetAttrModOp(int iAttrIndex, int iAttrModOp) {
      void* val = (void*)IPS64CAST(iAttrModOp);
      return IPWorksSSL_LDAPS_Set(m_pObj, 4, iAttrIndex, val, 0);
    }
    inline int GetAttrValue(int iAttrIndex, char *&lpAttrValue, int &lenAttrValue) {
      lpAttrValue = (char*)IPWorksSSL_LDAPS_Get(m_pObj, 5, iAttrIndex, &lenAttrValue);
      return lpAttrValue ? 0 : lenAttrValue;
    }

    inline int SetAttrValue(int iAttrIndex, const char *lpAttrValue, int lenAttrValue) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 5, iAttrIndex, (void*)lpAttrValue, lenAttrValue);
    }

    inline int GetAuthMechanism() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 6, 0, 0);
      return (int)(long)val;
    }
    inline int SetAuthMechanism(int iAuthMechanism) {
      void* val = (void*)IPS64CAST(iAuthMechanism);
      return IPWorksSSL_LDAPS_Set(m_pObj, 6, 0, val, 0);
    }
    inline int GetConnected() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 7, 0, 0);
      return (int)(long)val;
    }
    inline int SetConnected(int bConnected) {
      void* val = (void*)IPS64CAST(bConnected);
      return IPWorksSSL_LDAPS_Set(m_pObj, 7, 0, val, 0);
    }
    inline int GetDeleteOldRDN() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 8, 0, 0);
      return (int)(long)val;
    }
    inline int SetDeleteOldRDN(int bDeleteOldRDN) {
      void* val = (void*)IPS64CAST(bDeleteOldRDN);
      return IPWorksSSL_LDAPS_Set(m_pObj, 8, 0, val, 0);
    }
    inline char* GetDN() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 9, 0, 0);
      return (char*)val;
    }

    inline int SetDN(const char *lpDN) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 9, 0, (void*)lpDN, 0);
    }

    inline int GetFirewallAutoDetect() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 10, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallAutoDetect(int bFirewallAutoDetect) {
      void* val = (void*)IPS64CAST(bFirewallAutoDetect);
      return IPWorksSSL_LDAPS_Set(m_pObj, 10, 0, val, 0);
    }
    inline int GetFirewallType() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 11, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallType(int iFirewallType) {
      void* val = (void*)IPS64CAST(iFirewallType);
      return IPWorksSSL_LDAPS_Set(m_pObj, 11, 0, val, 0);
    }
    inline char* GetFirewallHost() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 12, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallHost(const char *lpFirewallHost) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 12, 0, (void*)lpFirewallHost, 0);
    }

    inline char* GetFirewallPassword() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 13, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallPassword(const char *lpFirewallPassword) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 13, 0, (void*)lpFirewallPassword, 0);
    }

    inline int GetFirewallPort() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 14, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallPort(int lFirewallPort) {
      void* val = (void*)IPS64CAST(lFirewallPort);
      return IPWorksSSL_LDAPS_Set(m_pObj, 14, 0, val, 0);
    }
    inline char* GetFirewallUser() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 15, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallUser(const char *lpFirewallUser) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 15, 0, (void*)lpFirewallUser, 0);
    }

    inline int GetIdle() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 16, 0, 0);
      return (int)(long)val;
    }

    inline int GetLDAPVersion() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 17, 0, 0);
      return (int)(long)val;
    }
    inline int SetLDAPVersion(int iLDAPVersion) {
      void* val = (void*)IPS64CAST(iLDAPVersion);
      return IPWorksSSL_LDAPS_Set(m_pObj, 17, 0, val, 0);
    }
    inline char* GetLocalHost() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 18, 0, 0);
      return (char*)val;
    }

    inline int SetLocalHost(const char *lpLocalHost) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 18, 0, (void*)lpLocalHost, 0);
    }

    inline int GetMessageId() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 19, 0, 0);
      return (int)(long)val;
    }
    inline int SetMessageId(int iMessageId) {
      void* val = (void*)IPS64CAST(iMessageId);
      return IPWorksSSL_LDAPS_Set(m_pObj, 19, 0, val, 0);
    }
    inline int GetPageSize() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 20, 0, 0);
      return (int)(long)val;
    }
    inline int SetPageSize(int iPageSize) {
      void* val = (void*)IPS64CAST(iPageSize);
      return IPWorksSSL_LDAPS_Set(m_pObj, 20, 0, val, 0);
    }
    inline char* GetPassword() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 21, 0, 0);
      return (char*)val;
    }

    inline int SetPassword(const char *lpPassword) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 21, 0, (void*)lpPassword, 0);
    }

    inline int GetReferenceCount() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 22, 0, 0);
      return (int)(long)val;
    }

    inline char* GetReferenceURL(int iReferenceIndex) {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 23, iReferenceIndex, 0);
      return (char*)val;
    }


    inline int GetResultCode() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 24, 0, 0);
      return (int)(long)val;
    }

    inline char* GetResultDescription() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 25, 0, 0);
      return (char*)val;
    }


    inline char* GetResultDN() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 26, 0, 0);
      return (char*)val;
    }


    inline int GetSearchDerefAliases() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 27, 0, 0);
      return (int)(long)val;
    }
    inline int SetSearchDerefAliases(int iSearchDerefAliases) {
      void* val = (void*)IPS64CAST(iSearchDerefAliases);
      return IPWorksSSL_LDAPS_Set(m_pObj, 27, 0, val, 0);
    }
    inline int GetSearchReturnValues() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 28, 0, 0);
      return (int)(long)val;
    }
    inline int SetSearchReturnValues(int bSearchReturnValues) {
      void* val = (void*)IPS64CAST(bSearchReturnValues);
      return IPWorksSSL_LDAPS_Set(m_pObj, 28, 0, val, 0);
    }
    inline int GetSearchScope() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 29, 0, 0);
      return (int)(long)val;
    }
    inline int SetSearchScope(int iSearchScope) {
      void* val = (void*)IPS64CAST(iSearchScope);
      return IPWorksSSL_LDAPS_Set(m_pObj, 29, 0, val, 0);
    }
    inline int GetSearchSizeLimit() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 30, 0, 0);
      return (int)(long)val;
    }
    inline int SetSearchSizeLimit(int iSearchSizeLimit) {
      void* val = (void*)IPS64CAST(iSearchSizeLimit);
      return IPWorksSSL_LDAPS_Set(m_pObj, 30, 0, val, 0);
    }
    inline int GetSearchTimeLimit() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 31, 0, 0);
      return (int)(long)val;
    }
    inline int SetSearchTimeLimit(int iSearchTimeLimit) {
      void* val = (void*)IPS64CAST(iSearchTimeLimit);
      return IPWorksSSL_LDAPS_Set(m_pObj, 31, 0, val, 0);
    }
    inline char* GetServerName() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 32, 0, 0);
      return (char*)val;
    }

    inline int SetServerName(const char *lpServerName) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 32, 0, (void*)lpServerName, 0);
    }

    inline int GetServerPort() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 33, 0, 0);
      return (int)(long)val;
    }
    inline int SetServerPort(int lServerPort) {
      void* val = (void*)IPS64CAST(lServerPort);
      return IPWorksSSL_LDAPS_Set(m_pObj, 33, 0, val, 0);
    }
    inline char* GetSortAttributes() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 34, 0, 0);
      return (char*)val;
    }

    inline int SetSortAttributes(const char *lpSortAttributes) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 34, 0, (void*)lpSortAttributes, 0);
    }

    inline int GetSSLAcceptServerCertEncoded(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_LDAPS_Get(m_pObj, 35, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }

    inline int SetSSLAcceptServerCertEncoded(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 35, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }

    inline int GetSSLCertEncoded(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_LDAPS_Get(m_pObj, 36, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }

    inline int SetSSLCertEncoded(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 36, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }

    inline int GetSSLCertStore(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_LDAPS_Get(m_pObj, 37, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }

    inline int SetSSLCertStore(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 37, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }

    inline char* GetSSLCertStorePassword() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 38, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertStorePassword(const char *lpSSLCertStorePassword) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 38, 0, (void*)lpSSLCertStorePassword, 0);
    }

    inline int GetSSLCertStoreType() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 39, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLCertStoreType(int iSSLCertStoreType) {
      void* val = (void*)IPS64CAST(iSSLCertStoreType);
      return IPWorksSSL_LDAPS_Set(m_pObj, 39, 0, val, 0);
    }
    inline char* GetSSLCertSubject() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 40, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertSubject(const char *lpSSLCertSubject) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 40, 0, (void*)lpSSLCertSubject, 0);
    }

    inline int GetSSLServerCertEncoded(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_LDAPS_Get(m_pObj, 41, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }


    inline int GetSSLStartMode() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 42, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLStartMode(int iSSLStartMode) {
      void* val = (void*)IPS64CAST(iSSLStartMode);
      return IPWorksSSL_LDAPS_Set(m_pObj, 42, 0, val, 0);
    }
    inline int GetTimeout() {
      void* val = IPWorksSSL_LDAPS_Get(m_pObj, 43, 0, 0);
      return (int)(long)val;
    }
    inline int SetTimeout(int iTimeout) {
      void* val = (void*)IPS64CAST(iTimeout);
      return IPWorksSSL_LDAPS_Set(m_pObj, 43, 0, val, 0);
    }

  public: //methods

    inline int Abandon(int iMessageId) {
      void *param[1+1] = {(void*)IPS64CAST(iMessageId), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_LDAPS_Do(m_pObj, 2, 1, param, cbparam);
      
      
    }
    inline int Add() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_LDAPS_Do(m_pObj, 3, 0, param, cbparam);
      
      
    }
    inline char* Attr(const char* lpszAttrType) {
      void *param[1+1] = {(void*)IPS64CAST(lpszAttrType), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_LDAPS_Do(m_pObj, 4, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline int Bind() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_LDAPS_Do(m_pObj, 5, 0, param, cbparam);
      
      
    }
    inline int Compare() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_LDAPS_Do(m_pObj, 6, 0, param, cbparam);
      
      
    }
    inline char* Config(const char* lpszConfigurationString) {
      void *param[1+1] = {(void*)IPS64CAST(lpszConfigurationString), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_LDAPS_Do(m_pObj, 7, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline int Delete() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_LDAPS_Do(m_pObj, 8, 0, param, cbparam);
      
      
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_LDAPS_Do(m_pObj, 9, 0, param, cbparam);
      
      
    }
    inline int ExtendedRequest(const char* lpszRequestName, const char* lpRequestValue, int lenRequestValue) {
      void *param[2+1] = {(void*)IPS64CAST(lpszRequestName), (void*)IPS64CAST(lpRequestValue), 0};
      int cbparam[2+1] = {0, lenRequestValue, 0};
      return IPWorksSSL_LDAPS_Do(m_pObj, 10, 2, param, cbparam);
      
      
    }
    inline int Interrupt() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_LDAPS_Do(m_pObj, 11, 0, param, cbparam);
      
      
    }
    inline int Modify() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_LDAPS_Do(m_pObj, 12, 0, param, cbparam);
      
      
    }
    inline int ModifyRDN(const char* lpszNewRDN) {
      void *param[1+1] = {(void*)IPS64CAST(lpszNewRDN), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_LDAPS_Do(m_pObj, 13, 1, param, cbparam);
      
      
    }
    inline int MoveToDN(const char* lpszNewSuperior) {
      void *param[1+1] = {(void*)IPS64CAST(lpszNewSuperior), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_LDAPS_Do(m_pObj, 14, 1, param, cbparam);
      
      
    }
    inline int Search(const char* lpszSearchFilter) {
      void *param[1+1] = {(void*)IPS64CAST(lpszSearchFilter), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_LDAPS_Do(m_pObj, 15, 1, param, cbparam);
      
      
    }
    inline int Unbind() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_LDAPS_Do(m_pObj, 16, 0, param, cbparam);
      
      
    }

};


#ifdef WIN32 //UNICODE

typedef struct {
  int StatusCode;
  LPWSTR Description;
  int reserved;
} LDAPSConnectedEventParamsW;

typedef struct {
  LPWSTR ConnectionEvent;
  int StatusCode;
  LPWSTR Description;
  int reserved;
} LDAPSConnectionStatusEventParamsW;

typedef struct {
  int StatusCode;
  LPWSTR Description;
  int reserved;
} LDAPSDisconnectedEventParamsW;

typedef struct {
  int ErrorCode;
  LPWSTR Description;
  int reserved;
} LDAPSErrorEventParamsW;

typedef struct {
  int MessageId;
  LPWSTR DN;
  int ResultCode;
  LPWSTR Description;
  LPWSTR ResponseName;
  LPWSTR ResponseValue;
  int lenResponseValue;
  int reserved;
} LDAPSExtendedResponseEventParamsW;

typedef struct {
  int MessageId;
  LPWSTR DN;
  int ResultCode;
  LPWSTR Description;
  int reserved;
} LDAPSResultEventParamsW;

typedef struct {
  int MessageId;
  LPWSTR DN;
  int ResultCode;
  LPWSTR Description;
  int reserved;
} LDAPSSearchCompleteEventParamsW;

typedef struct {
  int MessageId;
  LPWSTR DN;
  int ResultCode;
  LPWSTR Description;
  int CancelSearch;
  int reserved;
} LDAPSSearchPageEventParamsW;

typedef struct {
  int MessageId;
  LPWSTR DN;
  int reserved;
} LDAPSSearchResultEventParamsW;

typedef struct {
  int MessageId;
  LPWSTR LdapUrl;
  int reserved;
} LDAPSSearchResultReferenceEventParamsW;

typedef struct {
  LPWSTR CertEncoded;
  LPWSTR CertSubject;
  LPWSTR CertIssuer;
  LPWSTR Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} LDAPSSSLServerAuthenticationEventParamsW;

typedef struct {
  LPWSTR Message;
  int reserved;
} LDAPSSSLStatusEventParamsW;



class LDAPSW : public LDAPS {

  public: //properties
  




    inline LPWSTR GetAttrType(int iAttrIndex) {
      return (LPWSTR)IPWorksSSL_LDAPS_Get(m_pObj, 10000+3, iAttrIndex, 0);
    }

    inline int SetAttrType(int iAttrIndex, LPWSTR lpAttrType) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 10000+3, iAttrIndex, (void*)lpAttrType, 0);
    }



    inline LPWSTR GetAttrValue(int iAttrIndex) {
      return (LPWSTR)IPWorksSSL_LDAPS_Get(m_pObj, 10000+5, iAttrIndex, 0);
    }

    inline int SetAttrValue(int iAttrIndex, LPWSTR lpAttrValue) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 10000+5, iAttrIndex, (void*)lpAttrValue, 0);
    }
    inline int GetAttrValueB(int iAttrIndex, char *&lpAttrValue, int &lenAttrValue) {
      lpAttrValue = (char*)IPWorksSSL_LDAPS_Get(m_pObj, 5, iAttrIndex, &lenAttrValue);
      return lpAttrValue ? 0 : lenAttrValue;
    }
    inline int SetAttrValueB(int iAttrIndex, const char *lpAttrValue, int lenAttrValue) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 5, iAttrIndex, (void*)lpAttrValue, lenAttrValue);
    }






    inline LPWSTR GetDN() {
      return (LPWSTR)IPWorksSSL_LDAPS_Get(m_pObj, 10000+9, 0, 0);
    }

    inline int SetDN(LPWSTR lpDN) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 10000+9, 0, (void*)lpDN, 0);
    }





    inline LPWSTR GetFirewallHost() {
      return (LPWSTR)IPWorksSSL_LDAPS_Get(m_pObj, 10000+12, 0, 0);
    }

    inline int SetFirewallHost(LPWSTR lpFirewallHost) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 10000+12, 0, (void*)lpFirewallHost, 0);
    }

    inline LPWSTR GetFirewallPassword() {
      return (LPWSTR)IPWorksSSL_LDAPS_Get(m_pObj, 10000+13, 0, 0);
    }

    inline int SetFirewallPassword(LPWSTR lpFirewallPassword) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 10000+13, 0, (void*)lpFirewallPassword, 0);
    }



    inline LPWSTR GetFirewallUser() {
      return (LPWSTR)IPWorksSSL_LDAPS_Get(m_pObj, 10000+15, 0, 0);
    }

    inline int SetFirewallUser(LPWSTR lpFirewallUser) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 10000+15, 0, (void*)lpFirewallUser, 0);
    }





    inline LPWSTR GetLocalHost() {
      return (LPWSTR)IPWorksSSL_LDAPS_Get(m_pObj, 10000+18, 0, 0);
    }

    inline int SetLocalHost(LPWSTR lpLocalHost) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 10000+18, 0, (void*)lpLocalHost, 0);
    }





    inline LPWSTR GetPassword() {
      return (LPWSTR)IPWorksSSL_LDAPS_Get(m_pObj, 10000+21, 0, 0);
    }

    inline int SetPassword(LPWSTR lpPassword) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 10000+21, 0, (void*)lpPassword, 0);
    }



    inline LPWSTR GetReferenceURL(int iReferenceIndex) {
      return (LPWSTR)IPWorksSSL_LDAPS_Get(m_pObj, 10000+23, iReferenceIndex, 0);
    }





    inline LPWSTR GetResultDescription() {
      return (LPWSTR)IPWorksSSL_LDAPS_Get(m_pObj, 10000+25, 0, 0);
    }



    inline LPWSTR GetResultDN() {
      return (LPWSTR)IPWorksSSL_LDAPS_Get(m_pObj, 10000+26, 0, 0);
    }













    inline LPWSTR GetServerName() {
      return (LPWSTR)IPWorksSSL_LDAPS_Get(m_pObj, 10000+32, 0, 0);
    }

    inline int SetServerName(LPWSTR lpServerName) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 10000+32, 0, (void*)lpServerName, 0);
    }



    inline LPWSTR GetSortAttributes() {
      return (LPWSTR)IPWorksSSL_LDAPS_Get(m_pObj, 10000+34, 0, 0);
    }

    inline int SetSortAttributes(LPWSTR lpSortAttributes) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 10000+34, 0, (void*)lpSortAttributes, 0);
    }

    inline LPWSTR GetSSLAcceptServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_LDAPS_Get(m_pObj, 10000+35, 0, 0);
    }

    inline int SetSSLAcceptServerCertEncoded(LPWSTR lpSSLAcceptServerCertEncoded) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 10000+35, 0, (void*)lpSSLAcceptServerCertEncoded, 0);
    }
    inline int GetSSLAcceptServerCertEncodedB(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_LDAPS_Get(m_pObj, 35, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }
    inline int SetSSLAcceptServerCertEncodedB(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 35, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }
    inline LPWSTR GetSSLCertEncoded() {
      return (LPWSTR)IPWorksSSL_LDAPS_Get(m_pObj, 10000+36, 0, 0);
    }

    inline int SetSSLCertEncoded(LPWSTR lpSSLCertEncoded) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 10000+36, 0, (void*)lpSSLCertEncoded, 0);
    }
    inline int GetSSLCertEncodedB(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_LDAPS_Get(m_pObj, 36, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }
    inline int SetSSLCertEncodedB(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 36, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }
    inline LPWSTR GetSSLCertStore() {
      return (LPWSTR)IPWorksSSL_LDAPS_Get(m_pObj, 10000+37, 0, 0);
    }

    inline int SetSSLCertStore(LPWSTR lpSSLCertStore) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 10000+37, 0, (void*)lpSSLCertStore, 0);
    }
    inline int GetSSLCertStoreB(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_LDAPS_Get(m_pObj, 37, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }
    inline int SetSSLCertStoreB(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 37, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }
    inline LPWSTR GetSSLCertStorePassword() {
      return (LPWSTR)IPWorksSSL_LDAPS_Get(m_pObj, 10000+38, 0, 0);
    }

    inline int SetSSLCertStorePassword(LPWSTR lpSSLCertStorePassword) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 10000+38, 0, (void*)lpSSLCertStorePassword, 0);
    }



    inline LPWSTR GetSSLCertSubject() {
      return (LPWSTR)IPWorksSSL_LDAPS_Get(m_pObj, 10000+40, 0, 0);
    }

    inline int SetSSLCertSubject(LPWSTR lpSSLCertSubject) {
      return IPWorksSSL_LDAPS_Set(m_pObj, 10000+40, 0, (void*)lpSSLCertSubject, 0);
    }

    inline LPWSTR GetSSLServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_LDAPS_Get(m_pObj, 10000+41, 0, 0);
    }


    inline int GetSSLServerCertEncodedB(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_LDAPS_Get(m_pObj, 41, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }







  public: //events
  
    virtual int FireConnected(LDAPSConnectedEventParamsW *e) {return 0;}
    virtual int FireConnectionStatus(LDAPSConnectionStatusEventParamsW *e) {return 0;}
    virtual int FireDisconnected(LDAPSDisconnectedEventParamsW *e) {return 0;}
    virtual int FireError(LDAPSErrorEventParamsW *e) {return 0;}
    virtual int FireExtendedResponse(LDAPSExtendedResponseEventParamsW *e) {return 0;}
    virtual int FireResult(LDAPSResultEventParamsW *e) {return 0;}
    virtual int FireSearchComplete(LDAPSSearchCompleteEventParamsW *e) {return 0;}
    virtual int FireSearchPage(LDAPSSearchPageEventParamsW *e) {return 0;}
    virtual int FireSearchResult(LDAPSSearchResultEventParamsW *e) {return 0;}
    virtual int FireSearchResultReference(LDAPSSearchResultReferenceEventParamsW *e) {return 0;}
    virtual int FireSSLServerAuthentication(LDAPSSSLServerAuthenticationEventParamsW *e) {return 0;}
    virtual int FireSSLStatus(LDAPSSSLStatusEventParamsW *e) {return 0;}


  protected:
  
    virtual int LDAPSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {
    	int ret_code = 0;
      switch (event_id) {
         case 1: {
            LDAPSConnectedEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireConnected(&e);
            break;
         }
         case 2: {
            LDAPSConnectionStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]),  0};
            ret_code = FireConnectionStatus(&e);
            break;
         }
         case 3: {
            LDAPSDisconnectedEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireDisconnected(&e);
            break;
         }
         case 4: {
            LDAPSErrorEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireError(&e);
            break;
         }
         case 5: {
            LDAPSExtendedResponseEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (LPWSTR)IPS64CAST(param[4]), (LPWSTR)IPS64CAST(param[5]), (int)IPS64CAST(cbparam[5]),  0};
            ret_code = FireExtendedResponse(&e);
            break;
         }
         case 6: {
            LDAPSResultEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]),  0};
            ret_code = FireResult(&e);
            break;
         }
         case 7: {
            LDAPSSearchCompleteEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]),  0};
            ret_code = FireSearchComplete(&e);
            break;
         }
         case 8: {
            LDAPSSearchPageEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (int)IPS64CAST(param[4]),  0};
            ret_code = FireSearchPage(&e);
            param[4] = (void*)(e.CancelSearch);
            break;
         }
         case 9: {
            LDAPSSearchResultEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireSearchResult(&e);
            break;
         }
         case 10: {
            LDAPSSearchResultReferenceEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireSearchResultReference(&e);
            break;
         }
         case 11: {
            LDAPSSSLServerAuthenticationEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = FireSSLServerAuthentication(&e);
            param[4] = (void*)(e.Accept);
            break;
         }
         case 12: {
            LDAPSSSLStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]),  0};
            ret_code = FireSSLStatus(&e);
            break;
         }

      }
      return ret_code;
    }
  
  public: //event overrides

    virtual int FireConnected(LDAPSConnectedEventParams *e) {return -10000;}
    virtual int FireConnectionStatus(LDAPSConnectionStatusEventParams *e) {return -10000;}
    virtual int FireDisconnected(LDAPSDisconnectedEventParams *e) {return -10000;}
    virtual int FireError(LDAPSErrorEventParams *e) {return -10000;}
    virtual int FireExtendedResponse(LDAPSExtendedResponseEventParams *e) {return -10000;}
    virtual int FireResult(LDAPSResultEventParams *e) {return -10000;}
    virtual int FireSearchComplete(LDAPSSearchCompleteEventParams *e) {return -10000;}
    virtual int FireSearchPage(LDAPSSearchPageEventParams *e) {return -10000;}
    virtual int FireSearchResult(LDAPSSearchResultEventParams *e) {return -10000;}
    virtual int FireSearchResultReference(LDAPSSearchResultReferenceEventParams *e) {return -10000;}
    virtual int FireSSLServerAuthentication(LDAPSSSLServerAuthenticationEventParams *e) {return -10000;}
    virtual int FireSSLStatus(LDAPSSSLStatusEventParams *e) {return -10000;}

  public: //methods

    inline int Abandon(int iMessageId) {
      void *param[1+1] = {(void*)iMessageId, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_LDAPS_Do(m_pObj, 10000+2, 1, param, cbparam);
      
    }
    inline int Add() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_LDAPS_Do(m_pObj, 10000+3, 0, param, cbparam);
      
    }
    inline LPWSTR Attr(LPWSTR lpszAttrType) {
      void *param[1+1] = {(void*)lpszAttrType, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_LDAPS_Do(m_pObj, 10000+4, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline int Bind() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_LDAPS_Do(m_pObj, 10000+5, 0, param, cbparam);
      
    }
    inline int Compare() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_LDAPS_Do(m_pObj, 10000+6, 0, param, cbparam);
      
    }
    inline LPWSTR Config(LPWSTR lpszConfigurationString) {
      void *param[1+1] = {(void*)lpszConfigurationString, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_LDAPS_Do(m_pObj, 10000+7, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline int Delete() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_LDAPS_Do(m_pObj, 10000+8, 0, param, cbparam);
      
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_LDAPS_Do(m_pObj, 10000+9, 0, param, cbparam);
      
    }
    inline int ExtendedRequest(LPWSTR lpszRequestName, LPWSTR lpRequestValue, int lenRequestValue) {
      void *param[2+1] = {(void*)lpszRequestName, (void*)lpRequestValue, 0};
      int cbparam[2+1] = {0, lenRequestValue, 0};
      return IPWorksSSL_LDAPS_Do(m_pObj, 10000+10, 2, param, cbparam);
      
    }
    inline int Interrupt() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_LDAPS_Do(m_pObj, 10000+11, 0, param, cbparam);
      
    }
    inline int Modify() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_LDAPS_Do(m_pObj, 10000+12, 0, param, cbparam);
      
    }
    inline int ModifyRDN(LPWSTR lpszNewRDN) {
      void *param[1+1] = {(void*)lpszNewRDN, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_LDAPS_Do(m_pObj, 10000+13, 1, param, cbparam);
      
    }
    inline int MoveToDN(LPWSTR lpszNewSuperior) {
      void *param[1+1] = {(void*)lpszNewSuperior, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_LDAPS_Do(m_pObj, 10000+14, 1, param, cbparam);
      
    }
    inline int Search(LPWSTR lpszSearchFilter) {
      void *param[1+1] = {(void*)lpszSearchFilter, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_LDAPS_Do(m_pObj, 10000+15, 1, param, cbparam);
      
    }
    inline int Unbind() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_LDAPS_Do(m_pObj, 10000+16, 0, param, cbparam);
      
    }

};

#endif //WIN32

#endif //_IPWORKSSSL_LDAPS_H_




