/******************************************************************
   IP*Works! SSL V9 C++ Edition
   Copyright (c) 2013 /n software inc. - All rights reserved.
*******************************************************************/

#ifndef _IPWORKSSSL_ODATAS_H_
#define _IPWORKSSSL_ODATAS_H_

#define IPWORKSSSL_ONLY_TYPES
#include "ipworksssl.h"
#include "ipworksssl.key"

//AuthSchemes
#define AUTH_BASIC                                         0
#define AUTH_DIGEST                                        1
#define AUTH_PROPRIETARY                                   2
#define AUTH_NONE                                          3
#define AUTH_NTLM                                          4
#define AUTH_NEGOTIATE                                     5
#define AUTH_OAUTH                                         6

//AtomTextTypes
#define TT_TEXT                                            0
#define TT_HTML                                            1
#define TT_XHTML                                           2

//FirewallTypes
#define FW_NONE                                            0
#define FW_TUNNEL                                          1
#define FW_SOCKS4                                          2
#define FW_SOCKS5                                          3

//ProxySSLTypes
#define PS_AUTOMATIC                                       0
#define PS_ALWAYS                                          1
#define PS_NEVER                                           2
#define PS_TUNNEL                                          3

//AssociationMultiplicities
#define NM_ZERO_ONE                                        0
#define NM_ONE                                             1
#define NM_MANY                                            2

//AssociationDeleteActions
#define DA_NONE                                            0
#define DA_CASCADE                                         1

//CertStoreTypes
#define CST_USER                                           0
#define CST_MACHINE                                        1
#define CST_PFXFILE                                        2
#define CST_PFXBLOB                                        3
#define CST_JKSFILE                                        4
#define CST_JKSBLOB                                        5
#define CST_PEMKEY_FILE                                    6
#define CST_PEMKEY_BLOB                                    7
#define CST_PUBLIC_KEY_FILE                                8
#define CST_PUBLIC_KEY_BLOB                                9
#define CST_SSHPUBLIC_KEY_BLOB                             10
#define CST_P7BFILE                                        11
#define CST_P7BBLOB                                        12
#define CST_SSHPUBLIC_KEY_FILE                             13


extern "C" void* IPWORKSSSL_CALL IPWorksSSL_ODataS_Create(PIPWORKSSSL_CALLBACK lpSink, void *lpContext, char *lpOemKey);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_ODataS_Destroy(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_ODataS_CheckIndex(void *lpObj, int propid, int arridx);
extern "C" void* IPWORKSSSL_CALL IPWorksSSL_ODataS_Get(void *lpObj, int propid, int arridx, int *lpcbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_ODataS_Set(void *lpObj, int propid, int arridx, const void *val, int cbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_ODataS_Do(void *lpObj, int methid, int cparam, void *param[], int cbparam[]);
extern "C" char* IPWORKSSSL_CALL IPWorksSSL_ODataS_GetLastError(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_ODataS_GetLastErrorCode(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_ODataS_StaticInit(void *hInst);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_ODataS_StaticDestroy();

#ifdef WIN32
#include <windows.h>
#pragma warning(disable:4311) 
#pragma warning(disable:4312) 
#endif

typedef struct {
  int StatusCode;
  const char* Description;
  int reserved;
} ODataSConnectedEventParams;

typedef struct {
  int StatusCode;
  const char* Description;
  int reserved;
} ODataSDisconnectedEventParams;

typedef struct {
  int Direction;
  int reserved;
} ODataSEndTransferEventParams;

typedef struct {
  const char* Name;
  int reserved;
} ODataSEntityEventParams;

typedef struct {
  const char* Id;
  const char* Title;
  const char* Summary;
  const char* ETag;
  int reserved;
} ODataSEntryEventParams;

typedef struct {
  int ErrorCode;
  const char* Description;
  int reserved;
} ODataSErrorEventParams;

typedef struct {
  const char* Field;
  const char* Value;
  int reserved;
} ODataSHeaderEventParams;

typedef struct {
  const char* Name;
  const char* TypeName;
  int Nullable;
  int reserved;
} ODataSPropertyEventParams;

typedef struct {
  const char* Name;
  const char* Value;
  const char* Expires;
  const char* Domain;
  const char* Path;
  int Secure;
  int reserved;
} ODataSSetCookieEventParams;

typedef struct {
  const char* CertEncoded;
  const char* CertSubject;
  const char* CertIssuer;
  const char* Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} ODataSSSLServerAuthenticationEventParams;

typedef struct {
  const char* Message;
  int reserved;
} ODataSSSLStatusEventParams;

typedef struct {
  int Direction;
  int reserved;
} ODataSStartTransferEventParams;

typedef struct {
  const char* HTTPVersion;
  int StatusCode;
  const char* Description;
  int reserved;
} ODataSStatusEventParams;

typedef struct {
  int Direction;
  ns_int64 *pBytesTransferred;
  int PercentDone;
  const char* Text;
  int lenText;
  int reserved;
} ODataSTransferEventParams;



class ODataS {
  
  public: //events
  
    virtual int FireConnected(ODataSConnectedEventParams *e) {return 0;}
    virtual int FireDisconnected(ODataSDisconnectedEventParams *e) {return 0;}
    virtual int FireEndTransfer(ODataSEndTransferEventParams *e) {return 0;}
    virtual int FireEntity(ODataSEntityEventParams *e) {return 0;}
    virtual int FireEntry(ODataSEntryEventParams *e) {return 0;}
    virtual int FireError(ODataSErrorEventParams *e) {return 0;}
    virtual int FireHeader(ODataSHeaderEventParams *e) {return 0;}
    virtual int FireProperty(ODataSPropertyEventParams *e) {return 0;}
    virtual int FireSetCookie(ODataSSetCookieEventParams *e) {return 0;}
    virtual int FireSSLServerAuthentication(ODataSSSLServerAuthenticationEventParams *e) {return 0;}
    virtual int FireSSLStatus(ODataSSSLStatusEventParams *e) {return 0;}
    virtual int FireStartTransfer(ODataSStartTransferEventParams *e) {return 0;}
    virtual int FireStatus(ODataSStatusEventParams *e) {return 0;}
    virtual int FireTransfer(ODataSTransferEventParams *e) {return 0;}


  protected:

    void *m_pObj;
    
    static int IPWORKSSSL_CALL ODataSEventSink(void *lpObj, int event_id, int cparam, void *param[], int cbparam[]) {
      int ret_code = 0;
      if (event_id > 10000) return ((ODataS*)lpObj)->ODataSEventSinkW(event_id - 10000, cparam, param, cbparam);
      switch (event_id) {
         case 1: {
            ODataSConnectedEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((ODataS*)lpObj)->FireConnected(&e);
            break;
         }
         case 2: {
            ODataSDisconnectedEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((ODataS*)lpObj)->FireDisconnected(&e);
            break;
         }
         case 3: {
            ODataSEndTransferEventParams e = {(int)IPS64CAST(param[0]),  0};
            ret_code = ((ODataS*)lpObj)->FireEndTransfer(&e);
            break;
         }
         case 4: {
            ODataSEntityEventParams e = {(char*)IPS64CAST(param[0]),  0};
            ret_code = ((ODataS*)lpObj)->FireEntity(&e);
            break;
         }
         case 5: {
            ODataSEntryEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]),  0};
            ret_code = ((ODataS*)lpObj)->FireEntry(&e);
            break;
         }
         case 6: {
            ODataSErrorEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((ODataS*)lpObj)->FireError(&e);
            break;
         }
         case 7: {
            ODataSHeaderEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((ODataS*)lpObj)->FireHeader(&e);
            break;
         }
         case 8: {
            ODataSPropertyEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]),  0};
            ret_code = ((ODataS*)lpObj)->FireProperty(&e);
            break;
         }
         case 9: {
            ODataSSetCookieEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (char*)IPS64CAST(param[4]), (int)IPS64CAST(param[5]),  0};
            ret_code = ((ODataS*)lpObj)->FireSetCookie(&e);
            break;
         }
         case 10: {
            ODataSSSLServerAuthenticationEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = ((ODataS*)lpObj)->FireSSLServerAuthentication(&e);
            param[4] = (void*)IPS64CAST(e.Accept);
            break;
         }
         case 11: {
            ODataSSSLStatusEventParams e = {(char*)IPS64CAST(param[0]),  0};
            ret_code = ((ODataS*)lpObj)->FireSSLStatus(&e);
            break;
         }
         case 12: {
            ODataSStartTransferEventParams e = {(int)IPS64CAST(param[0]),  0};
            ret_code = ((ODataS*)lpObj)->FireStartTransfer(&e);
            break;
         }
         case 13: {
            ODataSStatusEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]),  0};
            ret_code = ((ODataS*)lpObj)->FireStatus(&e);
            break;
         }
         case 14: {
            ODataSTransferEventParams e = {(int)IPS64CAST(param[0]), (ns_int64*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (int)IPS64CAST(cbparam[3]),  0};
            ret_code = ((ODataS*)lpObj)->FireTransfer(&e);
            break;
         }

      }
      return ret_code;
    }

    virtual int ODataSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {return 0;}

  public:

    ODataS(char *lpOemKey = (char*)IPWORKSSSL_OEMKEY_81) {
      m_pObj = IPWorksSSL_ODataS_Create(ODataSEventSink, (void*)this, (char*)lpOemKey);
    }

    virtual ~ODataS() {
      IPWorksSSL_ODataS_Destroy(m_pObj);
    }

  public:

    inline char *GetLastError() {
      return IPWorksSSL_ODataS_GetLastError(m_pObj);
    }
    
    inline int GetLastErrorCode() {
      return IPWorksSSL_ODataS_GetLastErrorCode(m_pObj);
    }

    inline char *VERSION() {
      return (char*)IPWorksSSL_ODataS_Get(m_pObj, 0, 0, 0);
    }

  public: //properties

    inline char* GetAuthorization() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 1, 0, 0);
      return (char*)val;
    }

    inline int SetAuthorization(const char *lpAuthorization) {
      return IPWorksSSL_ODataS_Set(m_pObj, 1, 0, (void*)lpAuthorization, 0);
    }

    inline int GetAuthScheme() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 2, 0, 0);
      return (int)(long)val;
    }
    inline int SetAuthScheme(int iAuthScheme) {
      void* val = (void*)IPS64CAST(iAuthScheme);
      return IPWorksSSL_ODataS_Set(m_pObj, 2, 0, val, 0);
    }
    inline char* GetChannelAuthorEmail() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 3, 0, 0);
      return (char*)val;
    }

    inline int SetChannelAuthorEmail(const char *lpChannelAuthorEmail) {
      return IPWorksSSL_ODataS_Set(m_pObj, 3, 0, (void*)lpChannelAuthorEmail, 0);
    }

    inline char* GetChannelAuthorName() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 4, 0, 0);
      return (char*)val;
    }

    inline int SetChannelAuthorName(const char *lpChannelAuthorName) {
      return IPWorksSSL_ODataS_Set(m_pObj, 4, 0, (void*)lpChannelAuthorName, 0);
    }

    inline char* GetChannelAuthorUri() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 5, 0, 0);
      return (char*)val;
    }

    inline int SetChannelAuthorUri(const char *lpChannelAuthorUri) {
      return IPWorksSSL_ODataS_Set(m_pObj, 5, 0, (void*)lpChannelAuthorUri, 0);
    }

    inline char* GetChannelCategoryLabel() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 6, 0, 0);
      return (char*)val;
    }

    inline int SetChannelCategoryLabel(const char *lpChannelCategoryLabel) {
      return IPWorksSSL_ODataS_Set(m_pObj, 6, 0, (void*)lpChannelCategoryLabel, 0);
    }

    inline char* GetChannelCategoryScheme() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 7, 0, 0);
      return (char*)val;
    }

    inline int SetChannelCategoryScheme(const char *lpChannelCategoryScheme) {
      return IPWorksSSL_ODataS_Set(m_pObj, 7, 0, (void*)lpChannelCategoryScheme, 0);
    }

    inline char* GetChannelCategoryTerm() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 8, 0, 0);
      return (char*)val;
    }

    inline int SetChannelCategoryTerm(const char *lpChannelCategoryTerm) {
      return IPWorksSSL_ODataS_Set(m_pObj, 8, 0, (void*)lpChannelCategoryTerm, 0);
    }

    inline char* GetChannelContributorEmail() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 9, 0, 0);
      return (char*)val;
    }

    inline int SetChannelContributorEmail(const char *lpChannelContributorEmail) {
      return IPWorksSSL_ODataS_Set(m_pObj, 9, 0, (void*)lpChannelContributorEmail, 0);
    }

    inline char* GetChannelContributorName() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 10, 0, 0);
      return (char*)val;
    }

    inline int SetChannelContributorName(const char *lpChannelContributorName) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10, 0, (void*)lpChannelContributorName, 0);
    }

    inline char* GetChannelContributorUri() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 11, 0, 0);
      return (char*)val;
    }

    inline int SetChannelContributorUri(const char *lpChannelContributorUri) {
      return IPWorksSSL_ODataS_Set(m_pObj, 11, 0, (void*)lpChannelContributorUri, 0);
    }

    inline char* GetChannelGenerator() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 12, 0, 0);
      return (char*)val;
    }

    inline int SetChannelGenerator(const char *lpChannelGenerator) {
      return IPWorksSSL_ODataS_Set(m_pObj, 12, 0, (void*)lpChannelGenerator, 0);
    }

    inline char* GetChannelIcon() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 13, 0, 0);
      return (char*)val;
    }

    inline int SetChannelIcon(const char *lpChannelIcon) {
      return IPWorksSSL_ODataS_Set(m_pObj, 13, 0, (void*)lpChannelIcon, 0);
    }

    inline char* GetChannelId() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 14, 0, 0);
      return (char*)val;
    }

    inline int SetChannelId(const char *lpChannelId) {
      return IPWorksSSL_ODataS_Set(m_pObj, 14, 0, (void*)lpChannelId, 0);
    }

    inline char* GetChannelLinkHref() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 15, 0, 0);
      return (char*)val;
    }

    inline int SetChannelLinkHref(const char *lpChannelLinkHref) {
      return IPWorksSSL_ODataS_Set(m_pObj, 15, 0, (void*)lpChannelLinkHref, 0);
    }

    inline char* GetChannelLinkHrefLang() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 16, 0, 0);
      return (char*)val;
    }

    inline int SetChannelLinkHrefLang(const char *lpChannelLinkHrefLang) {
      return IPWorksSSL_ODataS_Set(m_pObj, 16, 0, (void*)lpChannelLinkHrefLang, 0);
    }

    inline char* GetChannelLinkLength() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 17, 0, 0);
      return (char*)val;
    }

    inline int SetChannelLinkLength(const char *lpChannelLinkLength) {
      return IPWorksSSL_ODataS_Set(m_pObj, 17, 0, (void*)lpChannelLinkLength, 0);
    }

    inline char* GetChannelLinkRel() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 18, 0, 0);
      return (char*)val;
    }

    inline int SetChannelLinkRel(const char *lpChannelLinkRel) {
      return IPWorksSSL_ODataS_Set(m_pObj, 18, 0, (void*)lpChannelLinkRel, 0);
    }

    inline char* GetChannelLinkTitle() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 19, 0, 0);
      return (char*)val;
    }

    inline int SetChannelLinkTitle(const char *lpChannelLinkTitle) {
      return IPWorksSSL_ODataS_Set(m_pObj, 19, 0, (void*)lpChannelLinkTitle, 0);
    }

    inline char* GetChannelLinkType() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 20, 0, 0);
      return (char*)val;
    }

    inline int SetChannelLinkType(const char *lpChannelLinkType) {
      return IPWorksSSL_ODataS_Set(m_pObj, 20, 0, (void*)lpChannelLinkType, 0);
    }

    inline char* GetChannelLogo() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 21, 0, 0);
      return (char*)val;
    }

    inline int SetChannelLogo(const char *lpChannelLogo) {
      return IPWorksSSL_ODataS_Set(m_pObj, 21, 0, (void*)lpChannelLogo, 0);
    }

    inline char* GetChannelRights() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 22, 0, 0);
      return (char*)val;
    }

    inline int SetChannelRights(const char *lpChannelRights) {
      return IPWorksSSL_ODataS_Set(m_pObj, 22, 0, (void*)lpChannelRights, 0);
    }

    inline int GetChannelRightsTextType() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 23, 0, 0);
      return (int)(long)val;
    }
    inline int SetChannelRightsTextType(int iChannelRightsTextType) {
      void* val = (void*)IPS64CAST(iChannelRightsTextType);
      return IPWorksSSL_ODataS_Set(m_pObj, 23, 0, val, 0);
    }
    inline char* GetChannelSubtitle() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 24, 0, 0);
      return (char*)val;
    }

    inline int SetChannelSubtitle(const char *lpChannelSubtitle) {
      return IPWorksSSL_ODataS_Set(m_pObj, 24, 0, (void*)lpChannelSubtitle, 0);
    }

    inline int GetChannelSubtitleTextType() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 25, 0, 0);
      return (int)(long)val;
    }
    inline int SetChannelSubtitleTextType(int iChannelSubtitleTextType) {
      void* val = (void*)IPS64CAST(iChannelSubtitleTextType);
      return IPWorksSSL_ODataS_Set(m_pObj, 25, 0, val, 0);
    }
    inline char* GetChannelTitle() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 26, 0, 0);
      return (char*)val;
    }

    inline int SetChannelTitle(const char *lpChannelTitle) {
      return IPWorksSSL_ODataS_Set(m_pObj, 26, 0, (void*)lpChannelTitle, 0);
    }

    inline int GetChannelTitleTextType() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 27, 0, 0);
      return (int)(long)val;
    }
    inline int SetChannelTitleTextType(int iChannelTitleTextType) {
      void* val = (void*)IPS64CAST(iChannelTitleTextType);
      return IPWorksSSL_ODataS_Set(m_pObj, 27, 0, val, 0);
    }
    inline char* GetChannelUpdated() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 28, 0, 0);
      return (char*)val;
    }

    inline int SetChannelUpdated(const char *lpChannelUpdated) {
      return IPWorksSSL_ODataS_Set(m_pObj, 28, 0, (void*)lpChannelUpdated, 0);
    }

    inline int GetCookieCount() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 29, 0, 0);
      return (int)(long)val;
    }
    inline int SetCookieCount(int iCookieCount) {
      void* val = (void*)IPS64CAST(iCookieCount);
      return IPWorksSSL_ODataS_Set(m_pObj, 29, 0, val, 0);
    }
    inline char* GetCookieDomain(int iCookieIndex) {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 30, iCookieIndex, 0);
      return (char*)val;
    }


    inline char* GetCookieExpiration(int iCookieIndex) {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 31, iCookieIndex, 0);
      return (char*)val;
    }


    inline char* GetCookieName(int iCookieIndex) {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 32, iCookieIndex, 0);
      return (char*)val;
    }

    inline int SetCookieName(int iCookieIndex, const char *lpCookieName) {
      return IPWorksSSL_ODataS_Set(m_pObj, 32, iCookieIndex, (void*)lpCookieName, 0);
    }

    inline char* GetCookiePath(int iCookieIndex) {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 33, iCookieIndex, 0);
      return (char*)val;
    }


    inline int GetCookieSecure(int iCookieIndex) {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 34, iCookieIndex, 0);
      return (int)(long)val;
    }

    inline char* GetCookieValue(int iCookieIndex) {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 35, iCookieIndex, 0);
      return (char*)val;
    }

    inline int SetCookieValue(int iCookieIndex, const char *lpCookieValue) {
      return IPWorksSSL_ODataS_Set(m_pObj, 35, iCookieIndex, (void*)lpCookieValue, 0);
    }

    inline char* GetEntryAuthor() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 36, 0, 0);
      return (char*)val;
    }

    inline int SetEntryAuthor(const char *lpEntryAuthor) {
      return IPWorksSSL_ODataS_Set(m_pObj, 36, 0, (void*)lpEntryAuthor, 0);
    }

    inline int GetEntryCount() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 37, 0, 0);
      return (int)(long)val;
    }

    inline char* GetEntryETag() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 38, 0, 0);
      return (char*)val;
    }

    inline int SetEntryETag(const char *lpEntryETag) {
      return IPWorksSSL_ODataS_Set(m_pObj, 38, 0, (void*)lpEntryETag, 0);
    }

    inline char* GetEntryId() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 39, 0, 0);
      return (char*)val;
    }

    inline int SetEntryId(const char *lpEntryId) {
      return IPWorksSSL_ODataS_Set(m_pObj, 39, 0, (void*)lpEntryId, 0);
    }

    inline int GetEntryIndex() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 40, 0, 0);
      return (int)(long)val;
    }
    inline int SetEntryIndex(int iEntryIndex) {
      void* val = (void*)IPS64CAST(iEntryIndex);
      return IPWorksSSL_ODataS_Set(m_pObj, 40, 0, val, 0);
    }
    inline int GetEntryLinksCount() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 41, 0, 0);
      return (int)(long)val;
    }
    inline int SetEntryLinksCount(int iEntryLinksCount) {
      void* val = (void*)IPS64CAST(iEntryLinksCount);
      return IPWorksSSL_ODataS_Set(m_pObj, 41, 0, val, 0);
    }
    inline char* GetEntryLinksResourcePath(int iEntryLinksIndex) {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 42, iEntryLinksIndex, 0);
      return (char*)val;
    }


    inline char* GetEntryLinksTitle(int iEntryLinksIndex) {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 43, iEntryLinksIndex, 0);
      return (char*)val;
    }


    inline int GetEntryPropertiesCount() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 44, 0, 0);
      return (int)(long)val;
    }
    inline int SetEntryPropertiesCount(int iEntryPropertiesCount) {
      void* val = (void*)IPS64CAST(iEntryPropertiesCount);
      return IPWorksSSL_ODataS_Set(m_pObj, 44, 0, val, 0);
    }
    inline char* GetEntryPropertiesAttributes(int iEntryPropertiesIndex) {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 45, iEntryPropertiesIndex, 0);
      return (char*)val;
    }

    inline int SetEntryPropertiesAttributes(int iEntryPropertiesIndex, const char *lpEntryPropertiesAttributes) {
      return IPWorksSSL_ODataS_Set(m_pObj, 45, iEntryPropertiesIndex, (void*)lpEntryPropertiesAttributes, 0);
    }

    inline int GetEntryPropertiesIsNull(int iEntryPropertiesIndex) {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 46, iEntryPropertiesIndex, 0);
      return (int)(long)val;
    }
    inline int SetEntryPropertiesIsNull(int iEntryPropertiesIndex, int bEntryPropertiesIsNull) {
      void* val = (void*)IPS64CAST(bEntryPropertiesIsNull);
      return IPWorksSSL_ODataS_Set(m_pObj, 46, iEntryPropertiesIndex, val, 0);
    }
    inline char* GetEntryPropertiesName(int iEntryPropertiesIndex) {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 47, iEntryPropertiesIndex, 0);
      return (char*)val;
    }

    inline int SetEntryPropertiesName(int iEntryPropertiesIndex, const char *lpEntryPropertiesName) {
      return IPWorksSSL_ODataS_Set(m_pObj, 47, iEntryPropertiesIndex, (void*)lpEntryPropertiesName, 0);
    }

    inline char* GetEntryPropertiesType(int iEntryPropertiesIndex) {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 48, iEntryPropertiesIndex, 0);
      return (char*)val;
    }

    inline int SetEntryPropertiesType(int iEntryPropertiesIndex, const char *lpEntryPropertiesType) {
      return IPWorksSSL_ODataS_Set(m_pObj, 48, iEntryPropertiesIndex, (void*)lpEntryPropertiesType, 0);
    }

    inline char* GetEntryPropertiesValue(int iEntryPropertiesIndex) {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 49, iEntryPropertiesIndex, 0);
      return (char*)val;
    }

    inline int SetEntryPropertiesValue(int iEntryPropertiesIndex, const char *lpEntryPropertiesValue) {
      return IPWorksSSL_ODataS_Set(m_pObj, 49, iEntryPropertiesIndex, (void*)lpEntryPropertiesValue, 0);
    }

    inline char* GetEntryPropertiesXPath(int iEntryPropertiesIndex) {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 50, iEntryPropertiesIndex, 0);
      return (char*)val;
    }


    inline char* GetEntrySummary() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 51, 0, 0);
      return (char*)val;
    }

    inline int SetEntrySummary(const char *lpEntrySummary) {
      return IPWorksSSL_ODataS_Set(m_pObj, 51, 0, (void*)lpEntrySummary, 0);
    }

    inline char* GetEntryTitle() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 52, 0, 0);
      return (char*)val;
    }

    inline int SetEntryTitle(const char *lpEntryTitle) {
      return IPWorksSSL_ODataS_Set(m_pObj, 52, 0, (void*)lpEntryTitle, 0);
    }

    inline char* GetEntryUpdated() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 53, 0, 0);
      return (char*)val;
    }


    inline char* GetEntryXML() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 54, 0, 0);
      return (char*)val;
    }

    inline int SetEntryXML(const char *lpEntryXML) {
      return IPWorksSSL_ODataS_Set(m_pObj, 54, 0, (void*)lpEntryXML, 0);
    }

    inline int GetFirewallAutoDetect() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 55, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallAutoDetect(int bFirewallAutoDetect) {
      void* val = (void*)IPS64CAST(bFirewallAutoDetect);
      return IPWorksSSL_ODataS_Set(m_pObj, 55, 0, val, 0);
    }
    inline int GetFirewallType() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 56, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallType(int iFirewallType) {
      void* val = (void*)IPS64CAST(iFirewallType);
      return IPWorksSSL_ODataS_Set(m_pObj, 56, 0, val, 0);
    }
    inline char* GetFirewallHost() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 57, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallHost(const char *lpFirewallHost) {
      return IPWorksSSL_ODataS_Set(m_pObj, 57, 0, (void*)lpFirewallHost, 0);
    }

    inline char* GetFirewallPassword() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 58, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallPassword(const char *lpFirewallPassword) {
      return IPWorksSSL_ODataS_Set(m_pObj, 58, 0, (void*)lpFirewallPassword, 0);
    }

    inline int GetFirewallPort() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 59, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallPort(int lFirewallPort) {
      void* val = (void*)IPS64CAST(lFirewallPort);
      return IPWorksSSL_ODataS_Set(m_pObj, 59, 0, val, 0);
    }
    inline char* GetFirewallUser() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 60, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallUser(const char *lpFirewallUser) {
      return IPWorksSSL_ODataS_Set(m_pObj, 60, 0, (void*)lpFirewallUser, 0);
    }

    inline int GetIdle() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 61, 0, 0);
      return (int)(long)val;
    }

    inline char* GetLocalHost() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 62, 0, 0);
      return (char*)val;
    }

    inline int SetLocalHost(const char *lpLocalHost) {
      return IPWorksSSL_ODataS_Set(m_pObj, 62, 0, (void*)lpLocalHost, 0);
    }

    inline char* GetOtherHeaders() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 63, 0, 0);
      return (char*)val;
    }

    inline int SetOtherHeaders(const char *lpOtherHeaders) {
      return IPWorksSSL_ODataS_Set(m_pObj, 63, 0, (void*)lpOtherHeaders, 0);
    }

    inline char* GetOtherQueryOptions() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 64, 0, 0);
      return (char*)val;
    }

    inline int SetOtherQueryOptions(const char *lpOtherQueryOptions) {
      return IPWorksSSL_ODataS_Set(m_pObj, 64, 0, (void*)lpOtherQueryOptions, 0);
    }

    inline int GetParsedHeaderCount() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 65, 0, 0);
      return (int)(long)val;
    }

    inline char* GetParsedHeaderField(int iParsedHeaderIndex) {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 66, iParsedHeaderIndex, 0);
      return (char*)val;
    }


    inline char* GetParsedHeaderValue(int iParsedHeaderIndex) {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 67, iParsedHeaderIndex, 0);
      return (char*)val;
    }


    inline char* GetPassword() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 68, 0, 0);
      return (char*)val;
    }

    inline int SetPassword(const char *lpPassword) {
      return IPWorksSSL_ODataS_Set(m_pObj, 68, 0, (void*)lpPassword, 0);
    }

    inline int GetProxyAuthScheme() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 69, 0, 0);
      return (int)(long)val;
    }
    inline int SetProxyAuthScheme(int iProxyAuthScheme) {
      void* val = (void*)IPS64CAST(iProxyAuthScheme);
      return IPWorksSSL_ODataS_Set(m_pObj, 69, 0, val, 0);
    }
    inline int GetProxyAutoDetect() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 70, 0, 0);
      return (int)(long)val;
    }
    inline int SetProxyAutoDetect(int bProxyAutoDetect) {
      void* val = (void*)IPS64CAST(bProxyAutoDetect);
      return IPWorksSSL_ODataS_Set(m_pObj, 70, 0, val, 0);
    }
    inline char* GetProxyPassword() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 71, 0, 0);
      return (char*)val;
    }

    inline int SetProxyPassword(const char *lpProxyPassword) {
      return IPWorksSSL_ODataS_Set(m_pObj, 71, 0, (void*)lpProxyPassword, 0);
    }

    inline int GetProxyPort() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 72, 0, 0);
      return (int)(long)val;
    }
    inline int SetProxyPort(int lProxyPort) {
      void* val = (void*)IPS64CAST(lProxyPort);
      return IPWorksSSL_ODataS_Set(m_pObj, 72, 0, val, 0);
    }
    inline char* GetProxyServer() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 73, 0, 0);
      return (char*)val;
    }

    inline int SetProxyServer(const char *lpProxyServer) {
      return IPWorksSSL_ODataS_Set(m_pObj, 73, 0, (void*)lpProxyServer, 0);
    }

    inline int GetProxySSL() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 74, 0, 0);
      return (int)(long)val;
    }
    inline int SetProxySSL(int iProxySSL) {
      void* val = (void*)IPS64CAST(iProxySSL);
      return IPWorksSSL_ODataS_Set(m_pObj, 74, 0, val, 0);
    }
    inline char* GetProxyUser() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 75, 0, 0);
      return (char*)val;
    }

    inline int SetProxyUser(const char *lpProxyUser) {
      return IPWorksSSL_ODataS_Set(m_pObj, 75, 0, (void*)lpProxyUser, 0);
    }

    inline char* GetQueryFilter() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 76, 0, 0);
      return (char*)val;
    }

    inline int SetQueryFilter(const char *lpQueryFilter) {
      return IPWorksSSL_ODataS_Set(m_pObj, 76, 0, (void*)lpQueryFilter, 0);
    }

    inline char* GetQueryOrderBy() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 77, 0, 0);
      return (char*)val;
    }

    inline int SetQueryOrderBy(const char *lpQueryOrderBy) {
      return IPWorksSSL_ODataS_Set(m_pObj, 77, 0, (void*)lpQueryOrderBy, 0);
    }

    inline char* GetQuerySelect() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 78, 0, 0);
      return (char*)val;
    }

    inline int SetQuerySelect(const char *lpQuerySelect) {
      return IPWorksSSL_ODataS_Set(m_pObj, 78, 0, (void*)lpQuerySelect, 0);
    }

    inline char* GetQuerySkip() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 79, 0, 0);
      return (char*)val;
    }

    inline int SetQuerySkip(const char *lpQuerySkip) {
      return IPWorksSSL_ODataS_Set(m_pObj, 79, 0, (void*)lpQuerySkip, 0);
    }

    inline char* GetQuerySkipToken() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 80, 0, 0);
      return (char*)val;
    }

    inline int SetQuerySkipToken(const char *lpQuerySkipToken) {
      return IPWorksSSL_ODataS_Set(m_pObj, 80, 0, (void*)lpQuerySkipToken, 0);
    }

    inline char* GetQueryTop() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 81, 0, 0);
      return (char*)val;
    }

    inline int SetQueryTop(const char *lpQueryTop) {
      return IPWorksSSL_ODataS_Set(m_pObj, 81, 0, (void*)lpQueryTop, 0);
    }

    inline char* GetResourcePath() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 82, 0, 0);
      return (char*)val;
    }

    inline int SetResourcePath(const char *lpResourcePath) {
      return IPWorksSSL_ODataS_Set(m_pObj, 82, 0, (void*)lpResourcePath, 0);
    }

    inline char* GetSchemaAssociationDescription() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 83, 0, 0);
      return (char*)val;
    }


    inline char* GetSchemaAssociationFromRole() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 84, 0, 0);
      return (char*)val;
    }


    inline char* GetSchemaAssociationFromRoleDescription() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 85, 0, 0);
      return (char*)val;
    }


    inline char* GetSchemaAssociationFromRoleEntity() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 86, 0, 0);
      return (char*)val;
    }


    inline int GetSchemaAssociationFromRoleMultiplicity() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 87, 0, 0);
      return (int)(long)val;
    }

    inline char* GetSchemaAssociationFromRoleSummary() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 88, 0, 0);
      return (char*)val;
    }


    inline char* GetSchemaAssociationName() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 89, 0, 0);
      return (char*)val;
    }


    inline int GetSchemaAssociationOnDelete() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 90, 0, 0);
      return (int)(long)val;
    }

    inline char* GetSchemaAssociationRelationship() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 91, 0, 0);
      return (char*)val;
    }


    inline char* GetSchemaAssociationSummary() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 92, 0, 0);
      return (char*)val;
    }


    inline char* GetSchemaAssociationToRole() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 93, 0, 0);
      return (char*)val;
    }


    inline char* GetSchemaAssociationToRoleDescription() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 94, 0, 0);
      return (char*)val;
    }


    inline char* GetSchemaAssociationToRoleEntity() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 95, 0, 0);
      return (char*)val;
    }


    inline int GetSchemaAssociationToRoleMultiplicity() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 96, 0, 0);
      return (int)(long)val;
    }

    inline char* GetSchemaAssociationToRoleSummary() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 97, 0, 0);
      return (char*)val;
    }


    inline int GetSchemaAssociationCount() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 98, 0, 0);
      return (int)(long)val;
    }

    inline int GetSchemaAssociationIndex() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 99, 0, 0);
      return (int)(long)val;
    }
    inline int SetSchemaAssociationIndex(int iSchemaAssociationIndex) {
      void* val = (void*)IPS64CAST(iSchemaAssociationIndex);
      return IPWorksSSL_ODataS_Set(m_pObj, 99, 0, val, 0);
    }
    inline char* GetSchemaEntityDescription() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 100, 0, 0);
      return (char*)val;
    }


    inline char* GetSchemaEntityName() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 101, 0, 0);
      return (char*)val;
    }


    inline char* GetSchemaEntitySummary() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 102, 0, 0);
      return (char*)val;
    }


    inline int GetSchemaEntityCount() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 103, 0, 0);
      return (int)(long)val;
    }

    inline int GetSchemaEntityIndex() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 104, 0, 0);
      return (int)(long)val;
    }
    inline int SetSchemaEntityIndex(int iSchemaEntityIndex) {
      void* val = (void*)IPS64CAST(iSchemaEntityIndex);
      return IPWorksSSL_ODataS_Set(m_pObj, 104, 0, val, 0);
    }
    inline int GetSchemaKeysCount() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 105, 0, 0);
      return (int)(long)val;
    }

    inline char* GetSchemaKeys(int iSchemaKeysIndex) {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 106, iSchemaKeysIndex, 0);
      return (char*)val;
    }


    inline char* GetSchemaPropertyAttributes() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 107, 0, 0);
      return (char*)val;
    }


    inline char* GetSchemaPropertyDescription() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 108, 0, 0);
      return (char*)val;
    }


    inline int GetSchemaPropertyIsComplexType() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 109, 0, 0);
      return (int)(long)val;
    }

    inline char* GetSchemaPropertyName() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 110, 0, 0);
      return (char*)val;
    }


    inline int GetSchemaPropertyNullable() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 111, 0, 0);
      return (int)(long)val;
    }

    inline char* GetSchemaPropertySummary() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 112, 0, 0);
      return (char*)val;
    }


    inline char* GetSchemaPropertyType() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 113, 0, 0);
      return (char*)val;
    }


    inline int GetSchemaPropertyCount() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 114, 0, 0);
      return (int)(long)val;
    }

    inline int GetSchemaPropertyIndex() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 115, 0, 0);
      return (int)(long)val;
    }
    inline int SetSchemaPropertyIndex(int iSchemaPropertyIndex) {
      void* val = (void*)IPS64CAST(iSchemaPropertyIndex);
      return IPWorksSSL_ODataS_Set(m_pObj, 115, 0, val, 0);
    }
    inline char* GetSchemaTypeFieldAttributes() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 116, 0, 0);
      return (char*)val;
    }


    inline char* GetSchemaTypeFieldDescription() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 117, 0, 0);
      return (char*)val;
    }


    inline int GetSchemaTypeFieldIsComplexType() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 118, 0, 0);
      return (int)(long)val;
    }

    inline char* GetSchemaTypeFieldName() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 119, 0, 0);
      return (char*)val;
    }


    inline int GetSchemaTypeFieldNullable() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 120, 0, 0);
      return (int)(long)val;
    }

    inline char* GetSchemaTypeFieldSummary() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 121, 0, 0);
      return (char*)val;
    }


    inline char* GetSchemaTypeFieldType() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 122, 0, 0);
      return (char*)val;
    }


    inline int GetSchemaTypeFieldCount() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 123, 0, 0);
      return (int)(long)val;
    }

    inline int GetSchemaTypeFieldIndex() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 124, 0, 0);
      return (int)(long)val;
    }
    inline int SetSchemaTypeFieldIndex(int iSchemaTypeFieldIndex) {
      void* val = (void*)IPS64CAST(iSchemaTypeFieldIndex);
      return IPWorksSSL_ODataS_Set(m_pObj, 124, 0, val, 0);
    }
    inline char* GetServiceRootURI() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 125, 0, 0);
      return (char*)val;
    }

    inline int SetServiceRootURI(const char *lpServiceRootURI) {
      return IPWorksSSL_ODataS_Set(m_pObj, 125, 0, (void*)lpServiceRootURI, 0);
    }

    inline int GetSSLAcceptServerCertEncoded(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_ODataS_Get(m_pObj, 126, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }

    inline int SetSSLAcceptServerCertEncoded(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_ODataS_Set(m_pObj, 126, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }

    inline int GetSSLCertEncoded(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_ODataS_Get(m_pObj, 127, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }

    inline int SetSSLCertEncoded(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_ODataS_Set(m_pObj, 127, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }

    inline int GetSSLCertStore(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_ODataS_Get(m_pObj, 128, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }

    inline int SetSSLCertStore(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_ODataS_Set(m_pObj, 128, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }

    inline char* GetSSLCertStorePassword() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 129, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertStorePassword(const char *lpSSLCertStorePassword) {
      return IPWorksSSL_ODataS_Set(m_pObj, 129, 0, (void*)lpSSLCertStorePassword, 0);
    }

    inline int GetSSLCertStoreType() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 130, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLCertStoreType(int iSSLCertStoreType) {
      void* val = (void*)IPS64CAST(iSSLCertStoreType);
      return IPWorksSSL_ODataS_Set(m_pObj, 130, 0, val, 0);
    }
    inline char* GetSSLCertSubject() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 131, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertSubject(const char *lpSSLCertSubject) {
      return IPWorksSSL_ODataS_Set(m_pObj, 131, 0, (void*)lpSSLCertSubject, 0);
    }

    inline int GetSSLServerCertEncoded(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_ODataS_Get(m_pObj, 132, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }


    inline int GetTimeout() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 133, 0, 0);
      return (int)(long)val;
    }
    inline int SetTimeout(int iTimeout) {
      void* val = (void*)IPS64CAST(iTimeout);
      return IPWorksSSL_ODataS_Set(m_pObj, 133, 0, val, 0);
    }
    inline int GetTransferredData(char *&lpTransferredData, int &lenTransferredData) {
      lpTransferredData = (char*)IPWorksSSL_ODataS_Get(m_pObj, 134, 0, &lenTransferredData);
      return lpTransferredData ? 0 : lenTransferredData;
    }


    inline char* GetUser() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 135, 0, 0);
      return (char*)val;
    }

    inline int SetUser(const char *lpUser) {
      return IPWorksSSL_ODataS_Set(m_pObj, 135, 0, (void*)lpUser, 0);
    }

    inline int GetAttrCount() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 136, 0, 0);
      return (int)(long)val;
    }

    inline char* GetAttrName(int iAttrIndex) {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 137, iAttrIndex, 0);
      return (char*)val;
    }


    inline char* GetAttrNamespace(int iAttrIndex) {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 138, iAttrIndex, 0);
      return (char*)val;
    }


    inline char* GetAttrPrefix(int iAttrIndex) {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 139, iAttrIndex, 0);
      return (char*)val;
    }


    inline char* GetAttrValue(int iAttrIndex) {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 140, iAttrIndex, 0);
      return (char*)val;
    }


    inline int GetXChildrenCount() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 141, 0, 0);
      return (int)(long)val;
    }

    inline char* GetXChildName(int iXChildIndex) {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 142, iXChildIndex, 0);
      return (char*)val;
    }


    inline char* GetXChildNamespace(int iXChildIndex) {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 143, iXChildIndex, 0);
      return (char*)val;
    }


    inline char* GetXChildPrefix(int iXChildIndex) {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 144, iXChildIndex, 0);
      return (char*)val;
    }


    inline char* GetXChildXText(int iXChildIndex) {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 145, iXChildIndex, 0);
      return (char*)val;
    }


    inline char* GetXElement() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 146, 0, 0);
      return (char*)val;
    }


    inline char* GetXNamespace() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 147, 0, 0);
      return (char*)val;
    }


    inline char* GetXParent() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 148, 0, 0);
      return (char*)val;
    }


    inline char* GetXPath() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 149, 0, 0);
      return (char*)val;
    }

    inline int SetXPath(const char *lpXPath) {
      return IPWorksSSL_ODataS_Set(m_pObj, 149, 0, (void*)lpXPath, 0);
    }

    inline char* GetXPrefix() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 150, 0, 0);
      return (char*)val;
    }


    inline char* GetXSubTree() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 151, 0, 0);
      return (char*)val;
    }


    inline char* GetXText() {
      void* val = IPWorksSSL_ODataS_Get(m_pObj, 152, 0, 0);
      return (char*)val;
    }



  public: //methods

    inline char* Config(const char* lpszConfigurationString) {
      void *param[1+1] = {(void*)IPS64CAST(lpszConfigurationString), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_ODataS_Do(m_pObj, 2, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline int CreateEntry() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_ODataS_Do(m_pObj, 3, 0, param, cbparam);
      
      
    }
    inline int CustomRequest(const char* lpszHTTPMethod, const char* lpszURL, const char* lpszPostData) {
      void *param[3+1] = {(void*)IPS64CAST(lpszHTTPMethod), (void*)IPS64CAST(lpszURL), (void*)IPS64CAST(lpszPostData), 0};
      int cbparam[3+1] = {0, 0, 0, 0};
      return IPWorksSSL_ODataS_Do(m_pObj, 4, 3, param, cbparam);
      
      
    }
    inline int DeleteEntry() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_ODataS_Do(m_pObj, 5, 0, param, cbparam);
      
      
    }
    inline char* GetEntryProperty(const char* lpszXPath) {
      void *param[1+1] = {(void*)IPS64CAST(lpszXPath), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_ODataS_Do(m_pObj, 6, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline int GetSchema() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_ODataS_Do(m_pObj, 7, 0, param, cbparam);
      
      
    }
    inline int HasXPath(const char* lpszxpath) {
      void *param[1+1] = {(void*)IPS64CAST(lpszxpath), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_ODataS_Do(m_pObj, 8, 1, param, cbparam);
      
      return (int)IPS64CAST(param[1]);
    }
    inline int QueryService() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_ODataS_Do(m_pObj, 9, 0, param, cbparam);
      
      
    }
    inline int Reset() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_ODataS_Do(m_pObj, 10, 0, param, cbparam);
      
      
    }
    inline int SetEntryProperty(const char* lpszXPath, const char* lpszValue) {
      void *param[2+1] = {(void*)IPS64CAST(lpszXPath), (void*)IPS64CAST(lpszValue), 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_ODataS_Do(m_pObj, 11, 2, param, cbparam);
      
      
    }
    inline int SetSchemaEntity(const char* lpszName) {
      void *param[1+1] = {(void*)IPS64CAST(lpszName), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_ODataS_Do(m_pObj, 12, 1, param, cbparam);
      
      
    }
    inline int SetSchemaProperty(const char* lpszName) {
      void *param[1+1] = {(void*)IPS64CAST(lpszName), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_ODataS_Do(m_pObj, 13, 1, param, cbparam);
      
      
    }
    inline int SetSchemaType(const char* lpszName) {
      void *param[1+1] = {(void*)IPS64CAST(lpszName), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_ODataS_Do(m_pObj, 14, 1, param, cbparam);
      
      
    }
    inline int UpdateEntry() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_ODataS_Do(m_pObj, 15, 0, param, cbparam);
      
      
    }
    inline int UpdateProperty(const char* lpszXPath, const char* lpszValue) {
      void *param[2+1] = {(void*)IPS64CAST(lpszXPath), (void*)IPS64CAST(lpszValue), 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_ODataS_Do(m_pObj, 16, 2, param, cbparam);
      
      
    }

};


#ifdef WIN32 //UNICODE

typedef struct {
  int StatusCode;
  LPWSTR Description;
  int reserved;
} ODataSConnectedEventParamsW;

typedef struct {
  int StatusCode;
  LPWSTR Description;
  int reserved;
} ODataSDisconnectedEventParamsW;

typedef struct {
  int Direction;
  int reserved;
} ODataSEndTransferEventParamsW;

typedef struct {
  LPWSTR Name;
  int reserved;
} ODataSEntityEventParamsW;

typedef struct {
  LPWSTR Id;
  LPWSTR Title;
  LPWSTR Summary;
  LPWSTR ETag;
  int reserved;
} ODataSEntryEventParamsW;

typedef struct {
  int ErrorCode;
  LPWSTR Description;
  int reserved;
} ODataSErrorEventParamsW;

typedef struct {
  LPWSTR Field;
  LPWSTR Value;
  int reserved;
} ODataSHeaderEventParamsW;

typedef struct {
  LPWSTR Name;
  LPWSTR TypeName;
  int Nullable;
  int reserved;
} ODataSPropertyEventParamsW;

typedef struct {
  LPWSTR Name;
  LPWSTR Value;
  LPWSTR Expires;
  LPWSTR Domain;
  LPWSTR Path;
  int Secure;
  int reserved;
} ODataSSetCookieEventParamsW;

typedef struct {
  LPWSTR CertEncoded;
  LPWSTR CertSubject;
  LPWSTR CertIssuer;
  LPWSTR Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} ODataSSSLServerAuthenticationEventParamsW;

typedef struct {
  LPWSTR Message;
  int reserved;
} ODataSSSLStatusEventParamsW;

typedef struct {
  int Direction;
  int reserved;
} ODataSStartTransferEventParamsW;

typedef struct {
  LPWSTR HTTPVersion;
  int StatusCode;
  LPWSTR Description;
  int reserved;
} ODataSStatusEventParamsW;

typedef struct {
  int Direction;
  ns_int64 *pBytesTransferred;
  int PercentDone;
  LPWSTR Text;
  int lenText;
  int reserved;
} ODataSTransferEventParamsW;



class ODataSW : public ODataS {

  public: //properties
  
    inline LPWSTR GetAuthorization() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+1, 0, 0);
    }

    inline int SetAuthorization(LPWSTR lpAuthorization) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+1, 0, (void*)lpAuthorization, 0);
    }



    inline LPWSTR GetChannelAuthorEmail() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+3, 0, 0);
    }

    inline int SetChannelAuthorEmail(LPWSTR lpChannelAuthorEmail) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+3, 0, (void*)lpChannelAuthorEmail, 0);
    }

    inline LPWSTR GetChannelAuthorName() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+4, 0, 0);
    }

    inline int SetChannelAuthorName(LPWSTR lpChannelAuthorName) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+4, 0, (void*)lpChannelAuthorName, 0);
    }

    inline LPWSTR GetChannelAuthorUri() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+5, 0, 0);
    }

    inline int SetChannelAuthorUri(LPWSTR lpChannelAuthorUri) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+5, 0, (void*)lpChannelAuthorUri, 0);
    }

    inline LPWSTR GetChannelCategoryLabel() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+6, 0, 0);
    }

    inline int SetChannelCategoryLabel(LPWSTR lpChannelCategoryLabel) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+6, 0, (void*)lpChannelCategoryLabel, 0);
    }

    inline LPWSTR GetChannelCategoryScheme() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+7, 0, 0);
    }

    inline int SetChannelCategoryScheme(LPWSTR lpChannelCategoryScheme) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+7, 0, (void*)lpChannelCategoryScheme, 0);
    }

    inline LPWSTR GetChannelCategoryTerm() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+8, 0, 0);
    }

    inline int SetChannelCategoryTerm(LPWSTR lpChannelCategoryTerm) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+8, 0, (void*)lpChannelCategoryTerm, 0);
    }

    inline LPWSTR GetChannelContributorEmail() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+9, 0, 0);
    }

    inline int SetChannelContributorEmail(LPWSTR lpChannelContributorEmail) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+9, 0, (void*)lpChannelContributorEmail, 0);
    }

    inline LPWSTR GetChannelContributorName() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+10, 0, 0);
    }

    inline int SetChannelContributorName(LPWSTR lpChannelContributorName) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+10, 0, (void*)lpChannelContributorName, 0);
    }

    inline LPWSTR GetChannelContributorUri() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+11, 0, 0);
    }

    inline int SetChannelContributorUri(LPWSTR lpChannelContributorUri) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+11, 0, (void*)lpChannelContributorUri, 0);
    }

    inline LPWSTR GetChannelGenerator() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+12, 0, 0);
    }

    inline int SetChannelGenerator(LPWSTR lpChannelGenerator) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+12, 0, (void*)lpChannelGenerator, 0);
    }

    inline LPWSTR GetChannelIcon() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+13, 0, 0);
    }

    inline int SetChannelIcon(LPWSTR lpChannelIcon) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+13, 0, (void*)lpChannelIcon, 0);
    }

    inline LPWSTR GetChannelId() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+14, 0, 0);
    }

    inline int SetChannelId(LPWSTR lpChannelId) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+14, 0, (void*)lpChannelId, 0);
    }

    inline LPWSTR GetChannelLinkHref() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+15, 0, 0);
    }

    inline int SetChannelLinkHref(LPWSTR lpChannelLinkHref) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+15, 0, (void*)lpChannelLinkHref, 0);
    }

    inline LPWSTR GetChannelLinkHrefLang() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+16, 0, 0);
    }

    inline int SetChannelLinkHrefLang(LPWSTR lpChannelLinkHrefLang) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+16, 0, (void*)lpChannelLinkHrefLang, 0);
    }

    inline LPWSTR GetChannelLinkLength() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+17, 0, 0);
    }

    inline int SetChannelLinkLength(LPWSTR lpChannelLinkLength) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+17, 0, (void*)lpChannelLinkLength, 0);
    }

    inline LPWSTR GetChannelLinkRel() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+18, 0, 0);
    }

    inline int SetChannelLinkRel(LPWSTR lpChannelLinkRel) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+18, 0, (void*)lpChannelLinkRel, 0);
    }

    inline LPWSTR GetChannelLinkTitle() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+19, 0, 0);
    }

    inline int SetChannelLinkTitle(LPWSTR lpChannelLinkTitle) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+19, 0, (void*)lpChannelLinkTitle, 0);
    }

    inline LPWSTR GetChannelLinkType() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+20, 0, 0);
    }

    inline int SetChannelLinkType(LPWSTR lpChannelLinkType) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+20, 0, (void*)lpChannelLinkType, 0);
    }

    inline LPWSTR GetChannelLogo() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+21, 0, 0);
    }

    inline int SetChannelLogo(LPWSTR lpChannelLogo) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+21, 0, (void*)lpChannelLogo, 0);
    }

    inline LPWSTR GetChannelRights() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+22, 0, 0);
    }

    inline int SetChannelRights(LPWSTR lpChannelRights) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+22, 0, (void*)lpChannelRights, 0);
    }



    inline LPWSTR GetChannelSubtitle() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+24, 0, 0);
    }

    inline int SetChannelSubtitle(LPWSTR lpChannelSubtitle) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+24, 0, (void*)lpChannelSubtitle, 0);
    }



    inline LPWSTR GetChannelTitle() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+26, 0, 0);
    }

    inline int SetChannelTitle(LPWSTR lpChannelTitle) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+26, 0, (void*)lpChannelTitle, 0);
    }



    inline LPWSTR GetChannelUpdated() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+28, 0, 0);
    }

    inline int SetChannelUpdated(LPWSTR lpChannelUpdated) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+28, 0, (void*)lpChannelUpdated, 0);
    }



    inline LPWSTR GetCookieDomain(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+30, iCookieIndex, 0);
    }



    inline LPWSTR GetCookieExpiration(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+31, iCookieIndex, 0);
    }



    inline LPWSTR GetCookieName(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+32, iCookieIndex, 0);
    }

    inline int SetCookieName(int iCookieIndex, LPWSTR lpCookieName) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+32, iCookieIndex, (void*)lpCookieName, 0);
    }

    inline LPWSTR GetCookiePath(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+33, iCookieIndex, 0);
    }





    inline LPWSTR GetCookieValue(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+35, iCookieIndex, 0);
    }

    inline int SetCookieValue(int iCookieIndex, LPWSTR lpCookieValue) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+35, iCookieIndex, (void*)lpCookieValue, 0);
    }

    inline LPWSTR GetEntryAuthor() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+36, 0, 0);
    }

    inline int SetEntryAuthor(LPWSTR lpEntryAuthor) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+36, 0, (void*)lpEntryAuthor, 0);
    }



    inline LPWSTR GetEntryETag() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+38, 0, 0);
    }

    inline int SetEntryETag(LPWSTR lpEntryETag) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+38, 0, (void*)lpEntryETag, 0);
    }

    inline LPWSTR GetEntryId() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+39, 0, 0);
    }

    inline int SetEntryId(LPWSTR lpEntryId) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+39, 0, (void*)lpEntryId, 0);
    }





    inline LPWSTR GetEntryLinksResourcePath(int iEntryLinksIndex) {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+42, iEntryLinksIndex, 0);
    }



    inline LPWSTR GetEntryLinksTitle(int iEntryLinksIndex) {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+43, iEntryLinksIndex, 0);
    }





    inline LPWSTR GetEntryPropertiesAttributes(int iEntryPropertiesIndex) {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+45, iEntryPropertiesIndex, 0);
    }

    inline int SetEntryPropertiesAttributes(int iEntryPropertiesIndex, LPWSTR lpEntryPropertiesAttributes) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+45, iEntryPropertiesIndex, (void*)lpEntryPropertiesAttributes, 0);
    }



    inline LPWSTR GetEntryPropertiesName(int iEntryPropertiesIndex) {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+47, iEntryPropertiesIndex, 0);
    }

    inline int SetEntryPropertiesName(int iEntryPropertiesIndex, LPWSTR lpEntryPropertiesName) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+47, iEntryPropertiesIndex, (void*)lpEntryPropertiesName, 0);
    }

    inline LPWSTR GetEntryPropertiesType(int iEntryPropertiesIndex) {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+48, iEntryPropertiesIndex, 0);
    }

    inline int SetEntryPropertiesType(int iEntryPropertiesIndex, LPWSTR lpEntryPropertiesType) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+48, iEntryPropertiesIndex, (void*)lpEntryPropertiesType, 0);
    }

    inline LPWSTR GetEntryPropertiesValue(int iEntryPropertiesIndex) {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+49, iEntryPropertiesIndex, 0);
    }

    inline int SetEntryPropertiesValue(int iEntryPropertiesIndex, LPWSTR lpEntryPropertiesValue) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+49, iEntryPropertiesIndex, (void*)lpEntryPropertiesValue, 0);
    }

    inline LPWSTR GetEntryPropertiesXPath(int iEntryPropertiesIndex) {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+50, iEntryPropertiesIndex, 0);
    }



    inline LPWSTR GetEntrySummary() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+51, 0, 0);
    }

    inline int SetEntrySummary(LPWSTR lpEntrySummary) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+51, 0, (void*)lpEntrySummary, 0);
    }

    inline LPWSTR GetEntryTitle() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+52, 0, 0);
    }

    inline int SetEntryTitle(LPWSTR lpEntryTitle) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+52, 0, (void*)lpEntryTitle, 0);
    }

    inline LPWSTR GetEntryUpdated() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+53, 0, 0);
    }



    inline LPWSTR GetEntryXML() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+54, 0, 0);
    }

    inline int SetEntryXML(LPWSTR lpEntryXML) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+54, 0, (void*)lpEntryXML, 0);
    }





    inline LPWSTR GetFirewallHost() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+57, 0, 0);
    }

    inline int SetFirewallHost(LPWSTR lpFirewallHost) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+57, 0, (void*)lpFirewallHost, 0);
    }

    inline LPWSTR GetFirewallPassword() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+58, 0, 0);
    }

    inline int SetFirewallPassword(LPWSTR lpFirewallPassword) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+58, 0, (void*)lpFirewallPassword, 0);
    }



    inline LPWSTR GetFirewallUser() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+60, 0, 0);
    }

    inline int SetFirewallUser(LPWSTR lpFirewallUser) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+60, 0, (void*)lpFirewallUser, 0);
    }



    inline LPWSTR GetLocalHost() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+62, 0, 0);
    }

    inline int SetLocalHost(LPWSTR lpLocalHost) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+62, 0, (void*)lpLocalHost, 0);
    }

    inline LPWSTR GetOtherHeaders() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+63, 0, 0);
    }

    inline int SetOtherHeaders(LPWSTR lpOtherHeaders) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+63, 0, (void*)lpOtherHeaders, 0);
    }

    inline LPWSTR GetOtherQueryOptions() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+64, 0, 0);
    }

    inline int SetOtherQueryOptions(LPWSTR lpOtherQueryOptions) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+64, 0, (void*)lpOtherQueryOptions, 0);
    }



    inline LPWSTR GetParsedHeaderField(int iParsedHeaderIndex) {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+66, iParsedHeaderIndex, 0);
    }



    inline LPWSTR GetParsedHeaderValue(int iParsedHeaderIndex) {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+67, iParsedHeaderIndex, 0);
    }



    inline LPWSTR GetPassword() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+68, 0, 0);
    }

    inline int SetPassword(LPWSTR lpPassword) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+68, 0, (void*)lpPassword, 0);
    }





    inline LPWSTR GetProxyPassword() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+71, 0, 0);
    }

    inline int SetProxyPassword(LPWSTR lpProxyPassword) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+71, 0, (void*)lpProxyPassword, 0);
    }



    inline LPWSTR GetProxyServer() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+73, 0, 0);
    }

    inline int SetProxyServer(LPWSTR lpProxyServer) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+73, 0, (void*)lpProxyServer, 0);
    }



    inline LPWSTR GetProxyUser() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+75, 0, 0);
    }

    inline int SetProxyUser(LPWSTR lpProxyUser) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+75, 0, (void*)lpProxyUser, 0);
    }

    inline LPWSTR GetQueryFilter() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+76, 0, 0);
    }

    inline int SetQueryFilter(LPWSTR lpQueryFilter) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+76, 0, (void*)lpQueryFilter, 0);
    }

    inline LPWSTR GetQueryOrderBy() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+77, 0, 0);
    }

    inline int SetQueryOrderBy(LPWSTR lpQueryOrderBy) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+77, 0, (void*)lpQueryOrderBy, 0);
    }

    inline LPWSTR GetQuerySelect() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+78, 0, 0);
    }

    inline int SetQuerySelect(LPWSTR lpQuerySelect) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+78, 0, (void*)lpQuerySelect, 0);
    }

    inline LPWSTR GetQuerySkip() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+79, 0, 0);
    }

    inline int SetQuerySkip(LPWSTR lpQuerySkip) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+79, 0, (void*)lpQuerySkip, 0);
    }

    inline LPWSTR GetQuerySkipToken() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+80, 0, 0);
    }

    inline int SetQuerySkipToken(LPWSTR lpQuerySkipToken) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+80, 0, (void*)lpQuerySkipToken, 0);
    }

    inline LPWSTR GetQueryTop() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+81, 0, 0);
    }

    inline int SetQueryTop(LPWSTR lpQueryTop) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+81, 0, (void*)lpQueryTop, 0);
    }

    inline LPWSTR GetResourcePath() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+82, 0, 0);
    }

    inline int SetResourcePath(LPWSTR lpResourcePath) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+82, 0, (void*)lpResourcePath, 0);
    }

    inline LPWSTR GetSchemaAssociationDescription() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+83, 0, 0);
    }



    inline LPWSTR GetSchemaAssociationFromRole() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+84, 0, 0);
    }



    inline LPWSTR GetSchemaAssociationFromRoleDescription() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+85, 0, 0);
    }



    inline LPWSTR GetSchemaAssociationFromRoleEntity() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+86, 0, 0);
    }





    inline LPWSTR GetSchemaAssociationFromRoleSummary() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+88, 0, 0);
    }



    inline LPWSTR GetSchemaAssociationName() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+89, 0, 0);
    }





    inline LPWSTR GetSchemaAssociationRelationship() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+91, 0, 0);
    }



    inline LPWSTR GetSchemaAssociationSummary() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+92, 0, 0);
    }



    inline LPWSTR GetSchemaAssociationToRole() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+93, 0, 0);
    }



    inline LPWSTR GetSchemaAssociationToRoleDescription() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+94, 0, 0);
    }



    inline LPWSTR GetSchemaAssociationToRoleEntity() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+95, 0, 0);
    }





    inline LPWSTR GetSchemaAssociationToRoleSummary() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+97, 0, 0);
    }







    inline LPWSTR GetSchemaEntityDescription() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+100, 0, 0);
    }



    inline LPWSTR GetSchemaEntityName() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+101, 0, 0);
    }



    inline LPWSTR GetSchemaEntitySummary() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+102, 0, 0);
    }









    inline LPWSTR GetSchemaKeys(int iSchemaKeysIndex) {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+106, iSchemaKeysIndex, 0);
    }



    inline LPWSTR GetSchemaPropertyAttributes() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+107, 0, 0);
    }



    inline LPWSTR GetSchemaPropertyDescription() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+108, 0, 0);
    }





    inline LPWSTR GetSchemaPropertyName() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+110, 0, 0);
    }





    inline LPWSTR GetSchemaPropertySummary() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+112, 0, 0);
    }



    inline LPWSTR GetSchemaPropertyType() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+113, 0, 0);
    }







    inline LPWSTR GetSchemaTypeFieldAttributes() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+116, 0, 0);
    }



    inline LPWSTR GetSchemaTypeFieldDescription() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+117, 0, 0);
    }





    inline LPWSTR GetSchemaTypeFieldName() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+119, 0, 0);
    }





    inline LPWSTR GetSchemaTypeFieldSummary() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+121, 0, 0);
    }



    inline LPWSTR GetSchemaTypeFieldType() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+122, 0, 0);
    }







    inline LPWSTR GetServiceRootURI() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+125, 0, 0);
    }

    inline int SetServiceRootURI(LPWSTR lpServiceRootURI) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+125, 0, (void*)lpServiceRootURI, 0);
    }

    inline LPWSTR GetSSLAcceptServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+126, 0, 0);
    }

    inline int SetSSLAcceptServerCertEncoded(LPWSTR lpSSLAcceptServerCertEncoded) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+126, 0, (void*)lpSSLAcceptServerCertEncoded, 0);
    }
    inline int GetSSLAcceptServerCertEncodedB(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_ODataS_Get(m_pObj, 126, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }
    inline int SetSSLAcceptServerCertEncodedB(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_ODataS_Set(m_pObj, 126, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }
    inline LPWSTR GetSSLCertEncoded() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+127, 0, 0);
    }

    inline int SetSSLCertEncoded(LPWSTR lpSSLCertEncoded) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+127, 0, (void*)lpSSLCertEncoded, 0);
    }
    inline int GetSSLCertEncodedB(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_ODataS_Get(m_pObj, 127, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }
    inline int SetSSLCertEncodedB(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_ODataS_Set(m_pObj, 127, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }
    inline LPWSTR GetSSLCertStore() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+128, 0, 0);
    }

    inline int SetSSLCertStore(LPWSTR lpSSLCertStore) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+128, 0, (void*)lpSSLCertStore, 0);
    }
    inline int GetSSLCertStoreB(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_ODataS_Get(m_pObj, 128, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }
    inline int SetSSLCertStoreB(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_ODataS_Set(m_pObj, 128, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }
    inline LPWSTR GetSSLCertStorePassword() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+129, 0, 0);
    }

    inline int SetSSLCertStorePassword(LPWSTR lpSSLCertStorePassword) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+129, 0, (void*)lpSSLCertStorePassword, 0);
    }



    inline LPWSTR GetSSLCertSubject() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+131, 0, 0);
    }

    inline int SetSSLCertSubject(LPWSTR lpSSLCertSubject) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+131, 0, (void*)lpSSLCertSubject, 0);
    }

    inline LPWSTR GetSSLServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+132, 0, 0);
    }


    inline int GetSSLServerCertEncodedB(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_ODataS_Get(m_pObj, 132, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }



    inline LPWSTR GetTransferredData() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+134, 0, 0);
    }


    inline int GetTransferredDataB(char *&lpTransferredData, int &lenTransferredData) {
      lpTransferredData = (char*)IPWorksSSL_ODataS_Get(m_pObj, 134, 0, &lenTransferredData);
      return lpTransferredData ? 0 : lenTransferredData;
    }

    inline LPWSTR GetUser() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+135, 0, 0);
    }

    inline int SetUser(LPWSTR lpUser) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+135, 0, (void*)lpUser, 0);
    }



    inline LPWSTR GetAttrName(int iAttrIndex) {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+137, iAttrIndex, 0);
    }



    inline LPWSTR GetAttrNamespace(int iAttrIndex) {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+138, iAttrIndex, 0);
    }



    inline LPWSTR GetAttrPrefix(int iAttrIndex) {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+139, iAttrIndex, 0);
    }



    inline LPWSTR GetAttrValue(int iAttrIndex) {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+140, iAttrIndex, 0);
    }





    inline LPWSTR GetXChildName(int iXChildIndex) {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+142, iXChildIndex, 0);
    }



    inline LPWSTR GetXChildNamespace(int iXChildIndex) {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+143, iXChildIndex, 0);
    }



    inline LPWSTR GetXChildPrefix(int iXChildIndex) {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+144, iXChildIndex, 0);
    }



    inline LPWSTR GetXChildXText(int iXChildIndex) {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+145, iXChildIndex, 0);
    }



    inline LPWSTR GetXElement() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+146, 0, 0);
    }



    inline LPWSTR GetXNamespace() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+147, 0, 0);
    }



    inline LPWSTR GetXParent() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+148, 0, 0);
    }



    inline LPWSTR GetXPath() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+149, 0, 0);
    }

    inline int SetXPath(LPWSTR lpXPath) {
      return IPWorksSSL_ODataS_Set(m_pObj, 10000+149, 0, (void*)lpXPath, 0);
    }

    inline LPWSTR GetXPrefix() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+150, 0, 0);
    }



    inline LPWSTR GetXSubTree() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+151, 0, 0);
    }



    inline LPWSTR GetXText() {
      return (LPWSTR)IPWorksSSL_ODataS_Get(m_pObj, 10000+152, 0, 0);
    }





  public: //events
  
    virtual int FireConnected(ODataSConnectedEventParamsW *e) {return 0;}
    virtual int FireDisconnected(ODataSDisconnectedEventParamsW *e) {return 0;}
    virtual int FireEndTransfer(ODataSEndTransferEventParamsW *e) {return 0;}
    virtual int FireEntity(ODataSEntityEventParamsW *e) {return 0;}
    virtual int FireEntry(ODataSEntryEventParamsW *e) {return 0;}
    virtual int FireError(ODataSErrorEventParamsW *e) {return 0;}
    virtual int FireHeader(ODataSHeaderEventParamsW *e) {return 0;}
    virtual int FireProperty(ODataSPropertyEventParamsW *e) {return 0;}
    virtual int FireSetCookie(ODataSSetCookieEventParamsW *e) {return 0;}
    virtual int FireSSLServerAuthentication(ODataSSSLServerAuthenticationEventParamsW *e) {return 0;}
    virtual int FireSSLStatus(ODataSSSLStatusEventParamsW *e) {return 0;}
    virtual int FireStartTransfer(ODataSStartTransferEventParamsW *e) {return 0;}
    virtual int FireStatus(ODataSStatusEventParamsW *e) {return 0;}
    virtual int FireTransfer(ODataSTransferEventParamsW *e) {return 0;}


  protected:
  
    virtual int ODataSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {
    	int ret_code = 0;
      switch (event_id) {
         case 1: {
            ODataSConnectedEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireConnected(&e);
            break;
         }
         case 2: {
            ODataSDisconnectedEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireDisconnected(&e);
            break;
         }
         case 3: {
            ODataSEndTransferEventParamsW e = {(int)IPS64CAST(param[0]),  0};
            ret_code = FireEndTransfer(&e);
            break;
         }
         case 4: {
            ODataSEntityEventParamsW e = {(LPWSTR)IPS64CAST(param[0]),  0};
            ret_code = FireEntity(&e);
            break;
         }
         case 5: {
            ODataSEntryEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]),  0};
            ret_code = FireEntry(&e);
            break;
         }
         case 6: {
            ODataSErrorEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireError(&e);
            break;
         }
         case 7: {
            ODataSHeaderEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireHeader(&e);
            break;
         }
         case 8: {
            ODataSPropertyEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (int)IPS64CAST(param[2]),  0};
            ret_code = FireProperty(&e);
            break;
         }
         case 9: {
            ODataSSetCookieEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (LPWSTR)IPS64CAST(param[4]), (int)IPS64CAST(param[5]),  0};
            ret_code = FireSetCookie(&e);
            break;
         }
         case 10: {
            ODataSSSLServerAuthenticationEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = FireSSLServerAuthentication(&e);
            param[4] = (void*)(e.Accept);
            break;
         }
         case 11: {
            ODataSSSLStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]),  0};
            ret_code = FireSSLStatus(&e);
            break;
         }
         case 12: {
            ODataSStartTransferEventParamsW e = {(int)IPS64CAST(param[0]),  0};
            ret_code = FireStartTransfer(&e);
            break;
         }
         case 13: {
            ODataSStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]),  0};
            ret_code = FireStatus(&e);
            break;
         }
         case 14: {
            ODataSTransferEventParamsW e = {(int)IPS64CAST(param[0]), (ns_int64*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (int)IPS64CAST(cbparam[3]),  0};
            ret_code = FireTransfer(&e);
            break;
         }

      }
      return ret_code;
    }
  
  public: //event overrides

    virtual int FireConnected(ODataSConnectedEventParams *e) {return -10000;}
    virtual int FireDisconnected(ODataSDisconnectedEventParams *e) {return -10000;}
    virtual int FireEndTransfer(ODataSEndTransferEventParams *e) {return -10000;}
    virtual int FireEntity(ODataSEntityEventParams *e) {return -10000;}
    virtual int FireEntry(ODataSEntryEventParams *e) {return -10000;}
    virtual int FireError(ODataSErrorEventParams *e) {return -10000;}
    virtual int FireHeader(ODataSHeaderEventParams *e) {return -10000;}
    virtual int FireProperty(ODataSPropertyEventParams *e) {return -10000;}
    virtual int FireSetCookie(ODataSSetCookieEventParams *e) {return -10000;}
    virtual int FireSSLServerAuthentication(ODataSSSLServerAuthenticationEventParams *e) {return -10000;}
    virtual int FireSSLStatus(ODataSSSLStatusEventParams *e) {return -10000;}
    virtual int FireStartTransfer(ODataSStartTransferEventParams *e) {return -10000;}
    virtual int FireStatus(ODataSStatusEventParams *e) {return -10000;}
    virtual int FireTransfer(ODataSTransferEventParams *e) {return -10000;}

  public: //methods

    inline LPWSTR Config(LPWSTR lpszConfigurationString) {
      void *param[1+1] = {(void*)lpszConfigurationString, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_ODataS_Do(m_pObj, 10000+2, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline int CreateEntry() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_ODataS_Do(m_pObj, 10000+3, 0, param, cbparam);
      
    }
    inline int CustomRequest(LPWSTR lpszHTTPMethod, LPWSTR lpszURL, LPWSTR lpszPostData) {
      void *param[3+1] = {(void*)lpszHTTPMethod, (void*)lpszURL, (void*)lpszPostData, 0};
      int cbparam[3+1] = {0, 0, 0, 0};
      return IPWorksSSL_ODataS_Do(m_pObj, 10000+4, 3, param, cbparam);
      
    }
    inline int DeleteEntry() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_ODataS_Do(m_pObj, 10000+5, 0, param, cbparam);
      
    }
    inline LPWSTR GetEntryProperty(LPWSTR lpszXPath) {
      void *param[1+1] = {(void*)lpszXPath, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_ODataS_Do(m_pObj, 10000+6, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline int GetSchema() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_ODataS_Do(m_pObj, 10000+7, 0, param, cbparam);
      
    }
    inline int HasXPath(LPWSTR lpszxpath) {
      void *param[1+1] = {(void*)lpszxpath, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_ODataS_Do(m_pObj, 10000+8, 1, param, cbparam);
      return (int)IPS64CAST(param[1]);
    }
    inline int QueryService() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_ODataS_Do(m_pObj, 10000+9, 0, param, cbparam);
      
    }
    inline int Reset() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_ODataS_Do(m_pObj, 10000+10, 0, param, cbparam);
      
    }
    inline int SetEntryProperty(LPWSTR lpszXPath, LPWSTR lpszValue) {
      void *param[2+1] = {(void*)lpszXPath, (void*)lpszValue, 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_ODataS_Do(m_pObj, 10000+11, 2, param, cbparam);
      
    }
    inline int SetSchemaEntity(LPWSTR lpszName) {
      void *param[1+1] = {(void*)lpszName, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_ODataS_Do(m_pObj, 10000+12, 1, param, cbparam);
      
    }
    inline int SetSchemaProperty(LPWSTR lpszName) {
      void *param[1+1] = {(void*)lpszName, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_ODataS_Do(m_pObj, 10000+13, 1, param, cbparam);
      
    }
    inline int SetSchemaType(LPWSTR lpszName) {
      void *param[1+1] = {(void*)lpszName, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_ODataS_Do(m_pObj, 10000+14, 1, param, cbparam);
      
    }
    inline int UpdateEntry() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_ODataS_Do(m_pObj, 10000+15, 0, param, cbparam);
      
    }
    inline int UpdateProperty(LPWSTR lpszXPath, LPWSTR lpszValue) {
      void *param[2+1] = {(void*)lpszXPath, (void*)lpszValue, 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_ODataS_Do(m_pObj, 10000+16, 2, param, cbparam);
      
    }

};

#endif //WIN32

#endif //_IPWORKSSSL_ODATAS_H_




