/******************************************************************
   IP*Works! SSL V9 C++ Edition
   Copyright (c) 2013 /n software inc. - All rights reserved.
*******************************************************************/

#ifndef _IPWORKSSSL_POPS_H_
#define _IPWORKSSSL_POPS_H_

#define IPWORKSSSL_ONLY_TYPES
#include "ipworksssl.h"
#include "ipworksssl.key"

//AuthMechanisms
#define AM_USER_PASSWORD                                   0
#define AM_CRAMMD5                                         1
#define AM_NTLM                                            2
#define AM_APOP                                            3
#define AM_SASLPLAIN                                       4
#define AM_SASLDIGEST_MD5                                  5
#define AM_KERBEROS                                        6

//FirewallTypes
#define FW_NONE                                            0
#define FW_TUNNEL                                          1
#define FW_SOCKS4                                          2
#define FW_SOCKS5                                          3

//EmailRecipientTypes
#define RT_TO                                              0
#define RT_CC                                              1
#define RT_BCC                                             2

//CertStoreTypes
#define CST_USER                                           0
#define CST_MACHINE                                        1
#define CST_PFXFILE                                        2
#define CST_PFXBLOB                                        3
#define CST_JKSFILE                                        4
#define CST_JKSBLOB                                        5
#define CST_PEMKEY_FILE                                    6
#define CST_PEMKEY_BLOB                                    7
#define CST_PUBLIC_KEY_FILE                                8
#define CST_PUBLIC_KEY_BLOB                                9
#define CST_SSHPUBLIC_KEY_BLOB                             10
#define CST_P7BFILE                                        11
#define CST_P7BBLOB                                        12
#define CST_SSHPUBLIC_KEY_FILE                             13

//TSSLStartMode
#define SSL_AUTOMATIC                                      0
#define SSL_IMPLICIT                                       1
#define SSL_EXPLICIT                                       2
#define SSL_NONE                                           3


extern "C" void* IPWORKSSSL_CALL IPWorksSSL_POPS_Create(PIPWORKSSSL_CALLBACK lpSink, void *lpContext, char *lpOemKey);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_POPS_Destroy(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_POPS_CheckIndex(void *lpObj, int propid, int arridx);
extern "C" void* IPWORKSSSL_CALL IPWorksSSL_POPS_Get(void *lpObj, int propid, int arridx, int *lpcbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_POPS_Set(void *lpObj, int propid, int arridx, const void *val, int cbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_POPS_Do(void *lpObj, int methid, int cparam, void *param[], int cbparam[]);
extern "C" char* IPWORKSSSL_CALL IPWorksSSL_POPS_GetLastError(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_POPS_GetLastErrorCode(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_POPS_StaticInit(void *hInst);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_POPS_StaticDestroy();

#ifdef WIN32
#include <windows.h>
#pragma warning(disable:4311) 
#pragma warning(disable:4312) 
#endif

typedef struct {
  const char* ConnectionEvent;
  int StatusCode;
  const char* Description;
  int reserved;
} POPSConnectionStatusEventParams;

typedef struct {
  int Direction;
  int reserved;
} POPSEndTransferEventParams;

typedef struct {
  int ErrorCode;
  const char* Description;
  int reserved;
} POPSErrorEventParams;

typedef struct {
  const char* Field;
  const char* Value;
  int reserved;
} POPSHeaderEventParams;

typedef struct {
  int MessageNumber;
  const char* MessageUID;
  int MessageSize;
  int reserved;
} POPSMessageListEventParams;

typedef struct {
  int Direction;
  const char* Message;
  int reserved;
} POPSPITrailEventParams;

typedef struct {
  const char* CertEncoded;
  const char* CertSubject;
  const char* CertIssuer;
  const char* Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} POPSSSLServerAuthenticationEventParams;

typedef struct {
  const char* Message;
  int reserved;
} POPSSSLStatusEventParams;

typedef struct {
  int Direction;
  int reserved;
} POPSStartTransferEventParams;

typedef struct {
  int Direction;
  ns_int64 *pBytesTransferred;
  int PercentDone;
  const char* Text;
  int EOL;
  int lenText;
  int reserved;
} POPSTransferEventParams;



class POPS {
  
  public: //events
  
    virtual int FireConnectionStatus(POPSConnectionStatusEventParams *e) {return 0;}
    virtual int FireEndTransfer(POPSEndTransferEventParams *e) {return 0;}
    virtual int FireError(POPSErrorEventParams *e) {return 0;}
    virtual int FireHeader(POPSHeaderEventParams *e) {return 0;}
    virtual int FireMessageList(POPSMessageListEventParams *e) {return 0;}
    virtual int FirePITrail(POPSPITrailEventParams *e) {return 0;}
    virtual int FireSSLServerAuthentication(POPSSSLServerAuthenticationEventParams *e) {return 0;}
    virtual int FireSSLStatus(POPSSSLStatusEventParams *e) {return 0;}
    virtual int FireStartTransfer(POPSStartTransferEventParams *e) {return 0;}
    virtual int FireTransfer(POPSTransferEventParams *e) {return 0;}


  protected:

    void *m_pObj;
    
    static int IPWORKSSSL_CALL POPSEventSink(void *lpObj, int event_id, int cparam, void *param[], int cbparam[]) {
      int ret_code = 0;
      if (event_id > 10000) return ((POPS*)lpObj)->POPSEventSinkW(event_id - 10000, cparam, param, cbparam);
      switch (event_id) {
         case 1: {
            POPSConnectionStatusEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]),  0};
            ret_code = ((POPS*)lpObj)->FireConnectionStatus(&e);
            break;
         }
         case 2: {
            POPSEndTransferEventParams e = {(int)IPS64CAST(param[0]),  0};
            ret_code = ((POPS*)lpObj)->FireEndTransfer(&e);
            break;
         }
         case 3: {
            POPSErrorEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((POPS*)lpObj)->FireError(&e);
            break;
         }
         case 4: {
            POPSHeaderEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((POPS*)lpObj)->FireHeader(&e);
            break;
         }
         case 5: {
            POPSMessageListEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]),  0};
            ret_code = ((POPS*)lpObj)->FireMessageList(&e);
            break;
         }
         case 6: {
            POPSPITrailEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((POPS*)lpObj)->FirePITrail(&e);
            break;
         }
         case 7: {
            POPSSSLServerAuthenticationEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = ((POPS*)lpObj)->FireSSLServerAuthentication(&e);
            param[4] = (void*)IPS64CAST(e.Accept);
            break;
         }
         case 8: {
            POPSSSLStatusEventParams e = {(char*)IPS64CAST(param[0]),  0};
            ret_code = ((POPS*)lpObj)->FireSSLStatus(&e);
            break;
         }
         case 9: {
            POPSStartTransferEventParams e = {(int)IPS64CAST(param[0]),  0};
            ret_code = ((POPS*)lpObj)->FireStartTransfer(&e);
            break;
         }
         case 10: {
            POPSTransferEventParams e = {(int)IPS64CAST(param[0]), (ns_int64*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[3]),  0};
            ret_code = ((POPS*)lpObj)->FireTransfer(&e);
            break;
         }

      }
      return ret_code;
    }

    virtual int POPSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {return 0;}

  public:

    POPS(char *lpOemKey = (char*)IPWORKSSSL_OEMKEY_24) {
      m_pObj = IPWorksSSL_POPS_Create(POPSEventSink, (void*)this, (char*)lpOemKey);
    }

    virtual ~POPS() {
      IPWorksSSL_POPS_Destroy(m_pObj);
    }

  public:

    inline char *GetLastError() {
      return IPWorksSSL_POPS_GetLastError(m_pObj);
    }
    
    inline int GetLastErrorCode() {
      return IPWorksSSL_POPS_GetLastErrorCode(m_pObj);
    }

    inline char *VERSION() {
      return (char*)IPWorksSSL_POPS_Get(m_pObj, 0, 0, 0);
    }

  public: //properties

    inline int GetAuthMechanism() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 1, 0, 0);
      return (int)(long)val;
    }
    inline int SetAuthMechanism(int iAuthMechanism) {
      void* val = (void*)IPS64CAST(iAuthMechanism);
      return IPWorksSSL_POPS_Set(m_pObj, 1, 0, val, 0);
    }

    inline int SetCommand(const char *lpCommand) {
      return IPWorksSSL_POPS_Set(m_pObj, 2, 0, (void*)lpCommand, 0);
    }

    inline int GetConnected() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 3, 0, 0);
      return (int)(long)val;
    }
    inline int SetConnected(int bConnected) {
      void* val = (void*)IPS64CAST(bConnected);
      return IPWorksSSL_POPS_Set(m_pObj, 3, 0, val, 0);
    }
    inline int GetFirewallAutoDetect() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 4, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallAutoDetect(int bFirewallAutoDetect) {
      void* val = (void*)IPS64CAST(bFirewallAutoDetect);
      return IPWorksSSL_POPS_Set(m_pObj, 4, 0, val, 0);
    }
    inline int GetFirewallType() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 5, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallType(int iFirewallType) {
      void* val = (void*)IPS64CAST(iFirewallType);
      return IPWorksSSL_POPS_Set(m_pObj, 5, 0, val, 0);
    }
    inline char* GetFirewallHost() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 6, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallHost(const char *lpFirewallHost) {
      return IPWorksSSL_POPS_Set(m_pObj, 6, 0, (void*)lpFirewallHost, 0);
    }

    inline char* GetFirewallPassword() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 7, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallPassword(const char *lpFirewallPassword) {
      return IPWorksSSL_POPS_Set(m_pObj, 7, 0, (void*)lpFirewallPassword, 0);
    }

    inline int GetFirewallPort() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 8, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallPort(int lFirewallPort) {
      void* val = (void*)IPS64CAST(lFirewallPort);
      return IPWorksSSL_POPS_Set(m_pObj, 8, 0, val, 0);
    }
    inline char* GetFirewallUser() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 9, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallUser(const char *lpFirewallUser) {
      return IPWorksSSL_POPS_Set(m_pObj, 9, 0, (void*)lpFirewallUser, 0);
    }

    inline int GetIdle() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 10, 0, 0);
      return (int)(long)val;
    }

    inline int GetIncludeHeaders() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 11, 0, 0);
      return (int)(long)val;
    }
    inline int SetIncludeHeaders(int bIncludeHeaders) {
      void* val = (void*)IPS64CAST(bIncludeHeaders);
      return IPWorksSSL_POPS_Set(m_pObj, 11, 0, val, 0);
    }
    inline char* GetLastReply() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 12, 0, 0);
      return (char*)val;
    }


    inline char* GetLocalFile() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 13, 0, 0);
      return (char*)val;
    }

    inline int SetLocalFile(const char *lpLocalFile) {
      return IPWorksSSL_POPS_Set(m_pObj, 13, 0, (void*)lpLocalFile, 0);
    }

    inline char* GetLocalHost() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 14, 0, 0);
      return (char*)val;
    }

    inline int SetLocalHost(const char *lpLocalHost) {
      return IPWorksSSL_POPS_Set(m_pObj, 14, 0, (void*)lpLocalHost, 0);
    }

    inline int GetMailPort() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 15, 0, 0);
      return (int)(long)val;
    }
    inline int SetMailPort(int lMailPort) {
      void* val = (void*)IPS64CAST(lMailPort);
      return IPWorksSSL_POPS_Set(m_pObj, 15, 0, val, 0);
    }
    inline char* GetMailServer() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 16, 0, 0);
      return (char*)val;
    }

    inline int SetMailServer(const char *lpMailServer) {
      return IPWorksSSL_POPS_Set(m_pObj, 16, 0, (void*)lpMailServer, 0);
    }

    inline int GetMaxLines() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 17, 0, 0);
      return (int)(long)val;
    }
    inline int SetMaxLines(int lMaxLines) {
      void* val = (void*)IPS64CAST(lMaxLines);
      return IPWorksSSL_POPS_Set(m_pObj, 17, 0, val, 0);
    }
    inline int GetMessage(char *&lpMessage, int &lenMessage) {
      lpMessage = (char*)IPWorksSSL_POPS_Get(m_pObj, 18, 0, &lenMessage);
      return lpMessage ? 0 : lenMessage;
    }


    inline char* GetMessageCc() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 19, 0, 0);
      return (char*)val;
    }


    inline int GetMessageCount() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 20, 0, 0);
      return (int)(long)val;
    }

    inline char* GetMessageDate() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 21, 0, 0);
      return (char*)val;
    }


    inline char* GetMessageFrom() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 22, 0, 0);
      return (char*)val;
    }


    inline int GetMessageHeaderCount() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 23, 0, 0);
      return (int)(long)val;
    }

    inline char* GetMessageHeaderField(int iMessageHeaderIndex) {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 24, iMessageHeaderIndex, 0);
      return (char*)val;
    }


    inline char* GetMessageHeaderValue(int iMessageHeaderIndex) {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 25, iMessageHeaderIndex, 0);
      return (char*)val;
    }


    inline char* GetMessageHeadersString() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 26, 0, 0);
      return (char*)val;
    }


    inline int GetMessageNumber() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 27, 0, 0);
      return (int)(long)val;
    }
    inline int SetMessageNumber(int lMessageNumber) {
      void* val = (void*)IPS64CAST(lMessageNumber);
      return IPWorksSSL_POPS_Set(m_pObj, 27, 0, val, 0);
    }
    inline int GetMessageRecipientCount() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 28, 0, 0);
      return (int)(long)val;
    }

    inline char* GetMessageRecipientAddress(int iRecipientIndex) {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 29, iRecipientIndex, 0);
      return (char*)val;
    }


    inline char* GetMessageRecipientName(int iRecipientIndex) {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 30, iRecipientIndex, 0);
      return (char*)val;
    }


    inline char* GetMessageRecipientOptions(int iRecipientIndex) {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 31, iRecipientIndex, 0);
      return (char*)val;
    }


    inline int GetMessageRecipientType(int iRecipientIndex) {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 32, iRecipientIndex, 0);
      return (int)(long)val;
    }

    inline char* GetMessageReplyTo() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 33, 0, 0);
      return (char*)val;
    }


    inline int GetMessageSize() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 34, 0, 0);
      return (int)(long)val;
    }

    inline char* GetMessageSubject() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 35, 0, 0);
      return (char*)val;
    }


    inline char* GetMessageText() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 36, 0, 0);
      return (char*)val;
    }


    inline char* GetMessageTo() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 37, 0, 0);
      return (char*)val;
    }


    inline char* GetMessageUID() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 38, 0, 0);
      return (char*)val;
    }


    inline char* GetPassword() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 39, 0, 0);
      return (char*)val;
    }

    inline int SetPassword(const char *lpPassword) {
      return IPWorksSSL_POPS_Set(m_pObj, 39, 0, (void*)lpPassword, 0);
    }

    inline int GetSSLAcceptServerCertEncoded(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_POPS_Get(m_pObj, 40, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }

    inline int SetSSLAcceptServerCertEncoded(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_POPS_Set(m_pObj, 40, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }

    inline int GetSSLCertEncoded(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_POPS_Get(m_pObj, 41, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }

    inline int SetSSLCertEncoded(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_POPS_Set(m_pObj, 41, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }

    inline int GetSSLCertStore(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_POPS_Get(m_pObj, 42, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }

    inline int SetSSLCertStore(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_POPS_Set(m_pObj, 42, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }

    inline char* GetSSLCertStorePassword() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 43, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertStorePassword(const char *lpSSLCertStorePassword) {
      return IPWorksSSL_POPS_Set(m_pObj, 43, 0, (void*)lpSSLCertStorePassword, 0);
    }

    inline int GetSSLCertStoreType() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 44, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLCertStoreType(int iSSLCertStoreType) {
      void* val = (void*)IPS64CAST(iSSLCertStoreType);
      return IPWorksSSL_POPS_Set(m_pObj, 44, 0, val, 0);
    }
    inline char* GetSSLCertSubject() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 45, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertSubject(const char *lpSSLCertSubject) {
      return IPWorksSSL_POPS_Set(m_pObj, 45, 0, (void*)lpSSLCertSubject, 0);
    }

    inline int GetSSLServerCertEncoded(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_POPS_Get(m_pObj, 46, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }


    inline int GetSSLStartMode() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 47, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLStartMode(int iSSLStartMode) {
      void* val = (void*)IPS64CAST(iSSLStartMode);
      return IPWorksSSL_POPS_Set(m_pObj, 47, 0, val, 0);
    }
    inline int GetTimeout() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 48, 0, 0);
      return (int)(long)val;
    }
    inline int SetTimeout(int iTimeout) {
      void* val = (void*)IPS64CAST(iTimeout);
      return IPWorksSSL_POPS_Set(m_pObj, 48, 0, val, 0);
    }
    inline ns_int64 GetTotalSize() {
      ns_int64 *pval = (ns_int64*)IPWorksSSL_POPS_Get(m_pObj, 49, 0, 0);
      return *pval;
    }


    inline char* GetUser() {
      void* val = IPWorksSSL_POPS_Get(m_pObj, 50, 0, 0);
      return (char*)val;
    }

    inline int SetUser(const char *lpUser) {
      return IPWorksSSL_POPS_Set(m_pObj, 50, 0, (void*)lpUser, 0);
    }


  public: //methods

    inline char* Config(const char* lpszConfigurationString) {
      void *param[1+1] = {(void*)IPS64CAST(lpszConfigurationString), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_POPS_Do(m_pObj, 2, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline int Connect() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_POPS_Do(m_pObj, 3, 0, param, cbparam);
      
      
    }
    inline int Delete() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_POPS_Do(m_pObj, 4, 0, param, cbparam);
      
      
    }
    inline int Disconnect() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_POPS_Do(m_pObj, 5, 0, param, cbparam);
      
      
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_POPS_Do(m_pObj, 6, 0, param, cbparam);
      
      
    }
    inline int Interrupt() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_POPS_Do(m_pObj, 7, 0, param, cbparam);
      
      
    }
    inline int ListMessageSizes() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_POPS_Do(m_pObj, 8, 0, param, cbparam);
      
      
    }
    inline int ListMessageUIDs() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_POPS_Do(m_pObj, 9, 0, param, cbparam);
      
      
    }
    inline char* LocalizeDate(const char* lpszDateTime) {
      void *param[1+1] = {(void*)IPS64CAST(lpszDateTime), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_POPS_Do(m_pObj, 10, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline int Reset() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_POPS_Do(m_pObj, 11, 0, param, cbparam);
      
      
    }
    inline int Retrieve() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_POPS_Do(m_pObj, 12, 0, param, cbparam);
      
      
    }
    inline int RetrieveHeaders() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_POPS_Do(m_pObj, 13, 0, param, cbparam);
      
      
    }

};


#ifdef WIN32 //UNICODE

typedef struct {
  LPWSTR ConnectionEvent;
  int StatusCode;
  LPWSTR Description;
  int reserved;
} POPSConnectionStatusEventParamsW;

typedef struct {
  int Direction;
  int reserved;
} POPSEndTransferEventParamsW;

typedef struct {
  int ErrorCode;
  LPWSTR Description;
  int reserved;
} POPSErrorEventParamsW;

typedef struct {
  LPWSTR Field;
  LPWSTR Value;
  int reserved;
} POPSHeaderEventParamsW;

typedef struct {
  int MessageNumber;
  LPWSTR MessageUID;
  int MessageSize;
  int reserved;
} POPSMessageListEventParamsW;

typedef struct {
  int Direction;
  LPWSTR Message;
  int reserved;
} POPSPITrailEventParamsW;

typedef struct {
  LPWSTR CertEncoded;
  LPWSTR CertSubject;
  LPWSTR CertIssuer;
  LPWSTR Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} POPSSSLServerAuthenticationEventParamsW;

typedef struct {
  LPWSTR Message;
  int reserved;
} POPSSSLStatusEventParamsW;

typedef struct {
  int Direction;
  int reserved;
} POPSStartTransferEventParamsW;

typedef struct {
  int Direction;
  ns_int64 *pBytesTransferred;
  int PercentDone;
  LPWSTR Text;
  int EOL;
  int lenText;
  int reserved;
} POPSTransferEventParamsW;



class POPSW : public POPS {

  public: //properties
  




    inline int SetCommand(LPWSTR lpCommand) {
      return IPWorksSSL_POPS_Set(m_pObj, 10000+2, 0, (void*)lpCommand, 0);
    }







    inline LPWSTR GetFirewallHost() {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+6, 0, 0);
    }

    inline int SetFirewallHost(LPWSTR lpFirewallHost) {
      return IPWorksSSL_POPS_Set(m_pObj, 10000+6, 0, (void*)lpFirewallHost, 0);
    }

    inline LPWSTR GetFirewallPassword() {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+7, 0, 0);
    }

    inline int SetFirewallPassword(LPWSTR lpFirewallPassword) {
      return IPWorksSSL_POPS_Set(m_pObj, 10000+7, 0, (void*)lpFirewallPassword, 0);
    }



    inline LPWSTR GetFirewallUser() {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+9, 0, 0);
    }

    inline int SetFirewallUser(LPWSTR lpFirewallUser) {
      return IPWorksSSL_POPS_Set(m_pObj, 10000+9, 0, (void*)lpFirewallUser, 0);
    }





    inline LPWSTR GetLastReply() {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+12, 0, 0);
    }



    inline LPWSTR GetLocalFile() {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+10000+13, 0, 0);
    }

    inline int SetLocalFile(LPWSTR lpLocalFile) {
      return IPWorksSSL_POPS_Set(m_pObj, 10000+10000+13, 0, (void*)lpLocalFile, 0);
    }

    inline LPWSTR GetLocalHost() {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+14, 0, 0);
    }

    inline int SetLocalHost(LPWSTR lpLocalHost) {
      return IPWorksSSL_POPS_Set(m_pObj, 10000+14, 0, (void*)lpLocalHost, 0);
    }



    inline LPWSTR GetMailServer() {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+16, 0, 0);
    }

    inline int SetMailServer(LPWSTR lpMailServer) {
      return IPWorksSSL_POPS_Set(m_pObj, 10000+16, 0, (void*)lpMailServer, 0);
    }



    inline LPWSTR GetMessage() {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+18, 0, 0);
    }


    inline int GetMessageB(char *&lpMessage, int &lenMessage) {
      lpMessage = (char*)IPWorksSSL_POPS_Get(m_pObj, 18, 0, &lenMessage);
      return lpMessage ? 0 : lenMessage;
    }

    inline LPWSTR GetMessageCc() {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+19, 0, 0);
    }





    inline LPWSTR GetMessageDate() {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+21, 0, 0);
    }



    inline LPWSTR GetMessageFrom() {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+22, 0, 0);
    }





    inline LPWSTR GetMessageHeaderField(int iMessageHeaderIndex) {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+24, iMessageHeaderIndex, 0);
    }



    inline LPWSTR GetMessageHeaderValue(int iMessageHeaderIndex) {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+25, iMessageHeaderIndex, 0);
    }



    inline LPWSTR GetMessageHeadersString() {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+26, 0, 0);
    }







    inline LPWSTR GetMessageRecipientAddress(int iRecipientIndex) {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+29, iRecipientIndex, 0);
    }



    inline LPWSTR GetMessageRecipientName(int iRecipientIndex) {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+30, iRecipientIndex, 0);
    }



    inline LPWSTR GetMessageRecipientOptions(int iRecipientIndex) {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+31, iRecipientIndex, 0);
    }





    inline LPWSTR GetMessageReplyTo() {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+33, 0, 0);
    }





    inline LPWSTR GetMessageSubject() {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+35, 0, 0);
    }



    inline LPWSTR GetMessageText() {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+36, 0, 0);
    }



    inline LPWSTR GetMessageTo() {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+37, 0, 0);
    }



    inline LPWSTR GetMessageUID() {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+38, 0, 0);
    }



    inline LPWSTR GetPassword() {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+39, 0, 0);
    }

    inline int SetPassword(LPWSTR lpPassword) {
      return IPWorksSSL_POPS_Set(m_pObj, 10000+39, 0, (void*)lpPassword, 0);
    }

    inline LPWSTR GetSSLAcceptServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+40, 0, 0);
    }

    inline int SetSSLAcceptServerCertEncoded(LPWSTR lpSSLAcceptServerCertEncoded) {
      return IPWorksSSL_POPS_Set(m_pObj, 10000+40, 0, (void*)lpSSLAcceptServerCertEncoded, 0);
    }
    inline int GetSSLAcceptServerCertEncodedB(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_POPS_Get(m_pObj, 40, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }
    inline int SetSSLAcceptServerCertEncodedB(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_POPS_Set(m_pObj, 40, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }
    inline LPWSTR GetSSLCertEncoded() {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+41, 0, 0);
    }

    inline int SetSSLCertEncoded(LPWSTR lpSSLCertEncoded) {
      return IPWorksSSL_POPS_Set(m_pObj, 10000+41, 0, (void*)lpSSLCertEncoded, 0);
    }
    inline int GetSSLCertEncodedB(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_POPS_Get(m_pObj, 41, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }
    inline int SetSSLCertEncodedB(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_POPS_Set(m_pObj, 41, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }
    inline LPWSTR GetSSLCertStore() {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+42, 0, 0);
    }

    inline int SetSSLCertStore(LPWSTR lpSSLCertStore) {
      return IPWorksSSL_POPS_Set(m_pObj, 10000+42, 0, (void*)lpSSLCertStore, 0);
    }
    inline int GetSSLCertStoreB(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_POPS_Get(m_pObj, 42, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }
    inline int SetSSLCertStoreB(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_POPS_Set(m_pObj, 42, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }
    inline LPWSTR GetSSLCertStorePassword() {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+43, 0, 0);
    }

    inline int SetSSLCertStorePassword(LPWSTR lpSSLCertStorePassword) {
      return IPWorksSSL_POPS_Set(m_pObj, 10000+43, 0, (void*)lpSSLCertStorePassword, 0);
    }



    inline LPWSTR GetSSLCertSubject() {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+45, 0, 0);
    }

    inline int SetSSLCertSubject(LPWSTR lpSSLCertSubject) {
      return IPWorksSSL_POPS_Set(m_pObj, 10000+45, 0, (void*)lpSSLCertSubject, 0);
    }

    inline LPWSTR GetSSLServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+46, 0, 0);
    }


    inline int GetSSLServerCertEncodedB(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_POPS_Get(m_pObj, 46, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }







    inline LPWSTR GetUser() {
      return (LPWSTR)IPWorksSSL_POPS_Get(m_pObj, 10000+50, 0, 0);
    }

    inline int SetUser(LPWSTR lpUser) {
      return IPWorksSSL_POPS_Set(m_pObj, 10000+50, 0, (void*)lpUser, 0);
    }



  public: //events
  
    virtual int FireConnectionStatus(POPSConnectionStatusEventParamsW *e) {return 0;}
    virtual int FireEndTransfer(POPSEndTransferEventParamsW *e) {return 0;}
    virtual int FireError(POPSErrorEventParamsW *e) {return 0;}
    virtual int FireHeader(POPSHeaderEventParamsW *e) {return 0;}
    virtual int FireMessageList(POPSMessageListEventParamsW *e) {return 0;}
    virtual int FirePITrail(POPSPITrailEventParamsW *e) {return 0;}
    virtual int FireSSLServerAuthentication(POPSSSLServerAuthenticationEventParamsW *e) {return 0;}
    virtual int FireSSLStatus(POPSSSLStatusEventParamsW *e) {return 0;}
    virtual int FireStartTransfer(POPSStartTransferEventParamsW *e) {return 0;}
    virtual int FireTransfer(POPSTransferEventParamsW *e) {return 0;}


  protected:
  
    virtual int POPSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {
    	int ret_code = 0;
      switch (event_id) {
         case 1: {
            POPSConnectionStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]),  0};
            ret_code = FireConnectionStatus(&e);
            break;
         }
         case 2: {
            POPSEndTransferEventParamsW e = {(int)IPS64CAST(param[0]),  0};
            ret_code = FireEndTransfer(&e);
            break;
         }
         case 3: {
            POPSErrorEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireError(&e);
            break;
         }
         case 4: {
            POPSHeaderEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireHeader(&e);
            break;
         }
         case 5: {
            POPSMessageListEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (int)IPS64CAST(param[2]),  0};
            ret_code = FireMessageList(&e);
            break;
         }
         case 6: {
            POPSPITrailEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FirePITrail(&e);
            break;
         }
         case 7: {
            POPSSSLServerAuthenticationEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = FireSSLServerAuthentication(&e);
            param[4] = (void*)(e.Accept);
            break;
         }
         case 8: {
            POPSSSLStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]),  0};
            ret_code = FireSSLStatus(&e);
            break;
         }
         case 9: {
            POPSStartTransferEventParamsW e = {(int)IPS64CAST(param[0]),  0};
            ret_code = FireStartTransfer(&e);
            break;
         }
         case 10: {
            POPSTransferEventParamsW e = {(int)IPS64CAST(param[0]), (ns_int64*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[3]),  0};
            ret_code = FireTransfer(&e);
            break;
         }

      }
      return ret_code;
    }
  
  public: //event overrides

    virtual int FireConnectionStatus(POPSConnectionStatusEventParams *e) {return -10000;}
    virtual int FireEndTransfer(POPSEndTransferEventParams *e) {return -10000;}
    virtual int FireError(POPSErrorEventParams *e) {return -10000;}
    virtual int FireHeader(POPSHeaderEventParams *e) {return -10000;}
    virtual int FireMessageList(POPSMessageListEventParams *e) {return -10000;}
    virtual int FirePITrail(POPSPITrailEventParams *e) {return -10000;}
    virtual int FireSSLServerAuthentication(POPSSSLServerAuthenticationEventParams *e) {return -10000;}
    virtual int FireSSLStatus(POPSSSLStatusEventParams *e) {return -10000;}
    virtual int FireStartTransfer(POPSStartTransferEventParams *e) {return -10000;}
    virtual int FireTransfer(POPSTransferEventParams *e) {return -10000;}

  public: //methods

    inline LPWSTR Config(LPWSTR lpszConfigurationString) {
      void *param[1+1] = {(void*)lpszConfigurationString, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_POPS_Do(m_pObj, 10000+2, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline int Connect() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_POPS_Do(m_pObj, 10000+3, 0, param, cbparam);
      
    }
    inline int Delete() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_POPS_Do(m_pObj, 10000+4, 0, param, cbparam);
      
    }
    inline int Disconnect() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_POPS_Do(m_pObj, 10000+5, 0, param, cbparam);
      
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_POPS_Do(m_pObj, 10000+6, 0, param, cbparam);
      
    }
    inline int Interrupt() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_POPS_Do(m_pObj, 10000+7, 0, param, cbparam);
      
    }
    inline int ListMessageSizes() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_POPS_Do(m_pObj, 10000+8, 0, param, cbparam);
      
    }
    inline int ListMessageUIDs() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_POPS_Do(m_pObj, 10000+9, 0, param, cbparam);
      
    }
    inline LPWSTR LocalizeDate(LPWSTR lpszDateTime) {
      void *param[1+1] = {(void*)lpszDateTime, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_POPS_Do(m_pObj, 10000+10, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline int Reset() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_POPS_Do(m_pObj, 10000+11, 0, param, cbparam);
      
    }
    inline int Retrieve() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_POPS_Do(m_pObj, 10000+12, 0, param, cbparam);
      
    }
    inline int RetrieveHeaders() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_POPS_Do(m_pObj, 10000+13, 0, param, cbparam);
      
    }

};

#endif //WIN32

#endif //_IPWORKSSSL_POPS_H_




