/******************************************************************
   IP*Works! SSL V9 C++ Edition
   Copyright (c) 2013 /n software inc. - All rights reserved.
*******************************************************************/

#ifndef _IPWORKSSSL_RSSS_H_
#define _IPWORKSSSL_RSSS_H_

#define IPWORKSSSL_ONLY_TYPES
#include "ipworksssl.h"
#include "ipworksssl.key"

//AuthSchemes
#define AUTH_BASIC                                         0
#define AUTH_DIGEST                                        1
#define AUTH_PROPRIETARY                                   2
#define AUTH_NONE                                          3
#define AUTH_NTLM                                          4
#define AUTH_NEGOTIATE                                     5
#define AUTH_OAUTH                                         6

//FirewallTypes
#define FW_NONE                                            0
#define FW_TUNNEL                                          1
#define FW_SOCKS4                                          2
#define FW_SOCKS5                                          3

//TFollowRedirects
#define FR_NEVER                                           0
#define FR_ALWAYS                                          1
#define FR_SAME_SCHEME                                     2

//ProxySSLTypes
#define PS_AUTOMATIC                                       0
#define PS_ALWAYS                                          1
#define PS_NEVER                                           2
#define PS_TUNNEL                                          3

//TRSSVersion
#define RSS_VERSION_UNDEFINED                              0
#define RSS_VERSION_091                                    1
#define RSS_VERSION_092                                    2
#define RSS_VERSION_200                                    3

//CertStoreTypes
#define CST_USER                                           0
#define CST_MACHINE                                        1
#define CST_PFXFILE                                        2
#define CST_PFXBLOB                                        3
#define CST_JKSFILE                                        4
#define CST_JKSBLOB                                        5
#define CST_PEMKEY_FILE                                    6
#define CST_PEMKEY_BLOB                                    7
#define CST_PUBLIC_KEY_FILE                                8
#define CST_PUBLIC_KEY_BLOB                                9
#define CST_SSHPUBLIC_KEY_BLOB                             10
#define CST_P7BFILE                                        11
#define CST_P7BBLOB                                        12
#define CST_SSHPUBLIC_KEY_FILE                             13


extern "C" void* IPWORKSSSL_CALL IPWorksSSL_RSSS_Create(PIPWORKSSSL_CALLBACK lpSink, void *lpContext, char *lpOemKey);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_RSSS_Destroy(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_RSSS_CheckIndex(void *lpObj, int propid, int arridx);
extern "C" void* IPWORKSSSL_CALL IPWorksSSL_RSSS_Get(void *lpObj, int propid, int arridx, int *lpcbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_RSSS_Set(void *lpObj, int propid, int arridx, const void *val, int cbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_RSSS_Do(void *lpObj, int methid, int cparam, void *param[], int cbparam[]);
extern "C" char* IPWORKSSSL_CALL IPWorksSSL_RSSS_GetLastError(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_RSSS_GetLastErrorCode(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_RSSS_StaticInit(void *hInst);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_RSSS_StaticDestroy();

#ifdef WIN32
#include <windows.h>
#pragma warning(disable:4311) 
#pragma warning(disable:4312) 
#endif

typedef struct {
  int StatusCode;
  const char* Description;
  int reserved;
} RSSSConnectedEventParams;

typedef struct {
  const char* ConnectionEvent;
  int StatusCode;
  const char* Description;
  int reserved;
} RSSSConnectionStatusEventParams;

typedef struct {
  int StatusCode;
  const char* Description;
  int reserved;
} RSSSDisconnectedEventParams;

typedef struct {
  int Direction;
  int reserved;
} RSSSEndTransferEventParams;

typedef struct {
  int ErrorCode;
  const char* Description;
  int reserved;
} RSSSErrorEventParams;

typedef struct {
  const char* Field;
  const char* Value;
  int reserved;
} RSSSHeaderEventParams;

typedef struct {
  const char* Field;
  const char* Value;
  int reserved;
} RSSSOPMLHeaderEventParams;

typedef struct {
  int Level;
  const char* Text;
  const char* TypeName;
  const char* Title;
  const char* URL;
  const char* XMLURL;
  const char* HTMLURL;
  const char* Created;
  const char* Description;
  const char* OtherAttrs;
  int reserved;
} RSSSOPMLOutlineEventParams;

typedef struct {
  const char* Location;
  int Accept;
  int reserved;
} RSSSRedirectEventParams;

typedef struct {
  const char* Name;
  const char* Value;
  const char* Expires;
  const char* Domain;
  const char* Path;
  int Secure;
  int reserved;
} RSSSSetCookieEventParams;

typedef struct {
  const char* CertEncoded;
  const char* CertSubject;
  const char* CertIssuer;
  const char* Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} RSSSSSLServerAuthenticationEventParams;

typedef struct {
  const char* Message;
  int reserved;
} RSSSSSLStatusEventParams;

typedef struct {
  int Direction;
  int reserved;
} RSSSStartTransferEventParams;

typedef struct {
  const char* HTTPVersion;
  int StatusCode;
  const char* Description;
  int reserved;
} RSSSStatusEventParams;

typedef struct {
  int Direction;
  ns_int64 *pBytesTransferred;
  int PercentDone;
  const char* Text;
  int lenText;
  int reserved;
} RSSSTransferEventParams;



class RSSS {
  
  public: //events
  
    virtual int FireConnected(RSSSConnectedEventParams *e) {return 0;}
    virtual int FireConnectionStatus(RSSSConnectionStatusEventParams *e) {return 0;}
    virtual int FireDisconnected(RSSSDisconnectedEventParams *e) {return 0;}
    virtual int FireEndTransfer(RSSSEndTransferEventParams *e) {return 0;}
    virtual int FireError(RSSSErrorEventParams *e) {return 0;}
    virtual int FireHeader(RSSSHeaderEventParams *e) {return 0;}
    virtual int FireOPMLHeader(RSSSOPMLHeaderEventParams *e) {return 0;}
    virtual int FireOPMLOutline(RSSSOPMLOutlineEventParams *e) {return 0;}
    virtual int FireRedirect(RSSSRedirectEventParams *e) {return 0;}
    virtual int FireSetCookie(RSSSSetCookieEventParams *e) {return 0;}
    virtual int FireSSLServerAuthentication(RSSSSSLServerAuthenticationEventParams *e) {return 0;}
    virtual int FireSSLStatus(RSSSSSLStatusEventParams *e) {return 0;}
    virtual int FireStartTransfer(RSSSStartTransferEventParams *e) {return 0;}
    virtual int FireStatus(RSSSStatusEventParams *e) {return 0;}
    virtual int FireTransfer(RSSSTransferEventParams *e) {return 0;}


  protected:

    void *m_pObj;
    
    static int IPWORKSSSL_CALL RSSSEventSink(void *lpObj, int event_id, int cparam, void *param[], int cbparam[]) {
      int ret_code = 0;
      if (event_id > 10000) return ((RSSS*)lpObj)->RSSSEventSinkW(event_id - 10000, cparam, param, cbparam);
      switch (event_id) {
         case 1: {
            RSSSConnectedEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((RSSS*)lpObj)->FireConnected(&e);
            break;
         }
         case 2: {
            RSSSConnectionStatusEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]),  0};
            ret_code = ((RSSS*)lpObj)->FireConnectionStatus(&e);
            break;
         }
         case 3: {
            RSSSDisconnectedEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((RSSS*)lpObj)->FireDisconnected(&e);
            break;
         }
         case 4: {
            RSSSEndTransferEventParams e = {(int)IPS64CAST(param[0]),  0};
            ret_code = ((RSSS*)lpObj)->FireEndTransfer(&e);
            break;
         }
         case 5: {
            RSSSErrorEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((RSSS*)lpObj)->FireError(&e);
            break;
         }
         case 6: {
            RSSSHeaderEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((RSSS*)lpObj)->FireHeader(&e);
            break;
         }
         case 7: {
            RSSSOPMLHeaderEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((RSSS*)lpObj)->FireOPMLHeader(&e);
            break;
         }
         case 8: {
            RSSSOPMLOutlineEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (char*)IPS64CAST(param[4]), (char*)IPS64CAST(param[5]), (char*)IPS64CAST(param[6]), (char*)IPS64CAST(param[7]), (char*)IPS64CAST(param[8]), (char*)IPS64CAST(param[9]),  0};
            ret_code = ((RSSS*)lpObj)->FireOPMLOutline(&e);
            break;
         }
         case 9: {
            RSSSRedirectEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]),  0};
            ret_code = ((RSSS*)lpObj)->FireRedirect(&e);
            param[1] = (void*)IPS64CAST(e.Accept);
            break;
         }
         case 10: {
            RSSSSetCookieEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (char*)IPS64CAST(param[4]), (int)IPS64CAST(param[5]),  0};
            ret_code = ((RSSS*)lpObj)->FireSetCookie(&e);
            break;
         }
         case 11: {
            RSSSSSLServerAuthenticationEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = ((RSSS*)lpObj)->FireSSLServerAuthentication(&e);
            param[4] = (void*)IPS64CAST(e.Accept);
            break;
         }
         case 12: {
            RSSSSSLStatusEventParams e = {(char*)IPS64CAST(param[0]),  0};
            ret_code = ((RSSS*)lpObj)->FireSSLStatus(&e);
            break;
         }
         case 13: {
            RSSSStartTransferEventParams e = {(int)IPS64CAST(param[0]),  0};
            ret_code = ((RSSS*)lpObj)->FireStartTransfer(&e);
            break;
         }
         case 14: {
            RSSSStatusEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]),  0};
            ret_code = ((RSSS*)lpObj)->FireStatus(&e);
            break;
         }
         case 15: {
            RSSSTransferEventParams e = {(int)IPS64CAST(param[0]), (ns_int64*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (int)IPS64CAST(cbparam[3]),  0};
            ret_code = ((RSSS*)lpObj)->FireTransfer(&e);
            break;
         }

      }
      return ret_code;
    }

    virtual int RSSSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {return 0;}

  public:

    RSSS(char *lpOemKey = (char*)IPWORKSSSL_OEMKEY_59) {
      m_pObj = IPWorksSSL_RSSS_Create(RSSSEventSink, (void*)this, (char*)lpOemKey);
    }

    virtual ~RSSS() {
      IPWorksSSL_RSSS_Destroy(m_pObj);
    }

  public:

    inline char *GetLastError() {
      return IPWorksSSL_RSSS_GetLastError(m_pObj);
    }
    
    inline int GetLastErrorCode() {
      return IPWorksSSL_RSSS_GetLastErrorCode(m_pObj);
    }

    inline char *VERSION() {
      return (char*)IPWorksSSL_RSSS_Get(m_pObj, 0, 0, 0);
    }

  public: //properties

    inline char* GetAccept() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 1, 0, 0);
      return (char*)val;
    }

    inline int SetAccept(const char *lpAccept) {
      return IPWorksSSL_RSSS_Set(m_pObj, 1, 0, (void*)lpAccept, 0);
    }

    inline char* GetAuthorization() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 2, 0, 0);
      return (char*)val;
    }

    inline int SetAuthorization(const char *lpAuthorization) {
      return IPWorksSSL_RSSS_Set(m_pObj, 2, 0, (void*)lpAuthorization, 0);
    }

    inline int GetAuthScheme() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 3, 0, 0);
      return (int)(long)val;
    }
    inline int SetAuthScheme(int iAuthScheme) {
      void* val = (void*)IPS64CAST(iAuthScheme);
      return IPWorksSSL_RSSS_Set(m_pObj, 3, 0, val, 0);
    }
    inline char* GetChannelCategory() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 4, 0, 0);
      return (char*)val;
    }

    inline int SetChannelCategory(const char *lpChannelCategory) {
      return IPWorksSSL_RSSS_Set(m_pObj, 4, 0, (void*)lpChannelCategory, 0);
    }

    inline char* GetChannelCategoryDomain() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 5, 0, 0);
      return (char*)val;
    }

    inline int SetChannelCategoryDomain(const char *lpChannelCategoryDomain) {
      return IPWorksSSL_RSSS_Set(m_pObj, 5, 0, (void*)lpChannelCategoryDomain, 0);
    }

    inline char* GetChannelCopyright() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 6, 0, 0);
      return (char*)val;
    }

    inline int SetChannelCopyright(const char *lpChannelCopyright) {
      return IPWorksSSL_RSSS_Set(m_pObj, 6, 0, (void*)lpChannelCopyright, 0);
    }

    inline char* GetChannelDescription() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 7, 0, 0);
      return (char*)val;
    }

    inline int SetChannelDescription(const char *lpChannelDescription) {
      return IPWorksSSL_RSSS_Set(m_pObj, 7, 0, (void*)lpChannelDescription, 0);
    }

    inline char* GetChannelDocs() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 8, 0, 0);
      return (char*)val;
    }

    inline int SetChannelDocs(const char *lpChannelDocs) {
      return IPWorksSSL_RSSS_Set(m_pObj, 8, 0, (void*)lpChannelDocs, 0);
    }

    inline char* GetChannelGenerator() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 9, 0, 0);
      return (char*)val;
    }

    inline int SetChannelGenerator(const char *lpChannelGenerator) {
      return IPWorksSSL_RSSS_Set(m_pObj, 9, 0, (void*)lpChannelGenerator, 0);
    }

    inline char* GetChannelImageDescription() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 10, 0, 0);
      return (char*)val;
    }

    inline int SetChannelImageDescription(const char *lpChannelImageDescription) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10, 0, (void*)lpChannelImageDescription, 0);
    }

    inline char* GetChannelImageHeight() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 11, 0, 0);
      return (char*)val;
    }

    inline int SetChannelImageHeight(const char *lpChannelImageHeight) {
      return IPWorksSSL_RSSS_Set(m_pObj, 11, 0, (void*)lpChannelImageHeight, 0);
    }

    inline char* GetChannelImageLink() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 12, 0, 0);
      return (char*)val;
    }

    inline int SetChannelImageLink(const char *lpChannelImageLink) {
      return IPWorksSSL_RSSS_Set(m_pObj, 12, 0, (void*)lpChannelImageLink, 0);
    }

    inline char* GetChannelImageTitle() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 13, 0, 0);
      return (char*)val;
    }

    inline int SetChannelImageTitle(const char *lpChannelImageTitle) {
      return IPWorksSSL_RSSS_Set(m_pObj, 13, 0, (void*)lpChannelImageTitle, 0);
    }

    inline char* GetChannelImageUrl() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 14, 0, 0);
      return (char*)val;
    }

    inline int SetChannelImageUrl(const char *lpChannelImageUrl) {
      return IPWorksSSL_RSSS_Set(m_pObj, 14, 0, (void*)lpChannelImageUrl, 0);
    }

    inline char* GetChannelImageWidth() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 15, 0, 0);
      return (char*)val;
    }

    inline int SetChannelImageWidth(const char *lpChannelImageWidth) {
      return IPWorksSSL_RSSS_Set(m_pObj, 15, 0, (void*)lpChannelImageWidth, 0);
    }

    inline char* GetChannelLanguage() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 16, 0, 0);
      return (char*)val;
    }

    inline int SetChannelLanguage(const char *lpChannelLanguage) {
      return IPWorksSSL_RSSS_Set(m_pObj, 16, 0, (void*)lpChannelLanguage, 0);
    }

    inline char* GetChannelLastBuildDate() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 17, 0, 0);
      return (char*)val;
    }

    inline int SetChannelLastBuildDate(const char *lpChannelLastBuildDate) {
      return IPWorksSSL_RSSS_Set(m_pObj, 17, 0, (void*)lpChannelLastBuildDate, 0);
    }

    inline char* GetChannelLink() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 18, 0, 0);
      return (char*)val;
    }

    inline int SetChannelLink(const char *lpChannelLink) {
      return IPWorksSSL_RSSS_Set(m_pObj, 18, 0, (void*)lpChannelLink, 0);
    }

    inline char* GetChannelManagingEditor() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 19, 0, 0);
      return (char*)val;
    }

    inline int SetChannelManagingEditor(const char *lpChannelManagingEditor) {
      return IPWorksSSL_RSSS_Set(m_pObj, 19, 0, (void*)lpChannelManagingEditor, 0);
    }

    inline char* GetChannelPubDate() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 20, 0, 0);
      return (char*)val;
    }

    inline int SetChannelPubDate(const char *lpChannelPubDate) {
      return IPWorksSSL_RSSS_Set(m_pObj, 20, 0, (void*)lpChannelPubDate, 0);
    }

    inline char* GetChannelSkipDays() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 21, 0, 0);
      return (char*)val;
    }

    inline int SetChannelSkipDays(const char *lpChannelSkipDays) {
      return IPWorksSSL_RSSS_Set(m_pObj, 21, 0, (void*)lpChannelSkipDays, 0);
    }

    inline char* GetChannelSkipHours() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 22, 0, 0);
      return (char*)val;
    }

    inline int SetChannelSkipHours(const char *lpChannelSkipHours) {
      return IPWorksSSL_RSSS_Set(m_pObj, 22, 0, (void*)lpChannelSkipHours, 0);
    }

    inline char* GetChannelTitle() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 23, 0, 0);
      return (char*)val;
    }

    inline int SetChannelTitle(const char *lpChannelTitle) {
      return IPWorksSSL_RSSS_Set(m_pObj, 23, 0, (void*)lpChannelTitle, 0);
    }

    inline char* GetChannelTtl() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 24, 0, 0);
      return (char*)val;
    }

    inline int SetChannelTtl(const char *lpChannelTtl) {
      return IPWorksSSL_RSSS_Set(m_pObj, 24, 0, (void*)lpChannelTtl, 0);
    }

    inline char* GetChannelWebMaster() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 25, 0, 0);
      return (char*)val;
    }

    inline int SetChannelWebMaster(const char *lpChannelWebMaster) {
      return IPWorksSSL_RSSS_Set(m_pObj, 25, 0, (void*)lpChannelWebMaster, 0);
    }

    inline int GetConnected() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 26, 0, 0);
      return (int)(long)val;
    }
    inline int SetConnected(int bConnected) {
      void* val = (void*)IPS64CAST(bConnected);
      return IPWorksSSL_RSSS_Set(m_pObj, 26, 0, val, 0);
    }
    inline int GetCookieCount() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 27, 0, 0);
      return (int)(long)val;
    }
    inline int SetCookieCount(int iCookieCount) {
      void* val = (void*)IPS64CAST(iCookieCount);
      return IPWorksSSL_RSSS_Set(m_pObj, 27, 0, val, 0);
    }
    inline char* GetCookieDomain(int iCookieIndex) {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 28, iCookieIndex, 0);
      return (char*)val;
    }


    inline char* GetCookieExpiration(int iCookieIndex) {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 29, iCookieIndex, 0);
      return (char*)val;
    }


    inline char* GetCookieName(int iCookieIndex) {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 30, iCookieIndex, 0);
      return (char*)val;
    }

    inline int SetCookieName(int iCookieIndex, const char *lpCookieName) {
      return IPWorksSSL_RSSS_Set(m_pObj, 30, iCookieIndex, (void*)lpCookieName, 0);
    }

    inline char* GetCookiePath(int iCookieIndex) {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 31, iCookieIndex, 0);
      return (char*)val;
    }


    inline int GetCookieSecure(int iCookieIndex) {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 32, iCookieIndex, 0);
      return (int)(long)val;
    }

    inline char* GetCookieValue(int iCookieIndex) {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 33, iCookieIndex, 0);
      return (char*)val;
    }

    inline int SetCookieValue(int iCookieIndex, const char *lpCookieValue) {
      return IPWorksSSL_RSSS_Set(m_pObj, 33, iCookieIndex, (void*)lpCookieValue, 0);
    }

    inline int GetFirewallAutoDetect() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 34, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallAutoDetect(int bFirewallAutoDetect) {
      void* val = (void*)IPS64CAST(bFirewallAutoDetect);
      return IPWorksSSL_RSSS_Set(m_pObj, 34, 0, val, 0);
    }
    inline int GetFirewallType() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 35, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallType(int iFirewallType) {
      void* val = (void*)IPS64CAST(iFirewallType);
      return IPWorksSSL_RSSS_Set(m_pObj, 35, 0, val, 0);
    }
    inline char* GetFirewallHost() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 36, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallHost(const char *lpFirewallHost) {
      return IPWorksSSL_RSSS_Set(m_pObj, 36, 0, (void*)lpFirewallHost, 0);
    }

    inline char* GetFirewallPassword() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 37, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallPassword(const char *lpFirewallPassword) {
      return IPWorksSSL_RSSS_Set(m_pObj, 37, 0, (void*)lpFirewallPassword, 0);
    }

    inline int GetFirewallPort() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 38, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallPort(int lFirewallPort) {
      void* val = (void*)IPS64CAST(lFirewallPort);
      return IPWorksSSL_RSSS_Set(m_pObj, 38, 0, val, 0);
    }
    inline char* GetFirewallUser() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 39, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallUser(const char *lpFirewallUser) {
      return IPWorksSSL_RSSS_Set(m_pObj, 39, 0, (void*)lpFirewallUser, 0);
    }

    inline int GetFollowRedirects() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 40, 0, 0);
      return (int)(long)val;
    }
    inline int SetFollowRedirects(int iFollowRedirects) {
      void* val = (void*)IPS64CAST(iFollowRedirects);
      return IPWorksSSL_RSSS_Set(m_pObj, 40, 0, val, 0);
    }
    inline int GetIdle() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 41, 0, 0);
      return (int)(long)val;
    }

    inline char* GetIfModifiedSince() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 42, 0, 0);
      return (char*)val;
    }

    inline int SetIfModifiedSince(const char *lpIfModifiedSince) {
      return IPWorksSSL_RSSS_Set(m_pObj, 42, 0, (void*)lpIfModifiedSince, 0);
    }

    inline int GetItemCount() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 43, 0, 0);
      return (int)(long)val;
    }
    inline int SetItemCount(int iItemCount) {
      void* val = (void*)IPS64CAST(iItemCount);
      return IPWorksSSL_RSSS_Set(m_pObj, 43, 0, val, 0);
    }
    inline char* GetItemAuthor(int iItemIndex) {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 44, iItemIndex, 0);
      return (char*)val;
    }

    inline int SetItemAuthor(int iItemIndex, const char *lpItemAuthor) {
      return IPWorksSSL_RSSS_Set(m_pObj, 44, iItemIndex, (void*)lpItemAuthor, 0);
    }

    inline char* GetItemCategory(int iItemIndex) {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 45, iItemIndex, 0);
      return (char*)val;
    }

    inline int SetItemCategory(int iItemIndex, const char *lpItemCategory) {
      return IPWorksSSL_RSSS_Set(m_pObj, 45, iItemIndex, (void*)lpItemCategory, 0);
    }

    inline char* GetItemCategoryDomain(int iItemIndex) {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 46, iItemIndex, 0);
      return (char*)val;
    }

    inline int SetItemCategoryDomain(int iItemIndex, const char *lpItemCategoryDomain) {
      return IPWorksSSL_RSSS_Set(m_pObj, 46, iItemIndex, (void*)lpItemCategoryDomain, 0);
    }

    inline char* GetItemComments(int iItemIndex) {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 47, iItemIndex, 0);
      return (char*)val;
    }

    inline int SetItemComments(int iItemIndex, const char *lpItemComments) {
      return IPWorksSSL_RSSS_Set(m_pObj, 47, iItemIndex, (void*)lpItemComments, 0);
    }

    inline char* GetItemDescription(int iItemIndex) {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 48, iItemIndex, 0);
      return (char*)val;
    }

    inline int SetItemDescription(int iItemIndex, const char *lpItemDescription) {
      return IPWorksSSL_RSSS_Set(m_pObj, 48, iItemIndex, (void*)lpItemDescription, 0);
    }

    inline char* GetItemEnclosureLength(int iItemIndex) {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 49, iItemIndex, 0);
      return (char*)val;
    }

    inline int SetItemEnclosureLength(int iItemIndex, const char *lpItemEnclosureLength) {
      return IPWorksSSL_RSSS_Set(m_pObj, 49, iItemIndex, (void*)lpItemEnclosureLength, 0);
    }

    inline char* GetItemEnclosureType(int iItemIndex) {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 50, iItemIndex, 0);
      return (char*)val;
    }

    inline int SetItemEnclosureType(int iItemIndex, const char *lpItemEnclosureType) {
      return IPWorksSSL_RSSS_Set(m_pObj, 50, iItemIndex, (void*)lpItemEnclosureType, 0);
    }

    inline char* GetItemEnclosureUrl(int iItemIndex) {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 51, iItemIndex, 0);
      return (char*)val;
    }

    inline int SetItemEnclosureUrl(int iItemIndex, const char *lpItemEnclosureUrl) {
      return IPWorksSSL_RSSS_Set(m_pObj, 51, iItemIndex, (void*)lpItemEnclosureUrl, 0);
    }

    inline char* GetItemGuid(int iItemIndex) {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 52, iItemIndex, 0);
      return (char*)val;
    }

    inline int SetItemGuid(int iItemIndex, const char *lpItemGuid) {
      return IPWorksSSL_RSSS_Set(m_pObj, 52, iItemIndex, (void*)lpItemGuid, 0);
    }

    inline int GetItemGuidIsPermaLink(int iItemIndex) {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 53, iItemIndex, 0);
      return (int)(long)val;
    }
    inline int SetItemGuidIsPermaLink(int iItemIndex, int bItemGuidIsPermaLink) {
      void* val = (void*)IPS64CAST(bItemGuidIsPermaLink);
      return IPWorksSSL_RSSS_Set(m_pObj, 53, iItemIndex, val, 0);
    }
    inline char* GetItemXML(int iItemIndex) {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 54, iItemIndex, 0);
      return (char*)val;
    }

    inline int SetItemXML(int iItemIndex, const char *lpItemXML) {
      return IPWorksSSL_RSSS_Set(m_pObj, 54, iItemIndex, (void*)lpItemXML, 0);
    }

    inline char* GetItemLink(int iItemIndex) {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 55, iItemIndex, 0);
      return (char*)val;
    }

    inline int SetItemLink(int iItemIndex, const char *lpItemLink) {
      return IPWorksSSL_RSSS_Set(m_pObj, 55, iItemIndex, (void*)lpItemLink, 0);
    }

    inline char* GetItemPubDate(int iItemIndex) {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 56, iItemIndex, 0);
      return (char*)val;
    }

    inline int SetItemPubDate(int iItemIndex, const char *lpItemPubDate) {
      return IPWorksSSL_RSSS_Set(m_pObj, 56, iItemIndex, (void*)lpItemPubDate, 0);
    }

    inline char* GetItemSource(int iItemIndex) {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 57, iItemIndex, 0);
      return (char*)val;
    }

    inline int SetItemSource(int iItemIndex, const char *lpItemSource) {
      return IPWorksSSL_RSSS_Set(m_pObj, 57, iItemIndex, (void*)lpItemSource, 0);
    }

    inline char* GetItemSourceURL(int iItemIndex) {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 58, iItemIndex, 0);
      return (char*)val;
    }

    inline int SetItemSourceURL(int iItemIndex, const char *lpItemSourceURL) {
      return IPWorksSSL_RSSS_Set(m_pObj, 58, iItemIndex, (void*)lpItemSourceURL, 0);
    }

    inline char* GetItemTitle(int iItemIndex) {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 59, iItemIndex, 0);
      return (char*)val;
    }

    inline int SetItemTitle(int iItemIndex, const char *lpItemTitle) {
      return IPWorksSSL_RSSS_Set(m_pObj, 59, iItemIndex, (void*)lpItemTitle, 0);
    }

    inline char* GetLocalFile() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 60, 0, 0);
      return (char*)val;
    }

    inline int SetLocalFile(const char *lpLocalFile) {
      return IPWorksSSL_RSSS_Set(m_pObj, 60, 0, (void*)lpLocalFile, 0);
    }

    inline char* GetLocalHost() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 61, 0, 0);
      return (char*)val;
    }

    inline int SetLocalHost(const char *lpLocalHost) {
      return IPWorksSSL_RSSS_Set(m_pObj, 61, 0, (void*)lpLocalHost, 0);
    }

    inline int GetNamespaceCount() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 62, 0, 0);
      return (int)(long)val;
    }
    inline int SetNamespaceCount(int iNamespaceCount) {
      void* val = (void*)IPS64CAST(iNamespaceCount);
      return IPWorksSSL_RSSS_Set(m_pObj, 62, 0, val, 0);
    }
    inline char* GetNamespacePrefix(int iNamespaceIndex) {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 63, iNamespaceIndex, 0);
      return (char*)val;
    }

    inline int SetNamespacePrefix(int iNamespaceIndex, const char *lpNamespacePrefix) {
      return IPWorksSSL_RSSS_Set(m_pObj, 63, iNamespaceIndex, (void*)lpNamespacePrefix, 0);
    }

    inline char* GetNamespaceURI(int iNamespaceIndex) {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 64, iNamespaceIndex, 0);
      return (char*)val;
    }

    inline int SetNamespaceURI(int iNamespaceIndex, const char *lpNamespaceURI) {
      return IPWorksSSL_RSSS_Set(m_pObj, 64, iNamespaceIndex, (void*)lpNamespaceURI, 0);
    }

    inline char* GetOtherHeaders() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 65, 0, 0);
      return (char*)val;
    }

    inline int SetOtherHeaders(const char *lpOtherHeaders) {
      return IPWorksSSL_RSSS_Set(m_pObj, 65, 0, (void*)lpOtherHeaders, 0);
    }

    inline int GetParsedHeaderCount() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 66, 0, 0);
      return (int)(long)val;
    }

    inline char* GetParsedHeaderField(int iParsedHeaderIndex) {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 67, iParsedHeaderIndex, 0);
      return (char*)val;
    }


    inline char* GetParsedHeaderValue(int iParsedHeaderIndex) {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 68, iParsedHeaderIndex, 0);
      return (char*)val;
    }


    inline char* GetPassword() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 69, 0, 0);
      return (char*)val;
    }

    inline int SetPassword(const char *lpPassword) {
      return IPWorksSSL_RSSS_Set(m_pObj, 69, 0, (void*)lpPassword, 0);
    }

    inline int GetProxyAuthScheme() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 70, 0, 0);
      return (int)(long)val;
    }
    inline int SetProxyAuthScheme(int iProxyAuthScheme) {
      void* val = (void*)IPS64CAST(iProxyAuthScheme);
      return IPWorksSSL_RSSS_Set(m_pObj, 70, 0, val, 0);
    }
    inline int GetProxyAutoDetect() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 71, 0, 0);
      return (int)(long)val;
    }
    inline int SetProxyAutoDetect(int bProxyAutoDetect) {
      void* val = (void*)IPS64CAST(bProxyAutoDetect);
      return IPWorksSSL_RSSS_Set(m_pObj, 71, 0, val, 0);
    }
    inline char* GetProxyPassword() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 72, 0, 0);
      return (char*)val;
    }

    inline int SetProxyPassword(const char *lpProxyPassword) {
      return IPWorksSSL_RSSS_Set(m_pObj, 72, 0, (void*)lpProxyPassword, 0);
    }

    inline int GetProxyPort() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 73, 0, 0);
      return (int)(long)val;
    }
    inline int SetProxyPort(int lProxyPort) {
      void* val = (void*)IPS64CAST(lProxyPort);
      return IPWorksSSL_RSSS_Set(m_pObj, 73, 0, val, 0);
    }
    inline char* GetProxyServer() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 74, 0, 0);
      return (char*)val;
    }

    inline int SetProxyServer(const char *lpProxyServer) {
      return IPWorksSSL_RSSS_Set(m_pObj, 74, 0, (void*)lpProxyServer, 0);
    }

    inline int GetProxySSL() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 75, 0, 0);
      return (int)(long)val;
    }
    inline int SetProxySSL(int iProxySSL) {
      void* val = (void*)IPS64CAST(iProxySSL);
      return IPWorksSSL_RSSS_Set(m_pObj, 75, 0, val, 0);
    }
    inline char* GetProxyUser() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 76, 0, 0);
      return (char*)val;
    }

    inline int SetProxyUser(const char *lpProxyUser) {
      return IPWorksSSL_RSSS_Set(m_pObj, 76, 0, (void*)lpProxyUser, 0);
    }

    inline char* GetReferer() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 77, 0, 0);
      return (char*)val;
    }

    inline int SetReferer(const char *lpReferer) {
      return IPWorksSSL_RSSS_Set(m_pObj, 77, 0, (void*)lpReferer, 0);
    }

    inline char* GetRSSData() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 78, 0, 0);
      return (char*)val;
    }

    inline int SetRSSData(const char *lpRSSData) {
      return IPWorksSSL_RSSS_Set(m_pObj, 78, 0, (void*)lpRSSData, 0);
    }

    inline int GetRSSVersion() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 79, 0, 0);
      return (int)(long)val;
    }
    inline int SetRSSVersion(int iRSSVersion) {
      void* val = (void*)IPS64CAST(iRSSVersion);
      return IPWorksSSL_RSSS_Set(m_pObj, 79, 0, val, 0);
    }
    inline int GetSSLAcceptServerCertEncoded(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_RSSS_Get(m_pObj, 80, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }

    inline int SetSSLAcceptServerCertEncoded(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_RSSS_Set(m_pObj, 80, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }

    inline int GetSSLCertEncoded(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_RSSS_Get(m_pObj, 81, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }

    inline int SetSSLCertEncoded(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_RSSS_Set(m_pObj, 81, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }

    inline int GetSSLCertStore(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_RSSS_Get(m_pObj, 82, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }

    inline int SetSSLCertStore(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_RSSS_Set(m_pObj, 82, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }

    inline char* GetSSLCertStorePassword() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 83, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertStorePassword(const char *lpSSLCertStorePassword) {
      return IPWorksSSL_RSSS_Set(m_pObj, 83, 0, (void*)lpSSLCertStorePassword, 0);
    }

    inline int GetSSLCertStoreType() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 84, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLCertStoreType(int iSSLCertStoreType) {
      void* val = (void*)IPS64CAST(iSSLCertStoreType);
      return IPWorksSSL_RSSS_Set(m_pObj, 84, 0, val, 0);
    }
    inline char* GetSSLCertSubject() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 85, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertSubject(const char *lpSSLCertSubject) {
      return IPWorksSSL_RSSS_Set(m_pObj, 85, 0, (void*)lpSSLCertSubject, 0);
    }

    inline int GetSSLServerCertEncoded(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_RSSS_Get(m_pObj, 86, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }


    inline int GetTimeout() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 87, 0, 0);
      return (int)(long)val;
    }
    inline int SetTimeout(int iTimeout) {
      void* val = (void*)IPS64CAST(iTimeout);
      return IPWorksSSL_RSSS_Set(m_pObj, 87, 0, val, 0);
    }
    inline int GetTransferredData(char *&lpTransferredData, int &lenTransferredData) {
      lpTransferredData = (char*)IPWorksSSL_RSSS_Get(m_pObj, 88, 0, &lenTransferredData);
      return lpTransferredData ? 0 : lenTransferredData;
    }


    inline ns_int64 GetTransferredDataLimit() {
      ns_int64 *pval = (ns_int64*)IPWorksSSL_RSSS_Get(m_pObj, 89, 0, 0);
      return *pval;
    }

    inline int SetTransferredDataLimit(ns_int64 lTransferredDataLimit) {
      void* val = (void*)(&lTransferredDataLimit);
      return IPWorksSSL_RSSS_Set(m_pObj, 89, 0, val, 0);
    }

    inline char* GetTransferredHeaders() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 90, 0, 0);
      return (char*)val;
    }


    inline char* GetUser() {
      void* val = IPWorksSSL_RSSS_Get(m_pObj, 91, 0, 0);
      return (char*)val;
    }

    inline int SetUser(const char *lpUser) {
      return IPWorksSSL_RSSS_Set(m_pObj, 91, 0, (void*)lpUser, 0);
    }


  public: //methods

    inline int AddCookie(const char* lpszCookieName, const char* lpszCookieValue) {
      void *param[2+1] = {(void*)IPS64CAST(lpszCookieName), (void*)IPS64CAST(lpszCookieValue), 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_RSSS_Do(m_pObj, 2, 2, param, cbparam);
      
      
    }
    inline int AddItem(const char* lpszTitle, const char* lpszDescription, const char* lpszLink) {
      void *param[3+1] = {(void*)IPS64CAST(lpszTitle), (void*)IPS64CAST(lpszDescription), (void*)IPS64CAST(lpszLink), 0};
      int cbparam[3+1] = {0, 0, 0, 0};
      return IPWorksSSL_RSSS_Do(m_pObj, 3, 3, param, cbparam);
      
      
    }
    inline int AddNamespace(const char* lpszPrefix, const char* lpszNamespaceURI) {
      void *param[2+1] = {(void*)IPS64CAST(lpszPrefix), (void*)IPS64CAST(lpszNamespaceURI), 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_RSSS_Do(m_pObj, 4, 2, param, cbparam);
      
      
    }
    inline char* Config(const char* lpszConfigurationString) {
      void *param[1+1] = {(void*)IPS64CAST(lpszConfigurationString), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_RSSS_Do(m_pObj, 5, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_RSSS_Do(m_pObj, 6, 0, param, cbparam);
      
      
    }
    inline int GetFeed(const char* lpszURL) {
      void *param[1+1] = {(void*)IPS64CAST(lpszURL), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_RSSS_Do(m_pObj, 7, 1, param, cbparam);
      
      
    }
    inline char* GetProperty(const char* lpszPropertyName) {
      void *param[1+1] = {(void*)IPS64CAST(lpszPropertyName), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_RSSS_Do(m_pObj, 8, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline int GetURL(const char* lpszURL) {
      void *param[1+1] = {(void*)IPS64CAST(lpszURL), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_RSSS_Do(m_pObj, 9, 1, param, cbparam);
      
      
    }
    inline int Interrupt() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_RSSS_Do(m_pObj, 10, 0, param, cbparam);
      
      
    }
    inline char* OPMLAttr(const char* lpszAttrName) {
      void *param[1+1] = {(void*)IPS64CAST(lpszAttrName), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_RSSS_Do(m_pObj, 11, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline int Put(const char* lpszURL) {
      void *param[1+1] = {(void*)IPS64CAST(lpszURL), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_RSSS_Do(m_pObj, 12, 1, param, cbparam);
      
      
    }
    inline int ReadFile(const char* lpszFileName) {
      void *param[1+1] = {(void*)IPS64CAST(lpszFileName), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_RSSS_Do(m_pObj, 13, 1, param, cbparam);
      
      
    }
    inline int ReadOPML(const char* lpszOPMLFile) {
      void *param[1+1] = {(void*)IPS64CAST(lpszOPMLFile), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_RSSS_Do(m_pObj, 14, 1, param, cbparam);
      
      
    }
    inline int Reset() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_RSSS_Do(m_pObj, 15, 0, param, cbparam);
      
      
    }
    inline int SetProperty(const char* lpszPropertyName, const char* lpszPropertyValue) {
      void *param[2+1] = {(void*)IPS64CAST(lpszPropertyName), (void*)IPS64CAST(lpszPropertyValue), 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_RSSS_Do(m_pObj, 16, 2, param, cbparam);
      
      
    }
    inline int WriteFile(const char* lpszFilename) {
      void *param[1+1] = {(void*)IPS64CAST(lpszFilename), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_RSSS_Do(m_pObj, 17, 1, param, cbparam);
      
      
    }

};


#ifdef WIN32 //UNICODE

typedef struct {
  int StatusCode;
  LPWSTR Description;
  int reserved;
} RSSSConnectedEventParamsW;

typedef struct {
  LPWSTR ConnectionEvent;
  int StatusCode;
  LPWSTR Description;
  int reserved;
} RSSSConnectionStatusEventParamsW;

typedef struct {
  int StatusCode;
  LPWSTR Description;
  int reserved;
} RSSSDisconnectedEventParamsW;

typedef struct {
  int Direction;
  int reserved;
} RSSSEndTransferEventParamsW;

typedef struct {
  int ErrorCode;
  LPWSTR Description;
  int reserved;
} RSSSErrorEventParamsW;

typedef struct {
  LPWSTR Field;
  LPWSTR Value;
  int reserved;
} RSSSHeaderEventParamsW;

typedef struct {
  LPWSTR Field;
  LPWSTR Value;
  int reserved;
} RSSSOPMLHeaderEventParamsW;

typedef struct {
  int Level;
  LPWSTR Text;
  LPWSTR TypeName;
  LPWSTR Title;
  LPWSTR URL;
  LPWSTR XMLURL;
  LPWSTR HTMLURL;
  LPWSTR Created;
  LPWSTR Description;
  LPWSTR OtherAttrs;
  int reserved;
} RSSSOPMLOutlineEventParamsW;

typedef struct {
  LPWSTR Location;
  int Accept;
  int reserved;
} RSSSRedirectEventParamsW;

typedef struct {
  LPWSTR Name;
  LPWSTR Value;
  LPWSTR Expires;
  LPWSTR Domain;
  LPWSTR Path;
  int Secure;
  int reserved;
} RSSSSetCookieEventParamsW;

typedef struct {
  LPWSTR CertEncoded;
  LPWSTR CertSubject;
  LPWSTR CertIssuer;
  LPWSTR Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} RSSSSSLServerAuthenticationEventParamsW;

typedef struct {
  LPWSTR Message;
  int reserved;
} RSSSSSLStatusEventParamsW;

typedef struct {
  int Direction;
  int reserved;
} RSSSStartTransferEventParamsW;

typedef struct {
  LPWSTR HTTPVersion;
  int StatusCode;
  LPWSTR Description;
  int reserved;
} RSSSStatusEventParamsW;

typedef struct {
  int Direction;
  ns_int64 *pBytesTransferred;
  int PercentDone;
  LPWSTR Text;
  int lenText;
  int reserved;
} RSSSTransferEventParamsW;



class RSSSW : public RSSS {

  public: //properties
  
    inline LPWSTR GetAccept() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+1, 0, 0);
    }

    inline int SetAccept(LPWSTR lpAccept) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+1, 0, (void*)lpAccept, 0);
    }

    inline LPWSTR GetAuthorization() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+2, 0, 0);
    }

    inline int SetAuthorization(LPWSTR lpAuthorization) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+2, 0, (void*)lpAuthorization, 0);
    }



    inline LPWSTR GetChannelCategory() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+4, 0, 0);
    }

    inline int SetChannelCategory(LPWSTR lpChannelCategory) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+4, 0, (void*)lpChannelCategory, 0);
    }

    inline LPWSTR GetChannelCategoryDomain() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+5, 0, 0);
    }

    inline int SetChannelCategoryDomain(LPWSTR lpChannelCategoryDomain) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+5, 0, (void*)lpChannelCategoryDomain, 0);
    }

    inline LPWSTR GetChannelCopyright() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+6, 0, 0);
    }

    inline int SetChannelCopyright(LPWSTR lpChannelCopyright) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+6, 0, (void*)lpChannelCopyright, 0);
    }

    inline LPWSTR GetChannelDescription() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+7, 0, 0);
    }

    inline int SetChannelDescription(LPWSTR lpChannelDescription) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+7, 0, (void*)lpChannelDescription, 0);
    }

    inline LPWSTR GetChannelDocs() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+8, 0, 0);
    }

    inline int SetChannelDocs(LPWSTR lpChannelDocs) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+8, 0, (void*)lpChannelDocs, 0);
    }

    inline LPWSTR GetChannelGenerator() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+9, 0, 0);
    }

    inline int SetChannelGenerator(LPWSTR lpChannelGenerator) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+9, 0, (void*)lpChannelGenerator, 0);
    }

    inline LPWSTR GetChannelImageDescription() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+10, 0, 0);
    }

    inline int SetChannelImageDescription(LPWSTR lpChannelImageDescription) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+10, 0, (void*)lpChannelImageDescription, 0);
    }

    inline LPWSTR GetChannelImageHeight() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+11, 0, 0);
    }

    inline int SetChannelImageHeight(LPWSTR lpChannelImageHeight) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+11, 0, (void*)lpChannelImageHeight, 0);
    }

    inline LPWSTR GetChannelImageLink() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+12, 0, 0);
    }

    inline int SetChannelImageLink(LPWSTR lpChannelImageLink) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+12, 0, (void*)lpChannelImageLink, 0);
    }

    inline LPWSTR GetChannelImageTitle() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+13, 0, 0);
    }

    inline int SetChannelImageTitle(LPWSTR lpChannelImageTitle) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+13, 0, (void*)lpChannelImageTitle, 0);
    }

    inline LPWSTR GetChannelImageUrl() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+14, 0, 0);
    }

    inline int SetChannelImageUrl(LPWSTR lpChannelImageUrl) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+14, 0, (void*)lpChannelImageUrl, 0);
    }

    inline LPWSTR GetChannelImageWidth() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+15, 0, 0);
    }

    inline int SetChannelImageWidth(LPWSTR lpChannelImageWidth) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+15, 0, (void*)lpChannelImageWidth, 0);
    }

    inline LPWSTR GetChannelLanguage() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+16, 0, 0);
    }

    inline int SetChannelLanguage(LPWSTR lpChannelLanguage) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+16, 0, (void*)lpChannelLanguage, 0);
    }

    inline LPWSTR GetChannelLastBuildDate() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+17, 0, 0);
    }

    inline int SetChannelLastBuildDate(LPWSTR lpChannelLastBuildDate) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+17, 0, (void*)lpChannelLastBuildDate, 0);
    }

    inline LPWSTR GetChannelLink() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+18, 0, 0);
    }

    inline int SetChannelLink(LPWSTR lpChannelLink) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+18, 0, (void*)lpChannelLink, 0);
    }

    inline LPWSTR GetChannelManagingEditor() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+19, 0, 0);
    }

    inline int SetChannelManagingEditor(LPWSTR lpChannelManagingEditor) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+19, 0, (void*)lpChannelManagingEditor, 0);
    }

    inline LPWSTR GetChannelPubDate() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+20, 0, 0);
    }

    inline int SetChannelPubDate(LPWSTR lpChannelPubDate) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+20, 0, (void*)lpChannelPubDate, 0);
    }

    inline LPWSTR GetChannelSkipDays() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+21, 0, 0);
    }

    inline int SetChannelSkipDays(LPWSTR lpChannelSkipDays) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+21, 0, (void*)lpChannelSkipDays, 0);
    }

    inline LPWSTR GetChannelSkipHours() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+22, 0, 0);
    }

    inline int SetChannelSkipHours(LPWSTR lpChannelSkipHours) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+22, 0, (void*)lpChannelSkipHours, 0);
    }

    inline LPWSTR GetChannelTitle() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+23, 0, 0);
    }

    inline int SetChannelTitle(LPWSTR lpChannelTitle) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+23, 0, (void*)lpChannelTitle, 0);
    }

    inline LPWSTR GetChannelTtl() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+24, 0, 0);
    }

    inline int SetChannelTtl(LPWSTR lpChannelTtl) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+24, 0, (void*)lpChannelTtl, 0);
    }

    inline LPWSTR GetChannelWebMaster() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+25, 0, 0);
    }

    inline int SetChannelWebMaster(LPWSTR lpChannelWebMaster) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+25, 0, (void*)lpChannelWebMaster, 0);
    }





    inline LPWSTR GetCookieDomain(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+28, iCookieIndex, 0);
    }



    inline LPWSTR GetCookieExpiration(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+29, iCookieIndex, 0);
    }



    inline LPWSTR GetCookieName(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+30, iCookieIndex, 0);
    }

    inline int SetCookieName(int iCookieIndex, LPWSTR lpCookieName) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+30, iCookieIndex, (void*)lpCookieName, 0);
    }

    inline LPWSTR GetCookiePath(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+31, iCookieIndex, 0);
    }





    inline LPWSTR GetCookieValue(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+33, iCookieIndex, 0);
    }

    inline int SetCookieValue(int iCookieIndex, LPWSTR lpCookieValue) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+33, iCookieIndex, (void*)lpCookieValue, 0);
    }





    inline LPWSTR GetFirewallHost() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+36, 0, 0);
    }

    inline int SetFirewallHost(LPWSTR lpFirewallHost) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+36, 0, (void*)lpFirewallHost, 0);
    }

    inline LPWSTR GetFirewallPassword() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+37, 0, 0);
    }

    inline int SetFirewallPassword(LPWSTR lpFirewallPassword) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+37, 0, (void*)lpFirewallPassword, 0);
    }



    inline LPWSTR GetFirewallUser() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+39, 0, 0);
    }

    inline int SetFirewallUser(LPWSTR lpFirewallUser) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+39, 0, (void*)lpFirewallUser, 0);
    }





    inline LPWSTR GetIfModifiedSince() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+42, 0, 0);
    }

    inline int SetIfModifiedSince(LPWSTR lpIfModifiedSince) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+42, 0, (void*)lpIfModifiedSince, 0);
    }



    inline LPWSTR GetItemAuthor(int iItemIndex) {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+44, iItemIndex, 0);
    }

    inline int SetItemAuthor(int iItemIndex, LPWSTR lpItemAuthor) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+44, iItemIndex, (void*)lpItemAuthor, 0);
    }

    inline LPWSTR GetItemCategory(int iItemIndex) {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+45, iItemIndex, 0);
    }

    inline int SetItemCategory(int iItemIndex, LPWSTR lpItemCategory) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+45, iItemIndex, (void*)lpItemCategory, 0);
    }

    inline LPWSTR GetItemCategoryDomain(int iItemIndex) {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+46, iItemIndex, 0);
    }

    inline int SetItemCategoryDomain(int iItemIndex, LPWSTR lpItemCategoryDomain) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+46, iItemIndex, (void*)lpItemCategoryDomain, 0);
    }

    inline LPWSTR GetItemComments(int iItemIndex) {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+47, iItemIndex, 0);
    }

    inline int SetItemComments(int iItemIndex, LPWSTR lpItemComments) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+47, iItemIndex, (void*)lpItemComments, 0);
    }

    inline LPWSTR GetItemDescription(int iItemIndex) {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+48, iItemIndex, 0);
    }

    inline int SetItemDescription(int iItemIndex, LPWSTR lpItemDescription) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+48, iItemIndex, (void*)lpItemDescription, 0);
    }

    inline LPWSTR GetItemEnclosureLength(int iItemIndex) {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+49, iItemIndex, 0);
    }

    inline int SetItemEnclosureLength(int iItemIndex, LPWSTR lpItemEnclosureLength) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+49, iItemIndex, (void*)lpItemEnclosureLength, 0);
    }

    inline LPWSTR GetItemEnclosureType(int iItemIndex) {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+50, iItemIndex, 0);
    }

    inline int SetItemEnclosureType(int iItemIndex, LPWSTR lpItemEnclosureType) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+50, iItemIndex, (void*)lpItemEnclosureType, 0);
    }

    inline LPWSTR GetItemEnclosureUrl(int iItemIndex) {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+51, iItemIndex, 0);
    }

    inline int SetItemEnclosureUrl(int iItemIndex, LPWSTR lpItemEnclosureUrl) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+51, iItemIndex, (void*)lpItemEnclosureUrl, 0);
    }

    inline LPWSTR GetItemGuid(int iItemIndex) {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+52, iItemIndex, 0);
    }

    inline int SetItemGuid(int iItemIndex, LPWSTR lpItemGuid) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+52, iItemIndex, (void*)lpItemGuid, 0);
    }



    inline LPWSTR GetItemXML(int iItemIndex) {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+54, iItemIndex, 0);
    }

    inline int SetItemXML(int iItemIndex, LPWSTR lpItemXML) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+54, iItemIndex, (void*)lpItemXML, 0);
    }

    inline LPWSTR GetItemLink(int iItemIndex) {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+55, iItemIndex, 0);
    }

    inline int SetItemLink(int iItemIndex, LPWSTR lpItemLink) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+55, iItemIndex, (void*)lpItemLink, 0);
    }

    inline LPWSTR GetItemPubDate(int iItemIndex) {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+56, iItemIndex, 0);
    }

    inline int SetItemPubDate(int iItemIndex, LPWSTR lpItemPubDate) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+56, iItemIndex, (void*)lpItemPubDate, 0);
    }

    inline LPWSTR GetItemSource(int iItemIndex) {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+57, iItemIndex, 0);
    }

    inline int SetItemSource(int iItemIndex, LPWSTR lpItemSource) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+57, iItemIndex, (void*)lpItemSource, 0);
    }

    inline LPWSTR GetItemSourceURL(int iItemIndex) {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+58, iItemIndex, 0);
    }

    inline int SetItemSourceURL(int iItemIndex, LPWSTR lpItemSourceURL) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+58, iItemIndex, (void*)lpItemSourceURL, 0);
    }

    inline LPWSTR GetItemTitle(int iItemIndex) {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+59, iItemIndex, 0);
    }

    inline int SetItemTitle(int iItemIndex, LPWSTR lpItemTitle) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+59, iItemIndex, (void*)lpItemTitle, 0);
    }

    inline LPWSTR GetLocalFile() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+10000+60, 0, 0);
    }

    inline int SetLocalFile(LPWSTR lpLocalFile) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+10000+60, 0, (void*)lpLocalFile, 0);
    }

    inline LPWSTR GetLocalHost() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+61, 0, 0);
    }

    inline int SetLocalHost(LPWSTR lpLocalHost) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+61, 0, (void*)lpLocalHost, 0);
    }



    inline LPWSTR GetNamespacePrefix(int iNamespaceIndex) {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+63, iNamespaceIndex, 0);
    }

    inline int SetNamespacePrefix(int iNamespaceIndex, LPWSTR lpNamespacePrefix) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+63, iNamespaceIndex, (void*)lpNamespacePrefix, 0);
    }

    inline LPWSTR GetNamespaceURI(int iNamespaceIndex) {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+64, iNamespaceIndex, 0);
    }

    inline int SetNamespaceURI(int iNamespaceIndex, LPWSTR lpNamespaceURI) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+64, iNamespaceIndex, (void*)lpNamespaceURI, 0);
    }

    inline LPWSTR GetOtherHeaders() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+65, 0, 0);
    }

    inline int SetOtherHeaders(LPWSTR lpOtherHeaders) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+65, 0, (void*)lpOtherHeaders, 0);
    }



    inline LPWSTR GetParsedHeaderField(int iParsedHeaderIndex) {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+67, iParsedHeaderIndex, 0);
    }



    inline LPWSTR GetParsedHeaderValue(int iParsedHeaderIndex) {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+68, iParsedHeaderIndex, 0);
    }



    inline LPWSTR GetPassword() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+69, 0, 0);
    }

    inline int SetPassword(LPWSTR lpPassword) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+69, 0, (void*)lpPassword, 0);
    }





    inline LPWSTR GetProxyPassword() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+72, 0, 0);
    }

    inline int SetProxyPassword(LPWSTR lpProxyPassword) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+72, 0, (void*)lpProxyPassword, 0);
    }



    inline LPWSTR GetProxyServer() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+74, 0, 0);
    }

    inline int SetProxyServer(LPWSTR lpProxyServer) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+74, 0, (void*)lpProxyServer, 0);
    }



    inline LPWSTR GetProxyUser() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+76, 0, 0);
    }

    inline int SetProxyUser(LPWSTR lpProxyUser) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+76, 0, (void*)lpProxyUser, 0);
    }

    inline LPWSTR GetReferer() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+77, 0, 0);
    }

    inline int SetReferer(LPWSTR lpReferer) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+77, 0, (void*)lpReferer, 0);
    }

    inline LPWSTR GetRSSData() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+78, 0, 0);
    }

    inline int SetRSSData(LPWSTR lpRSSData) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+78, 0, (void*)lpRSSData, 0);
    }



    inline LPWSTR GetSSLAcceptServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+80, 0, 0);
    }

    inline int SetSSLAcceptServerCertEncoded(LPWSTR lpSSLAcceptServerCertEncoded) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+80, 0, (void*)lpSSLAcceptServerCertEncoded, 0);
    }
    inline int GetSSLAcceptServerCertEncodedB(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_RSSS_Get(m_pObj, 80, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }
    inline int SetSSLAcceptServerCertEncodedB(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_RSSS_Set(m_pObj, 80, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }
    inline LPWSTR GetSSLCertEncoded() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+81, 0, 0);
    }

    inline int SetSSLCertEncoded(LPWSTR lpSSLCertEncoded) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+81, 0, (void*)lpSSLCertEncoded, 0);
    }
    inline int GetSSLCertEncodedB(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_RSSS_Get(m_pObj, 81, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }
    inline int SetSSLCertEncodedB(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_RSSS_Set(m_pObj, 81, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }
    inline LPWSTR GetSSLCertStore() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+82, 0, 0);
    }

    inline int SetSSLCertStore(LPWSTR lpSSLCertStore) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+82, 0, (void*)lpSSLCertStore, 0);
    }
    inline int GetSSLCertStoreB(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_RSSS_Get(m_pObj, 82, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }
    inline int SetSSLCertStoreB(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_RSSS_Set(m_pObj, 82, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }
    inline LPWSTR GetSSLCertStorePassword() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+83, 0, 0);
    }

    inline int SetSSLCertStorePassword(LPWSTR lpSSLCertStorePassword) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+83, 0, (void*)lpSSLCertStorePassword, 0);
    }



    inline LPWSTR GetSSLCertSubject() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+85, 0, 0);
    }

    inline int SetSSLCertSubject(LPWSTR lpSSLCertSubject) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+85, 0, (void*)lpSSLCertSubject, 0);
    }

    inline LPWSTR GetSSLServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+86, 0, 0);
    }


    inline int GetSSLServerCertEncodedB(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_RSSS_Get(m_pObj, 86, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }



    inline LPWSTR GetTransferredData() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+88, 0, 0);
    }


    inline int GetTransferredDataB(char *&lpTransferredData, int &lenTransferredData) {
      lpTransferredData = (char*)IPWorksSSL_RSSS_Get(m_pObj, 88, 0, &lenTransferredData);
      return lpTransferredData ? 0 : lenTransferredData;
    }



    inline LPWSTR GetTransferredHeaders() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+90, 0, 0);
    }



    inline LPWSTR GetUser() {
      return (LPWSTR)IPWorksSSL_RSSS_Get(m_pObj, 10000+91, 0, 0);
    }

    inline int SetUser(LPWSTR lpUser) {
      return IPWorksSSL_RSSS_Set(m_pObj, 10000+91, 0, (void*)lpUser, 0);
    }



  public: //events
  
    virtual int FireConnected(RSSSConnectedEventParamsW *e) {return 0;}
    virtual int FireConnectionStatus(RSSSConnectionStatusEventParamsW *e) {return 0;}
    virtual int FireDisconnected(RSSSDisconnectedEventParamsW *e) {return 0;}
    virtual int FireEndTransfer(RSSSEndTransferEventParamsW *e) {return 0;}
    virtual int FireError(RSSSErrorEventParamsW *e) {return 0;}
    virtual int FireHeader(RSSSHeaderEventParamsW *e) {return 0;}
    virtual int FireOPMLHeader(RSSSOPMLHeaderEventParamsW *e) {return 0;}
    virtual int FireOPMLOutline(RSSSOPMLOutlineEventParamsW *e) {return 0;}
    virtual int FireRedirect(RSSSRedirectEventParamsW *e) {return 0;}
    virtual int FireSetCookie(RSSSSetCookieEventParamsW *e) {return 0;}
    virtual int FireSSLServerAuthentication(RSSSSSLServerAuthenticationEventParamsW *e) {return 0;}
    virtual int FireSSLStatus(RSSSSSLStatusEventParamsW *e) {return 0;}
    virtual int FireStartTransfer(RSSSStartTransferEventParamsW *e) {return 0;}
    virtual int FireStatus(RSSSStatusEventParamsW *e) {return 0;}
    virtual int FireTransfer(RSSSTransferEventParamsW *e) {return 0;}


  protected:
  
    virtual int RSSSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {
    	int ret_code = 0;
      switch (event_id) {
         case 1: {
            RSSSConnectedEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireConnected(&e);
            break;
         }
         case 2: {
            RSSSConnectionStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]),  0};
            ret_code = FireConnectionStatus(&e);
            break;
         }
         case 3: {
            RSSSDisconnectedEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireDisconnected(&e);
            break;
         }
         case 4: {
            RSSSEndTransferEventParamsW e = {(int)IPS64CAST(param[0]),  0};
            ret_code = FireEndTransfer(&e);
            break;
         }
         case 5: {
            RSSSErrorEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireError(&e);
            break;
         }
         case 6: {
            RSSSHeaderEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireHeader(&e);
            break;
         }
         case 7: {
            RSSSOPMLHeaderEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireOPMLHeader(&e);
            break;
         }
         case 8: {
            RSSSOPMLOutlineEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (LPWSTR)IPS64CAST(param[4]), (LPWSTR)IPS64CAST(param[5]), (LPWSTR)IPS64CAST(param[6]), (LPWSTR)IPS64CAST(param[7]), (LPWSTR)IPS64CAST(param[8]), (LPWSTR)IPS64CAST(param[9]),  0};
            ret_code = FireOPMLOutline(&e);
            break;
         }
         case 9: {
            RSSSRedirectEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]),  0};
            ret_code = FireRedirect(&e);
            param[1] = (void*)(e.Accept);
            break;
         }
         case 10: {
            RSSSSetCookieEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (LPWSTR)IPS64CAST(param[4]), (int)IPS64CAST(param[5]),  0};
            ret_code = FireSetCookie(&e);
            break;
         }
         case 11: {
            RSSSSSLServerAuthenticationEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = FireSSLServerAuthentication(&e);
            param[4] = (void*)(e.Accept);
            break;
         }
         case 12: {
            RSSSSSLStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]),  0};
            ret_code = FireSSLStatus(&e);
            break;
         }
         case 13: {
            RSSSStartTransferEventParamsW e = {(int)IPS64CAST(param[0]),  0};
            ret_code = FireStartTransfer(&e);
            break;
         }
         case 14: {
            RSSSStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]),  0};
            ret_code = FireStatus(&e);
            break;
         }
         case 15: {
            RSSSTransferEventParamsW e = {(int)IPS64CAST(param[0]), (ns_int64*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (int)IPS64CAST(cbparam[3]),  0};
            ret_code = FireTransfer(&e);
            break;
         }

      }
      return ret_code;
    }
  
  public: //event overrides

    virtual int FireConnected(RSSSConnectedEventParams *e) {return -10000;}
    virtual int FireConnectionStatus(RSSSConnectionStatusEventParams *e) {return -10000;}
    virtual int FireDisconnected(RSSSDisconnectedEventParams *e) {return -10000;}
    virtual int FireEndTransfer(RSSSEndTransferEventParams *e) {return -10000;}
    virtual int FireError(RSSSErrorEventParams *e) {return -10000;}
    virtual int FireHeader(RSSSHeaderEventParams *e) {return -10000;}
    virtual int FireOPMLHeader(RSSSOPMLHeaderEventParams *e) {return -10000;}
    virtual int FireOPMLOutline(RSSSOPMLOutlineEventParams *e) {return -10000;}
    virtual int FireRedirect(RSSSRedirectEventParams *e) {return -10000;}
    virtual int FireSetCookie(RSSSSetCookieEventParams *e) {return -10000;}
    virtual int FireSSLServerAuthentication(RSSSSSLServerAuthenticationEventParams *e) {return -10000;}
    virtual int FireSSLStatus(RSSSSSLStatusEventParams *e) {return -10000;}
    virtual int FireStartTransfer(RSSSStartTransferEventParams *e) {return -10000;}
    virtual int FireStatus(RSSSStatusEventParams *e) {return -10000;}
    virtual int FireTransfer(RSSSTransferEventParams *e) {return -10000;}

  public: //methods

    inline int AddCookie(LPWSTR lpszCookieName, LPWSTR lpszCookieValue) {
      void *param[2+1] = {(void*)lpszCookieName, (void*)lpszCookieValue, 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_RSSS_Do(m_pObj, 10000+2, 2, param, cbparam);
      
    }
    inline int AddItem(LPWSTR lpszTitle, LPWSTR lpszDescription, LPWSTR lpszLink) {
      void *param[3+1] = {(void*)lpszTitle, (void*)lpszDescription, (void*)lpszLink, 0};
      int cbparam[3+1] = {0, 0, 0, 0};
      return IPWorksSSL_RSSS_Do(m_pObj, 10000+3, 3, param, cbparam);
      
    }
    inline int AddNamespace(LPWSTR lpszPrefix, LPWSTR lpszNamespaceURI) {
      void *param[2+1] = {(void*)lpszPrefix, (void*)lpszNamespaceURI, 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_RSSS_Do(m_pObj, 10000+4, 2, param, cbparam);
      
    }
    inline LPWSTR Config(LPWSTR lpszConfigurationString) {
      void *param[1+1] = {(void*)lpszConfigurationString, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_RSSS_Do(m_pObj, 10000+5, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_RSSS_Do(m_pObj, 10000+6, 0, param, cbparam);
      
    }
    inline int GetFeed(LPWSTR lpszURL) {
      void *param[1+1] = {(void*)lpszURL, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_RSSS_Do(m_pObj, 10000+7, 1, param, cbparam);
      
    }
    inline LPWSTR GetProperty(LPWSTR lpszPropertyName) {
      void *param[1+1] = {(void*)lpszPropertyName, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_RSSS_Do(m_pObj, 10000+8, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline int GetURL(LPWSTR lpszURL) {
      void *param[1+1] = {(void*)lpszURL, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_RSSS_Do(m_pObj, 10000+9, 1, param, cbparam);
      
    }
    inline int Interrupt() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_RSSS_Do(m_pObj, 10000+10, 0, param, cbparam);
      
    }
    inline LPWSTR OPMLAttr(LPWSTR lpszAttrName) {
      void *param[1+1] = {(void*)lpszAttrName, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_RSSS_Do(m_pObj, 10000+11, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline int Put(LPWSTR lpszURL) {
      void *param[1+1] = {(void*)lpszURL, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_RSSS_Do(m_pObj, 10000+12, 1, param, cbparam);
      
    }
    inline int ReadFile(LPWSTR lpszFileName) {
      void *param[1+1] = {(void*)lpszFileName, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_RSSS_Do(m_pObj, 10000+13, 1, param, cbparam);
      
    }
    inline int ReadOPML(LPWSTR lpszOPMLFile) {
      void *param[1+1] = {(void*)lpszOPMLFile, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_RSSS_Do(m_pObj, 10000+14, 1, param, cbparam);
      
    }
    inline int Reset() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_RSSS_Do(m_pObj, 10000+15, 0, param, cbparam);
      
    }
    inline int SetProperty(LPWSTR lpszPropertyName, LPWSTR lpszPropertyValue) {
      void *param[2+1] = {(void*)lpszPropertyName, (void*)lpszPropertyValue, 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_RSSS_Do(m_pObj, 10000+16, 2, param, cbparam);
      
    }
    inline int WriteFile(LPWSTR lpszFilename) {
      void *param[1+1] = {(void*)lpszFilename, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_RSSS_Do(m_pObj, 10000+17, 1, param, cbparam);
      
    }

};

#endif //WIN32

#endif //_IPWORKSSSL_RSSS_H_




