/******************************************************************
   IP*Works! SSL V9 C++ Edition
   Copyright (c) 2013 /n software inc. - All rights reserved.
*******************************************************************/

#ifndef _IPWORKSSSL_SNPPS_H_
#define _IPWORKSSSL_SNPPS_H_

#define IPWORKSSSL_ONLY_TYPES
#include "ipworksssl.h"
#include "ipworksssl.key"

//FirewallTypes
#define FW_NONE                                            0
#define FW_TUNNEL                                          1
#define FW_SOCKS4                                          2
#define FW_SOCKS5                                          3

//CertStoreTypes
#define CST_USER                                           0
#define CST_MACHINE                                        1
#define CST_PFXFILE                                        2
#define CST_PFXBLOB                                        3
#define CST_JKSFILE                                        4
#define CST_JKSBLOB                                        5
#define CST_PEMKEY_FILE                                    6
#define CST_PEMKEY_BLOB                                    7
#define CST_PUBLIC_KEY_FILE                                8
#define CST_PUBLIC_KEY_BLOB                                9
#define CST_SSHPUBLIC_KEY_BLOB                             10
#define CST_P7BFILE                                        11
#define CST_P7BBLOB                                        12
#define CST_SSHPUBLIC_KEY_FILE                             13

//TSSLStartMode
#define SSL_AUTOMATIC                                      0
#define SSL_IMPLICIT                                       1
#define SSL_EXPLICIT                                       2
#define SSL_NONE                                           3


extern "C" void* IPWORKSSSL_CALL IPWorksSSL_SNPPS_Create(PIPWORKSSSL_CALLBACK lpSink, void *lpContext, char *lpOemKey);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_SNPPS_Destroy(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_SNPPS_CheckIndex(void *lpObj, int propid, int arridx);
extern "C" void* IPWORKSSSL_CALL IPWorksSSL_SNPPS_Get(void *lpObj, int propid, int arridx, int *lpcbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_SNPPS_Set(void *lpObj, int propid, int arridx, const void *val, int cbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_SNPPS_Do(void *lpObj, int methid, int cparam, void *param[], int cbparam[]);
extern "C" char* IPWORKSSSL_CALL IPWorksSSL_SNPPS_GetLastError(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_SNPPS_GetLastErrorCode(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_SNPPS_StaticInit(void *hInst);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_SNPPS_StaticDestroy();

#ifdef WIN32
#include <windows.h>
#pragma warning(disable:4311) 
#pragma warning(disable:4312) 
#endif

typedef struct {
  const char* ConnectionEvent;
  int StatusCode;
  const char* Description;
  int reserved;
} SNPPSConnectionStatusEventParams;

typedef struct {
  int ErrorCode;
  const char* Description;
  int reserved;
} SNPPSErrorEventParams;

typedef struct {
  int Direction;
  const char* Message;
  int reserved;
} SNPPSPITrailEventParams;

typedef struct {
  const char* CertEncoded;
  const char* CertSubject;
  const char* CertIssuer;
  const char* Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} SNPPSSSLServerAuthenticationEventParams;

typedef struct {
  const char* Message;
  int reserved;
} SNPPSSSLStatusEventParams;



class SNPPS {
  
  public: //events
  
    virtual int FireConnectionStatus(SNPPSConnectionStatusEventParams *e) {return 0;}
    virtual int FireError(SNPPSErrorEventParams *e) {return 0;}
    virtual int FirePITrail(SNPPSPITrailEventParams *e) {return 0;}
    virtual int FireSSLServerAuthentication(SNPPSSSLServerAuthenticationEventParams *e) {return 0;}
    virtual int FireSSLStatus(SNPPSSSLStatusEventParams *e) {return 0;}


  protected:

    void *m_pObj;
    
    static int IPWORKSSSL_CALL SNPPSEventSink(void *lpObj, int event_id, int cparam, void *param[], int cbparam[]) {
      int ret_code = 0;
      if (event_id > 10000) return ((SNPPS*)lpObj)->SNPPSEventSinkW(event_id - 10000, cparam, param, cbparam);
      switch (event_id) {
         case 1: {
            SNPPSConnectionStatusEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]),  0};
            ret_code = ((SNPPS*)lpObj)->FireConnectionStatus(&e);
            break;
         }
         case 2: {
            SNPPSErrorEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((SNPPS*)lpObj)->FireError(&e);
            break;
         }
         case 3: {
            SNPPSPITrailEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((SNPPS*)lpObj)->FirePITrail(&e);
            break;
         }
         case 4: {
            SNPPSSSLServerAuthenticationEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = ((SNPPS*)lpObj)->FireSSLServerAuthentication(&e);
            param[4] = (void*)IPS64CAST(e.Accept);
            break;
         }
         case 5: {
            SNPPSSSLStatusEventParams e = {(char*)IPS64CAST(param[0]),  0};
            ret_code = ((SNPPS*)lpObj)->FireSSLStatus(&e);
            break;
         }

      }
      return ret_code;
    }

    virtual int SNPPSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {return 0;}

  public:

    SNPPS(char *lpOemKey = (char*)IPWORKSSSL_OEMKEY_32) {
      m_pObj = IPWorksSSL_SNPPS_Create(SNPPSEventSink, (void*)this, (char*)lpOemKey);
    }

    virtual ~SNPPS() {
      IPWorksSSL_SNPPS_Destroy(m_pObj);
    }

  public:

    inline char *GetLastError() {
      return IPWorksSSL_SNPPS_GetLastError(m_pObj);
    }
    
    inline int GetLastErrorCode() {
      return IPWorksSSL_SNPPS_GetLastErrorCode(m_pObj);
    }

    inline char *VERSION() {
      return (char*)IPWorksSSL_SNPPS_Get(m_pObj, 0, 0, 0);
    }

  public: //properties

    inline char* GetCallerId() {
      void* val = IPWorksSSL_SNPPS_Get(m_pObj, 1, 0, 0);
      return (char*)val;
    }

    inline int SetCallerId(const char *lpCallerId) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 1, 0, (void*)lpCallerId, 0);
    }


    inline int SetCommand(const char *lpCommand) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 2, 0, (void*)lpCommand, 0);
    }

    inline int GetConnected() {
      void* val = IPWorksSSL_SNPPS_Get(m_pObj, 3, 0, 0);
      return (int)(long)val;
    }
    inline int SetConnected(int bConnected) {
      void* val = (void*)IPS64CAST(bConnected);
      return IPWorksSSL_SNPPS_Set(m_pObj, 3, 0, val, 0);
    }
    inline int GetFirewallAutoDetect() {
      void* val = IPWorksSSL_SNPPS_Get(m_pObj, 4, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallAutoDetect(int bFirewallAutoDetect) {
      void* val = (void*)IPS64CAST(bFirewallAutoDetect);
      return IPWorksSSL_SNPPS_Set(m_pObj, 4, 0, val, 0);
    }
    inline int GetFirewallType() {
      void* val = IPWorksSSL_SNPPS_Get(m_pObj, 5, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallType(int iFirewallType) {
      void* val = (void*)IPS64CAST(iFirewallType);
      return IPWorksSSL_SNPPS_Set(m_pObj, 5, 0, val, 0);
    }
    inline char* GetFirewallHost() {
      void* val = IPWorksSSL_SNPPS_Get(m_pObj, 6, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallHost(const char *lpFirewallHost) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 6, 0, (void*)lpFirewallHost, 0);
    }

    inline char* GetFirewallPassword() {
      void* val = IPWorksSSL_SNPPS_Get(m_pObj, 7, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallPassword(const char *lpFirewallPassword) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 7, 0, (void*)lpFirewallPassword, 0);
    }

    inline int GetFirewallPort() {
      void* val = IPWorksSSL_SNPPS_Get(m_pObj, 8, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallPort(int lFirewallPort) {
      void* val = (void*)IPS64CAST(lFirewallPort);
      return IPWorksSSL_SNPPS_Set(m_pObj, 8, 0, val, 0);
    }
    inline char* GetFirewallUser() {
      void* val = IPWorksSSL_SNPPS_Get(m_pObj, 9, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallUser(const char *lpFirewallUser) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 9, 0, (void*)lpFirewallUser, 0);
    }

    inline int GetIdle() {
      void* val = IPWorksSSL_SNPPS_Get(m_pObj, 10, 0, 0);
      return (int)(long)val;
    }

    inline char* GetLastReply() {
      void* val = IPWorksSSL_SNPPS_Get(m_pObj, 11, 0, 0);
      return (char*)val;
    }


    inline char* GetLocalHost() {
      void* val = IPWorksSSL_SNPPS_Get(m_pObj, 12, 0, 0);
      return (char*)val;
    }

    inline int SetLocalHost(const char *lpLocalHost) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 12, 0, (void*)lpLocalHost, 0);
    }

    inline char* GetMessage() {
      void* val = IPWorksSSL_SNPPS_Get(m_pObj, 13, 0, 0);
      return (char*)val;
    }

    inline int SetMessage(const char *lpMessage) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 13, 0, (void*)lpMessage, 0);
    }

    inline char* GetPagerId() {
      void* val = IPWorksSSL_SNPPS_Get(m_pObj, 14, 0, 0);
      return (char*)val;
    }

    inline int SetPagerId(const char *lpPagerId) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 14, 0, (void*)lpPagerId, 0);
    }

    inline char* GetServerName() {
      void* val = IPWorksSSL_SNPPS_Get(m_pObj, 15, 0, 0);
      return (char*)val;
    }

    inline int SetServerName(const char *lpServerName) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 15, 0, (void*)lpServerName, 0);
    }

    inline int GetServerPort() {
      void* val = IPWorksSSL_SNPPS_Get(m_pObj, 16, 0, 0);
      return (int)(long)val;
    }
    inline int SetServerPort(int lServerPort) {
      void* val = (void*)IPS64CAST(lServerPort);
      return IPWorksSSL_SNPPS_Set(m_pObj, 16, 0, val, 0);
    }
    inline int GetSSLAcceptServerCertEncoded(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_SNPPS_Get(m_pObj, 17, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }

    inline int SetSSLAcceptServerCertEncoded(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 17, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }

    inline int GetSSLCertEncoded(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_SNPPS_Get(m_pObj, 18, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }

    inline int SetSSLCertEncoded(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 18, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }

    inline int GetSSLCertStore(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_SNPPS_Get(m_pObj, 19, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }

    inline int SetSSLCertStore(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 19, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }

    inline char* GetSSLCertStorePassword() {
      void* val = IPWorksSSL_SNPPS_Get(m_pObj, 20, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertStorePassword(const char *lpSSLCertStorePassword) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 20, 0, (void*)lpSSLCertStorePassword, 0);
    }

    inline int GetSSLCertStoreType() {
      void* val = IPWorksSSL_SNPPS_Get(m_pObj, 21, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLCertStoreType(int iSSLCertStoreType) {
      void* val = (void*)IPS64CAST(iSSLCertStoreType);
      return IPWorksSSL_SNPPS_Set(m_pObj, 21, 0, val, 0);
    }
    inline char* GetSSLCertSubject() {
      void* val = IPWorksSSL_SNPPS_Get(m_pObj, 22, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertSubject(const char *lpSSLCertSubject) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 22, 0, (void*)lpSSLCertSubject, 0);
    }

    inline int GetSSLServerCertEncoded(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_SNPPS_Get(m_pObj, 23, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }


    inline int GetSSLStartMode() {
      void* val = IPWorksSSL_SNPPS_Get(m_pObj, 24, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLStartMode(int iSSLStartMode) {
      void* val = (void*)IPS64CAST(iSSLStartMode);
      return IPWorksSSL_SNPPS_Set(m_pObj, 24, 0, val, 0);
    }
    inline int GetTimeout() {
      void* val = IPWorksSSL_SNPPS_Get(m_pObj, 25, 0, 0);
      return (int)(long)val;
    }
    inline int SetTimeout(int iTimeout) {
      void* val = (void*)IPS64CAST(iTimeout);
      return IPWorksSSL_SNPPS_Set(m_pObj, 25, 0, val, 0);
    }

  public: //methods

    inline char* Config(const char* lpszConfigurationString) {
      void *param[1+1] = {(void*)IPS64CAST(lpszConfigurationString), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_SNPPS_Do(m_pObj, 2, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline int Connect() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SNPPS_Do(m_pObj, 3, 0, param, cbparam);
      
      
    }
    inline int Disconnect() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SNPPS_Do(m_pObj, 4, 0, param, cbparam);
      
      
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SNPPS_Do(m_pObj, 5, 0, param, cbparam);
      
      
    }
    inline int Interrupt() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SNPPS_Do(m_pObj, 6, 0, param, cbparam);
      
      
    }
    inline int Reset() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SNPPS_Do(m_pObj, 7, 0, param, cbparam);
      
      
    }
    inline int Send() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SNPPS_Do(m_pObj, 8, 0, param, cbparam);
      
      
    }

};


#ifdef WIN32 //UNICODE

typedef struct {
  LPWSTR ConnectionEvent;
  int StatusCode;
  LPWSTR Description;
  int reserved;
} SNPPSConnectionStatusEventParamsW;

typedef struct {
  int ErrorCode;
  LPWSTR Description;
  int reserved;
} SNPPSErrorEventParamsW;

typedef struct {
  int Direction;
  LPWSTR Message;
  int reserved;
} SNPPSPITrailEventParamsW;

typedef struct {
  LPWSTR CertEncoded;
  LPWSTR CertSubject;
  LPWSTR CertIssuer;
  LPWSTR Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} SNPPSSSLServerAuthenticationEventParamsW;

typedef struct {
  LPWSTR Message;
  int reserved;
} SNPPSSSLStatusEventParamsW;



class SNPPSW : public SNPPS {

  public: //properties
  
    inline LPWSTR GetCallerId() {
      return (LPWSTR)IPWorksSSL_SNPPS_Get(m_pObj, 10000+1, 0, 0);
    }

    inline int SetCallerId(LPWSTR lpCallerId) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 10000+1, 0, (void*)lpCallerId, 0);
    }



    inline int SetCommand(LPWSTR lpCommand) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 10000+2, 0, (void*)lpCommand, 0);
    }







    inline LPWSTR GetFirewallHost() {
      return (LPWSTR)IPWorksSSL_SNPPS_Get(m_pObj, 10000+6, 0, 0);
    }

    inline int SetFirewallHost(LPWSTR lpFirewallHost) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 10000+6, 0, (void*)lpFirewallHost, 0);
    }

    inline LPWSTR GetFirewallPassword() {
      return (LPWSTR)IPWorksSSL_SNPPS_Get(m_pObj, 10000+7, 0, 0);
    }

    inline int SetFirewallPassword(LPWSTR lpFirewallPassword) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 10000+7, 0, (void*)lpFirewallPassword, 0);
    }



    inline LPWSTR GetFirewallUser() {
      return (LPWSTR)IPWorksSSL_SNPPS_Get(m_pObj, 10000+9, 0, 0);
    }

    inline int SetFirewallUser(LPWSTR lpFirewallUser) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 10000+9, 0, (void*)lpFirewallUser, 0);
    }



    inline LPWSTR GetLastReply() {
      return (LPWSTR)IPWorksSSL_SNPPS_Get(m_pObj, 10000+11, 0, 0);
    }



    inline LPWSTR GetLocalHost() {
      return (LPWSTR)IPWorksSSL_SNPPS_Get(m_pObj, 10000+12, 0, 0);
    }

    inline int SetLocalHost(LPWSTR lpLocalHost) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 10000+12, 0, (void*)lpLocalHost, 0);
    }

    inline LPWSTR GetMessage() {
      return (LPWSTR)IPWorksSSL_SNPPS_Get(m_pObj, 10000+13, 0, 0);
    }

    inline int SetMessage(LPWSTR lpMessage) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 10000+13, 0, (void*)lpMessage, 0);
    }

    inline LPWSTR GetPagerId() {
      return (LPWSTR)IPWorksSSL_SNPPS_Get(m_pObj, 10000+14, 0, 0);
    }

    inline int SetPagerId(LPWSTR lpPagerId) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 10000+14, 0, (void*)lpPagerId, 0);
    }

    inline LPWSTR GetServerName() {
      return (LPWSTR)IPWorksSSL_SNPPS_Get(m_pObj, 10000+15, 0, 0);
    }

    inline int SetServerName(LPWSTR lpServerName) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 10000+15, 0, (void*)lpServerName, 0);
    }



    inline LPWSTR GetSSLAcceptServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_SNPPS_Get(m_pObj, 10000+17, 0, 0);
    }

    inline int SetSSLAcceptServerCertEncoded(LPWSTR lpSSLAcceptServerCertEncoded) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 10000+17, 0, (void*)lpSSLAcceptServerCertEncoded, 0);
    }
    inline int GetSSLAcceptServerCertEncodedB(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_SNPPS_Get(m_pObj, 17, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }
    inline int SetSSLAcceptServerCertEncodedB(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 17, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }
    inline LPWSTR GetSSLCertEncoded() {
      return (LPWSTR)IPWorksSSL_SNPPS_Get(m_pObj, 10000+18, 0, 0);
    }

    inline int SetSSLCertEncoded(LPWSTR lpSSLCertEncoded) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 10000+18, 0, (void*)lpSSLCertEncoded, 0);
    }
    inline int GetSSLCertEncodedB(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_SNPPS_Get(m_pObj, 18, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }
    inline int SetSSLCertEncodedB(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 18, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }
    inline LPWSTR GetSSLCertStore() {
      return (LPWSTR)IPWorksSSL_SNPPS_Get(m_pObj, 10000+19, 0, 0);
    }

    inline int SetSSLCertStore(LPWSTR lpSSLCertStore) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 10000+19, 0, (void*)lpSSLCertStore, 0);
    }
    inline int GetSSLCertStoreB(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_SNPPS_Get(m_pObj, 19, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }
    inline int SetSSLCertStoreB(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 19, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }
    inline LPWSTR GetSSLCertStorePassword() {
      return (LPWSTR)IPWorksSSL_SNPPS_Get(m_pObj, 10000+20, 0, 0);
    }

    inline int SetSSLCertStorePassword(LPWSTR lpSSLCertStorePassword) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 10000+20, 0, (void*)lpSSLCertStorePassword, 0);
    }



    inline LPWSTR GetSSLCertSubject() {
      return (LPWSTR)IPWorksSSL_SNPPS_Get(m_pObj, 10000+22, 0, 0);
    }

    inline int SetSSLCertSubject(LPWSTR lpSSLCertSubject) {
      return IPWorksSSL_SNPPS_Set(m_pObj, 10000+22, 0, (void*)lpSSLCertSubject, 0);
    }

    inline LPWSTR GetSSLServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_SNPPS_Get(m_pObj, 10000+23, 0, 0);
    }


    inline int GetSSLServerCertEncodedB(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_SNPPS_Get(m_pObj, 23, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }







  public: //events
  
    virtual int FireConnectionStatus(SNPPSConnectionStatusEventParamsW *e) {return 0;}
    virtual int FireError(SNPPSErrorEventParamsW *e) {return 0;}
    virtual int FirePITrail(SNPPSPITrailEventParamsW *e) {return 0;}
    virtual int FireSSLServerAuthentication(SNPPSSSLServerAuthenticationEventParamsW *e) {return 0;}
    virtual int FireSSLStatus(SNPPSSSLStatusEventParamsW *e) {return 0;}


  protected:
  
    virtual int SNPPSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {
    	int ret_code = 0;
      switch (event_id) {
         case 1: {
            SNPPSConnectionStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]),  0};
            ret_code = FireConnectionStatus(&e);
            break;
         }
         case 2: {
            SNPPSErrorEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireError(&e);
            break;
         }
         case 3: {
            SNPPSPITrailEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FirePITrail(&e);
            break;
         }
         case 4: {
            SNPPSSSLServerAuthenticationEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = FireSSLServerAuthentication(&e);
            param[4] = (void*)(e.Accept);
            break;
         }
         case 5: {
            SNPPSSSLStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]),  0};
            ret_code = FireSSLStatus(&e);
            break;
         }

      }
      return ret_code;
    }
  
  public: //event overrides

    virtual int FireConnectionStatus(SNPPSConnectionStatusEventParams *e) {return -10000;}
    virtual int FireError(SNPPSErrorEventParams *e) {return -10000;}
    virtual int FirePITrail(SNPPSPITrailEventParams *e) {return -10000;}
    virtual int FireSSLServerAuthentication(SNPPSSSLServerAuthenticationEventParams *e) {return -10000;}
    virtual int FireSSLStatus(SNPPSSSLStatusEventParams *e) {return -10000;}

  public: //methods

    inline LPWSTR Config(LPWSTR lpszConfigurationString) {
      void *param[1+1] = {(void*)lpszConfigurationString, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_SNPPS_Do(m_pObj, 10000+2, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline int Connect() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SNPPS_Do(m_pObj, 10000+3, 0, param, cbparam);
      
    }
    inline int Disconnect() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SNPPS_Do(m_pObj, 10000+4, 0, param, cbparam);
      
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SNPPS_Do(m_pObj, 10000+5, 0, param, cbparam);
      
    }
    inline int Interrupt() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SNPPS_Do(m_pObj, 10000+6, 0, param, cbparam);
      
    }
    inline int Reset() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SNPPS_Do(m_pObj, 10000+7, 0, param, cbparam);
      
    }
    inline int Send() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SNPPS_Do(m_pObj, 10000+8, 0, param, cbparam);
      
    }

};

#endif //WIN32

#endif //_IPWORKSSSL_SNPPS_H_




