/******************************************************************
   IP*Works! SSL V9 C++ Edition
   Copyright (c) 2013 /n software inc. - All rights reserved.
*******************************************************************/

#ifndef _IPWORKSSSL_SOAPS_H_
#define _IPWORKSSSL_SOAPS_H_

#define IPWORKSSSL_ONLY_TYPES
#include "ipworksssl.h"
#include "ipworksssl.key"

//AuthSchemes
#define AUTH_BASIC                                         0
#define AUTH_DIGEST                                        1
#define AUTH_PROPRIETARY                                   2
#define AUTH_NONE                                          3
#define AUTH_NTLM                                          4
#define AUTH_NEGOTIATE                                     5
#define AUTH_OAUTH                                         6

//FirewallTypes
#define FW_NONE                                            0
#define FW_TUNNEL                                          1
#define FW_SOCKS4                                          2
#define FW_SOCKS5                                          3

//TFollowRedirects
#define FR_NEVER                                           0
#define FR_ALWAYS                                          1
#define FR_SAME_SCHEME                                     2

//ProxySSLTypes
#define PS_AUTOMATIC                                       0
#define PS_ALWAYS                                          1
#define PS_NEVER                                           2
#define PS_TUNNEL                                          3

//CertStoreTypes
#define CST_USER                                           0
#define CST_MACHINE                                        1
#define CST_PFXFILE                                        2
#define CST_PFXBLOB                                        3
#define CST_JKSFILE                                        4
#define CST_JKSBLOB                                        5
#define CST_PEMKEY_FILE                                    6
#define CST_PEMKEY_BLOB                                    7
#define CST_PUBLIC_KEY_FILE                                8
#define CST_PUBLIC_KEY_BLOB                                9
#define CST_SSHPUBLIC_KEY_BLOB                             10
#define CST_P7BFILE                                        11
#define CST_P7BBLOB                                        12
#define CST_SSHPUBLIC_KEY_FILE                             13

//TValueFormat
#define VF_TEXT                                            0
#define VF_XML                                             1
#define VF_FULL_XML                                        2


extern "C" void* IPWORKSSSL_CALL IPWorksSSL_SOAPS_Create(PIPWORKSSSL_CALLBACK lpSink, void *lpContext, char *lpOemKey);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_SOAPS_Destroy(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_SOAPS_CheckIndex(void *lpObj, int propid, int arridx);
extern "C" void* IPWORKSSSL_CALL IPWorksSSL_SOAPS_Get(void *lpObj, int propid, int arridx, int *lpcbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_SOAPS_Set(void *lpObj, int propid, int arridx, const void *val, int cbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_SOAPS_Do(void *lpObj, int methid, int cparam, void *param[], int cbparam[]);
extern "C" char* IPWORKSSSL_CALL IPWorksSSL_SOAPS_GetLastError(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_SOAPS_GetLastErrorCode(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_SOAPS_StaticInit(void *hInst);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_SOAPS_StaticDestroy();

#ifdef WIN32
#include <windows.h>
#pragma warning(disable:4311) 
#pragma warning(disable:4312) 
#endif

typedef struct {
  int StatusCode;
  const char* Description;
  int reserved;
} SOAPSConnectedEventParams;

typedef struct {
  const char* ConnectionEvent;
  int StatusCode;
  const char* Description;
  int reserved;
} SOAPSConnectionStatusEventParams;

typedef struct {
  int StatusCode;
  const char* Description;
  int reserved;
} SOAPSDisconnectedEventParams;

typedef struct {
  int Direction;
  int reserved;
} SOAPSEndTransferEventParams;

typedef struct {
  int ErrorCode;
  const char* Description;
  int reserved;
} SOAPSErrorEventParams;

typedef struct {
  const char* Field;
  const char* Value;
  int reserved;
} SOAPSHeaderEventParams;

typedef struct {
  const char* Location;
  int Accept;
  int reserved;
} SOAPSRedirectEventParams;

typedef struct {
  const char* Name;
  const char* Value;
  const char* Expires;
  const char* Domain;
  const char* Path;
  int Secure;
  int reserved;
} SOAPSSetCookieEventParams;

typedef struct {
  const char* CertEncoded;
  const char* CertSubject;
  const char* CertIssuer;
  const char* Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} SOAPSSSLServerAuthenticationEventParams;

typedef struct {
  const char* Message;
  int reserved;
} SOAPSSSLStatusEventParams;

typedef struct {
  int Direction;
  int reserved;
} SOAPSStartTransferEventParams;

typedef struct {
  const char* HTTPVersion;
  int StatusCode;
  const char* Description;
  int reserved;
} SOAPSStatusEventParams;

typedef struct {
  int Direction;
  ns_int64 *pBytesTransferred;
  int PercentDone;
  int reserved;
} SOAPSTransferEventParams;



class SOAPS {
  
  public: //events
  
    virtual int FireConnected(SOAPSConnectedEventParams *e) {return 0;}
    virtual int FireConnectionStatus(SOAPSConnectionStatusEventParams *e) {return 0;}
    virtual int FireDisconnected(SOAPSDisconnectedEventParams *e) {return 0;}
    virtual int FireEndTransfer(SOAPSEndTransferEventParams *e) {return 0;}
    virtual int FireError(SOAPSErrorEventParams *e) {return 0;}
    virtual int FireHeader(SOAPSHeaderEventParams *e) {return 0;}
    virtual int FireRedirect(SOAPSRedirectEventParams *e) {return 0;}
    virtual int FireSetCookie(SOAPSSetCookieEventParams *e) {return 0;}
    virtual int FireSSLServerAuthentication(SOAPSSSLServerAuthenticationEventParams *e) {return 0;}
    virtual int FireSSLStatus(SOAPSSSLStatusEventParams *e) {return 0;}
    virtual int FireStartTransfer(SOAPSStartTransferEventParams *e) {return 0;}
    virtual int FireStatus(SOAPSStatusEventParams *e) {return 0;}
    virtual int FireTransfer(SOAPSTransferEventParams *e) {return 0;}


  protected:

    void *m_pObj;
    
    static int IPWORKSSSL_CALL SOAPSEventSink(void *lpObj, int event_id, int cparam, void *param[], int cbparam[]) {
      int ret_code = 0;
      if (event_id > 10000) return ((SOAPS*)lpObj)->SOAPSEventSinkW(event_id - 10000, cparam, param, cbparam);
      switch (event_id) {
         case 1: {
            SOAPSConnectedEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((SOAPS*)lpObj)->FireConnected(&e);
            break;
         }
         case 2: {
            SOAPSConnectionStatusEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]),  0};
            ret_code = ((SOAPS*)lpObj)->FireConnectionStatus(&e);
            break;
         }
         case 3: {
            SOAPSDisconnectedEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((SOAPS*)lpObj)->FireDisconnected(&e);
            break;
         }
         case 4: {
            SOAPSEndTransferEventParams e = {(int)IPS64CAST(param[0]),  0};
            ret_code = ((SOAPS*)lpObj)->FireEndTransfer(&e);
            break;
         }
         case 5: {
            SOAPSErrorEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((SOAPS*)lpObj)->FireError(&e);
            break;
         }
         case 6: {
            SOAPSHeaderEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((SOAPS*)lpObj)->FireHeader(&e);
            break;
         }
         case 7: {
            SOAPSRedirectEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]),  0};
            ret_code = ((SOAPS*)lpObj)->FireRedirect(&e);
            param[1] = (void*)IPS64CAST(e.Accept);
            break;
         }
         case 8: {
            SOAPSSetCookieEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (char*)IPS64CAST(param[4]), (int)IPS64CAST(param[5]),  0};
            ret_code = ((SOAPS*)lpObj)->FireSetCookie(&e);
            break;
         }
         case 9: {
            SOAPSSSLServerAuthenticationEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = ((SOAPS*)lpObj)->FireSSLServerAuthentication(&e);
            param[4] = (void*)IPS64CAST(e.Accept);
            break;
         }
         case 10: {
            SOAPSSSLStatusEventParams e = {(char*)IPS64CAST(param[0]),  0};
            ret_code = ((SOAPS*)lpObj)->FireSSLStatus(&e);
            break;
         }
         case 11: {
            SOAPSStartTransferEventParams e = {(int)IPS64CAST(param[0]),  0};
            ret_code = ((SOAPS*)lpObj)->FireStartTransfer(&e);
            break;
         }
         case 12: {
            SOAPSStatusEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]),  0};
            ret_code = ((SOAPS*)lpObj)->FireStatus(&e);
            break;
         }
         case 13: {
            SOAPSTransferEventParams e = {(int)IPS64CAST(param[0]), (ns_int64*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]),  0};
            ret_code = ((SOAPS*)lpObj)->FireTransfer(&e);
            break;
         }

      }
      return ret_code;
    }

    virtual int SOAPSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {return 0;}

  public:

    SOAPS(char *lpOemKey = (char*)IPWORKSSSL_OEMKEY_55) {
      m_pObj = IPWorksSSL_SOAPS_Create(SOAPSEventSink, (void*)this, (char*)lpOemKey);
    }

    virtual ~SOAPS() {
      IPWorksSSL_SOAPS_Destroy(m_pObj);
    }

  public:

    inline char *GetLastError() {
      return IPWorksSSL_SOAPS_GetLastError(m_pObj);
    }
    
    inline int GetLastErrorCode() {
      return IPWorksSSL_SOAPS_GetLastErrorCode(m_pObj);
    }

    inline char *VERSION() {
      return (char*)IPWorksSSL_SOAPS_Get(m_pObj, 0, 0, 0);
    }

  public: //properties

    inline char* GetAccept() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 1, 0, 0);
      return (char*)val;
    }

    inline int SetAccept(const char *lpAccept) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 1, 0, (void*)lpAccept, 0);
    }

    inline char* GetActionURI() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 2, 0, 0);
      return (char*)val;
    }

    inline int SetActionURI(const char *lpActionURI) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 2, 0, (void*)lpActionURI, 0);
    }

    inline char* GetAuthorization() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 3, 0, 0);
      return (char*)val;
    }

    inline int SetAuthorization(const char *lpAuthorization) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 3, 0, (void*)lpAuthorization, 0);
    }

    inline int GetAuthScheme() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 4, 0, 0);
      return (int)(long)val;
    }
    inline int SetAuthScheme(int iAuthScheme) {
      void* val = (void*)IPS64CAST(iAuthScheme);
      return IPWorksSSL_SOAPS_Set(m_pObj, 4, 0, val, 0);
    }
    inline int GetConnected() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 5, 0, 0);
      return (int)(long)val;
    }
    inline int SetConnected(int bConnected) {
      void* val = (void*)IPS64CAST(bConnected);
      return IPWorksSSL_SOAPS_Set(m_pObj, 5, 0, val, 0);
    }
    inline char* GetContentType() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 6, 0, 0);
      return (char*)val;
    }

    inline int SetContentType(const char *lpContentType) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 6, 0, (void*)lpContentType, 0);
    }

    inline int GetCookieCount() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 7, 0, 0);
      return (int)(long)val;
    }
    inline int SetCookieCount(int iCookieCount) {
      void* val = (void*)IPS64CAST(iCookieCount);
      return IPWorksSSL_SOAPS_Set(m_pObj, 7, 0, val, 0);
    }
    inline char* GetCookieDomain(int iCookieIndex) {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 8, iCookieIndex, 0);
      return (char*)val;
    }


    inline char* GetCookieExpiration(int iCookieIndex) {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 9, iCookieIndex, 0);
      return (char*)val;
    }


    inline char* GetCookieName(int iCookieIndex) {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 10, iCookieIndex, 0);
      return (char*)val;
    }

    inline int SetCookieName(int iCookieIndex, const char *lpCookieName) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10, iCookieIndex, (void*)lpCookieName, 0);
    }

    inline char* GetCookiePath(int iCookieIndex) {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 11, iCookieIndex, 0);
      return (char*)val;
    }


    inline int GetCookieSecure(int iCookieIndex) {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 12, iCookieIndex, 0);
      return (int)(long)val;
    }

    inline char* GetCookieValue(int iCookieIndex) {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 13, iCookieIndex, 0);
      return (char*)val;
    }

    inline int SetCookieValue(int iCookieIndex, const char *lpCookieValue) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 13, iCookieIndex, (void*)lpCookieValue, 0);
    }

    inline char* GetFaultActor() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 14, 0, 0);
      return (char*)val;
    }


    inline char* GetFaultCode() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 15, 0, 0);
      return (char*)val;
    }


    inline char* GetFaultString() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 16, 0, 0);
      return (char*)val;
    }


    inline int GetFirewallAutoDetect() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 17, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallAutoDetect(int bFirewallAutoDetect) {
      void* val = (void*)IPS64CAST(bFirewallAutoDetect);
      return IPWorksSSL_SOAPS_Set(m_pObj, 17, 0, val, 0);
    }
    inline int GetFirewallType() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 18, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallType(int iFirewallType) {
      void* val = (void*)IPS64CAST(iFirewallType);
      return IPWorksSSL_SOAPS_Set(m_pObj, 18, 0, val, 0);
    }
    inline char* GetFirewallHost() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 19, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallHost(const char *lpFirewallHost) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 19, 0, (void*)lpFirewallHost, 0);
    }

    inline char* GetFirewallPassword() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 20, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallPassword(const char *lpFirewallPassword) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 20, 0, (void*)lpFirewallPassword, 0);
    }

    inline int GetFirewallPort() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 21, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallPort(int lFirewallPort) {
      void* val = (void*)IPS64CAST(lFirewallPort);
      return IPWorksSSL_SOAPS_Set(m_pObj, 21, 0, val, 0);
    }
    inline char* GetFirewallUser() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 22, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallUser(const char *lpFirewallUser) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 22, 0, (void*)lpFirewallUser, 0);
    }

    inline int GetFollowRedirects() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 23, 0, 0);
      return (int)(long)val;
    }
    inline int SetFollowRedirects(int iFollowRedirects) {
      void* val = (void*)IPS64CAST(iFollowRedirects);
      return IPWorksSSL_SOAPS_Set(m_pObj, 23, 0, val, 0);
    }
    inline int GetIdle() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 24, 0, 0);
      return (int)(long)val;
    }

    inline char* GetIfModifiedSince() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 25, 0, 0);
      return (char*)val;
    }

    inline int SetIfModifiedSince(const char *lpIfModifiedSince) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 25, 0, (void*)lpIfModifiedSince, 0);
    }

    inline char* GetLocalFile() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 26, 0, 0);
      return (char*)val;
    }

    inline int SetLocalFile(const char *lpLocalFile) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 26, 0, (void*)lpLocalFile, 0);
    }

    inline char* GetLocalHost() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 27, 0, 0);
      return (char*)val;
    }

    inline int SetLocalHost(const char *lpLocalHost) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 27, 0, (void*)lpLocalHost, 0);
    }

    inline char* GetMethod() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 28, 0, 0);
      return (char*)val;
    }

    inline int SetMethod(const char *lpMethod) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 28, 0, (void*)lpMethod, 0);
    }

    inline char* GetMethodURI() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 29, 0, 0);
      return (char*)val;
    }

    inline int SetMethodURI(const char *lpMethodURI) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 29, 0, (void*)lpMethodURI, 0);
    }

    inline int GetNamespaceCount() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 30, 0, 0);
      return (int)(long)val;
    }
    inline int SetNamespaceCount(int iNamespaceCount) {
      void* val = (void*)IPS64CAST(iNamespaceCount);
      return IPWorksSSL_SOAPS_Set(m_pObj, 30, 0, val, 0);
    }
    inline char* GetNamespacePrefix(int iNamespaceIndex) {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 31, iNamespaceIndex, 0);
      return (char*)val;
    }

    inline int SetNamespacePrefix(int iNamespaceIndex, const char *lpNamespacePrefix) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 31, iNamespaceIndex, (void*)lpNamespacePrefix, 0);
    }

    inline char* GetNamespaceURI(int iNamespaceIndex) {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 32, iNamespaceIndex, 0);
      return (char*)val;
    }

    inline int SetNamespaceURI(int iNamespaceIndex, const char *lpNamespaceURI) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 32, iNamespaceIndex, (void*)lpNamespaceURI, 0);
    }

    inline char* GetOtherHeaders() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 33, 0, 0);
      return (char*)val;
    }

    inline int SetOtherHeaders(const char *lpOtherHeaders) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 33, 0, (void*)lpOtherHeaders, 0);
    }

    inline int GetParamCount() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 34, 0, 0);
      return (int)(long)val;
    }
    inline int SetParamCount(int iParamCount) {
      void* val = (void*)IPS64CAST(iParamCount);
      return IPWorksSSL_SOAPS_Set(m_pObj, 34, 0, val, 0);
    }
    inline char* GetParamAttrs(int iParamIndex) {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 35, iParamIndex, 0);
      return (char*)val;
    }

    inline int SetParamAttrs(int iParamIndex, const char *lpParamAttrs) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 35, iParamIndex, (void*)lpParamAttrs, 0);
    }

    inline char* GetParamName(int iParamIndex) {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 36, iParamIndex, 0);
      return (char*)val;
    }

    inline int SetParamName(int iParamIndex, const char *lpParamName) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 36, iParamIndex, (void*)lpParamName, 0);
    }

    inline char* GetParamValue(int iParamIndex) {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 37, iParamIndex, 0);
      return (char*)val;
    }

    inline int SetParamValue(int iParamIndex, const char *lpParamValue) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 37, iParamIndex, (void*)lpParamValue, 0);
    }

    inline int GetParsedHeaderCount() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 38, 0, 0);
      return (int)(long)val;
    }

    inline char* GetParsedHeaderField(int iParsedHeaderIndex) {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 39, iParsedHeaderIndex, 0);
      return (char*)val;
    }


    inline char* GetParsedHeaderValue(int iParsedHeaderIndex) {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 40, iParsedHeaderIndex, 0);
      return (char*)val;
    }


    inline char* GetPassword() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 41, 0, 0);
      return (char*)val;
    }

    inline int SetPassword(const char *lpPassword) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 41, 0, (void*)lpPassword, 0);
    }

    inline char* GetPragma() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 42, 0, 0);
      return (char*)val;
    }

    inline int SetPragma(const char *lpPragma) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 42, 0, (void*)lpPragma, 0);
    }

    inline int GetProxyAuthScheme() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 43, 0, 0);
      return (int)(long)val;
    }
    inline int SetProxyAuthScheme(int iProxyAuthScheme) {
      void* val = (void*)IPS64CAST(iProxyAuthScheme);
      return IPWorksSSL_SOAPS_Set(m_pObj, 43, 0, val, 0);
    }
    inline int GetProxyAutoDetect() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 44, 0, 0);
      return (int)(long)val;
    }
    inline int SetProxyAutoDetect(int bProxyAutoDetect) {
      void* val = (void*)IPS64CAST(bProxyAutoDetect);
      return IPWorksSSL_SOAPS_Set(m_pObj, 44, 0, val, 0);
    }
    inline char* GetProxyPassword() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 45, 0, 0);
      return (char*)val;
    }

    inline int SetProxyPassword(const char *lpProxyPassword) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 45, 0, (void*)lpProxyPassword, 0);
    }

    inline int GetProxyPort() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 46, 0, 0);
      return (int)(long)val;
    }
    inline int SetProxyPort(int lProxyPort) {
      void* val = (void*)IPS64CAST(lProxyPort);
      return IPWorksSSL_SOAPS_Set(m_pObj, 46, 0, val, 0);
    }
    inline char* GetProxyServer() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 47, 0, 0);
      return (char*)val;
    }

    inline int SetProxyServer(const char *lpProxyServer) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 47, 0, (void*)lpProxyServer, 0);
    }

    inline int GetProxySSL() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 48, 0, 0);
      return (int)(long)val;
    }
    inline int SetProxySSL(int iProxySSL) {
      void* val = (void*)IPS64CAST(iProxySSL);
      return IPWorksSSL_SOAPS_Set(m_pObj, 48, 0, val, 0);
    }
    inline char* GetProxyUser() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 49, 0, 0);
      return (char*)val;
    }

    inline int SetProxyUser(const char *lpProxyUser) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 49, 0, (void*)lpProxyUser, 0);
    }

    inline char* GetReferer() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 50, 0, 0);
      return (char*)val;
    }

    inline int SetReferer(const char *lpReferer) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 50, 0, (void*)lpReferer, 0);
    }

    inline char* GetReturnValue() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 51, 0, 0);
      return (char*)val;
    }


    inline char* GetSOAPEncoding() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 52, 0, 0);
      return (char*)val;
    }

    inline int SetSOAPEncoding(const char *lpSOAPEncoding) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 52, 0, (void*)lpSOAPEncoding, 0);
    }

    inline char* GetSOAPHeader() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 53, 0, 0);
      return (char*)val;
    }

    inline int SetSOAPHeader(const char *lpSOAPHeader) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 53, 0, (void*)lpSOAPHeader, 0);
    }

    inline char* GetSOAPPacket() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 54, 0, 0);
      return (char*)val;
    }

    inline int SetSOAPPacket(const char *lpSOAPPacket) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 54, 0, (void*)lpSOAPPacket, 0);
    }

    inline int GetSSLAcceptServerCertEncoded(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_SOAPS_Get(m_pObj, 55, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }

    inline int SetSSLAcceptServerCertEncoded(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 55, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }

    inline int GetSSLCertEncoded(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_SOAPS_Get(m_pObj, 56, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }

    inline int SetSSLCertEncoded(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 56, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }

    inline int GetSSLCertStore(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_SOAPS_Get(m_pObj, 57, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }

    inline int SetSSLCertStore(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 57, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }

    inline char* GetSSLCertStorePassword() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 58, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertStorePassword(const char *lpSSLCertStorePassword) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 58, 0, (void*)lpSSLCertStorePassword, 0);
    }

    inline int GetSSLCertStoreType() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 59, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLCertStoreType(int iSSLCertStoreType) {
      void* val = (void*)IPS64CAST(iSSLCertStoreType);
      return IPWorksSSL_SOAPS_Set(m_pObj, 59, 0, val, 0);
    }
    inline char* GetSSLCertSubject() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 60, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertSubject(const char *lpSSLCertSubject) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 60, 0, (void*)lpSSLCertSubject, 0);
    }

    inline int GetSSLServerCertEncoded(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_SOAPS_Get(m_pObj, 61, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }


    inline char* GetStatusLine() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 62, 0, 0);
      return (char*)val;
    }


    inline int GetTimeout() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 63, 0, 0);
      return (int)(long)val;
    }
    inline int SetTimeout(int iTimeout) {
      void* val = (void*)IPS64CAST(iTimeout);
      return IPWorksSSL_SOAPS_Set(m_pObj, 63, 0, val, 0);
    }
    inline int GetTransferredData(char *&lpTransferredData, int &lenTransferredData) {
      lpTransferredData = (char*)IPWorksSSL_SOAPS_Get(m_pObj, 64, 0, &lenTransferredData);
      return lpTransferredData ? 0 : lenTransferredData;
    }


    inline ns_int64 GetTransferredDataLimit() {
      ns_int64 *pval = (ns_int64*)IPWorksSSL_SOAPS_Get(m_pObj, 65, 0, 0);
      return *pval;
    }

    inline int SetTransferredDataLimit(ns_int64 lTransferredDataLimit) {
      void* val = (void*)(&lTransferredDataLimit);
      return IPWorksSSL_SOAPS_Set(m_pObj, 65, 0, val, 0);
    }

    inline char* GetTransferredHeaders() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 66, 0, 0);
      return (char*)val;
    }


    inline char* GetURL() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 67, 0, 0);
      return (char*)val;
    }

    inline int SetURL(const char *lpURL) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 67, 0, (void*)lpURL, 0);
    }

    inline char* GetUser() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 68, 0, 0);
      return (char*)val;
    }

    inline int SetUser(const char *lpUser) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 68, 0, (void*)lpUser, 0);
    }

    inline int GetValueFormat() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 69, 0, 0);
      return (int)(long)val;
    }
    inline int SetValueFormat(int iValueFormat) {
      void* val = (void*)IPS64CAST(iValueFormat);
      return IPWorksSSL_SOAPS_Set(m_pObj, 69, 0, val, 0);
    }
    inline int GetAttrCount() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 70, 0, 0);
      return (int)(long)val;
    }

    inline char* GetAttrName(int iAttrIndex) {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 71, iAttrIndex, 0);
      return (char*)val;
    }


    inline char* GetAttrNamespace(int iAttrIndex) {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 72, iAttrIndex, 0);
      return (char*)val;
    }


    inline char* GetAttrPrefix(int iAttrIndex) {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 73, iAttrIndex, 0);
      return (char*)val;
    }


    inline char* GetAttrValue(int iAttrIndex) {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 74, iAttrIndex, 0);
      return (char*)val;
    }


    inline int GetXChildren() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 75, 0, 0);
      return (int)(long)val;
    }

    inline char* GetXElement() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 76, 0, 0);
      return (char*)val;
    }


    inline char* GetXNamespace() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 77, 0, 0);
      return (char*)val;
    }


    inline char* GetXParent() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 78, 0, 0);
      return (char*)val;
    }


    inline char* GetXPath() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 79, 0, 0);
      return (char*)val;
    }

    inline int SetXPath(const char *lpXPath) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 79, 0, (void*)lpXPath, 0);
    }

    inline char* GetXPrefix() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 80, 0, 0);
      return (char*)val;
    }


    inline char* GetXText() {
      void* val = IPWorksSSL_SOAPS_Get(m_pObj, 81, 0, 0);
      return (char*)val;
    }



  public: //methods

    inline int AddCookie(const char* lpszCookieName, const char* lpszCookieValue) {
      void *param[2+1] = {(void*)IPS64CAST(lpszCookieName), (void*)IPS64CAST(lpszCookieValue), 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_SOAPS_Do(m_pObj, 2, 2, param, cbparam);
      
      
    }
    inline int AddNamespace(const char* lpszPrefix, const char* lpszNamespaceURI) {
      void *param[2+1] = {(void*)IPS64CAST(lpszPrefix), (void*)IPS64CAST(lpszNamespaceURI), 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_SOAPS_Do(m_pObj, 3, 2, param, cbparam);
      
      
    }
    inline int AddParam(const char* lpszParamName, const char* lpszParamValue) {
      void *param[2+1] = {(void*)IPS64CAST(lpszParamName), (void*)IPS64CAST(lpszParamValue), 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_SOAPS_Do(m_pObj, 4, 2, param, cbparam);
      
      
    }
    inline char* Attr(const char* lpszAttrName) {
      void *param[1+1] = {(void*)IPS64CAST(lpszAttrName), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_SOAPS_Do(m_pObj, 5, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline int BuildPacket() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SOAPS_Do(m_pObj, 6, 0, param, cbparam);
      
      
    }
    inline char* Config(const char* lpszConfigurationString) {
      void *param[1+1] = {(void*)IPS64CAST(lpszConfigurationString), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_SOAPS_Do(m_pObj, 7, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SOAPS_Do(m_pObj, 8, 0, param, cbparam);
      
      
    }
    inline int EvalPacket() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SOAPS_Do(m_pObj, 9, 0, param, cbparam);
      
      
    }
    inline int HasXPath(const char* lpszxpath) {
      void *param[1+1] = {(void*)IPS64CAST(lpszxpath), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_SOAPS_Do(m_pObj, 10, 1, param, cbparam);
      
      return (int)IPS64CAST(param[1]);
    }
    inline int Interrupt() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SOAPS_Do(m_pObj, 11, 0, param, cbparam);
      
      
    }
    inline int Reset() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SOAPS_Do(m_pObj, 12, 0, param, cbparam);
      
      
    }
    inline int SendPacket() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SOAPS_Do(m_pObj, 13, 0, param, cbparam);
      
      
    }
    inline int SendRequest() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SOAPS_Do(m_pObj, 14, 0, param, cbparam);
      
      
    }
    inline char* Value(const char* lpszParamName) {
      void *param[1+1] = {(void*)IPS64CAST(lpszParamName), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_SOAPS_Do(m_pObj, 15, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }

};


#ifdef WIN32 //UNICODE

typedef struct {
  int StatusCode;
  LPWSTR Description;
  int reserved;
} SOAPSConnectedEventParamsW;

typedef struct {
  LPWSTR ConnectionEvent;
  int StatusCode;
  LPWSTR Description;
  int reserved;
} SOAPSConnectionStatusEventParamsW;

typedef struct {
  int StatusCode;
  LPWSTR Description;
  int reserved;
} SOAPSDisconnectedEventParamsW;

typedef struct {
  int Direction;
  int reserved;
} SOAPSEndTransferEventParamsW;

typedef struct {
  int ErrorCode;
  LPWSTR Description;
  int reserved;
} SOAPSErrorEventParamsW;

typedef struct {
  LPWSTR Field;
  LPWSTR Value;
  int reserved;
} SOAPSHeaderEventParamsW;

typedef struct {
  LPWSTR Location;
  int Accept;
  int reserved;
} SOAPSRedirectEventParamsW;

typedef struct {
  LPWSTR Name;
  LPWSTR Value;
  LPWSTR Expires;
  LPWSTR Domain;
  LPWSTR Path;
  int Secure;
  int reserved;
} SOAPSSetCookieEventParamsW;

typedef struct {
  LPWSTR CertEncoded;
  LPWSTR CertSubject;
  LPWSTR CertIssuer;
  LPWSTR Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} SOAPSSSLServerAuthenticationEventParamsW;

typedef struct {
  LPWSTR Message;
  int reserved;
} SOAPSSSLStatusEventParamsW;

typedef struct {
  int Direction;
  int reserved;
} SOAPSStartTransferEventParamsW;

typedef struct {
  LPWSTR HTTPVersion;
  int StatusCode;
  LPWSTR Description;
  int reserved;
} SOAPSStatusEventParamsW;

typedef struct {
  int Direction;
  ns_int64 *pBytesTransferred;
  int PercentDone;
  int reserved;
} SOAPSTransferEventParamsW;



class SOAPSW : public SOAPS {

  public: //properties
  
    inline LPWSTR GetAccept() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+1, 0, 0);
    }

    inline int SetAccept(LPWSTR lpAccept) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+1, 0, (void*)lpAccept, 0);
    }

    inline LPWSTR GetActionURI() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+2, 0, 0);
    }

    inline int SetActionURI(LPWSTR lpActionURI) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+2, 0, (void*)lpActionURI, 0);
    }

    inline LPWSTR GetAuthorization() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+3, 0, 0);
    }

    inline int SetAuthorization(LPWSTR lpAuthorization) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+3, 0, (void*)lpAuthorization, 0);
    }





    inline LPWSTR GetContentType() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+6, 0, 0);
    }

    inline int SetContentType(LPWSTR lpContentType) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+6, 0, (void*)lpContentType, 0);
    }



    inline LPWSTR GetCookieDomain(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+8, iCookieIndex, 0);
    }



    inline LPWSTR GetCookieExpiration(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+9, iCookieIndex, 0);
    }



    inline LPWSTR GetCookieName(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+10, iCookieIndex, 0);
    }

    inline int SetCookieName(int iCookieIndex, LPWSTR lpCookieName) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+10, iCookieIndex, (void*)lpCookieName, 0);
    }

    inline LPWSTR GetCookiePath(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+11, iCookieIndex, 0);
    }





    inline LPWSTR GetCookieValue(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+13, iCookieIndex, 0);
    }

    inline int SetCookieValue(int iCookieIndex, LPWSTR lpCookieValue) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+13, iCookieIndex, (void*)lpCookieValue, 0);
    }

    inline LPWSTR GetFaultActor() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+14, 0, 0);
    }



    inline LPWSTR GetFaultCode() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+15, 0, 0);
    }



    inline LPWSTR GetFaultString() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+16, 0, 0);
    }







    inline LPWSTR GetFirewallHost() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+19, 0, 0);
    }

    inline int SetFirewallHost(LPWSTR lpFirewallHost) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+19, 0, (void*)lpFirewallHost, 0);
    }

    inline LPWSTR GetFirewallPassword() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+20, 0, 0);
    }

    inline int SetFirewallPassword(LPWSTR lpFirewallPassword) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+20, 0, (void*)lpFirewallPassword, 0);
    }



    inline LPWSTR GetFirewallUser() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+22, 0, 0);
    }

    inline int SetFirewallUser(LPWSTR lpFirewallUser) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+22, 0, (void*)lpFirewallUser, 0);
    }





    inline LPWSTR GetIfModifiedSince() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+25, 0, 0);
    }

    inline int SetIfModifiedSince(LPWSTR lpIfModifiedSince) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+25, 0, (void*)lpIfModifiedSince, 0);
    }

    inline LPWSTR GetLocalFile() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+10000+26, 0, 0);
    }

    inline int SetLocalFile(LPWSTR lpLocalFile) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+10000+26, 0, (void*)lpLocalFile, 0);
    }

    inline LPWSTR GetLocalHost() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+27, 0, 0);
    }

    inline int SetLocalHost(LPWSTR lpLocalHost) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+27, 0, (void*)lpLocalHost, 0);
    }

    inline LPWSTR GetMethod() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+28, 0, 0);
    }

    inline int SetMethod(LPWSTR lpMethod) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+28, 0, (void*)lpMethod, 0);
    }

    inline LPWSTR GetMethodURI() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+29, 0, 0);
    }

    inline int SetMethodURI(LPWSTR lpMethodURI) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+29, 0, (void*)lpMethodURI, 0);
    }



    inline LPWSTR GetNamespacePrefix(int iNamespaceIndex) {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+31, iNamespaceIndex, 0);
    }

    inline int SetNamespacePrefix(int iNamespaceIndex, LPWSTR lpNamespacePrefix) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+31, iNamespaceIndex, (void*)lpNamespacePrefix, 0);
    }

    inline LPWSTR GetNamespaceURI(int iNamespaceIndex) {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+32, iNamespaceIndex, 0);
    }

    inline int SetNamespaceURI(int iNamespaceIndex, LPWSTR lpNamespaceURI) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+32, iNamespaceIndex, (void*)lpNamespaceURI, 0);
    }

    inline LPWSTR GetOtherHeaders() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+33, 0, 0);
    }

    inline int SetOtherHeaders(LPWSTR lpOtherHeaders) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+33, 0, (void*)lpOtherHeaders, 0);
    }



    inline LPWSTR GetParamAttrs(int iParamIndex) {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+35, iParamIndex, 0);
    }

    inline int SetParamAttrs(int iParamIndex, LPWSTR lpParamAttrs) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+35, iParamIndex, (void*)lpParamAttrs, 0);
    }

    inline LPWSTR GetParamName(int iParamIndex) {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+36, iParamIndex, 0);
    }

    inline int SetParamName(int iParamIndex, LPWSTR lpParamName) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+36, iParamIndex, (void*)lpParamName, 0);
    }

    inline LPWSTR GetParamValue(int iParamIndex) {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+37, iParamIndex, 0);
    }

    inline int SetParamValue(int iParamIndex, LPWSTR lpParamValue) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+37, iParamIndex, (void*)lpParamValue, 0);
    }



    inline LPWSTR GetParsedHeaderField(int iParsedHeaderIndex) {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+39, iParsedHeaderIndex, 0);
    }



    inline LPWSTR GetParsedHeaderValue(int iParsedHeaderIndex) {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+40, iParsedHeaderIndex, 0);
    }



    inline LPWSTR GetPassword() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+41, 0, 0);
    }

    inline int SetPassword(LPWSTR lpPassword) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+41, 0, (void*)lpPassword, 0);
    }

    inline LPWSTR GetPragma() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+42, 0, 0);
    }

    inline int SetPragma(LPWSTR lpPragma) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+42, 0, (void*)lpPragma, 0);
    }





    inline LPWSTR GetProxyPassword() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+45, 0, 0);
    }

    inline int SetProxyPassword(LPWSTR lpProxyPassword) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+45, 0, (void*)lpProxyPassword, 0);
    }



    inline LPWSTR GetProxyServer() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+47, 0, 0);
    }

    inline int SetProxyServer(LPWSTR lpProxyServer) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+47, 0, (void*)lpProxyServer, 0);
    }



    inline LPWSTR GetProxyUser() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+49, 0, 0);
    }

    inline int SetProxyUser(LPWSTR lpProxyUser) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+49, 0, (void*)lpProxyUser, 0);
    }

    inline LPWSTR GetReferer() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+50, 0, 0);
    }

    inline int SetReferer(LPWSTR lpReferer) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+50, 0, (void*)lpReferer, 0);
    }

    inline LPWSTR GetReturnValue() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+51, 0, 0);
    }



    inline LPWSTR GetSOAPEncoding() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+52, 0, 0);
    }

    inline int SetSOAPEncoding(LPWSTR lpSOAPEncoding) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+52, 0, (void*)lpSOAPEncoding, 0);
    }

    inline LPWSTR GetSOAPHeader() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+53, 0, 0);
    }

    inline int SetSOAPHeader(LPWSTR lpSOAPHeader) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+53, 0, (void*)lpSOAPHeader, 0);
    }

    inline LPWSTR GetSOAPPacket() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+54, 0, 0);
    }

    inline int SetSOAPPacket(LPWSTR lpSOAPPacket) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+54, 0, (void*)lpSOAPPacket, 0);
    }

    inline LPWSTR GetSSLAcceptServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+55, 0, 0);
    }

    inline int SetSSLAcceptServerCertEncoded(LPWSTR lpSSLAcceptServerCertEncoded) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+55, 0, (void*)lpSSLAcceptServerCertEncoded, 0);
    }
    inline int GetSSLAcceptServerCertEncodedB(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_SOAPS_Get(m_pObj, 55, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }
    inline int SetSSLAcceptServerCertEncodedB(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 55, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }
    inline LPWSTR GetSSLCertEncoded() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+56, 0, 0);
    }

    inline int SetSSLCertEncoded(LPWSTR lpSSLCertEncoded) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+56, 0, (void*)lpSSLCertEncoded, 0);
    }
    inline int GetSSLCertEncodedB(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_SOAPS_Get(m_pObj, 56, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }
    inline int SetSSLCertEncodedB(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 56, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }
    inline LPWSTR GetSSLCertStore() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+57, 0, 0);
    }

    inline int SetSSLCertStore(LPWSTR lpSSLCertStore) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+57, 0, (void*)lpSSLCertStore, 0);
    }
    inline int GetSSLCertStoreB(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_SOAPS_Get(m_pObj, 57, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }
    inline int SetSSLCertStoreB(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 57, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }
    inline LPWSTR GetSSLCertStorePassword() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+58, 0, 0);
    }

    inline int SetSSLCertStorePassword(LPWSTR lpSSLCertStorePassword) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+58, 0, (void*)lpSSLCertStorePassword, 0);
    }



    inline LPWSTR GetSSLCertSubject() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+60, 0, 0);
    }

    inline int SetSSLCertSubject(LPWSTR lpSSLCertSubject) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+60, 0, (void*)lpSSLCertSubject, 0);
    }

    inline LPWSTR GetSSLServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+61, 0, 0);
    }


    inline int GetSSLServerCertEncodedB(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_SOAPS_Get(m_pObj, 61, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }

    inline LPWSTR GetStatusLine() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+62, 0, 0);
    }





    inline LPWSTR GetTransferredData() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+64, 0, 0);
    }


    inline int GetTransferredDataB(char *&lpTransferredData, int &lenTransferredData) {
      lpTransferredData = (char*)IPWorksSSL_SOAPS_Get(m_pObj, 64, 0, &lenTransferredData);
      return lpTransferredData ? 0 : lenTransferredData;
    }



    inline LPWSTR GetTransferredHeaders() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+66, 0, 0);
    }



    inline LPWSTR GetURL() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+67, 0, 0);
    }

    inline int SetURL(LPWSTR lpURL) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+67, 0, (void*)lpURL, 0);
    }

    inline LPWSTR GetUser() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+68, 0, 0);
    }

    inline int SetUser(LPWSTR lpUser) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+68, 0, (void*)lpUser, 0);
    }





    inline LPWSTR GetAttrName(int iAttrIndex) {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+71, iAttrIndex, 0);
    }



    inline LPWSTR GetAttrNamespace(int iAttrIndex) {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+72, iAttrIndex, 0);
    }



    inline LPWSTR GetAttrPrefix(int iAttrIndex) {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+73, iAttrIndex, 0);
    }



    inline LPWSTR GetAttrValue(int iAttrIndex) {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+74, iAttrIndex, 0);
    }





    inline LPWSTR GetXElement() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+76, 0, 0);
    }



    inline LPWSTR GetXNamespace() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+77, 0, 0);
    }



    inline LPWSTR GetXParent() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+78, 0, 0);
    }



    inline LPWSTR GetXPath() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+79, 0, 0);
    }

    inline int SetXPath(LPWSTR lpXPath) {
      return IPWorksSSL_SOAPS_Set(m_pObj, 10000+79, 0, (void*)lpXPath, 0);
    }

    inline LPWSTR GetXPrefix() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+80, 0, 0);
    }



    inline LPWSTR GetXText() {
      return (LPWSTR)IPWorksSSL_SOAPS_Get(m_pObj, 10000+81, 0, 0);
    }





  public: //events
  
    virtual int FireConnected(SOAPSConnectedEventParamsW *e) {return 0;}
    virtual int FireConnectionStatus(SOAPSConnectionStatusEventParamsW *e) {return 0;}
    virtual int FireDisconnected(SOAPSDisconnectedEventParamsW *e) {return 0;}
    virtual int FireEndTransfer(SOAPSEndTransferEventParamsW *e) {return 0;}
    virtual int FireError(SOAPSErrorEventParamsW *e) {return 0;}
    virtual int FireHeader(SOAPSHeaderEventParamsW *e) {return 0;}
    virtual int FireRedirect(SOAPSRedirectEventParamsW *e) {return 0;}
    virtual int FireSetCookie(SOAPSSetCookieEventParamsW *e) {return 0;}
    virtual int FireSSLServerAuthentication(SOAPSSSLServerAuthenticationEventParamsW *e) {return 0;}
    virtual int FireSSLStatus(SOAPSSSLStatusEventParamsW *e) {return 0;}
    virtual int FireStartTransfer(SOAPSStartTransferEventParamsW *e) {return 0;}
    virtual int FireStatus(SOAPSStatusEventParamsW *e) {return 0;}
    virtual int FireTransfer(SOAPSTransferEventParamsW *e) {return 0;}


  protected:
  
    virtual int SOAPSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {
    	int ret_code = 0;
      switch (event_id) {
         case 1: {
            SOAPSConnectedEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireConnected(&e);
            break;
         }
         case 2: {
            SOAPSConnectionStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]),  0};
            ret_code = FireConnectionStatus(&e);
            break;
         }
         case 3: {
            SOAPSDisconnectedEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireDisconnected(&e);
            break;
         }
         case 4: {
            SOAPSEndTransferEventParamsW e = {(int)IPS64CAST(param[0]),  0};
            ret_code = FireEndTransfer(&e);
            break;
         }
         case 5: {
            SOAPSErrorEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireError(&e);
            break;
         }
         case 6: {
            SOAPSHeaderEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireHeader(&e);
            break;
         }
         case 7: {
            SOAPSRedirectEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]),  0};
            ret_code = FireRedirect(&e);
            param[1] = (void*)(e.Accept);
            break;
         }
         case 8: {
            SOAPSSetCookieEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (LPWSTR)IPS64CAST(param[4]), (int)IPS64CAST(param[5]),  0};
            ret_code = FireSetCookie(&e);
            break;
         }
         case 9: {
            SOAPSSSLServerAuthenticationEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = FireSSLServerAuthentication(&e);
            param[4] = (void*)(e.Accept);
            break;
         }
         case 10: {
            SOAPSSSLStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]),  0};
            ret_code = FireSSLStatus(&e);
            break;
         }
         case 11: {
            SOAPSStartTransferEventParamsW e = {(int)IPS64CAST(param[0]),  0};
            ret_code = FireStartTransfer(&e);
            break;
         }
         case 12: {
            SOAPSStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]),  0};
            ret_code = FireStatus(&e);
            break;
         }
         case 13: {
            SOAPSTransferEventParamsW e = {(int)IPS64CAST(param[0]), (ns_int64*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]),  0};
            ret_code = FireTransfer(&e);
            break;
         }

      }
      return ret_code;
    }
  
  public: //event overrides

    virtual int FireConnected(SOAPSConnectedEventParams *e) {return -10000;}
    virtual int FireConnectionStatus(SOAPSConnectionStatusEventParams *e) {return -10000;}
    virtual int FireDisconnected(SOAPSDisconnectedEventParams *e) {return -10000;}
    virtual int FireEndTransfer(SOAPSEndTransferEventParams *e) {return -10000;}
    virtual int FireError(SOAPSErrorEventParams *e) {return -10000;}
    virtual int FireHeader(SOAPSHeaderEventParams *e) {return -10000;}
    virtual int FireRedirect(SOAPSRedirectEventParams *e) {return -10000;}
    virtual int FireSetCookie(SOAPSSetCookieEventParams *e) {return -10000;}
    virtual int FireSSLServerAuthentication(SOAPSSSLServerAuthenticationEventParams *e) {return -10000;}
    virtual int FireSSLStatus(SOAPSSSLStatusEventParams *e) {return -10000;}
    virtual int FireStartTransfer(SOAPSStartTransferEventParams *e) {return -10000;}
    virtual int FireStatus(SOAPSStatusEventParams *e) {return -10000;}
    virtual int FireTransfer(SOAPSTransferEventParams *e) {return -10000;}

  public: //methods

    inline int AddCookie(LPWSTR lpszCookieName, LPWSTR lpszCookieValue) {
      void *param[2+1] = {(void*)lpszCookieName, (void*)lpszCookieValue, 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_SOAPS_Do(m_pObj, 10000+2, 2, param, cbparam);
      
    }
    inline int AddNamespace(LPWSTR lpszPrefix, LPWSTR lpszNamespaceURI) {
      void *param[2+1] = {(void*)lpszPrefix, (void*)lpszNamespaceURI, 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_SOAPS_Do(m_pObj, 10000+3, 2, param, cbparam);
      
    }
    inline int AddParam(LPWSTR lpszParamName, LPWSTR lpszParamValue) {
      void *param[2+1] = {(void*)lpszParamName, (void*)lpszParamValue, 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_SOAPS_Do(m_pObj, 10000+4, 2, param, cbparam);
      
    }
    inline LPWSTR Attr(LPWSTR lpszAttrName) {
      void *param[1+1] = {(void*)lpszAttrName, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_SOAPS_Do(m_pObj, 10000+5, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline int BuildPacket() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SOAPS_Do(m_pObj, 10000+6, 0, param, cbparam);
      
    }
    inline LPWSTR Config(LPWSTR lpszConfigurationString) {
      void *param[1+1] = {(void*)lpszConfigurationString, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_SOAPS_Do(m_pObj, 10000+7, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SOAPS_Do(m_pObj, 10000+8, 0, param, cbparam);
      
    }
    inline int EvalPacket() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SOAPS_Do(m_pObj, 10000+9, 0, param, cbparam);
      
    }
    inline int HasXPath(LPWSTR lpszxpath) {
      void *param[1+1] = {(void*)lpszxpath, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_SOAPS_Do(m_pObj, 10000+10, 1, param, cbparam);
      return (int)IPS64CAST(param[1]);
    }
    inline int Interrupt() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SOAPS_Do(m_pObj, 10000+11, 0, param, cbparam);
      
    }
    inline int Reset() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SOAPS_Do(m_pObj, 10000+12, 0, param, cbparam);
      
    }
    inline int SendPacket() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SOAPS_Do(m_pObj, 10000+13, 0, param, cbparam);
      
    }
    inline int SendRequest() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_SOAPS_Do(m_pObj, 10000+14, 0, param, cbparam);
      
    }
    inline LPWSTR Value(LPWSTR lpszParamName) {
      void *param[1+1] = {(void*)lpszParamName, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_SOAPS_Do(m_pObj, 10000+15, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }

};

#endif //WIN32

#endif //_IPWORKSSSL_SOAPS_H_




