/******************************************************************
   IP*Works! SSL V9 C++ Edition
   Copyright (c) 2013 /n software inc. - All rights reserved.
*******************************************************************/

#ifndef _IPWORKSSSL_TELNETS_H_
#define _IPWORKSSSL_TELNETS_H_

#define IPWORKSSSL_ONLY_TYPES
#include "ipworksssl.h"
#include "ipworksssl.key"

//FirewallTypes
#define FW_NONE                                            0
#define FW_TUNNEL                                          1
#define FW_SOCKS4                                          2
#define FW_SOCKS5                                          3

//CertStoreTypes
#define CST_USER                                           0
#define CST_MACHINE                                        1
#define CST_PFXFILE                                        2
#define CST_PFXBLOB                                        3
#define CST_JKSFILE                                        4
#define CST_JKSBLOB                                        5
#define CST_PEMKEY_FILE                                    6
#define CST_PEMKEY_BLOB                                    7
#define CST_PUBLIC_KEY_FILE                                8
#define CST_PUBLIC_KEY_BLOB                                9
#define CST_SSHPUBLIC_KEY_BLOB                             10
#define CST_P7BFILE                                        11
#define CST_P7BBLOB                                        12
#define CST_SSHPUBLIC_KEY_FILE                             13

//TSSLStartMode
#define SSL_AUTOMATIC                                      0
#define SSL_IMPLICIT                                       1
#define SSL_EXPLICIT                                       2
#define SSL_NONE                                           3


extern "C" void* IPWORKSSSL_CALL IPWorksSSL_TelnetS_Create(PIPWORKSSSL_CALLBACK lpSink, void *lpContext, char *lpOemKey);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_TelnetS_Destroy(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_TelnetS_CheckIndex(void *lpObj, int propid, int arridx);
extern "C" void* IPWORKSSSL_CALL IPWorksSSL_TelnetS_Get(void *lpObj, int propid, int arridx, int *lpcbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_TelnetS_Set(void *lpObj, int propid, int arridx, const void *val, int cbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_TelnetS_Do(void *lpObj, int methid, int cparam, void *param[], int cbparam[]);
extern "C" char* IPWORKSSSL_CALL IPWorksSSL_TelnetS_GetLastError(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_TelnetS_GetLastErrorCode(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_TelnetS_StaticInit(void *hInst);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_TelnetS_StaticDestroy();

#ifdef WIN32
#include <windows.h>
#pragma warning(disable:4311) 
#pragma warning(disable:4312) 
#endif

typedef struct {
  int CommandCode;
  int reserved;
} TelnetSCommandEventParams;

typedef struct {
  int StatusCode;
  const char* Description;
  int reserved;
} TelnetSConnectedEventParams;

typedef struct {
  const char* ConnectionEvent;
  int StatusCode;
  const char* Description;
  int reserved;
} TelnetSConnectionStatusEventParams;

typedef struct {
  const char* Text;
  int lenText;
  int reserved;
} TelnetSDataInEventParams;

typedef struct {
  int StatusCode;
  const char* Description;
  int reserved;
} TelnetSDisconnectedEventParams;

typedef struct {
  int OptionCode;
  int reserved;
} TelnetSDoEventParams;

typedef struct {
  int OptionCode;
  int reserved;
} TelnetSDontEventParams;

typedef struct {
  int ErrorCode;
  const char* Description;
  int reserved;
} TelnetSErrorEventParams;

typedef struct {
  int reserved;
} TelnetSReadyToSendEventParams;

typedef struct {
  const char* CertEncoded;
  const char* CertSubject;
  const char* CertIssuer;
  const char* Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} TelnetSSSLServerAuthenticationEventParams;

typedef struct {
  const char* Message;
  int reserved;
} TelnetSSSLStatusEventParams;

typedef struct {
  const char* SubOption;
  int lenSubOption;
  int reserved;
} TelnetSSubOptionEventParams;

typedef struct {
  int OptionCode;
  int reserved;
} TelnetSWillEventParams;

typedef struct {
  int OptionCode;
  int reserved;
} TelnetSWontEventParams;



class TelnetS {
  
  public: //events
  
    virtual int FireCommand(TelnetSCommandEventParams *e) {return 0;}
    virtual int FireConnected(TelnetSConnectedEventParams *e) {return 0;}
    virtual int FireConnectionStatus(TelnetSConnectionStatusEventParams *e) {return 0;}
    virtual int FireDataIn(TelnetSDataInEventParams *e) {return 0;}
    virtual int FireDisconnected(TelnetSDisconnectedEventParams *e) {return 0;}
    virtual int FireDo(TelnetSDoEventParams *e) {return 0;}
    virtual int FireDont(TelnetSDontEventParams *e) {return 0;}
    virtual int FireError(TelnetSErrorEventParams *e) {return 0;}
    virtual int FireReadyToSend(TelnetSReadyToSendEventParams *e) {return 0;}
    virtual int FireSSLServerAuthentication(TelnetSSSLServerAuthenticationEventParams *e) {return 0;}
    virtual int FireSSLStatus(TelnetSSSLStatusEventParams *e) {return 0;}
    virtual int FireSubOption(TelnetSSubOptionEventParams *e) {return 0;}
    virtual int FireWill(TelnetSWillEventParams *e) {return 0;}
    virtual int FireWont(TelnetSWontEventParams *e) {return 0;}


  protected:

    void *m_pObj;
    
    static int IPWORKSSSL_CALL TelnetSEventSink(void *lpObj, int event_id, int cparam, void *param[], int cbparam[]) {
      int ret_code = 0;
      if (event_id > 10000) return ((TelnetS*)lpObj)->TelnetSEventSinkW(event_id - 10000, cparam, param, cbparam);
      switch (event_id) {
         case 1: {
            TelnetSCommandEventParams e = {(int)IPS64CAST(param[0]),  0};
            ret_code = ((TelnetS*)lpObj)->FireCommand(&e);
            break;
         }
         case 2: {
            TelnetSConnectedEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((TelnetS*)lpObj)->FireConnected(&e);
            break;
         }
         case 3: {
            TelnetSConnectionStatusEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]),  0};
            ret_code = ((TelnetS*)lpObj)->FireConnectionStatus(&e);
            break;
         }
         case 4: {
            TelnetSDataInEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = ((TelnetS*)lpObj)->FireDataIn(&e);
            break;
         }
         case 5: {
            TelnetSDisconnectedEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((TelnetS*)lpObj)->FireDisconnected(&e);
            break;
         }
         case 6: {
            TelnetSDoEventParams e = {(int)IPS64CAST(param[0]),  0};
            ret_code = ((TelnetS*)lpObj)->FireDo(&e);
            break;
         }
         case 7: {
            TelnetSDontEventParams e = {(int)IPS64CAST(param[0]),  0};
            ret_code = ((TelnetS*)lpObj)->FireDont(&e);
            break;
         }
         case 8: {
            TelnetSErrorEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((TelnetS*)lpObj)->FireError(&e);
            break;
         }
         case 9: {
            TelnetSReadyToSendEventParams e = { 0};
            ret_code = ((TelnetS*)lpObj)->FireReadyToSend(&e);
            break;
         }
         case 10: {
            TelnetSSSLServerAuthenticationEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = ((TelnetS*)lpObj)->FireSSLServerAuthentication(&e);
            param[4] = (void*)IPS64CAST(e.Accept);
            break;
         }
         case 11: {
            TelnetSSSLStatusEventParams e = {(char*)IPS64CAST(param[0]),  0};
            ret_code = ((TelnetS*)lpObj)->FireSSLStatus(&e);
            break;
         }
         case 12: {
            TelnetSSubOptionEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = ((TelnetS*)lpObj)->FireSubOption(&e);
            break;
         }
         case 13: {
            TelnetSWillEventParams e = {(int)IPS64CAST(param[0]),  0};
            ret_code = ((TelnetS*)lpObj)->FireWill(&e);
            break;
         }
         case 14: {
            TelnetSWontEventParams e = {(int)IPS64CAST(param[0]),  0};
            ret_code = ((TelnetS*)lpObj)->FireWont(&e);
            break;
         }

      }
      return ret_code;
    }

    virtual int TelnetSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {return 0;}

  public:

    TelnetS(char *lpOemKey = (char*)IPWORKSSSL_OEMKEY_19) {
      m_pObj = IPWorksSSL_TelnetS_Create(TelnetSEventSink, (void*)this, (char*)lpOemKey);
    }

    virtual ~TelnetS() {
      IPWorksSSL_TelnetS_Destroy(m_pObj);
    }

  public:

    inline char *GetLastError() {
      return IPWorksSSL_TelnetS_GetLastError(m_pObj);
    }
    
    inline int GetLastErrorCode() {
      return IPWorksSSL_TelnetS_GetLastErrorCode(m_pObj);
    }

    inline char *VERSION() {
      return (char*)IPWorksSSL_TelnetS_Get(m_pObj, 0, 0, 0);
    }

  public: //properties

    inline int GetAcceptData() {
      void* val = IPWorksSSL_TelnetS_Get(m_pObj, 1, 0, 0);
      return (int)(long)val;
    }
    inline int SetAcceptData(int bAcceptData) {
      void* val = (void*)IPS64CAST(bAcceptData);
      return IPWorksSSL_TelnetS_Set(m_pObj, 1, 0, val, 0);
    }
    inline int GetBytesSent() {
      void* val = IPWorksSSL_TelnetS_Get(m_pObj, 2, 0, 0);
      return (int)(long)val;
    }


    inline int SetCommand(int iCommand) {
      void* val = (void*)IPS64CAST(iCommand);
      return IPWorksSSL_TelnetS_Set(m_pObj, 3, 0, val, 0);
    }
    inline int GetConnected() {
      void* val = IPWorksSSL_TelnetS_Get(m_pObj, 4, 0, 0);
      return (int)(long)val;
    }
    inline int SetConnected(int bConnected) {
      void* val = (void*)IPS64CAST(bConnected);
      return IPWorksSSL_TelnetS_Set(m_pObj, 4, 0, val, 0);
    }

    inline int SetDataToSend(const char *lpDataToSend, int lenDataToSend) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 5, 0, (void*)lpDataToSend, lenDataToSend);
    }


    inline int SetDontOption(int iDontOption) {
      void* val = (void*)IPS64CAST(iDontOption);
      return IPWorksSSL_TelnetS_Set(m_pObj, 6, 0, val, 0);
    }

    inline int SetDoOption(int iDoOption) {
      void* val = (void*)IPS64CAST(iDoOption);
      return IPWorksSSL_TelnetS_Set(m_pObj, 7, 0, val, 0);
    }

    inline int SetDoSubOption(const char *lpDoSubOption, int lenDoSubOption) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 8, 0, (void*)lpDoSubOption, lenDoSubOption);
    }

    inline int GetFirewallAutoDetect() {
      void* val = IPWorksSSL_TelnetS_Get(m_pObj, 9, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallAutoDetect(int bFirewallAutoDetect) {
      void* val = (void*)IPS64CAST(bFirewallAutoDetect);
      return IPWorksSSL_TelnetS_Set(m_pObj, 9, 0, val, 0);
    }
    inline int GetFirewallType() {
      void* val = IPWorksSSL_TelnetS_Get(m_pObj, 10, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallType(int iFirewallType) {
      void* val = (void*)IPS64CAST(iFirewallType);
      return IPWorksSSL_TelnetS_Set(m_pObj, 10, 0, val, 0);
    }
    inline char* GetFirewallHost() {
      void* val = IPWorksSSL_TelnetS_Get(m_pObj, 11, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallHost(const char *lpFirewallHost) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 11, 0, (void*)lpFirewallHost, 0);
    }

    inline char* GetFirewallPassword() {
      void* val = IPWorksSSL_TelnetS_Get(m_pObj, 12, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallPassword(const char *lpFirewallPassword) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 12, 0, (void*)lpFirewallPassword, 0);
    }

    inline int GetFirewallPort() {
      void* val = IPWorksSSL_TelnetS_Get(m_pObj, 13, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallPort(int lFirewallPort) {
      void* val = (void*)IPS64CAST(lFirewallPort);
      return IPWorksSSL_TelnetS_Set(m_pObj, 13, 0, val, 0);
    }
    inline char* GetFirewallUser() {
      void* val = IPWorksSSL_TelnetS_Get(m_pObj, 14, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallUser(const char *lpFirewallUser) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 14, 0, (void*)lpFirewallUser, 0);
    }

    inline int GetKeepAlive() {
      void* val = IPWorksSSL_TelnetS_Get(m_pObj, 15, 0, 0);
      return (int)(long)val;
    }
    inline int SetKeepAlive(int bKeepAlive) {
      void* val = (void*)IPS64CAST(bKeepAlive);
      return IPWorksSSL_TelnetS_Set(m_pObj, 15, 0, val, 0);
    }
    inline int GetLinger() {
      void* val = IPWorksSSL_TelnetS_Get(m_pObj, 16, 0, 0);
      return (int)(long)val;
    }
    inline int SetLinger(int bLinger) {
      void* val = (void*)IPS64CAST(bLinger);
      return IPWorksSSL_TelnetS_Set(m_pObj, 16, 0, val, 0);
    }
    inline char* GetLocalHost() {
      void* val = IPWorksSSL_TelnetS_Get(m_pObj, 17, 0, 0);
      return (char*)val;
    }

    inline int SetLocalHost(const char *lpLocalHost) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 17, 0, (void*)lpLocalHost, 0);
    }

    inline int GetLocalPort() {
      void* val = IPWorksSSL_TelnetS_Get(m_pObj, 18, 0, 0);
      return (int)(long)val;
    }
    inline int SetLocalPort(int lLocalPort) {
      void* val = (void*)IPS64CAST(lLocalPort);
      return IPWorksSSL_TelnetS_Set(m_pObj, 18, 0, val, 0);
    }
    inline char* GetRemoteHost() {
      void* val = IPWorksSSL_TelnetS_Get(m_pObj, 19, 0, 0);
      return (char*)val;
    }

    inline int SetRemoteHost(const char *lpRemoteHost) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 19, 0, (void*)lpRemoteHost, 0);
    }

    inline int GetRemotePort() {
      void* val = IPWorksSSL_TelnetS_Get(m_pObj, 20, 0, 0);
      return (int)(long)val;
    }
    inline int SetRemotePort(int lRemotePort) {
      void* val = (void*)IPS64CAST(lRemotePort);
      return IPWorksSSL_TelnetS_Set(m_pObj, 20, 0, val, 0);
    }
    inline int GetSSLAcceptServerCertEncoded(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_TelnetS_Get(m_pObj, 21, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }

    inline int SetSSLAcceptServerCertEncoded(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 21, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }

    inline int GetSSLCertEncoded(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_TelnetS_Get(m_pObj, 22, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }

    inline int SetSSLCertEncoded(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 22, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }

    inline int GetSSLCertStore(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_TelnetS_Get(m_pObj, 23, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }

    inline int SetSSLCertStore(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 23, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }

    inline char* GetSSLCertStorePassword() {
      void* val = IPWorksSSL_TelnetS_Get(m_pObj, 24, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertStorePassword(const char *lpSSLCertStorePassword) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 24, 0, (void*)lpSSLCertStorePassword, 0);
    }

    inline int GetSSLCertStoreType() {
      void* val = IPWorksSSL_TelnetS_Get(m_pObj, 25, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLCertStoreType(int iSSLCertStoreType) {
      void* val = (void*)IPS64CAST(iSSLCertStoreType);
      return IPWorksSSL_TelnetS_Set(m_pObj, 25, 0, val, 0);
    }
    inline char* GetSSLCertSubject() {
      void* val = IPWorksSSL_TelnetS_Get(m_pObj, 26, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertSubject(const char *lpSSLCertSubject) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 26, 0, (void*)lpSSLCertSubject, 0);
    }

    inline int GetSSLServerCertEncoded(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_TelnetS_Get(m_pObj, 27, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }


    inline int GetSSLStartMode() {
      void* val = IPWorksSSL_TelnetS_Get(m_pObj, 28, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLStartMode(int iSSLStartMode) {
      void* val = (void*)IPS64CAST(iSSLStartMode);
      return IPWorksSSL_TelnetS_Set(m_pObj, 28, 0, val, 0);
    }
    inline int GetTimeout() {
      void* val = IPWorksSSL_TelnetS_Get(m_pObj, 29, 0, 0);
      return (int)(long)val;
    }
    inline int SetTimeout(int iTimeout) {
      void* val = (void*)IPS64CAST(iTimeout);
      return IPWorksSSL_TelnetS_Set(m_pObj, 29, 0, val, 0);
    }
    inline int GetTransparent() {
      void* val = IPWorksSSL_TelnetS_Get(m_pObj, 30, 0, 0);
      return (int)(long)val;
    }
    inline int SetTransparent(int bTransparent) {
      void* val = (void*)IPS64CAST(bTransparent);
      return IPWorksSSL_TelnetS_Set(m_pObj, 30, 0, val, 0);
    }

    inline int SetUrgentData(const char *lpUrgentData, int lenUrgentData) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 31, 0, (void*)lpUrgentData, lenUrgentData);
    }


    inline int SetWillOption(int iWillOption) {
      void* val = (void*)IPS64CAST(iWillOption);
      return IPWorksSSL_TelnetS_Set(m_pObj, 32, 0, val, 0);
    }

    inline int SetWontOption(int iWontOption) {
      void* val = (void*)IPS64CAST(iWontOption);
      return IPWorksSSL_TelnetS_Set(m_pObj, 33, 0, val, 0);
    }

  public: //methods

    inline char* Config(const char* lpszConfigurationString) {
      void *param[1+1] = {(void*)IPS64CAST(lpszConfigurationString), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_TelnetS_Do(m_pObj, 2, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline int Connect(const char* lpszHost) {
      void *param[1+1] = {(void*)IPS64CAST(lpszHost), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_TelnetS_Do(m_pObj, 3, 1, param, cbparam);
      
      
    }
    inline int Disconnect() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_TelnetS_Do(m_pObj, 4, 0, param, cbparam);
      
      
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_TelnetS_Do(m_pObj, 5, 0, param, cbparam);
      
      
    }
    inline int Send(const char* lpText, int lenText) {
      void *param[1+1] = {(void*)IPS64CAST(lpText), 0};
      int cbparam[1+1] = {lenText, 0};
      return IPWorksSSL_TelnetS_Do(m_pObj, 6, 1, param, cbparam);
      
      
    }

};


#ifdef WIN32 //UNICODE

typedef struct {
  int CommandCode;
  int reserved;
} TelnetSCommandEventParamsW;

typedef struct {
  int StatusCode;
  LPWSTR Description;
  int reserved;
} TelnetSConnectedEventParamsW;

typedef struct {
  LPWSTR ConnectionEvent;
  int StatusCode;
  LPWSTR Description;
  int reserved;
} TelnetSConnectionStatusEventParamsW;

typedef struct {
  LPWSTR Text;
  int lenText;
  int reserved;
} TelnetSDataInEventParamsW;

typedef struct {
  int StatusCode;
  LPWSTR Description;
  int reserved;
} TelnetSDisconnectedEventParamsW;

typedef struct {
  int OptionCode;
  int reserved;
} TelnetSDoEventParamsW;

typedef struct {
  int OptionCode;
  int reserved;
} TelnetSDontEventParamsW;

typedef struct {
  int ErrorCode;
  LPWSTR Description;
  int reserved;
} TelnetSErrorEventParamsW;

typedef struct {
  int reserved;
} TelnetSReadyToSendEventParamsW;

typedef struct {
  LPWSTR CertEncoded;
  LPWSTR CertSubject;
  LPWSTR CertIssuer;
  LPWSTR Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} TelnetSSSLServerAuthenticationEventParamsW;

typedef struct {
  LPWSTR Message;
  int reserved;
} TelnetSSSLStatusEventParamsW;

typedef struct {
  LPWSTR SubOption;
  int lenSubOption;
  int reserved;
} TelnetSSubOptionEventParamsW;

typedef struct {
  int OptionCode;
  int reserved;
} TelnetSWillEventParamsW;

typedef struct {
  int OptionCode;
  int reserved;
} TelnetSWontEventParamsW;



class TelnetSW : public TelnetS {

  public: //properties
  










    inline int SetDataToSend(LPWSTR lpDataToSend) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 10000+5, 0, (void*)lpDataToSend, 0);
    }

    inline int SetDataToSendB(const char *lpDataToSend, int lenDataToSend) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 5, 0, (void*)lpDataToSend, lenDataToSend);
    }






    inline int SetDoSubOption(LPWSTR lpDoSubOption) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 10000+8, 0, (void*)lpDoSubOption, 0);
    }

    inline int SetDoSubOptionB(const char *lpDoSubOption, int lenDoSubOption) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 8, 0, (void*)lpDoSubOption, lenDoSubOption);
    }




    inline LPWSTR GetFirewallHost() {
      return (LPWSTR)IPWorksSSL_TelnetS_Get(m_pObj, 10000+11, 0, 0);
    }

    inline int SetFirewallHost(LPWSTR lpFirewallHost) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 10000+11, 0, (void*)lpFirewallHost, 0);
    }

    inline LPWSTR GetFirewallPassword() {
      return (LPWSTR)IPWorksSSL_TelnetS_Get(m_pObj, 10000+12, 0, 0);
    }

    inline int SetFirewallPassword(LPWSTR lpFirewallPassword) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 10000+12, 0, (void*)lpFirewallPassword, 0);
    }



    inline LPWSTR GetFirewallUser() {
      return (LPWSTR)IPWorksSSL_TelnetS_Get(m_pObj, 10000+14, 0, 0);
    }

    inline int SetFirewallUser(LPWSTR lpFirewallUser) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 10000+14, 0, (void*)lpFirewallUser, 0);
    }





    inline LPWSTR GetLocalHost() {
      return (LPWSTR)IPWorksSSL_TelnetS_Get(m_pObj, 10000+17, 0, 0);
    }

    inline int SetLocalHost(LPWSTR lpLocalHost) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 10000+17, 0, (void*)lpLocalHost, 0);
    }



    inline LPWSTR GetRemoteHost() {
      return (LPWSTR)IPWorksSSL_TelnetS_Get(m_pObj, 10000+19, 0, 0);
    }

    inline int SetRemoteHost(LPWSTR lpRemoteHost) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 10000+19, 0, (void*)lpRemoteHost, 0);
    }



    inline LPWSTR GetSSLAcceptServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_TelnetS_Get(m_pObj, 10000+21, 0, 0);
    }

    inline int SetSSLAcceptServerCertEncoded(LPWSTR lpSSLAcceptServerCertEncoded) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 10000+21, 0, (void*)lpSSLAcceptServerCertEncoded, 0);
    }
    inline int GetSSLAcceptServerCertEncodedB(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_TelnetS_Get(m_pObj, 21, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }
    inline int SetSSLAcceptServerCertEncodedB(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 21, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }
    inline LPWSTR GetSSLCertEncoded() {
      return (LPWSTR)IPWorksSSL_TelnetS_Get(m_pObj, 10000+22, 0, 0);
    }

    inline int SetSSLCertEncoded(LPWSTR lpSSLCertEncoded) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 10000+22, 0, (void*)lpSSLCertEncoded, 0);
    }
    inline int GetSSLCertEncodedB(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_TelnetS_Get(m_pObj, 22, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }
    inline int SetSSLCertEncodedB(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 22, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }
    inline LPWSTR GetSSLCertStore() {
      return (LPWSTR)IPWorksSSL_TelnetS_Get(m_pObj, 10000+23, 0, 0);
    }

    inline int SetSSLCertStore(LPWSTR lpSSLCertStore) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 10000+23, 0, (void*)lpSSLCertStore, 0);
    }
    inline int GetSSLCertStoreB(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_TelnetS_Get(m_pObj, 23, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }
    inline int SetSSLCertStoreB(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 23, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }
    inline LPWSTR GetSSLCertStorePassword() {
      return (LPWSTR)IPWorksSSL_TelnetS_Get(m_pObj, 10000+24, 0, 0);
    }

    inline int SetSSLCertStorePassword(LPWSTR lpSSLCertStorePassword) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 10000+24, 0, (void*)lpSSLCertStorePassword, 0);
    }



    inline LPWSTR GetSSLCertSubject() {
      return (LPWSTR)IPWorksSSL_TelnetS_Get(m_pObj, 10000+26, 0, 0);
    }

    inline int SetSSLCertSubject(LPWSTR lpSSLCertSubject) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 10000+26, 0, (void*)lpSSLCertSubject, 0);
    }

    inline LPWSTR GetSSLServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_TelnetS_Get(m_pObj, 10000+27, 0, 0);
    }


    inline int GetSSLServerCertEncodedB(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_TelnetS_Get(m_pObj, 27, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }









    inline int SetUrgentData(LPWSTR lpUrgentData) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 10000+31, 0, (void*)lpUrgentData, 0);
    }

    inline int SetUrgentDataB(const char *lpUrgentData, int lenUrgentData) {
      return IPWorksSSL_TelnetS_Set(m_pObj, 31, 0, (void*)lpUrgentData, lenUrgentData);
    }






  public: //events
  
    virtual int FireCommand(TelnetSCommandEventParamsW *e) {return 0;}
    virtual int FireConnected(TelnetSConnectedEventParamsW *e) {return 0;}
    virtual int FireConnectionStatus(TelnetSConnectionStatusEventParamsW *e) {return 0;}
    virtual int FireDataIn(TelnetSDataInEventParamsW *e) {return 0;}
    virtual int FireDisconnected(TelnetSDisconnectedEventParamsW *e) {return 0;}
    virtual int FireDo(TelnetSDoEventParamsW *e) {return 0;}
    virtual int FireDont(TelnetSDontEventParamsW *e) {return 0;}
    virtual int FireError(TelnetSErrorEventParamsW *e) {return 0;}
    virtual int FireReadyToSend(TelnetSReadyToSendEventParamsW *e) {return 0;}
    virtual int FireSSLServerAuthentication(TelnetSSSLServerAuthenticationEventParamsW *e) {return 0;}
    virtual int FireSSLStatus(TelnetSSSLStatusEventParamsW *e) {return 0;}
    virtual int FireSubOption(TelnetSSubOptionEventParamsW *e) {return 0;}
    virtual int FireWill(TelnetSWillEventParamsW *e) {return 0;}
    virtual int FireWont(TelnetSWontEventParamsW *e) {return 0;}


  protected:
  
    virtual int TelnetSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {
    	int ret_code = 0;
      switch (event_id) {
         case 1: {
            TelnetSCommandEventParamsW e = {(int)IPS64CAST(param[0]),  0};
            ret_code = FireCommand(&e);
            break;
         }
         case 2: {
            TelnetSConnectedEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireConnected(&e);
            break;
         }
         case 3: {
            TelnetSConnectionStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]),  0};
            ret_code = FireConnectionStatus(&e);
            break;
         }
         case 4: {
            TelnetSDataInEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = FireDataIn(&e);
            break;
         }
         case 5: {
            TelnetSDisconnectedEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireDisconnected(&e);
            break;
         }
         case 6: {
            TelnetSDoEventParamsW e = {(int)IPS64CAST(param[0]),  0};
            ret_code = FireDo(&e);
            break;
         }
         case 7: {
            TelnetSDontEventParamsW e = {(int)IPS64CAST(param[0]),  0};
            ret_code = FireDont(&e);
            break;
         }
         case 8: {
            TelnetSErrorEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireError(&e);
            break;
         }
         case 9: {
            TelnetSReadyToSendEventParamsW e = { 0};
            ret_code = FireReadyToSend(&e);
            break;
         }
         case 10: {
            TelnetSSSLServerAuthenticationEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = FireSSLServerAuthentication(&e);
            param[4] = (void*)(e.Accept);
            break;
         }
         case 11: {
            TelnetSSSLStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]),  0};
            ret_code = FireSSLStatus(&e);
            break;
         }
         case 12: {
            TelnetSSubOptionEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = FireSubOption(&e);
            break;
         }
         case 13: {
            TelnetSWillEventParamsW e = {(int)IPS64CAST(param[0]),  0};
            ret_code = FireWill(&e);
            break;
         }
         case 14: {
            TelnetSWontEventParamsW e = {(int)IPS64CAST(param[0]),  0};
            ret_code = FireWont(&e);
            break;
         }

      }
      return ret_code;
    }
  
  public: //event overrides

    virtual int FireCommand(TelnetSCommandEventParams *e) {return -10000;}
    virtual int FireConnected(TelnetSConnectedEventParams *e) {return -10000;}
    virtual int FireConnectionStatus(TelnetSConnectionStatusEventParams *e) {return -10000;}
    virtual int FireDataIn(TelnetSDataInEventParams *e) {return -10000;}
    virtual int FireDisconnected(TelnetSDisconnectedEventParams *e) {return -10000;}
    virtual int FireDo(TelnetSDoEventParams *e) {return -10000;}
    virtual int FireDont(TelnetSDontEventParams *e) {return -10000;}
    virtual int FireError(TelnetSErrorEventParams *e) {return -10000;}
    virtual int FireReadyToSend(TelnetSReadyToSendEventParams *e) {return -10000;}
    virtual int FireSSLServerAuthentication(TelnetSSSLServerAuthenticationEventParams *e) {return -10000;}
    virtual int FireSSLStatus(TelnetSSSLStatusEventParams *e) {return -10000;}
    virtual int FireSubOption(TelnetSSubOptionEventParams *e) {return -10000;}
    virtual int FireWill(TelnetSWillEventParams *e) {return -10000;}
    virtual int FireWont(TelnetSWontEventParams *e) {return -10000;}

  public: //methods

    inline LPWSTR Config(LPWSTR lpszConfigurationString) {
      void *param[1+1] = {(void*)lpszConfigurationString, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_TelnetS_Do(m_pObj, 10000+2, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline int Connect(LPWSTR lpszHost) {
      void *param[1+1] = {(void*)lpszHost, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_TelnetS_Do(m_pObj, 10000+3, 1, param, cbparam);
      
    }
    inline int Disconnect() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_TelnetS_Do(m_pObj, 10000+4, 0, param, cbparam);
      
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_TelnetS_Do(m_pObj, 10000+5, 0, param, cbparam);
      
    }
    inline int Send(LPWSTR lpText, int lenText) {
      void *param[1+1] = {(void*)lpText, 0};
      int cbparam[1+1] = {lenText, 0};
      return IPWorksSSL_TelnetS_Do(m_pObj, 10000+6, 1, param, cbparam);
      
    }

};

#endif //WIN32

#endif //_IPWORKSSSL_TELNETS_H_




