/******************************************************************
   IP*Works! SSL V9 C++ Edition
   Copyright (c) 2013 /n software inc. - All rights reserved.
*******************************************************************/

#ifndef _IPWORKSSSL_WEBUPLOADS_H_
#define _IPWORKSSSL_WEBUPLOADS_H_

#define IPWORKSSSL_ONLY_TYPES
#include "ipworksssl.h"
#include "ipworksssl.key"

//AuthSchemes
#define AUTH_BASIC                                         0
#define AUTH_DIGEST                                        1
#define AUTH_PROPRIETARY                                   2
#define AUTH_NONE                                          3
#define AUTH_NTLM                                          4
#define AUTH_NEGOTIATE                                     5
#define AUTH_OAUTH                                         6

//FirewallTypes
#define FW_NONE                                            0
#define FW_TUNNEL                                          1
#define FW_SOCKS4                                          2
#define FW_SOCKS5                                          3

//TFollowRedirects
#define FR_NEVER                                           0
#define FR_ALWAYS                                          1
#define FR_SAME_SCHEME                                     2

//ProxySSLTypes
#define PS_AUTOMATIC                                       0
#define PS_ALWAYS                                          1
#define PS_NEVER                                           2
#define PS_TUNNEL                                          3

//CertStoreTypes
#define CST_USER                                           0
#define CST_MACHINE                                        1
#define CST_PFXFILE                                        2
#define CST_PFXBLOB                                        3
#define CST_JKSFILE                                        4
#define CST_JKSBLOB                                        5
#define CST_PEMKEY_FILE                                    6
#define CST_PEMKEY_BLOB                                    7
#define CST_PUBLIC_KEY_FILE                                8
#define CST_PUBLIC_KEY_BLOB                                9
#define CST_SSHPUBLIC_KEY_BLOB                             10
#define CST_P7BFILE                                        11
#define CST_P7BBLOB                                        12
#define CST_SSHPUBLIC_KEY_FILE                             13


extern "C" void* IPWORKSSSL_CALL IPWorksSSL_WebUploadS_Create(PIPWORKSSSL_CALLBACK lpSink, void *lpContext, char *lpOemKey);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_WebUploadS_Destroy(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_WebUploadS_CheckIndex(void *lpObj, int propid, int arridx);
extern "C" void* IPWORKSSSL_CALL IPWorksSSL_WebUploadS_Get(void *lpObj, int propid, int arridx, int *lpcbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_WebUploadS_Set(void *lpObj, int propid, int arridx, const void *val, int cbVal);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_WebUploadS_Do(void *lpObj, int methid, int cparam, void *param[], int cbparam[]);
extern "C" char* IPWORKSSSL_CALL IPWorksSSL_WebUploadS_GetLastError(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_WebUploadS_GetLastErrorCode(void *lpObj);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_WebUploadS_StaticInit(void *hInst);
extern "C" int   IPWORKSSSL_CALL IPWorksSSL_WebUploadS_StaticDestroy();

#ifdef WIN32
#include <windows.h>
#pragma warning(disable:4311) 
#pragma warning(disable:4312) 
#endif

typedef struct {
  int StatusCode;
  const char* Description;
  int reserved;
} WebUploadSConnectedEventParams;

typedef struct {
  const char* ConnectionEvent;
  int StatusCode;
  const char* Description;
  int reserved;
} WebUploadSConnectionStatusEventParams;

typedef struct {
  int StatusCode;
  const char* Description;
  int reserved;
} WebUploadSDisconnectedEventParams;

typedef struct {
  int Direction;
  int reserved;
} WebUploadSEndTransferEventParams;

typedef struct {
  int ErrorCode;
  const char* Description;
  int reserved;
} WebUploadSErrorEventParams;

typedef struct {
  const char* Field;
  const char* Value;
  int reserved;
} WebUploadSHeaderEventParams;

typedef struct {
  const char* Location;
  int Accept;
  int reserved;
} WebUploadSRedirectEventParams;

typedef struct {
  const char* Name;
  const char* Value;
  const char* Expires;
  const char* Domain;
  const char* Path;
  int Secure;
  int reserved;
} WebUploadSSetCookieEventParams;

typedef struct {
  const char* CertEncoded;
  const char* CertSubject;
  const char* CertIssuer;
  const char* Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} WebUploadSSSLServerAuthenticationEventParams;

typedef struct {
  const char* Message;
  int reserved;
} WebUploadSSSLStatusEventParams;

typedef struct {
  int Direction;
  int reserved;
} WebUploadSStartTransferEventParams;

typedef struct {
  const char* HTTPVersion;
  int StatusCode;
  const char* Description;
  int reserved;
} WebUploadSStatusEventParams;

typedef struct {
  int Direction;
  ns_int64 *pBytesTransferred;
  int PercentDone;
  const char* Text;
  int lenText;
  int reserved;
} WebUploadSTransferEventParams;

typedef struct {
  int PercentDone;
  int reserved;
} WebUploadSUploadProgressEventParams;



class WebUploadS {
  
  public: //events
  
    virtual int FireConnected(WebUploadSConnectedEventParams *e) {return 0;}
    virtual int FireConnectionStatus(WebUploadSConnectionStatusEventParams *e) {return 0;}
    virtual int FireDisconnected(WebUploadSDisconnectedEventParams *e) {return 0;}
    virtual int FireEndTransfer(WebUploadSEndTransferEventParams *e) {return 0;}
    virtual int FireError(WebUploadSErrorEventParams *e) {return 0;}
    virtual int FireHeader(WebUploadSHeaderEventParams *e) {return 0;}
    virtual int FireRedirect(WebUploadSRedirectEventParams *e) {return 0;}
    virtual int FireSetCookie(WebUploadSSetCookieEventParams *e) {return 0;}
    virtual int FireSSLServerAuthentication(WebUploadSSSLServerAuthenticationEventParams *e) {return 0;}
    virtual int FireSSLStatus(WebUploadSSSLStatusEventParams *e) {return 0;}
    virtual int FireStartTransfer(WebUploadSStartTransferEventParams *e) {return 0;}
    virtual int FireStatus(WebUploadSStatusEventParams *e) {return 0;}
    virtual int FireTransfer(WebUploadSTransferEventParams *e) {return 0;}
    virtual int FireUploadProgress(WebUploadSUploadProgressEventParams *e) {return 0;}


  protected:

    void *m_pObj;
    
    static int IPWORKSSSL_CALL WebUploadSEventSink(void *lpObj, int event_id, int cparam, void *param[], int cbparam[]) {
      int ret_code = 0;
      if (event_id > 10000) return ((WebUploadS*)lpObj)->WebUploadSEventSinkW(event_id - 10000, cparam, param, cbparam);
      switch (event_id) {
         case 1: {
            WebUploadSConnectedEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((WebUploadS*)lpObj)->FireConnected(&e);
            break;
         }
         case 2: {
            WebUploadSConnectionStatusEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]),  0};
            ret_code = ((WebUploadS*)lpObj)->FireConnectionStatus(&e);
            break;
         }
         case 3: {
            WebUploadSDisconnectedEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((WebUploadS*)lpObj)->FireDisconnected(&e);
            break;
         }
         case 4: {
            WebUploadSEndTransferEventParams e = {(int)IPS64CAST(param[0]),  0};
            ret_code = ((WebUploadS*)lpObj)->FireEndTransfer(&e);
            break;
         }
         case 5: {
            WebUploadSErrorEventParams e = {(int)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((WebUploadS*)lpObj)->FireError(&e);
            break;
         }
         case 6: {
            WebUploadSHeaderEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]),  0};
            ret_code = ((WebUploadS*)lpObj)->FireHeader(&e);
            break;
         }
         case 7: {
            WebUploadSRedirectEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]),  0};
            ret_code = ((WebUploadS*)lpObj)->FireRedirect(&e);
            param[1] = (void*)IPS64CAST(e.Accept);
            break;
         }
         case 8: {
            WebUploadSSetCookieEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (char*)IPS64CAST(param[4]), (int)IPS64CAST(param[5]),  0};
            ret_code = ((WebUploadS*)lpObj)->FireSetCookie(&e);
            break;
         }
         case 9: {
            WebUploadSSSLServerAuthenticationEventParams e = {(char*)IPS64CAST(param[0]), (char*)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = ((WebUploadS*)lpObj)->FireSSLServerAuthentication(&e);
            param[4] = (void*)IPS64CAST(e.Accept);
            break;
         }
         case 10: {
            WebUploadSSSLStatusEventParams e = {(char*)IPS64CAST(param[0]),  0};
            ret_code = ((WebUploadS*)lpObj)->FireSSLStatus(&e);
            break;
         }
         case 11: {
            WebUploadSStartTransferEventParams e = {(int)IPS64CAST(param[0]),  0};
            ret_code = ((WebUploadS*)lpObj)->FireStartTransfer(&e);
            break;
         }
         case 12: {
            WebUploadSStatusEventParams e = {(char*)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (char*)IPS64CAST(param[2]),  0};
            ret_code = ((WebUploadS*)lpObj)->FireStatus(&e);
            break;
         }
         case 13: {
            WebUploadSTransferEventParams e = {(int)IPS64CAST(param[0]), (ns_int64*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (char*)IPS64CAST(param[3]), (int)IPS64CAST(cbparam[3]),  0};
            ret_code = ((WebUploadS*)lpObj)->FireTransfer(&e);
            break;
         }
         case 14: {
            WebUploadSUploadProgressEventParams e = {(int)IPS64CAST(param[0]),  0};
            ret_code = ((WebUploadS*)lpObj)->FireUploadProgress(&e);
            break;
         }

      }
      return ret_code;
    }

    virtual int WebUploadSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {return 0;}

  public:

    WebUploadS(char *lpOemKey = (char*)IPWORKSSSL_OEMKEY_51) {
      m_pObj = IPWorksSSL_WebUploadS_Create(WebUploadSEventSink, (void*)this, (char*)lpOemKey);
    }

    virtual ~WebUploadS() {
      IPWorksSSL_WebUploadS_Destroy(m_pObj);
    }

  public:

    inline char *GetLastError() {
      return IPWorksSSL_WebUploadS_GetLastError(m_pObj);
    }
    
    inline int GetLastErrorCode() {
      return IPWorksSSL_WebUploadS_GetLastErrorCode(m_pObj);
    }

    inline char *VERSION() {
      return (char*)IPWorksSSL_WebUploadS_Get(m_pObj, 0, 0, 0);
    }

  public: //properties

    inline char* GetAccept() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 1, 0, 0);
      return (char*)val;
    }

    inline int SetAccept(const char *lpAccept) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 1, 0, (void*)lpAccept, 0);
    }

    inline char* GetAuthorization() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 2, 0, 0);
      return (char*)val;
    }

    inline int SetAuthorization(const char *lpAuthorization) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 2, 0, (void*)lpAuthorization, 0);
    }

    inline int GetAuthScheme() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 3, 0, 0);
      return (int)(long)val;
    }
    inline int SetAuthScheme(int iAuthScheme) {
      void* val = (void*)IPS64CAST(iAuthScheme);
      return IPWorksSSL_WebUploadS_Set(m_pObj, 3, 0, val, 0);
    }
    inline int GetConnected() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 4, 0, 0);
      return (int)(long)val;
    }
    inline int SetConnected(int bConnected) {
      void* val = (void*)IPS64CAST(bConnected);
      return IPWorksSSL_WebUploadS_Set(m_pObj, 4, 0, val, 0);
    }
    inline char* GetContentType() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 5, 0, 0);
      return (char*)val;
    }

    inline int SetContentType(const char *lpContentType) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 5, 0, (void*)lpContentType, 0);
    }

    inline int GetCookieCount() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 6, 0, 0);
      return (int)(long)val;
    }
    inline int SetCookieCount(int iCookieCount) {
      void* val = (void*)IPS64CAST(iCookieCount);
      return IPWorksSSL_WebUploadS_Set(m_pObj, 6, 0, val, 0);
    }
    inline char* GetCookieDomain(int iCookieIndex) {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 7, iCookieIndex, 0);
      return (char*)val;
    }


    inline char* GetCookieExpiration(int iCookieIndex) {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 8, iCookieIndex, 0);
      return (char*)val;
    }


    inline char* GetCookieName(int iCookieIndex) {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 9, iCookieIndex, 0);
      return (char*)val;
    }

    inline int SetCookieName(int iCookieIndex, const char *lpCookieName) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 9, iCookieIndex, (void*)lpCookieName, 0);
    }

    inline char* GetCookiePath(int iCookieIndex) {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 10, iCookieIndex, 0);
      return (char*)val;
    }


    inline int GetCookieSecure(int iCookieIndex) {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 11, iCookieIndex, 0);
      return (int)(long)val;
    }

    inline char* GetCookieValue(int iCookieIndex) {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 12, iCookieIndex, 0);
      return (char*)val;
    }

    inline int SetCookieValue(int iCookieIndex, const char *lpCookieValue) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 12, iCookieIndex, (void*)lpCookieValue, 0);
    }

    inline int GetFileCount() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 13, 0, 0);
      return (int)(long)val;
    }
    inline int SetFileCount(int iFileCount) {
      void* val = (void*)IPS64CAST(iFileCount);
      return IPWorksSSL_WebUploadS_Set(m_pObj, 13, 0, val, 0);
    }
    inline char* GetFileVarContentType(int iFileIndex) {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 14, iFileIndex, 0);
      return (char*)val;
    }

    inline int SetFileVarContentType(int iFileIndex, const char *lpFileVarContentType) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 14, iFileIndex, (void*)lpFileVarContentType, 0);
    }

    inline char* GetFileVarFile(int iFileIndex) {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 15, iFileIndex, 0);
      return (char*)val;
    }

    inline int SetFileVarFile(int iFileIndex, const char *lpFileVarFile) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 15, iFileIndex, (void*)lpFileVarFile, 0);
    }


    inline int SetFileVarInputData(int iFileIndex, const char *lpFileVarInputData, int lenFileVarInputData) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 16, iFileIndex, (void*)lpFileVarInputData, lenFileVarInputData);
    }

    inline char* GetFileVarName(int iFileIndex) {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 17, iFileIndex, 0);
      return (char*)val;
    }

    inline int SetFileVarName(int iFileIndex, const char *lpFileVarName) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 17, iFileIndex, (void*)lpFileVarName, 0);
    }

    inline int GetFirewallAutoDetect() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 18, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallAutoDetect(int bFirewallAutoDetect) {
      void* val = (void*)IPS64CAST(bFirewallAutoDetect);
      return IPWorksSSL_WebUploadS_Set(m_pObj, 18, 0, val, 0);
    }
    inline int GetFirewallType() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 19, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallType(int iFirewallType) {
      void* val = (void*)IPS64CAST(iFirewallType);
      return IPWorksSSL_WebUploadS_Set(m_pObj, 19, 0, val, 0);
    }
    inline char* GetFirewallHost() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 20, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallHost(const char *lpFirewallHost) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 20, 0, (void*)lpFirewallHost, 0);
    }

    inline char* GetFirewallPassword() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 21, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallPassword(const char *lpFirewallPassword) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 21, 0, (void*)lpFirewallPassword, 0);
    }

    inline int GetFirewallPort() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 22, 0, 0);
      return (int)(long)val;
    }
    inline int SetFirewallPort(int lFirewallPort) {
      void* val = (void*)IPS64CAST(lFirewallPort);
      return IPWorksSSL_WebUploadS_Set(m_pObj, 22, 0, val, 0);
    }
    inline char* GetFirewallUser() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 23, 0, 0);
      return (char*)val;
    }

    inline int SetFirewallUser(const char *lpFirewallUser) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 23, 0, (void*)lpFirewallUser, 0);
    }

    inline int GetFollowRedirects() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 24, 0, 0);
      return (int)(long)val;
    }
    inline int SetFollowRedirects(int iFollowRedirects) {
      void* val = (void*)IPS64CAST(iFollowRedirects);
      return IPWorksSSL_WebUploadS_Set(m_pObj, 24, 0, val, 0);
    }
    inline int GetFormVarCount() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 25, 0, 0);
      return (int)(long)val;
    }
    inline int SetFormVarCount(int iFormVarCount) {
      void* val = (void*)IPS64CAST(iFormVarCount);
      return IPWorksSSL_WebUploadS_Set(m_pObj, 25, 0, val, 0);
    }
    inline char* GetFormVarName(int iVarIndex) {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 26, iVarIndex, 0);
      return (char*)val;
    }

    inline int SetFormVarName(int iVarIndex, const char *lpFormVarName) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 26, iVarIndex, (void*)lpFormVarName, 0);
    }

    inline char* GetFormVarValue(int iVarIndex) {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 27, iVarIndex, 0);
      return (char*)val;
    }

    inline int SetFormVarValue(int iVarIndex, const char *lpFormVarValue) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 27, iVarIndex, (void*)lpFormVarValue, 0);
    }

    inline char* GetFrom() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 28, 0, 0);
      return (char*)val;
    }

    inline int SetFrom(const char *lpFrom) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 28, 0, (void*)lpFrom, 0);
    }

    inline int GetIdle() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 29, 0, 0);
      return (int)(long)val;
    }

    inline char* GetLocalFile() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 30, 0, 0);
      return (char*)val;
    }

    inline int SetLocalFile(const char *lpLocalFile) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 30, 0, (void*)lpLocalFile, 0);
    }

    inline char* GetLocalHost() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 31, 0, 0);
      return (char*)val;
    }

    inline int SetLocalHost(const char *lpLocalHost) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 31, 0, (void*)lpLocalHost, 0);
    }

    inline char* GetOtherHeaders() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 32, 0, 0);
      return (char*)val;
    }

    inline int SetOtherHeaders(const char *lpOtherHeaders) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 32, 0, (void*)lpOtherHeaders, 0);
    }

    inline int GetParsedHeaderCount() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 33, 0, 0);
      return (int)(long)val;
    }

    inline char* GetParsedHeaderField(int iParsedHeaderIndex) {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 34, iParsedHeaderIndex, 0);
      return (char*)val;
    }


    inline char* GetParsedHeaderValue(int iParsedHeaderIndex) {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 35, iParsedHeaderIndex, 0);
      return (char*)val;
    }


    inline char* GetPassword() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 36, 0, 0);
      return (char*)val;
    }

    inline int SetPassword(const char *lpPassword) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 36, 0, (void*)lpPassword, 0);
    }

    inline int GetProxyAuthScheme() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 37, 0, 0);
      return (int)(long)val;
    }
    inline int SetProxyAuthScheme(int iProxyAuthScheme) {
      void* val = (void*)IPS64CAST(iProxyAuthScheme);
      return IPWorksSSL_WebUploadS_Set(m_pObj, 37, 0, val, 0);
    }
    inline int GetProxyAutoDetect() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 38, 0, 0);
      return (int)(long)val;
    }
    inline int SetProxyAutoDetect(int bProxyAutoDetect) {
      void* val = (void*)IPS64CAST(bProxyAutoDetect);
      return IPWorksSSL_WebUploadS_Set(m_pObj, 38, 0, val, 0);
    }
    inline char* GetProxyPassword() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 39, 0, 0);
      return (char*)val;
    }

    inline int SetProxyPassword(const char *lpProxyPassword) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 39, 0, (void*)lpProxyPassword, 0);
    }

    inline int GetProxyPort() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 40, 0, 0);
      return (int)(long)val;
    }
    inline int SetProxyPort(int lProxyPort) {
      void* val = (void*)IPS64CAST(lProxyPort);
      return IPWorksSSL_WebUploadS_Set(m_pObj, 40, 0, val, 0);
    }
    inline char* GetProxyServer() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 41, 0, 0);
      return (char*)val;
    }

    inline int SetProxyServer(const char *lpProxyServer) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 41, 0, (void*)lpProxyServer, 0);
    }

    inline int GetProxySSL() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 42, 0, 0);
      return (int)(long)val;
    }
    inline int SetProxySSL(int iProxySSL) {
      void* val = (void*)IPS64CAST(iProxySSL);
      return IPWorksSSL_WebUploadS_Set(m_pObj, 42, 0, val, 0);
    }
    inline char* GetProxyUser() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 43, 0, 0);
      return (char*)val;
    }

    inline int SetProxyUser(const char *lpProxyUser) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 43, 0, (void*)lpProxyUser, 0);
    }

    inline char* GetReferer() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 44, 0, 0);
      return (char*)val;
    }

    inline int SetReferer(const char *lpReferer) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 44, 0, (void*)lpReferer, 0);
    }

    inline int GetSSLAcceptServerCertEncoded(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_WebUploadS_Get(m_pObj, 45, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }

    inline int SetSSLAcceptServerCertEncoded(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 45, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }

    inline int GetSSLCertEncoded(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_WebUploadS_Get(m_pObj, 46, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }

    inline int SetSSLCertEncoded(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 46, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }

    inline int GetSSLCertStore(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_WebUploadS_Get(m_pObj, 47, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }

    inline int SetSSLCertStore(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 47, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }

    inline char* GetSSLCertStorePassword() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 48, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertStorePassword(const char *lpSSLCertStorePassword) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 48, 0, (void*)lpSSLCertStorePassword, 0);
    }

    inline int GetSSLCertStoreType() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 49, 0, 0);
      return (int)(long)val;
    }
    inline int SetSSLCertStoreType(int iSSLCertStoreType) {
      void* val = (void*)IPS64CAST(iSSLCertStoreType);
      return IPWorksSSL_WebUploadS_Set(m_pObj, 49, 0, val, 0);
    }
    inline char* GetSSLCertSubject() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 50, 0, 0);
      return (char*)val;
    }

    inline int SetSSLCertSubject(const char *lpSSLCertSubject) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 50, 0, (void*)lpSSLCertSubject, 0);
    }

    inline int GetSSLServerCertEncoded(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_WebUploadS_Get(m_pObj, 51, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }


    inline char* GetStatusLine() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 52, 0, 0);
      return (char*)val;
    }


    inline int GetTimeout() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 53, 0, 0);
      return (int)(long)val;
    }
    inline int SetTimeout(int iTimeout) {
      void* val = (void*)IPS64CAST(iTimeout);
      return IPWorksSSL_WebUploadS_Set(m_pObj, 53, 0, val, 0);
    }
    inline int GetTransferredData(char *&lpTransferredData, int &lenTransferredData) {
      lpTransferredData = (char*)IPWorksSSL_WebUploadS_Get(m_pObj, 54, 0, &lenTransferredData);
      return lpTransferredData ? 0 : lenTransferredData;
    }


    inline ns_int64 GetTransferredDataLimit() {
      ns_int64 *pval = (ns_int64*)IPWorksSSL_WebUploadS_Get(m_pObj, 55, 0, 0);
      return *pval;
    }

    inline int SetTransferredDataLimit(ns_int64 lTransferredDataLimit) {
      void* val = (void*)(&lTransferredDataLimit);
      return IPWorksSSL_WebUploadS_Set(m_pObj, 55, 0, val, 0);
    }

    inline char* GetTransferredHeaders() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 56, 0, 0);
      return (char*)val;
    }


    inline char* GetURL() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 57, 0, 0);
      return (char*)val;
    }

    inline int SetURL(const char *lpURL) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 57, 0, (void*)lpURL, 0);
    }

    inline char* GetUser() {
      void* val = IPWorksSSL_WebUploadS_Get(m_pObj, 58, 0, 0);
      return (char*)val;
    }

    inline int SetUser(const char *lpUser) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 58, 0, (void*)lpUser, 0);
    }


  public: //methods

    inline int AddCookie(const char* lpszCookieName, const char* lpszCookieValue) {
      void *param[2+1] = {(void*)IPS64CAST(lpszCookieName), (void*)IPS64CAST(lpszCookieValue), 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_WebUploadS_Do(m_pObj, 2, 2, param, cbparam);
      
      
    }
    inline int AddFileVar(const char* lpszFileVar, const char* lpszFileName) {
      void *param[2+1] = {(void*)IPS64CAST(lpszFileVar), (void*)IPS64CAST(lpszFileName), 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_WebUploadS_Do(m_pObj, 3, 2, param, cbparam);
      
      
    }
    inline int AddFormVar(const char* lpszVarName, const char* lpszVarValue) {
      void *param[2+1] = {(void*)IPS64CAST(lpszVarName), (void*)IPS64CAST(lpszVarValue), 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_WebUploadS_Do(m_pObj, 4, 2, param, cbparam);
      
      
    }
    inline char* Config(const char* lpszConfigurationString) {
      void *param[1+1] = {(void*)IPS64CAST(lpszConfigurationString), 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_WebUploadS_Do(m_pObj, 5, 1, param, cbparam);
      
      return (char*)IPS64CAST(param[1]);
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_WebUploadS_Do(m_pObj, 6, 0, param, cbparam);
      
      
    }
    inline int Interrupt() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_WebUploadS_Do(m_pObj, 7, 0, param, cbparam);
      
      
    }
    inline int Reset() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_WebUploadS_Do(m_pObj, 8, 0, param, cbparam);
      
      
    }
    inline int Upload() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_WebUploadS_Do(m_pObj, 9, 0, param, cbparam);
      
      
    }
    inline int UploadTo(const char* lpszURL) {
      void *param[1+1] = {(void*)IPS64CAST(lpszURL), 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_WebUploadS_Do(m_pObj, 10, 1, param, cbparam);
      
      
    }

};


#ifdef WIN32 //UNICODE

typedef struct {
  int StatusCode;
  LPWSTR Description;
  int reserved;
} WebUploadSConnectedEventParamsW;

typedef struct {
  LPWSTR ConnectionEvent;
  int StatusCode;
  LPWSTR Description;
  int reserved;
} WebUploadSConnectionStatusEventParamsW;

typedef struct {
  int StatusCode;
  LPWSTR Description;
  int reserved;
} WebUploadSDisconnectedEventParamsW;

typedef struct {
  int Direction;
  int reserved;
} WebUploadSEndTransferEventParamsW;

typedef struct {
  int ErrorCode;
  LPWSTR Description;
  int reserved;
} WebUploadSErrorEventParamsW;

typedef struct {
  LPWSTR Field;
  LPWSTR Value;
  int reserved;
} WebUploadSHeaderEventParamsW;

typedef struct {
  LPWSTR Location;
  int Accept;
  int reserved;
} WebUploadSRedirectEventParamsW;

typedef struct {
  LPWSTR Name;
  LPWSTR Value;
  LPWSTR Expires;
  LPWSTR Domain;
  LPWSTR Path;
  int Secure;
  int reserved;
} WebUploadSSetCookieEventParamsW;

typedef struct {
  LPWSTR CertEncoded;
  LPWSTR CertSubject;
  LPWSTR CertIssuer;
  LPWSTR Status;
  int Accept;
  int lenCertEncoded;
  int reserved;
} WebUploadSSSLServerAuthenticationEventParamsW;

typedef struct {
  LPWSTR Message;
  int reserved;
} WebUploadSSSLStatusEventParamsW;

typedef struct {
  int Direction;
  int reserved;
} WebUploadSStartTransferEventParamsW;

typedef struct {
  LPWSTR HTTPVersion;
  int StatusCode;
  LPWSTR Description;
  int reserved;
} WebUploadSStatusEventParamsW;

typedef struct {
  int Direction;
  ns_int64 *pBytesTransferred;
  int PercentDone;
  LPWSTR Text;
  int lenText;
  int reserved;
} WebUploadSTransferEventParamsW;

typedef struct {
  int PercentDone;
  int reserved;
} WebUploadSUploadProgressEventParamsW;



class WebUploadSW : public WebUploadS {

  public: //properties
  
    inline LPWSTR GetAccept() {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+1, 0, 0);
    }

    inline int SetAccept(LPWSTR lpAccept) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+1, 0, (void*)lpAccept, 0);
    }

    inline LPWSTR GetAuthorization() {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+2, 0, 0);
    }

    inline int SetAuthorization(LPWSTR lpAuthorization) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+2, 0, (void*)lpAuthorization, 0);
    }





    inline LPWSTR GetContentType() {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+5, 0, 0);
    }

    inline int SetContentType(LPWSTR lpContentType) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+5, 0, (void*)lpContentType, 0);
    }



    inline LPWSTR GetCookieDomain(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+7, iCookieIndex, 0);
    }



    inline LPWSTR GetCookieExpiration(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+8, iCookieIndex, 0);
    }



    inline LPWSTR GetCookieName(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+9, iCookieIndex, 0);
    }

    inline int SetCookieName(int iCookieIndex, LPWSTR lpCookieName) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+9, iCookieIndex, (void*)lpCookieName, 0);
    }

    inline LPWSTR GetCookiePath(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+10, iCookieIndex, 0);
    }





    inline LPWSTR GetCookieValue(int iCookieIndex) {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+12, iCookieIndex, 0);
    }

    inline int SetCookieValue(int iCookieIndex, LPWSTR lpCookieValue) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+12, iCookieIndex, (void*)lpCookieValue, 0);
    }



    inline LPWSTR GetFileVarContentType(int iFileIndex) {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+14, iFileIndex, 0);
    }

    inline int SetFileVarContentType(int iFileIndex, LPWSTR lpFileVarContentType) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+14, iFileIndex, (void*)lpFileVarContentType, 0);
    }

    inline LPWSTR GetFileVarFile(int iFileIndex) {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+15, iFileIndex, 0);
    }

    inline int SetFileVarFile(int iFileIndex, LPWSTR lpFileVarFile) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+15, iFileIndex, (void*)lpFileVarFile, 0);
    }



    inline int SetFileVarInputData(int iFileIndex, LPWSTR lpFileVarInputData) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+16, iFileIndex, (void*)lpFileVarInputData, 0);
    }

    inline int SetFileVarInputDataB(int iFileIndex, const char *lpFileVarInputData, int lenFileVarInputData) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 16, iFileIndex, (void*)lpFileVarInputData, lenFileVarInputData);
    }
    inline LPWSTR GetFileVarName(int iFileIndex) {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+17, iFileIndex, 0);
    }

    inline int SetFileVarName(int iFileIndex, LPWSTR lpFileVarName) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+17, iFileIndex, (void*)lpFileVarName, 0);
    }





    inline LPWSTR GetFirewallHost() {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+20, 0, 0);
    }

    inline int SetFirewallHost(LPWSTR lpFirewallHost) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+20, 0, (void*)lpFirewallHost, 0);
    }

    inline LPWSTR GetFirewallPassword() {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+21, 0, 0);
    }

    inline int SetFirewallPassword(LPWSTR lpFirewallPassword) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+21, 0, (void*)lpFirewallPassword, 0);
    }



    inline LPWSTR GetFirewallUser() {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+23, 0, 0);
    }

    inline int SetFirewallUser(LPWSTR lpFirewallUser) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+23, 0, (void*)lpFirewallUser, 0);
    }





    inline LPWSTR GetFormVarName(int iVarIndex) {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+26, iVarIndex, 0);
    }

    inline int SetFormVarName(int iVarIndex, LPWSTR lpFormVarName) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+26, iVarIndex, (void*)lpFormVarName, 0);
    }

    inline LPWSTR GetFormVarValue(int iVarIndex) {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+27, iVarIndex, 0);
    }

    inline int SetFormVarValue(int iVarIndex, LPWSTR lpFormVarValue) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+27, iVarIndex, (void*)lpFormVarValue, 0);
    }

    inline LPWSTR GetFrom() {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+28, 0, 0);
    }

    inline int SetFrom(LPWSTR lpFrom) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+28, 0, (void*)lpFrom, 0);
    }



    inline LPWSTR GetLocalFile() {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+10000+30, 0, 0);
    }

    inline int SetLocalFile(LPWSTR lpLocalFile) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+10000+30, 0, (void*)lpLocalFile, 0);
    }

    inline LPWSTR GetLocalHost() {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+31, 0, 0);
    }

    inline int SetLocalHost(LPWSTR lpLocalHost) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+31, 0, (void*)lpLocalHost, 0);
    }

    inline LPWSTR GetOtherHeaders() {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+32, 0, 0);
    }

    inline int SetOtherHeaders(LPWSTR lpOtherHeaders) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+32, 0, (void*)lpOtherHeaders, 0);
    }



    inline LPWSTR GetParsedHeaderField(int iParsedHeaderIndex) {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+34, iParsedHeaderIndex, 0);
    }



    inline LPWSTR GetParsedHeaderValue(int iParsedHeaderIndex) {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+35, iParsedHeaderIndex, 0);
    }



    inline LPWSTR GetPassword() {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+36, 0, 0);
    }

    inline int SetPassword(LPWSTR lpPassword) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+36, 0, (void*)lpPassword, 0);
    }





    inline LPWSTR GetProxyPassword() {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+39, 0, 0);
    }

    inline int SetProxyPassword(LPWSTR lpProxyPassword) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+39, 0, (void*)lpProxyPassword, 0);
    }



    inline LPWSTR GetProxyServer() {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+41, 0, 0);
    }

    inline int SetProxyServer(LPWSTR lpProxyServer) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+41, 0, (void*)lpProxyServer, 0);
    }



    inline LPWSTR GetProxyUser() {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+43, 0, 0);
    }

    inline int SetProxyUser(LPWSTR lpProxyUser) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+43, 0, (void*)lpProxyUser, 0);
    }

    inline LPWSTR GetReferer() {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+44, 0, 0);
    }

    inline int SetReferer(LPWSTR lpReferer) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+44, 0, (void*)lpReferer, 0);
    }

    inline LPWSTR GetSSLAcceptServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+45, 0, 0);
    }

    inline int SetSSLAcceptServerCertEncoded(LPWSTR lpSSLAcceptServerCertEncoded) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+45, 0, (void*)lpSSLAcceptServerCertEncoded, 0);
    }
    inline int GetSSLAcceptServerCertEncodedB(char *&lpSSLAcceptServerCertEncoded, int &lenSSLAcceptServerCertEncoded) {
      lpSSLAcceptServerCertEncoded = (char*)IPWorksSSL_WebUploadS_Get(m_pObj, 45, 0, &lenSSLAcceptServerCertEncoded);
      return lpSSLAcceptServerCertEncoded ? 0 : lenSSLAcceptServerCertEncoded;
    }
    inline int SetSSLAcceptServerCertEncodedB(const char *lpSSLAcceptServerCertEncoded, int lenSSLAcceptServerCertEncoded) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 45, 0, (void*)lpSSLAcceptServerCertEncoded, lenSSLAcceptServerCertEncoded);
    }
    inline LPWSTR GetSSLCertEncoded() {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+46, 0, 0);
    }

    inline int SetSSLCertEncoded(LPWSTR lpSSLCertEncoded) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+46, 0, (void*)lpSSLCertEncoded, 0);
    }
    inline int GetSSLCertEncodedB(char *&lpSSLCertEncoded, int &lenSSLCertEncoded) {
      lpSSLCertEncoded = (char*)IPWorksSSL_WebUploadS_Get(m_pObj, 46, 0, &lenSSLCertEncoded);
      return lpSSLCertEncoded ? 0 : lenSSLCertEncoded;
    }
    inline int SetSSLCertEncodedB(const char *lpSSLCertEncoded, int lenSSLCertEncoded) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 46, 0, (void*)lpSSLCertEncoded, lenSSLCertEncoded);
    }
    inline LPWSTR GetSSLCertStore() {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+47, 0, 0);
    }

    inline int SetSSLCertStore(LPWSTR lpSSLCertStore) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+47, 0, (void*)lpSSLCertStore, 0);
    }
    inline int GetSSLCertStoreB(char *&lpSSLCertStore, int &lenSSLCertStore) {
      lpSSLCertStore = (char*)IPWorksSSL_WebUploadS_Get(m_pObj, 47, 0, &lenSSLCertStore);
      return lpSSLCertStore ? 0 : lenSSLCertStore;
    }
    inline int SetSSLCertStoreB(const char *lpSSLCertStore, int lenSSLCertStore) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 47, 0, (void*)lpSSLCertStore, lenSSLCertStore);
    }
    inline LPWSTR GetSSLCertStorePassword() {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+48, 0, 0);
    }

    inline int SetSSLCertStorePassword(LPWSTR lpSSLCertStorePassword) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+48, 0, (void*)lpSSLCertStorePassword, 0);
    }



    inline LPWSTR GetSSLCertSubject() {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+50, 0, 0);
    }

    inline int SetSSLCertSubject(LPWSTR lpSSLCertSubject) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+50, 0, (void*)lpSSLCertSubject, 0);
    }

    inline LPWSTR GetSSLServerCertEncoded() {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+51, 0, 0);
    }


    inline int GetSSLServerCertEncodedB(char *&lpSSLServerCertEncoded, int &lenSSLServerCertEncoded) {
      lpSSLServerCertEncoded = (char*)IPWorksSSL_WebUploadS_Get(m_pObj, 51, 0, &lenSSLServerCertEncoded);
      return lpSSLServerCertEncoded ? 0 : lenSSLServerCertEncoded;
    }

    inline LPWSTR GetStatusLine() {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+52, 0, 0);
    }





    inline LPWSTR GetTransferredData() {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+54, 0, 0);
    }


    inline int GetTransferredDataB(char *&lpTransferredData, int &lenTransferredData) {
      lpTransferredData = (char*)IPWorksSSL_WebUploadS_Get(m_pObj, 54, 0, &lenTransferredData);
      return lpTransferredData ? 0 : lenTransferredData;
    }



    inline LPWSTR GetTransferredHeaders() {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+56, 0, 0);
    }



    inline LPWSTR GetURL() {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+57, 0, 0);
    }

    inline int SetURL(LPWSTR lpURL) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+57, 0, (void*)lpURL, 0);
    }

    inline LPWSTR GetUser() {
      return (LPWSTR)IPWorksSSL_WebUploadS_Get(m_pObj, 10000+58, 0, 0);
    }

    inline int SetUser(LPWSTR lpUser) {
      return IPWorksSSL_WebUploadS_Set(m_pObj, 10000+58, 0, (void*)lpUser, 0);
    }



  public: //events
  
    virtual int FireConnected(WebUploadSConnectedEventParamsW *e) {return 0;}
    virtual int FireConnectionStatus(WebUploadSConnectionStatusEventParamsW *e) {return 0;}
    virtual int FireDisconnected(WebUploadSDisconnectedEventParamsW *e) {return 0;}
    virtual int FireEndTransfer(WebUploadSEndTransferEventParamsW *e) {return 0;}
    virtual int FireError(WebUploadSErrorEventParamsW *e) {return 0;}
    virtual int FireHeader(WebUploadSHeaderEventParamsW *e) {return 0;}
    virtual int FireRedirect(WebUploadSRedirectEventParamsW *e) {return 0;}
    virtual int FireSetCookie(WebUploadSSetCookieEventParamsW *e) {return 0;}
    virtual int FireSSLServerAuthentication(WebUploadSSSLServerAuthenticationEventParamsW *e) {return 0;}
    virtual int FireSSLStatus(WebUploadSSSLStatusEventParamsW *e) {return 0;}
    virtual int FireStartTransfer(WebUploadSStartTransferEventParamsW *e) {return 0;}
    virtual int FireStatus(WebUploadSStatusEventParamsW *e) {return 0;}
    virtual int FireTransfer(WebUploadSTransferEventParamsW *e) {return 0;}
    virtual int FireUploadProgress(WebUploadSUploadProgressEventParamsW *e) {return 0;}


  protected:
  
    virtual int WebUploadSEventSinkW(int event_id, int cparam, void *param[], int cbparam[]) {
    	int ret_code = 0;
      switch (event_id) {
         case 1: {
            WebUploadSConnectedEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireConnected(&e);
            break;
         }
         case 2: {
            WebUploadSConnectionStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]),  0};
            ret_code = FireConnectionStatus(&e);
            break;
         }
         case 3: {
            WebUploadSDisconnectedEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireDisconnected(&e);
            break;
         }
         case 4: {
            WebUploadSEndTransferEventParamsW e = {(int)IPS64CAST(param[0]),  0};
            ret_code = FireEndTransfer(&e);
            break;
         }
         case 5: {
            WebUploadSErrorEventParamsW e = {(int)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireError(&e);
            break;
         }
         case 6: {
            WebUploadSHeaderEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]),  0};
            ret_code = FireHeader(&e);
            break;
         }
         case 7: {
            WebUploadSRedirectEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]),  0};
            ret_code = FireRedirect(&e);
            param[1] = (void*)(e.Accept);
            break;
         }
         case 8: {
            WebUploadSSetCookieEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (LPWSTR)IPS64CAST(param[4]), (int)IPS64CAST(param[5]),  0};
            ret_code = FireSetCookie(&e);
            break;
         }
         case 9: {
            WebUploadSSSLServerAuthenticationEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (LPWSTR)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (int)IPS64CAST(param[4]), (int)IPS64CAST(cbparam[0]),  0};
            ret_code = FireSSLServerAuthentication(&e);
            param[4] = (void*)(e.Accept);
            break;
         }
         case 10: {
            WebUploadSSSLStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]),  0};
            ret_code = FireSSLStatus(&e);
            break;
         }
         case 11: {
            WebUploadSStartTransferEventParamsW e = {(int)IPS64CAST(param[0]),  0};
            ret_code = FireStartTransfer(&e);
            break;
         }
         case 12: {
            WebUploadSStatusEventParamsW e = {(LPWSTR)IPS64CAST(param[0]), (int)IPS64CAST(param[1]), (LPWSTR)IPS64CAST(param[2]),  0};
            ret_code = FireStatus(&e);
            break;
         }
         case 13: {
            WebUploadSTransferEventParamsW e = {(int)IPS64CAST(param[0]), (ns_int64*)IPS64CAST(param[1]), (int)IPS64CAST(param[2]), (LPWSTR)IPS64CAST(param[3]), (int)IPS64CAST(cbparam[3]),  0};
            ret_code = FireTransfer(&e);
            break;
         }
         case 14: {
            WebUploadSUploadProgressEventParamsW e = {(int)IPS64CAST(param[0]),  0};
            ret_code = FireUploadProgress(&e);
            break;
         }

      }
      return ret_code;
    }
  
  public: //event overrides

    virtual int FireConnected(WebUploadSConnectedEventParams *e) {return -10000;}
    virtual int FireConnectionStatus(WebUploadSConnectionStatusEventParams *e) {return -10000;}
    virtual int FireDisconnected(WebUploadSDisconnectedEventParams *e) {return -10000;}
    virtual int FireEndTransfer(WebUploadSEndTransferEventParams *e) {return -10000;}
    virtual int FireError(WebUploadSErrorEventParams *e) {return -10000;}
    virtual int FireHeader(WebUploadSHeaderEventParams *e) {return -10000;}
    virtual int FireRedirect(WebUploadSRedirectEventParams *e) {return -10000;}
    virtual int FireSetCookie(WebUploadSSetCookieEventParams *e) {return -10000;}
    virtual int FireSSLServerAuthentication(WebUploadSSSLServerAuthenticationEventParams *e) {return -10000;}
    virtual int FireSSLStatus(WebUploadSSSLStatusEventParams *e) {return -10000;}
    virtual int FireStartTransfer(WebUploadSStartTransferEventParams *e) {return -10000;}
    virtual int FireStatus(WebUploadSStatusEventParams *e) {return -10000;}
    virtual int FireTransfer(WebUploadSTransferEventParams *e) {return -10000;}
    virtual int FireUploadProgress(WebUploadSUploadProgressEventParams *e) {return -10000;}

  public: //methods

    inline int AddCookie(LPWSTR lpszCookieName, LPWSTR lpszCookieValue) {
      void *param[2+1] = {(void*)lpszCookieName, (void*)lpszCookieValue, 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_WebUploadS_Do(m_pObj, 10000+2, 2, param, cbparam);
      
    }
    inline int AddFileVar(LPWSTR lpszFileVar, LPWSTR lpszFileName) {
      void *param[2+1] = {(void*)lpszFileVar, (void*)lpszFileName, 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_WebUploadS_Do(m_pObj, 10000+3, 2, param, cbparam);
      
    }
    inline int AddFormVar(LPWSTR lpszVarName, LPWSTR lpszVarValue) {
      void *param[2+1] = {(void*)lpszVarName, (void*)lpszVarValue, 0};
      int cbparam[2+1] = {0, 0, 0};
      return IPWorksSSL_WebUploadS_Do(m_pObj, 10000+4, 2, param, cbparam);
      
    }
    inline LPWSTR Config(LPWSTR lpszConfigurationString) {
      void *param[1+1] = {(void*)lpszConfigurationString, 0};
      int cbparam[1+1] = {0, 0};
      IPWorksSSL_WebUploadS_Do(m_pObj, 10000+5, 1, param, cbparam);
      return (LPWSTR)IPS64CAST(param[1]);
    }
    inline int DoEvents() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_WebUploadS_Do(m_pObj, 10000+6, 0, param, cbparam);
      
    }
    inline int Interrupt() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_WebUploadS_Do(m_pObj, 10000+7, 0, param, cbparam);
      
    }
    inline int Reset() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_WebUploadS_Do(m_pObj, 10000+8, 0, param, cbparam);
      
    }
    inline int Upload() {
      void *param[0+1] = {0};
      int cbparam[0+1] = {0};
      return IPWorksSSL_WebUploadS_Do(m_pObj, 10000+9, 0, param, cbparam);
      
    }
    inline int UploadTo(LPWSTR lpszURL) {
      void *param[1+1] = {(void*)lpszURL, 0};
      int cbparam[1+1] = {0, 0};
      return IPWorksSSL_WebUploadS_Do(m_pObj, 10000+10, 1, param, cbparam);
      
    }

};

#endif //WIN32

#endif //_IPWORKSSSL_WEBUPLOADS_H_




